@@ -217,7 +217,7 @@ struct animmodel : model
 
         void preloadBIH()
         {
-            if(tex && tex->bpp==4 && !tex->alphamask) loadalphamask(tex);
+            if(tex && tex->type&Texture::ALPHA && !tex->alphamask) loadalphamask(tex);
         }
 
         void preloadshader()
@@ -280,7 +280,7 @@ struct animmodel : model
                 glBindTexture(GL_TEXTURE_2D, n->id);
                 glActiveTexture_(GL_TEXTURE0_ARB);
             }
-            if(s->bpp==4)
+            if(s->type&Texture::ALPHA)
             {
                 if(alphablend)
                 {
@@ -454,7 +454,7 @@ struct animmodel : model
                 u.mul(v1).sub(vec(e1).mul(v2));
                 v.mul(u1).sub(vec(e1).mul(u2));
 
-                if(vec().cross(e2, e1).dot(vec().cross(v, u)) < 0)
+                if(vec().cross(e2, e1).dot(vec().cross(v, u)) >= 0)
                 {
                     u.neg();
                     v.neg();
@@ -468,7 +468,7 @@ struct animmodel : model
 
                 loopj(3)
                 {
-                    tangent[t.vert[j]].add(u);
+                    tangent[t.vert[j]].sub(u);
                     bitangent[t.vert[j]].add(v);
                 }
             }
@@ -516,7 +516,7 @@ struct animmodel : model
 
         void gentris(int frame, vector<skin> &skins, vector<BIH::tri> *tris, const matrix3x4 &m)
         {
-            loopv(meshes) meshes[i]->gentris(frame, skins[i].tex && skins[i].tex->bpp == 4 ? skins[i].tex : NULL, tris, m);
+            loopv(meshes) meshes[i]->gentris(frame, skins[i].tex && skins[i].tex->type&Texture::ALPHA ? skins[i].tex : NULL, tris, m);
         }
 
         virtual int totalframes() const { return 1; }
@@ -687,11 +687,11 @@ struct animmodel : model
                 if(anims[animpart])
                 {
                     vector<animspec> &primary = anims[animpart][anim&ANIM_INDEX];
-                    if(&primary < &anims[animpart][NUMANIMS] && primary.length()) spec = &primary[uint(varseed + basetime)%primary.length()];
+                    if(&primary < &anims[animpart][game::numanims()] && primary.length()) spec = &primary[uint(varseed + basetime)%primary.length()];
                     if((anim>>ANIM_SECONDARY)&(ANIM_INDEX|ANIM_DIR))
                     {
                         vector<animspec> &secondary = anims[animpart][(anim>>ANIM_SECONDARY)&ANIM_INDEX];
-                        if(&secondary < &anims[animpart][NUMANIMS] && secondary.length())
+                        if(&secondary < &anims[animpart][game::numanims()] && secondary.length())
                         {
                             animspec &spec2 = secondary[uint(varseed + basetime2)%secondary.length()];
                             if(!spec || spec2.priority > spec->priority)
@@ -738,7 +738,7 @@ struct animmodel : model
             {
                 animinterpinfo &ai = d->animinterp[interp];
                 if((info.anim&ANIM_CLAMP)==ANIM_CLAMP) aitime = min(aitime, int(info.range*info.speed*0.5e-3f));
-                if(d->ragdoll && !(anim&ANIM_RAGDOLL)) 
+                if(d->ragdoll && !(anim&ANIM_RAGDOLL))
                 {
                     ai.prev.range = ai.cur.range = 0;
                     ai.lastswitch = -1;
@@ -879,7 +879,7 @@ struct animmodel : model
                 conoutf("invalid frame %d, range %d in model %s", frame, range, model->loadname);
                 return;
             }
-            if(!anims[animpart]) anims[animpart] = new vector<animspec>[NUMANIMS];
+            if(!anims[animpart]) anims[animpart] = new vector<animspec>[game::numanims()];
             animspec &spec = anims[animpart][num].add();
             spec.frame = frame;
             spec.range = range;
@@ -1410,7 +1410,7 @@ template<class MDL> struct modelloader
 };
 
 template<class MDL> MDL *modelloader<MDL>::loading = NULL;
-template<class MDL> string modelloader<MDL>::dir = "";
+template<class MDL> string modelloader<MDL>::dir = {'\0'}; // crashes clang if "" is used here
 
 template<class MDL, class MESH> struct modelcommands
 {
