@@ -8,7 +8,8 @@ vector<cline> conlines;
 int commandmillis = -1;
 string commandbuf;
 char *commandaction = NULL, *commandprompt = NULL;
-int commandpos = -1;
+enum { CF_COMPLETE = 1<<0, CF_EXECUTE = 1<<1 };
+int commandflags = 0, commandpos = -1;
 
 VARFP(maxcon, 10, 200, 1000, { while(conlines.length() > maxcon) delete[] conlines.pop().line; });
 
@@ -235,9 +236,9 @@ void bindkey(char *key, char *action, int state, const char *cmd)
     char *&binding = km->actions[state];
     if(!keypressed || keyaction!=binding) delete[] binding;
     // trim white-space to make searchbinds more reliable
-    while(isspace(*action)) action++;
+    while(iscubespace(*action)) action++;
     int len = strlen(action);
-    while(len>0 && isspace(action[len-1])) len--;
+    while(len>0 && iscubespace(action[len-1])) len--;
     binding = newstring(action, len);
 }
 
@@ -251,7 +252,7 @@ ICOMMAND(searchbinds,     "s", (char *action), searchbinds(action, keym::ACTION_
 ICOMMAND(searchspecbinds, "s", (char *action), searchbinds(action, keym::ACTION_SPECTATOR));
 ICOMMAND(searcheditbinds, "s", (char *action), searchbinds(action, keym::ACTION_EDITING));
 
-void inputcommand(char *init, char *action = NULL, char *prompt = NULL) // turns input to the command line on or off
+void inputcommand(char *init, char *action = NULL, char *prompt = NULL, char *flags = NULL) // turns input to the command line on or off
 {
     commandmillis = init ? totalmillis : -1;
     SDL_EnableUNICODE(commandmillis >= 0 ? 1 : 0);
@@ -262,58 +263,97 @@ void inputcommand(char *init, char *action = NULL, char *prompt = NULL) // turns
     commandpos = -1;
     if(action && action[0]) commandaction = newstring(action);
     if(prompt && prompt[0]) commandprompt = newstring(prompt);
+    commandflags = 0;
+    if(flags) while(*flags) switch(*flags++)
+    {
+        case 'c': commandflags |= CF_COMPLETE; break;
+        case 'x': commandflags |= CF_EXECUTE; break;
+        case 's': commandflags |= CF_COMPLETE|CF_EXECUTE; break;
+    }
+    else if(init) commandflags |= CF_COMPLETE|CF_EXECUTE;
 }
 
 ICOMMAND(saycommand, "C", (char *init), inputcommand(init));
-COMMAND(inputcommand, "sss");
+COMMAND(inputcommand, "ssss");
 
 #if !defined(WIN32) && !defined(__APPLE__)
 #include <X11/Xlib.h>
 #include <SDL_syswm.h>
 #endif
 
-void pasteconsole()
-{
-    #ifdef WIN32
-    if(!IsClipboardFormatAvailable(CF_TEXT)) return;
+ void pasteconsole()
+ {
+#ifdef WIN32
+    UINT fmt = CF_UNICODETEXT;
+    if(!IsClipboardFormatAvailable(fmt))
+    {
+        fmt = CF_TEXT;
+        if(!IsClipboardFormatAvailable(fmt)) return;
+    }
     if(!OpenClipboard(NULL)) return;
-    char *cb = (char *)GlobalLock(GetClipboardData(CF_TEXT));
-    concatstring(commandbuf, cb);
+    HANDLE h = GetClipboardData(fmt);
+    size_t commandlen = strlen(commandbuf);
+    int cblen = int(GlobalSize(h)), decoded = 0;
+    ushort *cb = (ushort *)GlobalLock(h);
+    switch(fmt)
+    {
+        case CF_UNICODETEXT:
+            decoded = min(int(sizeof(commandbuf)-1-commandlen), cblen/2);
+            loopi(decoded) commandbuf[commandlen++] = uni2cube(cb[i]);
+            break;
+        case CF_TEXT:
+            decoded = min(int(sizeof(commandbuf)-1-commandlen), cblen);
+            memcpy(&commandbuf[commandlen], cb, decoded);
+            break;
+    }
+    commandbuf[commandlen + decoded] = '\0';
     GlobalUnlock(cb);
     CloseClipboard();
-    #elif defined(__APPLE__)
-	extern void mac_pasteconsole(char *commandbuf);
-
-	mac_pasteconsole(commandbuf);
-	#else
+#elif defined(__APPLE__)
+    extern char *mac_pasteconsole(int *cblen);
+    int cblen = 0;
+    uchar *cb = (uchar *)mac_pasteconsole(&cblen);
+    if(!cb) return;
+    size_t commandlen = strlen(commandbuf);
+    int decoded = decodeutf8((uchar *)&commandbuf[commandlen], int(sizeof(commandbuf)-1-commandlen), cb, cblen);
+    commandbuf[commandlen + decoded] = '\0';
+    free(cb);
+    #else
     SDL_SysWMinfo wminfo;
     SDL_VERSION(&wminfo.version);
     wminfo.subsystem = SDL_SYSWM_X11;
     if(!SDL_GetWMInfo(&wminfo)) return;
     int cbsize;
-    char *cb = XFetchBytes(wminfo.info.x11.display, &cbsize);
+    uchar *cb = (uchar *)XFetchBytes(wminfo.info.x11.display, &cbsize);
     if(!cb || !cbsize) return;
     size_t commandlen = strlen(commandbuf);
-    for(char *cbline = cb, *cbend; commandlen + 1 < sizeof(commandbuf) && cbline < &cb[cbsize]; cbline = cbend + 1)
+    for(uchar *cbline = cb, *cbend; commandlen + 1 < sizeof(commandbuf) && cbline < &cb[cbsize]; cbline = cbend + 1)
     {
-        cbend = (char *)memchr(cbline, '\0', &cb[cbsize] - cbline);
+        cbend = (uchar *)memchr(cbline, '\0', &cb[cbsize] - cbline);
         if(!cbend) cbend = &cb[cbsize];
-        if(size_t(commandlen + cbend - cbline + 1) > sizeof(commandbuf)) cbend = cbline + sizeof(commandbuf) - commandlen - 1;
-        memcpy(&commandbuf[commandlen], cbline, cbend - cbline);
-        commandlen += cbend - cbline;
+        int cblen = int(cbend-cbline), commandmax = int(sizeof(commandbuf)-1-commandlen);
+        loopi(cblen) if((cbline[i]&0xC0) == 0x80)
+        {
+            commandlen += decodeutf8((uchar *)&commandbuf[commandlen], commandmax, cbline, cblen);
+            goto nextline;
+        }
+        cblen = min(cblen, commandmax);
+        loopi(cblen) commandbuf[commandlen++] = uni2cube(*cbline++);
+    nextline:
         commandbuf[commandlen] = '\n';
         if(commandlen + 1 < sizeof(commandbuf) && cbend < &cb[cbsize]) ++commandlen;
         commandbuf[commandlen] = '\0';
     }
     XFree(cb);
-    #endif
+#endif
 }
 
 struct hline
 {
     char *buf, *action, *prompt;
+    int flags;
 
-    hline() : buf(NULL), action(NULL), prompt(NULL) {}
+    hline() : buf(NULL), action(NULL), prompt(NULL), flags(0) {}
     ~hline()
     {
         DELETEA(buf);
@@ -329,13 +369,15 @@ struct hline
         DELETEA(commandprompt);
         if(action) commandaction = newstring(action);
         if(prompt) commandprompt = newstring(prompt);
+        commandflags = flags;
     }
 
     bool shouldsave()
     {
         return strcmp(commandbuf, buf) ||
                (commandaction ? !action || strcmp(commandaction, action) : action!=NULL) ||
-               (commandprompt ? !prompt || strcmp(commandprompt, prompt) : prompt!=NULL);
+               (commandprompt ? !prompt || strcmp(commandprompt, prompt) : prompt!=NULL) ||
+               commandflags != flags;
     }
 
     void save()
@@ -343,16 +385,17 @@ struct hline
         buf = newstring(commandbuf);
         if(commandaction) action = newstring(commandaction);
         if(commandprompt) prompt = newstring(commandprompt);
+        flags = commandflags;
     }
 
     void run()
     {
-        if(action)
+        if(flags&CF_EXECUTE && buf[0]=='/') execute(buf+1);
+        else if(action)
         {
             alias("commandbuf", buf);
             execute(action);
         }
-        else if(buf[0]=='/') execute(buf+1);
         else game::toserver(buf);
     }
 };
@@ -412,11 +455,7 @@ void execbind(keym &k, bool isdown)
     if(isdown)
     {
         int state = keym::ACTION_DEFAULT;
-		#ifndef NEWGUI
-        if(!mainmenu)
-		#else
-		if(!UI::mainmenu)
-		#endif
+        if(!UI::mainmenu)
         {
             if(editmode) state = keym::ACTION_EDITING;
             else if(player->state==CS_SPECTATOR) state = keym::ACTION_SPECTATOR;
@@ -495,9 +534,9 @@ void consolekey(int code, bool isdown, int cooked)
                 break;
 
             case SDLK_TAB:
-                if(!commandaction)
+                if(commandflags&CF_COMPLETE)
                 {
-                    complete(commandbuf);
+                    complete(commandbuf, commandflags&CF_EXECUTE ? "/" : NULL);
                     if(commandpos>=0 && commandpos>=(int)strlen(commandbuf)) commandpos = -1;
                 }
                 break;
@@ -555,19 +594,11 @@ void consolekey(int code, bool isdown, int cooked)
     }
 }
 
-#ifndef NEWGUI
-extern bool menukey(int code, bool isdown, int cooked);
-#endif
-
 void keypress(int code, bool isdown, int cooked)
 {
     keym *haskey = keyms.access(code);
     if(haskey && haskey->pressed) execbind(*haskey, isdown); // allow pressed keys to release
-    #ifdef NEWGUI
-    else if(!UI::keypress(code, isdown, cooked)) // 3D GUI mouse button intercept
-    #else
-    else if(!menukey(code, isdown, cooked)) // 3D GUI mouse button intercept
-    #endif
+    else if(!UI::keypress(code, isdown, cooked)) // UI mouse button intercept
     {
         if(commandmillis >= 0) consolekey(code, isdown, cooked);
         else if(haskey) execbind(*haskey, isdown);
@@ -595,7 +626,11 @@ void writebinds(stream *f)
         loopv(binds)
         {
             keym &km = *binds[i];
-            if(*km.actions[j]) f->printf("%s \"%s\" [%s]\n", cmds[j], km.name, km.actions[j]);
+            if(*km.actions[j])
+            {
+                if(validateblock(km.actions[j])) f->printf("%s %s [%s]\n", cmds[j], escapestring(km.name), km.actions[j]);
+                else f->printf("%s %s %s\n", cmds[j], escapestring(km.name), escapestring(km.actions[j]));
+            }
         }
     }
 }
@@ -705,26 +740,29 @@ void addlistcomplete(char *command, char *list)
 COMMANDN(complete, addfilecomplete, "sss");
 COMMANDN(listcomplete, addlistcomplete, "ss");
 
-void complete(char *s)
+void complete(char *s, const char *cmdprefix)
 {
-    if(*s!='/')
+    int cmdlen = 0;
+    if(cmdprefix)
     {
-        string t;
-        copystring(t, s);
-        copystring(s, "/");
-        concatstring(s, t);
+        cmdlen = strlen(cmdprefix);
+        if(strncmp(s, cmdprefix, cmdlen))
+        {
+            defformatstring(cmd)("%s%s", cmdprefix, s);
+            copystring(s, cmd);
+        }
     }
-    if(!s[1]) return;
-    if(!completesize) { completesize = (int)strlen(s)-1; lastcomplete[0] = '\0'; }
+    if(!s[cmdlen]) return;
+    if(!completesize) { completesize = (int)strlen(&s[cmdlen]); lastcomplete[0] = '\0'; }
 
     filesval *f = NULL;
     if(completesize)
     {
-        char *end = strchr(s, ' ');
+        char *end = strchr(&s[cmdlen], ' ');
         if(end)
         {
             string command;
-            copystring(command, s+1, min(size_t(end-s), sizeof(command)));
+            copystring(command, &s[cmdlen], min(size_t(end-&s[cmdlen]+1), sizeof(command)));
             filesval **hasfiles = completions.access(command);
             if(hasfiles) f = *hasfiles;
         }
@@ -732,31 +770,30 @@ void complete(char *s)
 
     const char *nextcomplete = NULL;
     string prefix;
-    copystring(prefix, "/");
     if(f) // complete using filenames
     {
-        int commandsize = strchr(s, ' ')+1-s;
+        int commandsize = strchr(&s[cmdlen], ' ')+1-s;
         copystring(prefix, s, min(size_t(commandsize+1), sizeof(prefix)));
         f->update();
         loopv(f->files)
         {
-            if(strncmp(f->files[i], s+commandsize, completesize+1-commandsize)==0 &&
+            if(strncmp(f->files[i], &s[commandsize], completesize+cmdlen-commandsize)==0 &&
                strcmp(f->files[i], lastcomplete) > 0 && (!nextcomplete || strcmp(f->files[i], nextcomplete) < 0))
                 nextcomplete = f->files[i];
         }
     }
     else // complete using command names
     {
+        if(cmdprefix) copystring(prefix, cmdprefix); else prefix[0] = '\0';
         enumerate(idents, ident, id,
-            if(strncmp(id.name, s+1, completesize)==0 &&
+            if(strncmp(id.name, &s[cmdlen], completesize)==0 &&
                strcmp(id.name, lastcomplete) > 0 && (!nextcomplete || strcmp(id.name, nextcomplete) < 0))
                 nextcomplete = id.name;
         );
     }
     if(nextcomplete)
     {
-        copystring(s, prefix);
-        concatstring(s, nextcomplete);
+        formatstring(s)("%s%s", prefix, nextcomplete);
         copystring(lastcomplete, nextcomplete);
     }
     else lastcomplete[0] = '\0';
@@ -776,8 +813,12 @@ void writecompletions(stream *f)
     {
         char *k = cmds[i];
         filesval *v = completions[k];
-        if(v->type==FILES_LIST) f->printf("listcomplete \"%s\" [%s]\n", k, v->dir);
-        else f->printf("complete \"%s\" \"%s\" \"%s\"\n", k, v->dir, v->ext ? v->ext : "*");
+        if(v->type==FILES_LIST)
+        {
+            if(validateblock(v->dir)) f->printf("listcomplete %s [%s]\n", escapeid(k), v->dir);
+            else f->printf("listcomplete %s %s\n", escapeid(k), escapestring(v->dir));
+        }
+        else f->printf("complete %s %s %s\n", escapeid(k), escapestring(v->dir), escapestring(v->ext ? v->ext : "*"));
     }
 }
 
