@@ -21,7 +21,7 @@
 #endif
 #endif
 
-#define CONFVERSION 9
+#define CONFVERSION 10
 
 // GL_ARB_multitexture
 extern PFNGLACTIVETEXTUREARBPROC       glActiveTexture_;
@@ -138,11 +138,14 @@ struct font
     char *name;
     vector<Texture *> texs;
     vector<charinfo> chars;
-    int charoffset, defaultw, defaulth;
+    int charoffset, defaultw, defaulth, scale;
+
+    font() : name(NULL) {}
+    ~font() { DELETEA(name); }
 };
 
-#define FONTH (curfont->defaulth)
-#define FONTW (curfont->defaultw)
+#define FONTH (curfont->scale)
+#define FONTW (FONTH/2)
 #define MINRESW 640
 #define MINRESH 480
 
@@ -160,7 +163,8 @@ extern void drawcubemap(int size, const vec &o, float yaw, float pitch, const cu
 extern void loadshaders();
 extern void setuptexparameters(int tnum, void *pixels, int clamp, int filter, GLenum format = GL_RGB, GLenum target = GL_TEXTURE_2D);
 extern void createtexture(int tnum, int w, int h, void *pixels, int clamp, int filter, GLenum component = GL_RGB, GLenum target = GL_TEXTURE_2D, int pw = 0, int ph = 0, int pitch = 0, bool resize = true, GLenum format = GL_FALSE);
-extern void blurtexture(int n, int bpp, int w, int h, uchar *dst, const uchar *src);
+extern void blurtexture(int n, int bpp, int w, int h, uchar *dst, const uchar *src, int margin = 0);
+extern void blurnormals(int n, int w, int h, bvec *dst, const bvec *src, int margin = 0);
 extern void renderpostfx();
 extern void initenvmaps();
 extern void genenvmaps();
@@ -261,7 +265,7 @@ extern void resetclipplanes();
 extern int getmippedtexture(cube &p, int orient);
 extern void forcemip(cube &c, bool fixtex = true);
 extern bool subdividecube(cube &c, bool fullcheck=true, bool brighten=true);
-extern void converttovectorworld();
+extern void edgespan2vectorcube(cube &c);
 extern int faceconvexity(ivec v[4]);
 extern int faceconvexity(ivec v[4], int &vis);
 extern int faceconvexity(vertinfo *verts, int numverts);
@@ -278,7 +282,6 @@ extern void genclipplanes(cube &c, int x, int y, int z, int size, clipplanes &p)
 extern bool visibleface(cube &c, int orient, int x, int y, int z, int size, uchar mat = MAT_AIR, uchar nmat = MAT_AIR, uchar matmask = MATF_VOLUME);
 extern int visibletris(cube &c, int orient, int x, int y, int z, int size, uchar nmat = MAT_AIR, uchar matmask = MAT_AIR);
 extern int visibleorient(cube &c, int orient);
-extern bool threeplaneintersect(plane &pl1, plane &pl2, plane &pl3, vec &dest);
 extern void genfaceverts(cube &c, int orient, ivec v[4]);
 extern int calcmergedsize(int orient, const ivec &co, int size, const vertinfo *verts, int numverts);
 extern void invalidatemerges(cube &c, const ivec &co, int size, bool msg);
@@ -293,11 +296,11 @@ static inline uchar octantrectangleoverlap(const ivec &c, int size, const ivec &
     ivec v(c);
     v.add(size);
     if(v.z <= o.z)     p &= 0xF0; // not in a -ve Z octant
-    if(v.z >= o.z+s.z) p &= 0x0F; // not in a +ve Z octant
+    else if(v.z >= o.z+s.z) p &= 0x0F; // not in a +ve Z octant
     if(v.y <= o.y)     p &= 0xCC; // not in a -ve Y octant
-    if(v.y >= o.y+s.y) p &= 0x33; // etc..
+    else if(v.y >= o.y+s.y) p &= 0x33; // etc..
     if(v.x <= o.x)     p &= 0xAA;
-    if(v.x >= o.x+s.x) p &= 0x55;
+    else if(v.x >= o.x+s.x) p &= 0x55;
     return p;
 }
 
@@ -385,7 +388,6 @@ extern void rendermatgrid(materialsurface *matbuf, int matsurfs);
 extern int optimizematsurfs(materialsurface *matbuf, int matsurfs);
 extern void setupmaterials(int start = 0, int len = 0);
 extern void rendermaterials();
-extern void drawmaterial(int orient, int x, int y, int z, int csize, int rsize, float offset);
 extern int visiblematerial(cube &c, int orient, int x, int y, int z, int size, uchar matmask = MATF_VOLUME);
 
 // water
@@ -442,7 +444,7 @@ extern void clientkeepalive();
 extern hashset<ident> idents;
 extern int identflags;
 
-extern void explodelist(const char *s, vector<char *> &elems);
+extern void explodelist(const char *s, vector<char *> &elems, int limit = -1);
 extern char *indexlist(const char *s, int pos);
 
 extern void clearoverrides();
@@ -458,7 +460,7 @@ extern int renderconsole(int w, int h, int abovehud);
 extern void conoutf(const char *s, ...);
 extern void conoutf(int type, const char *s, ...);
 extern void resetcomplete();
-extern void complete(char *s);
+extern void complete(char *s, const char *cmdprefix);
 const char *getkeyname(int code);
 extern const char *addreleaseaction(char *s);
 extern void writebinds(stream *f);
@@ -495,9 +497,6 @@ extern void swapbuffers();
 extern int getclockmillis();
 
 // menu
-#ifndef NEWGUI
-extern void menuprocess();
-#endif
 extern void addchange(const char *desc, int type);
 extern void clearchanges(int type);
 
@@ -533,7 +532,7 @@ struct mapmodelinfo { string name; model *m; };
 
 extern void findanims(const char *pattern, vector<int> &anims);
 extern void loadskin(const char *dir, const char *altdir, Texture *&skin, Texture *&masks);
-extern mapmodelinfo &getmminfo(int i);
+extern mapmodelinfo *getmminfo(int i);
 extern void startmodelquery(occludequery *query);
 extern void endmodelquery();
 extern void preloadmodelshaders();
@@ -575,18 +574,7 @@ extern double skyarea;
 extern void drawskybox(int farplane, bool limited);
 extern bool limitsky();
 
-// 3dgui
-#ifndef NEWGUI
-extern int mainmenu;
-extern void clearmainmenu();
-extern void g3d_render();
-extern bool g3d_windowhit(bool on, bool act);
-
-// menus
-extern void g3d_mainmenu();
-#else
-
-//new UI by neal
+//new UI by eihrul, courtesy of neal
 namespace UI
 {
     extern void resetcursor();
@@ -600,7 +588,6 @@ namespace UI
     extern void clearmainmenu();
     extern int mainmenu;
 }
-#endif
 
 // sound
 extern void writemapsounds(stream *f);
