@@ -41,11 +41,7 @@ static void inithemisphere(int hres, int depth)
     hemiverts = new vec[tris+1];
     hemiindices = new GLushort[tris*3];
     hemiverts[heminumverts++] = vec(0.0f, 0.0f, 1.0f); //build initial 'hres' sided pyramid
-    loopi(hres)
-    {
-        float a = PI2*float(i)/hres;
-        hemiverts[heminumverts++] = vec(cosf(a), sinf(a), 0.0f);
-    }
+    loopi(hres) hemiverts[heminumverts++] = vec(sincos360[(360*i)/hres], 0.0f);
     loopi(hres) genface(depth, 0, i+1, 1+(i+1)%hres);
 
     if(hasVBO)
