@@ -519,10 +519,10 @@ static uint generatelumel(lightmapworker *w, const float tolerance, uint lightma
 {
     vec avgray(0, 0, 0);
     float r = 0, g = 0, b = 0;
-	uint lightused = 0;
+    uint lightused = 0;
     loopv(lights)
     {
-		if(lightmask&(1<<i)) continue;
+        if(lightmask&(1<<i)) continue;
         const extentity &light = *lights[i];
         vec ray = target;
         ray.sub(light.o);
@@ -539,9 +539,8 @@ static uint generatelumel(lightmapworker *w, const float tolerance, uint lightma
         if(angle <= 0) continue;
         if(light.attached && light.attached->type==ET_SPOTLIGHT)
         {
-            vec spot(vec(light.attached->o).sub(light.o).normalize());
-            float maxatten = 1-cosf(max(1, min(90, int(light.attached->attr[0])))*RAD);
-            float spotatten = 1-(1-ray.dot(spot))/maxatten;
+            vec spot = vec(light.attached->o).sub(light.o).normalize();
+            float maxatten = sincos360[clamp(int(light.attached->attr[0]), 1, 89)].x, spotatten = (ray.dot(spot) - maxatten) / (1 - maxatten);
             if(spotatten <= 0) continue;
             attenuation *= spotatten;
         }
@@ -703,7 +702,7 @@ static inline void generatealpha(lightmapworker *w, float tolerance, const vec &
     }
 }
 
-VAR(edgetolerance, 1, 4, 8);
+VAR(edgetolerance, 1, 4, 64);
 VAR(adaptivesample, 0, 2, 2);
 
 enum
@@ -719,7 +718,7 @@ enum
 #define SURFACE_AMBIENT SURFACE_AMBIENT_BOTTOM
 #define SURFACE_LIGHTMAP SURFACE_LIGHTMAP_BOTTOM
 
-static bool generatelightmap(lightmapworker *w, float lpu, const lerpvert *lv, int numv, const vec &origin1, const vec &xstep1, const vec &ystep1, const vec &origin2, const vec &xstep2, const vec &ystep2, float side0, float sidestep)
+static bool generatelightmap(lightmapworker *w, float lpu, const lerpvert *lv, int numv, vec origin1, const vec &xstep1, const vec &ystep1, vec origin2, const vec &xstep2, const vec &ystep2, float side0, float sidestep)
 {
     static const float aacoords[8][2] =
     {
@@ -741,44 +740,54 @@ static bool generatelightmap(lightmapworker *w, float lpu, const lerpvert *lv, i
         offsets1[i] = vec(xstep1).mul(aacoords[i][0]).add(vec(ystep1).mul(aacoords[i][1]));
         offsets2[i] = vec(xstep2).mul(aacoords[i][0]).add(vec(ystep2).mul(aacoords[i][1]));
     }
-    if((w->type&LM_TYPE) == LM_BUMPMAP0) memset(w->raydata, 0, LM_MAXW*LM_MAXH*sizeof(vec));
+    if((w->type&LM_TYPE) == LM_BUMPMAP0) memset(w->raydata, 0, (LM_MAXW + 4)*(LM_MAXH + 4)*sizeof(vec));
+
+    origin1.sub(vec(ystep1).add(xstep1).mul(blurlms));
+    origin2.sub(vec(ystep2).add(xstep2).mul(blurlms));
 
     int aasample = min(1 << lmaa, 4);
     int stride = aasample*(w->w+1);
     vec *sample = w->colordata;
     uchar *skylight = w->ambient;
     lerpbounds start, end;
-    initlerpbounds(lv, numv, start, end);
-    float sidex = side0;
+    initlerpbounds(-blurlms, -blurlms, lv, numv, start, end);
+    float sidex = side0 + blurlms*sidestep;
     for(int y = 0; y < w->h; ++y, sidex += sidestep)
     {
         vec normal, nstep;
-        lerpnormal(y, lv, numv, start, end, normal, nstep);
+        lerpnormal(-blurlms, y - blurlms, lv, numv, start, end, normal, nstep);
 
         for(int x = 0; x < w->w; ++x, normal.add(nstep), skylight += w->bpp)
         {
+#define EDGE_TOLERANCE(x, y) \
+    (x < blurlms \
+     || x+1 > w->w - blurlms \
+     || y < blurlms \
+     || y+1 > w->h - blurlms \
+     ? edgetolerance : 1)
+            float t = EDGE_TOLERANCE(x, y) * tolerance;
             vec u = x < sidex ? vec(xstep1).mul(x).add(vec(ystep1).mul(y)).add(origin1) : vec(xstep2).mul(x).add(vec(ystep2).mul(y)).add(origin2);
-            lightused |= generatelumel(w, tolerance, 0, w->lights, u, vec(normal).normalize(), *sample, x, y);
+            lightused |= generatelumel(w, t, 0, w->lights, u, vec(normal).normalize(), *sample, x, y);
             if(hasskylight())
             {
                 if((w->type&LM_TYPE)==LM_BUMPMAP0 || !adaptivesample || sample->x<skylightcolor[0] || sample->y<skylightcolor[1] || sample->z<skylightcolor[2])
-                    calcskylight(w, u, normal, tolerance, skylight, lmshadows > 1 ? RAY_ALPHAPOLY : 0);
+                    calcskylight(w, u, normal, t, skylight, lmshadows > 1 ? RAY_ALPHAPOLY : 0);
                 else loopk(3) skylight[k] = max(skylightcolor[k], ambientcolor[k]);
             }
             else loopk(3) skylight[k] = ambientcolor[k];
-            if(w->type&LM_ALPHA) generatealpha(w, tolerance, u, skylight[3]);
+            if(w->type&LM_ALPHA) generatealpha(w, t, u, skylight[3]);
             sample += aasample;
         }
         sample += aasample;
     }
     if(adaptivesample > 1 && min(w->w, w->h) >= 2) lightmask = ~lightused;
     sample = w->colordata;
-    initlerpbounds(lv, numv, start, end);
-    sidex = side0;
+    initlerpbounds(-blurlms, -blurlms, lv, numv, start, end);
+    sidex = side0 + blurlms*sidestep;
     for(int y = 0; y < w->h; ++y, sidex += sidestep)
     {
         vec normal, nstep;
-        lerpnormal(y, lv, numv, start, end, normal, nstep);
+        lerpnormal(-blurlms, y - blurlms, lv, numv, start, end, normal, nstep);
 
         for(int x = 0; x < w->w; ++x, normal.add(nstep))
         {
@@ -787,23 +796,18 @@ static bool generatelightmap(lightmapworker *w, float lpu, const lerpvert *lv, i
                 loopi(aasample-1) *sample++ = center;
             else
             {
-#define EDGE_TOLERANCE(i) \
-    ((!x && aacoords[i][0] < 0) \
-     || (x+1==w->w && aacoords[i][0] > 0) \
-     || (!y && aacoords[i][1] < 0) \
-     || (y+1==w->h && aacoords[i][1] > 0) \
-     ? edgetolerance : 1)
+#define AA_EDGE_TOLERANCE(x, y, i) EDGE_TOLERANCE(x + aacoords[i][0], y + aacoords[i][1])
                 vec u = x < sidex ? vec(xstep1).mul(x).add(vec(ystep1).mul(y)).add(origin1) : vec(xstep2).mul(x).add(vec(ystep2).mul(y)).add(origin2);
                 const vec *offsets = x < sidex ? offsets1 : offsets2;
                 vec n = vec(normal).normalize();
                 loopi(aasample-1)
-                    generatelumel(w, EDGE_TOLERANCE(i+1) * tolerance, lightmask, w->lights, vec(u).add(offsets[i+1]), n, *sample++, x, y);
+                    generatelumel(w, AA_EDGE_TOLERANCE(x, y, i+1) * tolerance, lightmask, w->lights, vec(u).add(offsets[i+1]), n, *sample++, x, y);
                 if(lmaa == 3)
                 {
                     loopi(4)
                     {
                         vec s;
-                        generatelumel(w, EDGE_TOLERANCE(i+4) * tolerance, lightmask, w->lights, vec(u).add(offsets[i+4]), n, s, x, y);
+                        generatelumel(w, AA_EDGE_TOLERANCE(x, y, i+4) * tolerance, lightmask, w->lights, vec(u).add(offsets[i+4]), n, s, x, y);
                         center.add(s);
                     }
                     center.div(5);
@@ -815,7 +819,7 @@ static bool generatelightmap(lightmapworker *w, float lpu, const lerpvert *lv, i
             vec u = w->w < sidex ? vec(xstep1).mul(w->w).add(vec(ystep1).mul(y)).add(origin1) : vec(xstep2).mul(w->w).add(vec(ystep2).mul(y)).add(origin2);
             const vec *offsets = w->w < sidex ? offsets1 : offsets2;
             vec n = vec(normal).normalize();
-            generatelumel(w, tolerance, lightmask, w->lights, vec(u).add(offsets[1]), n, sample[1], w->w-1, y);
+            generatelumel(w, edgetolerance * tolerance, lightmask, w->lights, vec(u).add(offsets[1]), n, sample[1], w->w-1, y);
             if(aasample > 2)
                 generatelumel(w, edgetolerance * tolerance, lightmask, w->lights, vec(u).add(offsets[3]), n, sample[3], w->w-1, y);
         }
@@ -825,7 +829,7 @@ static bool generatelightmap(lightmapworker *w, float lpu, const lerpvert *lv, i
     if(aasample > 1)
     {
         vec normal, nstep;
-        lerpnormal(w->h, lv, numv, start, end, normal, nstep);
+        lerpnormal(-blurlms, w->h - blurlms, lv, numv, start, end, normal, nstep);
 
         for(int x = 0; x <= w->w; ++x, normal.add(nstep))
         {
@@ -854,9 +858,9 @@ static int finishlightmap(lightmapworker *w)
           cweight = weight * (lmaa == 3 ? 5.0f : 1.0f);
     uchar *skylight = w->ambient;
     vec *ray = w->raydata;
-    uchar *dstcolor = w->colorbuf;
+    uchar *dstcolor = blurlms && (w->w > 1 || w->h > 1) ? w->blur : w->colorbuf;
     uchar mincolor[4] = { 255, 255, 255, 255 }, maxcolor[4] = { 0, 0, 0, 0 };
-    bvec *dstray = w->raybuf;
+    bvec *dstray = blurlms && (w->w > 1 || w->h > 1) ? (bvec *)w->raydata : w->raybuf;
     bvec minray(255, 255, 255), maxray(0, 0, 0);
     loop(y, w->h)
     {
@@ -952,8 +956,10 @@ static int finishlightmap(lightmapworker *w)
     }
     if(blurlms && (w->w>1 || w->h>1))
     {
-        blurtexture(blurlms, w->bpp, w->w, w->h, w->blur, w->colorbuf);
-        memcpy(w->colorbuf, w->blur, w->bpp*w->w*w->h);
+        blurtexture(blurlms, w->bpp, w->w, w->h, w->colorbuf, w->blur, blurlms);
+        if((w->type&LM_TYPE) == LM_BUMPMAP0) blurnormals(blurlms, w->w, w->h, w->raybuf, (const bvec *)w->raydata, blurlms);
+        w->lastlightmap->w = (w->w -= 2*blurlms);
+        w->lastlightmap->h = (w->h -= 2*blurlms);
     }
     if(mincolor[3]==255) return SURFACE_LIGHTMAP_TOP;
     else if(maxcolor[3]==0) return SURFACE_LIGHTMAP_BOTTOM;
@@ -1350,6 +1356,11 @@ static int setupsurface(lightmapworker *w, plane planes[2], int numplanes, const
     int lw = clamp(int(ceil((cmax.x - cmin.x + 1)*lpu)), LM_MINW, LM_MAXW), lh = clamp(int(ceil((cmax.y - cmin.y + 1)*lpu)), LM_MINH, LM_MAXH);
     w->w = lw;
     w->h = lh;
+    if(!preview)
+    {
+        w->w += 2*blurlms;
+        w->h += 2*blurlms;
+    }
     if(!alloclightmap(w)) return NO_SURFACE;
 
     vec2 cscale = vec2(cmax).sub(cmin).div(vec2(lw-1, lh-1)),
@@ -1956,10 +1967,10 @@ lightmapworker::lightmapworker()
     buf = new uchar[LIGHTMAPBUFSIZE];
     bufstart = bufused = 0;
     firstlightmap = lastlightmap = curlightmaps = NULL;
-    ambient = new uchar[4*LM_MAXW*LM_MAXH];
-    blur = new uchar[4*LM_MAXW*LM_MAXH];
-    colordata = new vec[4*(LM_MAXW+1)*(LM_MAXH+1)];
-    raydata = new vec[LM_MAXW*LM_MAXH];
+    ambient = new uchar[4*(LM_MAXW + 4)*(LM_MAXH + 4)];
+    blur = new uchar[4*(LM_MAXW + 4)*(LM_MAXH + 4)];
+    colordata = new vec[4*(LM_MAXW+1 + 4)*(LM_MAXH+1 + 4)];
+    raydata = new vec[(LM_MAXW + 4)*(LM_MAXH + 4)];
     shadowraycache = newshadowraycache();
     blendmapcache = newblendmapcache();
     needspace = doneworking = false;
@@ -2095,11 +2106,11 @@ void calclight(int *quality)
         conoutf(CON_ERROR, "valid range for calclight quality is -1..1");
         return;
     }
-	extern string mname;
-	defformatstring(capt)("Sandbox %s: %s - Calculating Lightmaps...", version, mname);
-	SDL_WM_SetCaption(capt, NULL);
+    extern string mname;
+    defformatstring(capt)("Sandbox %s: %s - Calculating Lightmaps...", version, mname);
+    SDL_WM_SetCaption(capt, NULL);
 
-	renderbackground("computing lightmaps... (esc to abort)");
+    renderbackground("computing lightmaps... (esc to abort)");
 
     mpremip(true);
     optimizeblendmap();
@@ -2144,7 +2155,7 @@ void calclight(int *quality)
             lightmaps.length() ? lumels * 100.0 / (lightmaps.length() * LM_PACKW * LM_PACKH) : 0,
             lightmaps.length(),
             (end - start) / 1000.0f);
-	game::setwindowcaption();
+    game::setwindowcaption();
 }
 
 COMMAND(calclight, "i");
@@ -2159,10 +2170,10 @@ void patchlight(int *quality)
         conoutf(CON_ERROR, "valid range for patchlight quality is -1..1");
         return;
     }
-	extern string mname;
-	defformatstring(capt)("Sandbox %s: %s - Patching Lightmaps...", version, mname);
-	SDL_WM_SetCaption(capt, NULL);
-	renderbackground("patching lightmaps... (esc to abort)");
+    extern string mname;
+    defformatstring(capt)("Sandbox %s: %s - Patching Lightmaps...", version, mname);
+    SDL_WM_SetCaption(capt, NULL);
+    renderbackground("patching lightmaps... (esc to abort)");
     loadlayermasks();
     if(lightthreads > 1) preloadusedmapmodels(false, true);
     cleanuplightmaps();
@@ -2226,7 +2237,7 @@ void setfullbrightlevel(int fullbrightlevel)
 {
     if(lightmaptexs.length() > LMID_BRIGHT)
     {
-        uchar bright[3] = { fullbrightlevel, fullbrightlevel, fullbrightlevel };
+        uchar bright[3] = { uchar(fullbrightlevel), uchar(fullbrightlevel), uchar(fullbrightlevel) };
         createtexture(lightmaptexs[LMID_BRIGHT].id, 1, 1, bright, 0, 1);
     }
     initlights();
@@ -2404,7 +2415,7 @@ void genreservedlightmaptexs()
     createtexture(lightmaptexs[LMID_AMBIENT].id, 1, 1, unlit, 0, 1);
     bvec front(128, 128, 255);
     createtexture(lightmaptexs[LMID_AMBIENT1].id, 1, 1, &front, 0, 1);
-    uchar bright[3] = { fullbrightlevel, fullbrightlevel, fullbrightlevel };
+    uchar bright[3] = { uchar(fullbrightlevel), uchar(fullbrightlevel), uchar(fullbrightlevel) };
     createtexture(lightmaptexs[LMID_BRIGHT].id, 1, 1, bright, 0, 1);
     createtexture(lightmaptexs[LMID_BRIGHT1].id, 1, 1, &front, 0, 1);
     uchar dark[3] = { 0, 0, 0 };
@@ -2647,17 +2658,21 @@ void lightreaching(const vec &target, vec &color, vec &dir, bool fast, extentity
             continue;
 
         ray.div(mag);
-        if(shadowray(e.o, ray, mag, RAY_SHADOW | RAY_POLY, t) < mag)
-            continue;
+        if(mag < 1e-4f) ray = vec(0, 0, -1);
+        else
+        {
+            ray.div(mag);
+            if(shadowray(e.o, ray, mag, RAY_SHADOW | RAY_POLY, t) < mag)
+                continue;
+        }
         float intensity = 1;
         if(e.attr[0])
             intensity -= mag / float(e.attr[0]);
         if(e.attached && e.attached->type==ET_SPOTLIGHT)
         {
-            vec spot(vec(e.attached->o).sub(e.o).normalize());
-            float maxatten = 1-cosf(max(1, min(90, int(e.attached->attr[0])))*RAD);
-            float spotatten = 1-(1-ray.dot(spot))/maxatten;
-            if(spotatten<=0) continue;
+            vec spot = vec(e.attached->o).sub(e.o).normalize();
+            float maxatten = sincos360[clamp(int(e.attached->attr[0]), 1, 89)].x, spotatten = (ray.dot(spot) - maxatten) / (1 - maxatten);
+            if(spotatten <= 0) continue;
             intensity *= spotatten;
         }
 
@@ -2666,18 +2681,16 @@ void lightreaching(const vec &target, vec &color, vec &dir, bool fast, extentity
         //    conoutf(CON_DEBUG, "%d - %f %f", i, intensity, mag);
         //}
 
-        color.add(vec(e.attr[1], e.attr[2], e.attr[3]).mul(intensity/255));
-
-        intensity *= e.attr[1]*e.attr[2]*e.attr[3];
-
-        if(fabs(mag)<1e-3) dir.add(vec(0, 0, 1));
-        else dir.add(vec(e.o).sub(target).mul(intensity/mag));
+        vec lightcol = vec(e.attr[1], e.attr[2], e.attr[3]).mul(1.0f/255);
+        color.add(vec(lightcol).mul(intensity));
+        dir.add(vec(ray).mul(-intensity*lightcol.x*lightcol.y*lightcol.z));
     }
 
     if(sunlight && shadowray(target, sunlightdir, 1e16f, RAY_SHADOW | RAY_POLY | (skytexturelight ? RAY_SKIPSKY : 0), t) > 1e15f)
     {
-        color.add(vec(sunlightcolor.x, sunlightcolor.y, sunlightcolor.z).mul(sunlightscale/255));
-        dir.add(sunlightdir);
+        vec lightcol = vec(sunlightcolor.x, sunlightcolor.y, sunlightcolor.z).mul(sunlightscale/255);
+        color.add(lightcol);
+        dir.add(vec(sunlightdir).mul(lightcol.x*lightcol.y*lightcol.z));
     }
     if(hasskylight())
     {
@@ -2719,10 +2732,9 @@ entity *brightestlight(const vec &target, const vec &dir)
             intensity -= mag / float(e.attr[0]);
         if(e.attached && e.attached->type==ET_SPOTLIGHT)
         {
-            vec spot(vec(e.attached->o).sub(e.o).normalize());
-            float maxatten = 1-cosf(max(1, min(90, int(e.attached->attr[0])))*RAD);
-            float spotatten = 1-(1-ray.dot(spot))/maxatten;
-            if(spotatten<=0) continue;
+            vec spot = vec(e.attached->o).sub(e.o).normalize();
+            float maxatten = sincos360[clamp(int(e.attached->attr[0]), 1, 89)].x, spotatten = (ray.dot(spot) - maxatten) / (1 - maxatten);
+            if(spotatten <= 0) continue;
             intensity *= spotatten;
         }
 
