@@ -117,8 +117,8 @@ extern void calcnormals(bool lerptjoints = false);
 extern void clearnormals();
 extern void findnormal(const vec &key, const vec &surface, vec &v);
 extern void calclerpverts(const vec2 *c, const vec *n, lerpvert *lv, int &numv);
-extern void initlerpbounds(const lerpvert *lv, int numv, lerpbounds &start, lerpbounds &end);
-extern void lerpnormal(float v, const lerpvert *lv, int numv, lerpbounds &start, lerpbounds &end, vec &normal, vec &nstep);
+extern void initlerpbounds(float u, float v, const lerpvert *lv, int numv, lerpbounds &start, lerpbounds &end);
+extern void lerpnormal(float u, float v, const lerpvert *lv, int numv, lerpbounds &start, lerpbounds &end, vec &normal, vec &nstep);
 
 #define CHECK_CALCLIGHT_PROGRESS_LOCKED(exit, show_calclight_progress, before, after) \
     if(check_calclight_progress) \
