@@ -12,7 +12,7 @@ struct model
     int batch;
 
     model() : spinyaw(0), spinpitch(0), spinroll(0), offsetyaw(0), offsetpitch(0), offsetroll(0), collide(true), ellipsecollide(false), shadow(true), alphadepth(true), depthoffset(false), scale(1.0f), translate(0, 0, 0), bih(0), bbcenter(0, 0, 0), bbradius(0, 0, 0), bbextend(0, 0, 0), eyeheight(0.9f), collideradius(0), collideheight(0), batch(-1) {}
-    
+
     virtual ~model() { DELETEP(bih); }
     virtual void calcbb(int frame, vec &center, vec &radius) = 0;
     virtual void render(int anim, int basetime, int basetime2, const vec &o, float yaw, float pitch, float roll, dynent *d, modelattach *a = NULL, const vec &color = vec(0, 0, 0), const vec &dir = vec(0, 0, 0), float transparent = 1) = 0;
@@ -21,12 +21,13 @@ struct model
     virtual int type() const = 0;
     virtual BIH *setBIH() { return 0; }
     virtual bool envmapped() { return false; }
+    virtual bool skeletal() const { return false; }
 
     virtual void setshader(Shader *shader) {}
     virtual void setenvmap(float envmapmin, float envmapmax, Texture *envmap) {}
     virtual void setspec(float spec) {}
     virtual void setambient(float ambient) {}
-    virtual void setglow(float glow) {}
+    virtual void setglow(float glow, float glowdelta, float glowpulse) {}
     virtual void setglare(float specglare, float glowglare) {}
     virtual void setalphatest(float alpha) {}
     virtual void setalphablend(bool blend) {}
@@ -40,9 +41,6 @@ struct model
     virtual void startrender() {}
     virtual void endrender() {}
 
-    //__tha__ animations
-    virtual int numanims() = 0;
-
     void boundbox(int frame, vec &center, vec &radius)
     {
         if(frame) calcbb(frame, center, radius);
