@@ -444,7 +444,7 @@ struct aviwriter
         srcw &= ~1;
         srch &= ~1;
         const uint wfrac = (srcw<<12)/videow, hfrac = (srch<<12)/videoh,
-                   area = ((unsigned long long int)planesize<<12)/(srcw*srch + 1),
+                   area = ((ullong)planesize<<12)/(srcw*srch + 1),
                    dw = videow*wfrac, dh = videoh*hfrac;
 
         for(uint y = 0; y < dh;)
@@ -821,7 +821,7 @@ namespace recorder
     static SDL_cond *shouldencode = NULL, *shouldread = NULL;
 
     bool isrecording() { return file != NULL; }
-    
+
     float calcquality()
     {
         return 1.0f - float(dps)/float(dps+file->videofps); // strictly speaking should lock to read dps - 1.0=perfect, 0.5=half of frames are beingdropped
