@@ -94,6 +94,7 @@ void freecubeext(cube &c)
 
 void discardchildren(cube &c, bool fixtex, int depth)
 {
+    c.material = MAT_AIR;
     c.visible = 0;
     c.collide = 0;
     c.merged = 0;
@@ -297,9 +298,7 @@ cube &neighbourcube(cube &c, int orient, int x, int y, int z, int size, ivec &ro
 int getmippedtexture(cube &p, int orient)
 {
     cube *c = p.children;
-    int d = dimension(orient);
-    int dc = dimcoord(orient);
-    int tex[4] = { -1, -1, -1, -1 };
+    int d = dimension(orient), dc = dimcoord(orient), texs[4] = { -1, -1, -1, -1 }, numtexs = 0;
     loop(x, 2) loop(y, 2)
     {
         int n = octaindex(d, x, y, dc);
@@ -309,10 +308,11 @@ int getmippedtexture(cube &p, int orient)
             if(isempty(c[n]))
                 continue;
         }
-        loopk(3) if(tex[k] == c[n].texture[orient]) return tex[k];
-        if(c[n].texture[orient] > DEFAULT_SKY) tex[x*2+y] = c[n].texture[orient];
+        int tex = c[n].texture[orient];
+        if(tex > DEFAULT_SKY) loopi(numtexs) if(texs[i] == tex) return tex;
+        texs[numtexs++] = tex;
     }
-    loopk(4) if(tex[k] > DEFAULT_SKY) return tex[k];
+    loopirev(numtexs) if(!i || texs[i] > DEFAULT_SKY) return texs[i];
     return DEFAULT_GEOM;
 }
 
@@ -338,10 +338,9 @@ void forcemip(cube &c, bool fixtex)
         c.texture[j] = getmippedtexture(c, j);
 }
 
-int midedge(const ivec &a, const ivec &b, int xd, int yd, bool &perfect)
+static int midedge(const ivec &a, const ivec &b, int xd, int yd, bool &perfect)
 {
-    int ax = a[xd], bx = b[xd];
-    int ay = a[yd], by = b[yd];
+    int ax = a[xd], ay = a[yd], bx = b[xd], by = b[yd];
     if(ay==by) return ay;
     if(ax==bx) { perfect = false; return ay; }
     bool crossx = (ax<8 && bx>8) || (ax>8 && bx<8);
@@ -358,13 +357,21 @@ int midedge(const ivec &a, const ivec &b, int xd, int yd, bool &perfect)
     return crossy ? 8 : min(max(y, 0), 16);
 }
 
+static inline bool crosscenter(const ivec &a, const ivec &b, int xd, int yd)
+{
+    int ax = a[xd], ay = a[yd], bx = b[xd], by = b[yd];
+    return (((ax <= 8 && bx <= 8) || (ax >= 8 && bx >= 8)) &&
+            ((ay <= 8 && by <= 8) || (ay >= 8 && by >= 8))) ||
+           (ax + bx == 16 && ay + by == 16);
+}
+
 bool subdividecube(cube &c, bool fullcheck, bool brighten)
 {
     if(c.children) return true;
     if(c.ext) memset(c.ext->surfaces, 0, sizeof(c.ext->surfaces));
-	if(isempty(c) || isentirelysolid(c))
+    if(isempty(c) || isentirelysolid(c))
     {
-		c.children = newcubes(isempty(c) ? F_EMPTY : F_SOLID, c.material);
+        c.children = newcubes(isempty(c) ? F_EMPTY : F_SOLID, c.material);
         loopi(8)
         {
             loopl(6) c.children[i].texture[l] = c.texture[l];
@@ -406,12 +413,12 @@ bool subdividecube(cube &c, bool fullcheck, bool brighten)
         if(z ? c1 > c2 : c1 < c2)
         {
             e[1][1] = c1;
-            perfect = p1 && (c1 == c2 || v00[C[d]] + v11[C[d]] == 16);
+            perfect = p1 && (c1 == c2 || crosscenter(v00, v11, C[d], R[d]));
         }
         else
         {
             e[1][1] = c2;
-            perfect = p2 && (c1 == c2 || v01[C[d]] + v10[C[d]] == 16);
+            perfect = p2 && (c1 == c2 || crosscenter(v01, v10, C[d], R[d]));
         }
 
         loopi(8)
@@ -573,23 +580,22 @@ static inline int edgeval(cube &c, const ivec &p, int dim, int coord)
     return edgeget(cubeedge(c, dim, p[R[dim]]>>3, p[C[dim]]>>3), coord);
 }
 
-void genvertp(cube &c, ivec &p1, ivec &p2, ivec &p3, plane &pl)
+void genvertp(cube &c, ivec &p1, ivec &p2, ivec &p3, plane &pl, bool solid = false)
 {
     int dim = 0;
     if(p1.y==p2.y && p2.y==p3.y) dim = 1;
     else if(p1.z==p2.z && p2.z==p3.z) dim = 2;
 
     int coord = p1[dim];
-
     ivec v1(p1), v2(p2), v3(p3);
-    v1[D[dim]] = edgeval(c, p1, dim, coord);
-    v2[D[dim]] = edgeval(c, p2, dim, coord);
-    v3[D[dim]] = edgeval(c, p3, dim, coord);
+    v1[dim] = solid ? coord*8 : edgeval(c, p1, dim, coord);
+    v2[dim] = solid ? coord*8 : edgeval(c, p2, dim, coord);
+    v3[dim] = solid ? coord*8 : edgeval(c, p3, dim, coord);
 
     pl.toplane(v1.tovec(), v2.tovec(), v3.tovec());
 }
 
-bool threeplaneintersect(plane &pl1, plane &pl2, plane &pl3, vec &dest)
+static bool threeplaneintersect(plane &pl1, plane &pl2, plane &pl3, vec &dest)
 {
     vec &t1 = dest, t2, t3, t4;
     t1.cross(pl1, pl2); t4 = t1; t1.mul(pl3.offset);
@@ -604,22 +610,19 @@ bool threeplaneintersect(plane &pl1, plane &pl2, plane &pl3, vec &dest)
     return true;
 }
 
-void genedgespanvert(ivec &p, cube &c, vec &v)
+static void genedgespanvert(ivec &p, cube &c, vec &v)
 {
     ivec p1(8-p.x, p.y, p.z);
     ivec p2(p.x, 8-p.y, p.z);
     ivec p3(p.x, p.y, 8-p.z);
 
-    cube s;
-    solidfaces(s);
-
     plane plane1, plane2, plane3;
     genvertp(c, p, p1, p2, plane1);
     genvertp(c, p, p2, p3, plane2);
     genvertp(c, p, p3, p1, plane3);
-    if(plane1==plane2) genvertp(s, p, p1, p2, plane1);
-    if(plane1==plane3) genvertp(s, p, p1, p2, plane1);
-    if(plane2==plane3) genvertp(s, p, p2, p3, plane2);
+    if(plane1==plane2) genvertp(c, p, p1, p2, plane1, true);
+    if(plane1==plane3) genvertp(c, p, p1, p2, plane1, true);
+    if(plane2==plane3) genvertp(c, p, p2, p3, plane2, true);
 
     ASSERT(threeplaneintersect(plane1, plane2, plane3, v));
     //ASSERT(v.x>=0 && v.x<=8);
@@ -632,93 +635,69 @@ void genedgespanvert(ivec &p, cube &c, vec &v)
 
 void edgespan2vectorcube(cube &c)
 {
-    vec v;
-
-    if(c.children) loopi(8) edgespan2vectorcube(c.children[i]);
-
     if(isentirelysolid(c) || isempty(c)) return;
-
-    cube n = c;
-
-    loop(x,2) loop(y,2) loop(z,2)
+    cube o = c;
+    loop(x, 2) loop(y, 2) loop(z, 2)
     {
         ivec p(8*x, 8*y, 8*z);
-        genedgespanvert(p, c, v);
+        vec v;
+        genedgespanvert(p, o, v);
 
-        edgeset(cubeedge(n, 0, y, z), x, int(v.x+0.49f));
-        edgeset(cubeedge(n, 1, z, x), y, int(v.y+0.49f));
-        edgeset(cubeedge(n, 2, x, y), z, int(v.z+0.49f));
+        edgeset(cubeedge(c, 0, y, z), x, int(v.x+0.49f));
+        edgeset(cubeedge(c, 1, z, x), y, int(v.y+0.49f));
+        edgeset(cubeedge(c, 2, x, y), z, int(v.z+0.49f));
     }
-
-    c = n;
-}
-
-void converttovectorworld()
-{
-    conoutf(CON_WARN, "WARNING: old map, use savecurrentmap");
-    loopi(8) edgespan2vectorcube(worldroot[i]);
 }
 
 const ivec cubecoords[8] = // verts of bounding cube
 {
-    ivec(8, 8, 0),
-    ivec(0, 8, 0),
-    ivec(0, 8, 8),
-    ivec(8, 8, 8),
-    ivec(8, 0, 8),
-    ivec(0, 0, 8),
-    ivec(0, 0, 0),
-    ivec(8, 0, 0),
+#define GENCUBEVERT(n, x, y, z) ivec(x, y, z),
+    GENCUBEVERTS(0, 8, 0, 8, 0, 8)
+#undef GENCUBEVERT 
 };
 
 template<class T>
-static inline void genfacevert(cube &c, int i, T &v)
+static inline void gencubevert(cube &c, int i, T &v)
 {
-#define GENFACEVERT(v, X, Y, Z) { \
-        v.x = edgeget(cubeedge(c, 0, Y, Z), X); \
-        v.y = edgeget(cubeedge(c, 1, Z, X), Y); \
-        v.z = edgeget(cubeedge(c, 2, X, Y), Z); \
-    }
     switch(i)
     {
-    case 0: GENFACEVERT(v, 1, 1, 0); break;
-    case 1: GENFACEVERT(v, 0, 1, 0); break;
-    case 2: GENFACEVERT(v, 0, 1, 1); break;
-    case 3: GENFACEVERT(v, 1, 1, 1); break;
-    case 4: GENFACEVERT(v, 1, 0, 1); break;
-    case 5: GENFACEVERT(v, 0, 0, 1); break;
-    case 6: GENFACEVERT(v, 0, 0, 0); break;
-    case 7: GENFACEVERT(v, 1, 0, 0); break;
+#define GENCUBEVERT(n, x, y, z) \
+        case n: \
+            v = T(edgeget(cubeedge(c, 0, y, z), x), \
+                  edgeget(cubeedge(c, 1, z, x), y), \
+                  edgeget(cubeedge(c, 2, x, y), z)); \
+            break;
+        GENCUBEVERTS(0, 1, 0, 1, 0, 1)
+#undef GENCUBEVERT
     }
 }
 
 void genfaceverts(cube &c, int orient, ivec v[4])
 {
-#define GENFACEVERTS(X0,Y0,Z0, X1,Y1,Z1, X2,Y2,Z2, X3,Y3,Z3) { \
-        GENFACEVERT(v[0], X0, Y0, Z0); \
-        GENFACEVERT(v[1], X1, Y1, Z1); \
-        GENFACEVERT(v[2], X2, Y2, Z2); \
-        GENFACEVERT(v[3], X3, Y3, Z3); \
-    }
+
     switch(orient)
     {
-    case 0: GENFACEVERTS(0,1,1, 0,1,0, 0,0,0, 0,0,1); break;
-    case 1: GENFACEVERTS(1,1,1, 1,0,1, 1,0,0, 1,1,0); break;
-    case 2: GENFACEVERTS(1,0,1, 0,0,1, 0,0,0, 1,0,0); break;
-    case 3: GENFACEVERTS(0,1,0, 0,1,1, 1,1,1, 1,1,0); break;
-    case 4: GENFACEVERTS(0,0,0, 0,1,0, 1,1,0, 1,0,0); break;
-    case 5: GENFACEVERTS(0,0,1, 1,0,1, 1,1,1, 0,1,1); break;
+#define GENFACEORIENT(o, v0, v1, v2, v3) \
+        case o: v0 v1 v2 v3 break;
+#define GENFACEVERT(o, n, x,y,z, xv,yv,zv) \
+            v[n] = ivec(edgeget(cubeedge(c, 0, y, z), x), \
+                        edgeget(cubeedge(c, 1, z, x), y), \
+                        edgeget(cubeedge(c, 2, x, y), z));
+        GENFACEVERTS(0, 1, 0, 1, 0, 1, , , , , , )
+    #undef GENFACEORIENT
+    #undef GENFACEVERT
     }
 }
 
 const ivec facecoords[6][4] =
 {
-    { ivec(0,8,8), ivec(0,8,0), ivec(0,0,0), ivec(0,0,8) },
-    { ivec(8,8,8), ivec(8,0,8), ivec(8,0,0), ivec(8,8,0) },
-    { ivec(8,0,8), ivec(0,0,8), ivec(0,0,0), ivec(8,0,0) },
-    { ivec(0,8,0), ivec(0,8,8), ivec(8,8,8), ivec(8,8,0) },
-    { ivec(0,0,0), ivec(0,8,0), ivec(8,8,0), ivec(8,0,0) },
-    { ivec(0,0,8), ivec(8,0,8), ivec(8,8,8), ivec(0,8,8) },
+#define GENFACEORIENT(o, v0, v1, v2, v3) \
+    { v0, v1, v2, v3 },
+#define GENFACEVERT(o, n, x,y,z, xv,yv,zv) \
+        ivec(x,y,z)
+    GENFACEVERTS(0, 8, 0, 8, 0, 8, , , , , , )
+#undef GENFACEORIENT
+#undef GENFACEVERT
 };
 
 const uchar fv[6][4] = // indexes for cubecoords, per each vert of a face orientation
@@ -851,30 +830,49 @@ struct facevec
 
 static inline int genfacevecs(cube &cu, int orient, const ivec &pos, int size, bool solid, facevec *fvecs, const ivec *v = NULL)
 {
-    int dim = dimension(orient), coord = dimcoord(orient), c = C[dim], r = R[dim], winding = coord ? 0 : 3, touching = 0;
-    ivec buf[4];
+    int i = 0;
     if(solid)
     {
-        loopi(4)
+        switch(orient)
         {
-            const ivec &cc = facecoords[orient][i^winding];
-            fvecs[i] = facevec(cc[c]*size + (pos[c]<<3), cc[r]*size + (pos[r]<<3));
+        #define GENFACEORIENT(orient, v0, v1, v2, v3) \
+            case orient: \
+            { \
+                if(dimcoord(orient)) { v0 v1 v2 v3 } else { v3 v2 v1 v0 } \
+                break; \
+            }
+        #define GENFACEVERT(orient, vert, xv,yv,zv, x,y,z) \
+            { facevec &f = fvecs[i]; x ((xv)<<3); y ((yv)<<3); z ((zv)<<3); i++; }
+            GENFACEVERTS(pos.x, pos.x+size, pos.y, pos.y+size, pos.z, pos.z+size, f.x = , f.x = , f.y = , f.y = , (void), (void))
+        #undef GENFACEVERT
         }
         return 4;
     }
+    ivec buf[4];
     if(!v) { genfaceverts(cu, orient, buf); v = buf; }
     facevec prev(INT_MAX, INT_MAX);
-    loopi(4)
+    switch(orient)
     {
-        const ivec &cc = v[i^winding];
-        if(cc[dim] == coord*8)
-        {
-            fvecs[touching] = facevec(cc[c]*size + (pos[c]<<3), cc[r]*size + (pos[r]<<3));
-            if(fvecs[touching] != prev) prev = fvecs[touching++];
-        }
+    #define GENFACEVERT(orient, vert, sx,sy,sz, dx,dy,dz) \
+        { \
+            const ivec &e = v[vert]; \
+            ivec ef; \
+            ef.dx = e.sx; ef.dy = e.sy; ef.dz = e.sz; \
+            if(ef.z == dimcoord(orient)*8) \
+            { \
+                facevec &f = fvecs[i]; \
+                ivec pf; \
+                pf.dx = pos.sx; pf.dy = pos.sy; pf.dz = pos.sz; \
+                f = facevec(ef.x*size + (pf.x<<3), ef.y*size + (pf.y<<3)); \
+                if(f != prev) { prev = f; i++; } \
+            } \
+        } 
+        GENFACEVERTS(x, x, y, y, z, z, x, x, y, y, z, z)
+    #undef GENFACEORIENT
+    #undef GENFACEVERT
     }
-    if(fvecs[0] == prev) touching--;
-    return touching;
+    if(fvecs[0] == prev) i--;
+    return i;
 }
 
 static inline int clipfacevecy(const facevec &o, const facevec &dir, int cx, int cy, int size, facevec &r)
@@ -1106,7 +1104,7 @@ int visibletris(cube &c, int orient, int x, int y, int z, int size, uchar nmat,
     int opp = opposite(orient), numo = 0, numc;
     if(nsize > size || (nsize == size && !o.children))
     {
-        if(isempty(c) || notouchingface(o, opp)) return vis;
+        if(isempty(o) || notouchingface(o, opp)) return vis;
         if(nmat != MAT_AIR && (o.material&matmask) == nmat) return vis;
         if(isentirelysolid(o) || (touchingface(o, opp) && faceedges(o, opp) == F_SOLID)) return vis&notouch;
 
@@ -1152,7 +1150,7 @@ int visibletris(cube &c, int orient, int x, int y, int z, int size, uchar nmat,
 
 void calcvert(cube &c, int x, int y, int z, int size, ivec &v, int i, bool solid)
 {
-    if(solid) v = cubecoords[i]; else genfacevert(c, i, v);
+    if(solid) v = cubecoords[i]; else gencubevert(c, i, v);
     // avoid overflow
     if(size>=8) v.mul(size/8);
     else v.div(8/size);
@@ -1161,7 +1159,7 @@ void calcvert(cube &c, int x, int y, int z, int size, ivec &v, int i, bool solid
 
 void calcvert(cube &c, int x, int y, int z, int size, vec &v, int i, bool solid)
 {
-    if(solid) v = cubecoords[i].tovec(); else genfacevert(c, i, v);
+    if(solid) v = cubecoords[i].tovec(); else gencubevert(c, i, v);
     v.mul(size/8.0f).add(vec(x, y, z));
 }
 
@@ -1639,9 +1637,17 @@ void addmerge(cube &cu, int orient, const ivec &co, const ivec &n, int offset, p
         int numverts = oldsurf.numverts&MAXFACEVERTS;
         if(numverts == p.numverts)
         {
+            ivec v0 = verts[0].getxyz();
             const vertinfo *oldverts = cu.ext->verts() + oldsurf.verts;
-            loopk(numverts) if(verts[k].getxyz() != oldverts[k].getxyz()) goto nomatch;
-            return;
+            loopj(numverts) if(v0 == oldverts[j].getxyz())
+            {
+                for(int k = 1; k < numverts; k++)
+                {
+                    if(++j >= numverts) j = 0;
+                    if(verts[k].getxyz() != oldverts[j].getxyz()) goto nomatch;
+                }
+                return;
+            }
         nomatch:;
         }
     }
