@@ -202,7 +202,7 @@ static float shadowent(octaentities *oc, octaentities *last, const vec &o, const
     vec v(o), invray(ray.x ? 1/ray.x : 1e16f, ray.y ? 1/ray.y : 1e16f, ray.z ? 1/ray.z : 1e16f); \
     cube *levels[20]; \
     levels[worldscale] = worldroot; \
-    int lshift = worldscale; \
+    int lshift = worldscale, elvl = worldscale; \
     ivec lsizemask(invray.x>0 ? 1 : 0, invray.y>0 ? 1 : 0, invray.z>0 ? 1 : 0); \
 
 #define CHECKINSIDEWORLD \
@@ -232,10 +232,15 @@ static float shadowent(octaentities *oc, octaentities *last, const vec &o, const
         { \
             lshift--; \
             lc += octastep(x, y, z, lshift); \
-            if(lc->ext && lc->ext->ents && dent > 1e15f) \
+            if(lc->ext && lc->ext->ents && lshift < elvl) \
             { \
-                dent = disttoent(lc->ext->ents, oclast, o, ray, radius, mode, t); \
-                if(dent < 1e15f earlyexit) return min(dent, dist); \
+                float edist = disttoent(lc->ext->ents, oclast, o, ray, radius, mode, t); \
+                if(edist < 1e15f) \
+                { \
+                    if(earlyexit) return min(edist, dist); \
+                    elvl = lshift; \
+                    dent = min(dent, edist); \
+                } \
                 oclast = lc->ext->ents; \
             } \
             if(lc->children==NULL) break; \
@@ -279,7 +284,7 @@ float raycube(const vec &o, const vec &ray, float radius, int mode, int size, ex
     int closest = -1, x = int(v.x), y = int(v.y), z = int(v.z);
     for(;;)
     {
-        DOWNOCTREE(disttoent, && (mode&RAY_SHADOW));
+        DOWNOCTREE(disttoent, mode&RAY_SHADOW);
 
         int lsize = 1<<lshift;
 
@@ -322,7 +327,7 @@ float shadowray(const vec &o, const vec &ray, float radius, int mode, extentity
     int side = O_BOTTOM, x = int(v.x), y = int(v.y), z = int(v.z);
     for(;;)
     {
-        DOWNOCTREE(shadowent, );
+        DOWNOCTREE(shadowent, true);
 
         cube &c = *lc;
         ivec lo(x&(~0<<lshift), y&(~0<<lshift), z&(~0<<lshift));
@@ -372,7 +377,7 @@ float shadowray(ShadowRayCache *cache, const vec &o, const vec &ray, float radiu
     int side = O_BOTTOM, x = int(v.x), y = int(v.y), z = int(v.z);
     for(;;)
     {
-        DOWNOCTREE(shadowent, );
+        DOWNOCTREE(shadowent, true);
 
         cube &c = *lc;
         ivec lo(x&(~0<<lshift), y&(~0<<lshift), z&(~0<<lshift));
@@ -818,25 +823,25 @@ static inline bool clampcollide(const clipplanes &p, const E &entvol, const plan
         vec c = entvol.center();
         float fv = pw.x < p.o.x ? p.o.x-p.r.x : p.o.x+p.r.x, fdist = (w.x*fv + w.y*c.y + w.z*c.z + w.offset) / (w.y*w.y + w.z*w.z);
         vec fdir(fv - c.x, -w.y*fdist, -w.z*fdist);
-        if(entvol.supportpoint(fdir).squaredist(c) < fdir.squaredlen()) return true;
+        if((pw.y-c.y-fdir.y)*w.y + (pw.z-c.z-fdir.z)*w.z >= 0 && entvol.supportpoint(fdir).squaredist(c) < fdir.squaredlen()) return true;
     }
     if(w.y && (w.x || w.z) && fabs(pw.y - p.o.y) > p.r.y)
     {
         vec c = entvol.center();
         float fv = pw.y < p.o.y ? p.o.y-p.r.y : p.o.y+p.r.y, fdist = (w.x*c.x + w.y*fv + w.z*c.z + w.offset) / (w.x*w.x + w.z*w.z);
         vec fdir(-w.x*fdist, fv - c.y, -w.z*fdist);
-        if(entvol.supportpoint(fdir).squaredist(c) < fdir.squaredlen()) return true;
+        if((pw.x-c.x-fdir.x)*w.x + (pw.z-c.z-fdir.z)*w.z >= 0 && entvol.supportpoint(fdir).squaredist(c) < fdir.squaredlen()) return true;
     }
     if(w.z && (w.x || w.y) && fabs(pw.z - p.o.z) > p.r.z)
     {
         vec c = entvol.center();
         float fv = pw.z < p.o.z ? p.o.z-p.r.z : p.o.z+p.r.z, fdist = (w.x*c.x + w.y*c.y + w.z*fv + w.offset) / (w.x*w.x + w.y*w.y);
         vec fdir(-w.x*fdist, -w.y*fdist, fv - c.z);
-        if(entvol.supportpoint(fdir).squaredist(c) < fdir.squaredlen()) return true;
+        if((pw.x-c.x-fdir.x)*w.x + (pw.y-c.y-fdir.y)*w.y >= 0 && entvol.supportpoint(fdir).squaredist(c) < fdir.squaredlen()) return true;
     }
     return false;
 }
-    
+
 template<class E>
 static bool fuzzycollideplanes(physent *d, const vec &dir, float cutoff, cube &c, const ivec &co, int size) // collide with deformed cube geometry
 {
@@ -1642,9 +1647,12 @@ void vectoyawpitch(const vec &v, float &yaw, float &pitch)
     pitch = asin(v.z/v.magnitude())/RAD;
 }
 
-VARP(maxroll, 0, 3, 20);
+#define PHYSFRAMETIME 5
+
+VARP(maxroll, 0, 0, 20);
 FVAR(straferoll, 0, 0.033f, 90);
-VAR(floatspeed, 10, 100, 10000);
+FVAR(faderoll, 0, 0.95f, 1);
+VAR(floatspeed, 1, 100, 10000);
 
 void modifyvelocity(physent *pl, bool local, bool water, bool floating, int curtime)
 {
@@ -1657,7 +1665,7 @@ void modifyvelocity(physent *pl, bool local, bool water, bool floating, int curt
             pl->lastjump = lastmillis;
         }
     }
-    else if(pl->physstate >= PHYS_SLOPE || water || game::allowdoublejump())
+    else if(pl->physstate >= PHYS_SLOPE || water || game::allowdoublejump(pl))
     {
         if(water && !pl->inwater) pl->vel.div(8);
         if(pl->jumping)
@@ -1702,16 +1710,16 @@ void modifyvelocity(physent *pl, bool local, bool water, bool floating, int curt
 
     if(pl->type==ENT_PLAYER)
     {
-    	if(!floating)
-	{
-		d.mul(movespeed/100.0f);
-	}
+        if(!floating)
+        {
+            d.mul(movespeed/100.0f);
+        }
         if(floating)
         {
             if(pl==player)
-	    	d.mul(floatspeed/pl->maxspeed);
+                d.mul(floatspeed/pl->maxspeed);
         }
-        else if(!water && game::allowmove(pl)) d.mul((pl->move && !pl->strafe ? 1.3f : 1.0f) * (pl->physstate < PHYS_SLOPE ? 1.3f : 1.0f)); // EXPERIMENTAL
+        else if(!water && game::allowmove(pl)) d.mul((pl->move && !pl->strafe ? 1.3f : 1.0f) * (pl->physstate < PHYS_SLOPE ? 1.3f : 1.0f));
     }
     float fric = water && !floating ? 20.0f : (pl->physstate >= PHYS_SLOPE || floating ? 6.0f : 30.0f);
     pl->vel.lerp(d, pl->vel, pow(1 - 1/fric, curtime/20.0f));
@@ -1801,16 +1809,8 @@ bool moveplayer(physent *pl, int moveres, bool local, int curtime)
 
     // automatically apply smooth roll when strafing
 
-    if(pl->strafe==0)
-    {
-        pl->roll = pl->roll/(1+(float)sqrtf((float)curtime)/25);
-    }
-    else
-    {
-        pl->roll -= pl->strafe*curtime*straferoll;
-        if(pl->roll > maxroll) pl->roll = maxroll;
-        else if(pl->roll < -maxroll) pl->roll = -maxroll;
-    }
+    if(pl->strafe && maxroll) pl->roll = clamp(pl->roll - pow(clamp(1.0f + pl->strafe*pl->roll/maxroll, 0.0f, 1.0f), 0.33f)*pl->strafe*curtime*straferoll, -maxroll, maxroll);
+    else pl->roll *= curtime == PHYSFRAMETIME ? faderoll : pow(faderoll, curtime/float(PHYSFRAMETIME));
 
     // play sounds on water transitions
 
@@ -1828,8 +1828,6 @@ bool moveplayer(physent *pl, int moveres, bool local, int curtime)
     return true;
 }
 
-#define PHYSFRAMETIME 5
-
 int physsteps = 0, physframetime = PHYSFRAMETIME, lastphysframe = 0;
 
 void physicsframe()          // optimally schedule physics frames inside the graphics frames
