@@ -6,6 +6,7 @@ bool hasVBO = false, hasDRE = false, hasOQ = false, hasTR = false, hasFBO = fals
 int hasstencil = 0;
 
 VAR(renderpath, 1, 0, 0);
+VAR(glslversion, 1, 0, 0);
 
 // GL_ARB_vertex_buffer_object, GL_ARB_pixel_buffer_object
 PFNGLGENBUFFERSARBPROC       glGenBuffers_       = NULL;
@@ -145,7 +146,6 @@ VAR(apple_glsldepth_bug, 0, 0, 1);
 VAR(apple_ff_bug, 0, 0, 1);
 VAR(apple_vp_bug, 0, 0, 1);
 VAR(sdl_backingstore_bug, -1, 0, 1);
-VAR(intel_quadric_bug, 0, 0, 1);
 VAR(mesa_program_bug, 0, 0, 1);
 VAR(avoidshaders, 1, 0, 0);
 VAR(minimizetcusage, 1, 0, 0);
@@ -174,7 +174,7 @@ VAR(dbgexts, 0, 0, 1);
 bool hasext(const char *exts, const char *ext)
 {
     int len = strlen(ext);
-    for(const char *cur = exts; (cur = strstr(cur, ext)); cur += len)
+    if(len) for(const char *cur = exts; (cur = strstr(cur, ext)); cur += len)
     {
         if((cur == exts || cur[-1] == ' ') && (cur[len] == ' ' || !cur[len])) return true;
     }
@@ -196,11 +196,25 @@ void gl_checkextensions()
     sdl_backingstore_bug = -1;
 #endif
 
+    bool mesa = false, intel = false, ati = false, nvidia = false;
+    if(strstr(renderer, "Mesa") || strstr(version, "Mesa"))
+        mesa = true;
+    else if(strstr(vendor, "NVIDIA"))
+        nvidia = true;
+    else if(strstr(vendor, "ATI") || strstr(vendor, "Advanced Micro Devices"))
+        ati = true;
+    else if(strstr(vendor, "Intel"))
+        intel = true;
+
     //extern int shaderprecision;
     // default to low precision shaders on certain cards, can be overridden with -f3
     // char *weakcards[] = { "GeForce FX", "Quadro FX", "6200", "9500", "9550", "9600", "9700", "9800", "X300", "X600", "FireGL", "Intel", "Chrome", NULL }
     // if(shaderprecision==2) for(char **wc = weakcards; *wc; wc++) if(strstr(renderer, *wc)) shaderprecision = 1;
 
+    GLint val;
+    glGetIntegerv(GL_MAX_TEXTURE_SIZE, &val);
+    hwtexsize = val;
+
     if(hasext(exts, "GL_EXT_texture_env_combine") || hasext(exts, "GL_ARB_texture_env_combine"))
     {
         hasTE = true;
@@ -314,9 +328,9 @@ void gl_checkextensions()
     }
     else conoutf(CON_WARN, "WARNING: No framebuffer object support. (reflective water may be slow)");
 
-#if defined(__APPLE__)
+#ifdef __APPLE__
     // Intel HD3000 broke occlusion queries - either causing software fallback, or returning wrong results
-    if(!strstr(vendor, "Intel"))
+    if(!intel)
 #endif
     if(hasext(exts, "GL_ARB_occlusion_query"))
     {
@@ -334,7 +348,7 @@ void gl_checkextensions()
             hasOQ = true;
             if(dbgexts) conoutf(CON_INIT, "Using GL_ARB_occlusion_query extension.");
 #if defined(__APPLE__) && SDL_BYTEORDER == SDL_BIG_ENDIAN
-            if(strstr(vendor, "ATI") && (osversion<0x1050)) ati_oq_bug = 1;
+            if(ati && (osversion<0x1050)) ati_oq_bug = 1;
 #endif
             //if(ati_oq_bug) conoutf(CON_WARN, "WARNING: Using ATI occlusion query bug workaround. (use \"/ati_oq_bug 0\" to disable if unnecessary)");
         }
@@ -347,55 +361,6 @@ void gl_checkextensions()
         waterreflect = 0;
     }
 
-    extern int reservedynlighttc, reserveshadowmaptc, batchlightmaps, ffdynlights, fpdepthfx;
-    if(strstr(vendor, "ATI"))
-    {
-        //conoutf(CON_WARN, "WARNING: ATI cards may show garbage in skybox. (use \"/ati_skybox_bug 1\" to fix)");
-
-        reservedynlighttc = 2;
-        reserveshadowmaptc = 3;
-        minimizetcusage = 1;
-        emulatefog = 1;
-		if(hasTF && hasNVFB) fpdepthfx = 1;
-    }
-    else if(strstr(vendor, "NVIDIA"))
-    {
-        reservevpparams = 10;
-        rtsharefb = 0; // work-around for strange driver stalls involving when using many FBOs
-        extern int filltjoints;
-        if(!hasext(exts, "GL_EXT_gpu_shader4")) filltjoints = 0; // DX9 or less NV cards seem to not cause many sparklies
-
-        if(hasFBO && !hasTF) nvidia_scissor_bug = 1; // 5200 bug, clearing with scissor on an FBO messes up on reflections, may affect lesser cards too
-        extern int fpdepthfx;
-        if(hasTF && (!strstr(renderer, "GeForce") || !checkseries(renderer, 6000, 6600)))
-            fpdepthfx = 1; // FP filtering causes software fallback on 6200?
-    }
-    else if(strstr(vendor, "Intel"))
-    {
-        avoidshaders = 1;
-        intel_quadric_bug = 1;
-        maxtexsize = 256;
-        reservevpparams = 20;
-        batchlightmaps = 0;
-        ffdynlights = 0;
-
-        if(!hasOQ) waterrefract = 0;
-
-#ifdef __APPLE__
-        apple_vp_bug = 1;
-#endif
-    }
-    else if(strstr(vendor, "Tungsten") || strstr(vendor, "Mesa") || strstr(vendor, "DRI") || strstr(vendor, "Microsoft") || strstr(vendor, "S3 Graphics"))
-    {
-        avoidshaders = 1;
-        maxtexsize = 256;
-        reservevpparams = 20;
-        batchlightmaps = 0;
-        ffdynlights = 0;
-
-        if(!hasOQ) waterrefract = 0;
-    }
-
     if(hasext(exts, "GL_ARB_vertex_program") && hasext(exts, "GL_ARB_fragment_program"))
     {
         hasVP = hasFP = true;
@@ -410,8 +375,8 @@ void gl_checkextensions()
         glDisableVertexAttribArray_ = (PFNGLDISABLEVERTEXATTRIBARRAYARBPROC) getprocaddress("glDisableVertexAttribArrayARB");
         glVertexAttribPointer_ =      (PFNGLVERTEXATTRIBPOINTERARBPROC)      getprocaddress("glVertexAttribPointerARB");
 
-        if(strstr(vendor, "ATI")) ati_dph_bug = ati_line_bug = 1;
-        else if(strstr(vendor, "Tungsten")) mesa_program_bug = 1;
+        if(ati) ati_dph_bug = ati_line_bug = 1;
+        else if(mesa) mesa_program_bug = 1;
 
 #ifdef __APPLE__
         if(osversion>=0x1050) // fixed in 1055 for some hardware.. but not all..
@@ -463,14 +428,71 @@ void gl_checkextensions()
             if(hasVP && hasFP) apple_glsldepth_bug = 1;
 #endif
             //if(apple_glsldepth_bug) conoutf(CON_WARN, "WARNING: Using Apple GLSL depth bug workaround. (use \"/apple_glsldepth_bug 0\" to disable if unnecessary");
+
+            const char *str = (const char *)glGetString(GL_SHADING_LANGUAGE_VERSION_ARB);
+            uint majorversion, minorversion;
+            if(!str || sscanf(str, " %u.%u", &majorversion, &minorversion) != 2) glslversion = 100;
+            else glslversion = majorversion*100 + minorversion;
         }
     }
 
+    extern int reservedynlighttc, reserveshadowmaptc, batchlightmaps, ffdynlights, fpdepthfx;
+    if(ati)
+    {
+        //conoutf(CON_WARN, "WARNING: ATI cards may show garbage in skybox. (use \"/ati_skybox_bug 1\" to fix)");
+
+        reservedynlighttc = 2;
+        reserveshadowmaptc = 3;
+        minimizetcusage = 1;
+        emulatefog = 1;
+        if(hasTF && hasNVFB) fpdepthfx = 1;
+    }
+    else if(nvidia)
+    {
+        reservevpparams = 10;
+        rtsharefb = 0; // work-around for strange driver stalls involving when using many FBOs
+        extern int filltjoints;
+        if(!hasext(exts, "GL_EXT_gpu_shader4")) filltjoints = 0; // DX9 or less NV cards seem to not cause many sparklies
+
+        if(hasFBO && !hasTF) nvidia_scissor_bug = 1; // 5200 bug, clearing with scissor on an FBO messes up on reflections, may affect lesser cards too
+        extern int fpdepthfx;
+        if(hasTF && (!strstr(renderer, "GeForce") || !checkseries(renderer, 6000, 6600)))
+            fpdepthfx = 1; // FP filtering causes software fallback on 6200?
+    }
+    else
+    {
+        if(intel)
+        {
+#ifdef __APPLE__
+            apple_vp_bug = 1;
+#endif
+        }
+
+        if(!hasGLSL || glslversion < 130)
+        {
+            avoidshaders = 1;
+            if(hwtexsize < 4096)
+            {
+                maxtexsize = hwtexsize >= 2048 ? 512 : 256;
+                batchlightmaps = 0;
+            }
+            if(!hasTF) ffdynlights = 0;
+        }
+
+        reservevpparams = 20;
+
+        if(!hasOQ) waterrefract = 0;
+    }
+
     bool hasshaders = (hasVP && hasFP) || hasGLSL;
     if(hasshaders)
     {
-        extern int matskel;
-        if(!avoidshaders) matskel = 0;
+        extern int matskel, forceglsl;
+        if(!avoidshaders)
+        {
+            matskel = 0;
+            if(hasGLSL && glslversion >= 130) forceglsl = 1;
+        }
     }
 
     if(hasext(exts, "GL_NV_vertex_program2_option")) { usevp2 = 1; hasVP2 = true; }
@@ -496,7 +518,7 @@ void gl_checkextensions()
 
         useubo = 1;
         hasUBO = true;
-        if(strstr(vendor, "ATI")) ati_ubo_bug = 1;
+        if(ati) ati_ubo_bug = 1;
         if(dbgexts) conoutf(CON_INIT, "Using GL_ARB_uniform_buffer_object extension.");
     }
     else if(hasext(exts, "GL_EXT_bindable_uniform"))
@@ -507,7 +529,7 @@ void gl_checkextensions()
 
         usebue = 1;
         hasBUE = true;
-        if(strstr(vendor, "ATI")) ati_ubo_bug = 1;
+        if(ati) ati_ubo_bug = 1;
         if(dbgexts) conoutf(CON_INIT, "Using GL_EXT_bindable_uniform extension.");
     }
 
@@ -529,7 +551,7 @@ void gl_checkextensions()
     {
         glBlendEquation_ = (PFNGLBLENDEQUATIONEXTPROC) getprocaddress("glBlendEquationEXT");
         hasBE = true;
-        if(strstr(vendor, "ATI")) ati_minmax_bug = 1;
+        if(ati) ati_minmax_bug = 1;
         if(dbgexts) conoutf(CON_INIT, "Using GL_EXT_blend_minmax extension.");
     }
 
@@ -554,7 +576,7 @@ void gl_checkextensions()
         hwcubetexsize = val;
         hasCM = true;
         // On Catalyst 10.2, issuing an occlusion query on the first draw using a given cubemap texture causes a nasty crash
-        if(strstr(vendor, "ATI")) ati_cubemap_bug = 1;
+        if(ati) ati_cubemap_bug = 1;
         if(dbgexts) conoutf(CON_INIT, "Using GL_ARB_texture_cube_map extension.");
     }
     else conoutf(CON_WARN, "WARNING: No cube map texture support. (no reflective glass)");
@@ -610,7 +632,7 @@ void gl_checkextensions()
     if(hasext(exts, "GL_ARB_shadow"))
     {
         hasSGISH = hasSH = true;
-        if(strstr(vendor, "NVIDIA") || strstr(renderer, "Radeon HD")) hasNVPCF = true;
+        if(nvidia || (ati && strstr(renderer, "Radeon HD"))) hasNVPCF = true;
         if(dbgexts) conoutf(CON_INIT, "Using GL_ARB_shadow extension.");
     }
     else if(hasext(exts, "GL_SGIX_shadow"))
@@ -646,10 +668,6 @@ void gl_checkextensions()
             }
         }
     }
-
-    GLint val;
-    glGetIntegerv(GL_MAX_TEXTURE_SIZE, &val);
-    hwtexsize = val;
 }
 
 void glext(char *ext)
@@ -743,31 +761,24 @@ vec worldpos, camdir, camright, camup;
 
 void findorientation()
 {
-    float yaw, pitch;
-    #ifdef NEWGUI
-    if(editmode)
-    {
-        float x, y;
-        UI::getcursorpos(x, y);
-
-        vec dir1 = invmvpmatrix.perspectivetransform(vec(x*2-1, 1-2*y, 2-1)),
-        dir2 = invmvpmatrix.perspectivetransform(vec(x*2-1, 1-2*y, -1));
-        camdir = dir1.sub(dir2).normalize();
-        vectoyawpitch(camdir, yaw, pitch);
-    }
-    else
-    #endif
-    {
-        yaw = camera1->yaw;
-        pitch = camera1->pitch;
-        vecfromyawpitch(yaw, pitch, 1, 0, camdir);
-    }
+    float yaw = camera1->yaw, pitch = camera1->pitch;
+    vecfromyawpitch(yaw, pitch, 1, 0, camdir);
 
     vecfromyawpitch(yaw, 0, 0, -1, camright);
     vecfromyawpitch(yaw, pitch+90, 1, 0, camup);
 
-    if(raycubepos(camera1->o, camdir, worldpos, 0, RAY_CLIPMAT|RAY_SKIPFIRST) == -1)
-        worldpos = vec(camdir).mul(2*worldsize).add(camera1->o); //otherwise 3dgui won't work when outside of map
+    vec lookdir = camdir;
+
+    float x, y;
+    UI::getcursorpos(x, y);
+
+    vec dir1 = invmvpmatrix.perspectivetransform(vec(x*2-1, 1-2*y, 2-1)),
+    dir2 = invmvpmatrix.perspectivetransform(vec(x*2-1, 1-2*y, -1));
+
+    lookdir = dir1.sub(dir2).normalize();
+
+    if(raycubepos(camera1->o, lookdir, worldpos, 0, RAY_CLIPMAT|RAY_SKIPFIRST) == -1)
+        worldpos = vec(camdir).mul(2*worldsize).add(camera1->o);
 }
 
 void transplayer()
@@ -875,8 +886,8 @@ void mousemove(int dx, int dy)
 
     if(game::mousemove(dx, dy, cursens))
     {
-	    fixcamerarange();
-	    return; //game processed mouse movement, don't bother
+        fixcamerarange();
+        return; //game processed mouse movement, don't bother
     }
 
     camera1->yaw += dx*cursens;
@@ -937,12 +948,12 @@ glmatrixf mvmatrix, projmatrix, mvpmatrix, invmvmatrix, invmvpmatrix;
 
 void readmatrices()
 {
-	glGetFloatv(GL_MODELVIEW_MATRIX, mvmatrix.v);
-	glGetFloatv(GL_PROJECTION_MATRIX, projmatrix.v);
+    glGetFloatv(GL_MODELVIEW_MATRIX, mvmatrix.v);
+    glGetFloatv(GL_PROJECTION_MATRIX, projmatrix.v);
 
-	mvpmatrix.mul(projmatrix, mvmatrix);
-	invmvmatrix.invert(mvmatrix);
-	invmvpmatrix.invert(mvpmatrix);
+    mvpmatrix.mul(projmatrix, mvmatrix);
+    invmvmatrix.invert(mvmatrix);
+    invmvpmatrix.invert(mvpmatrix);
 }
 
 FVAR(nearplane, 1e-3f, 0.54f, 1e3f);
@@ -1676,9 +1687,7 @@ void drawminimap()
     glClearColor(fogc[0], fogc[1], fogc[2], fogc[3]);
     glClear(GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT);
 
-    glViewport(1, 1, size-2, size-2);
-    glScissor(1, 1, size-2, size-2);
-    glEnable(GL_SCISSOR_TEST);
+    glViewport(0, 0, size, size);
 
     glDisable(GL_FOG);
     glEnable(GL_CULL_FACE);
@@ -1715,7 +1724,6 @@ void drawminimap()
     glDisable(GL_CULL_FACE);
     glDisable(GL_FOG);
 
-    glDisable(GL_SCISSOR_TEST);
     glViewport(0, 0, screen->w, screen->h);
 
     camera1 = oldcamera;
@@ -1724,6 +1732,10 @@ void drawminimap()
     glBindTexture(GL_TEXTURE_2D, minimaptex);
     glCopyTexImage2D(GL_TEXTURE_2D, 0, GL_RGB5, 0, 0, size, size, 0);
     setuptexparameters(minimaptex, NULL, 3, 1, GL_RGB5, GL_TEXTURE_2D);
+    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_BORDER);
+    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_BORDER);
+    GLfloat border[4] = { minimapcolor.x/255.0f, minimapcolor.y/255.0f, minimapcolor.z/255.0f, 1.0f };
+    glTexParameterfv(GL_TEXTURE_2D, GL_TEXTURE_BORDER_COLOR, border);
     glBindTexture(GL_TEXTURE_2D, 0);
 }
 
@@ -1918,10 +1930,10 @@ void gl_drawframe(int w, int h)
     rendermaterials();
     renderalphageom();
 
-    renderparticles(true);
-
     if(wireframe && editmode) glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
 
+    renderparticles(true);
+
     glDisable(GL_FOG);
     glDisable(GL_CULL_FACE);
     glDisable(GL_DEPTH_TEST);
@@ -1931,13 +1943,36 @@ void gl_drawframe(int w, int h)
     if(fogmat==MAT_WATER || fogmat==MAT_LAVA) drawfogoverlay(fogmat, fogblend, abovemat);
     renderpostfx();
 
-    defaultshader->set();
+    extern int hidehud;
+    if(!hidehud && !UI::mainmenu)
+    {
+        glDisable(GL_TEXTURE_2D);
+        notextureshader->set();
+        glEnable(GL_DEPTH_TEST);
+        glDepthMask(GL_FALSE);
+
+        if(editmode)renderblendbrush();
+
+        if (editmode) rendereditcursor();
+        glEnable(GL_BLEND);
+        glBlendFunc(GL_ONE, GL_ONE);
+        extern int showenthelpers;
+        extern void renderhelpers();
+        if((showenthelpers==1 && game::showenthelpers()) || showenthelpers == 2) renderhelpers();
+        glDisable(GL_BLEND);
+
+        glDepthMask(GL_TRUE);
+        glDisable(GL_DEPTH_TEST);
+
+        glEnable(GL_TEXTURE_2D);
+        defaultshader->set();
+    }
+    else
+    {
+        defaultshader->set();
+    }
 
-    #ifndef NEWGUI
-    g3d_render();
-    #else
     UI::render();
-    #endif
 
     glDisable(GL_TEXTURE_2D);
     notextureshader->set();
@@ -1961,11 +1996,7 @@ void gl_drawmainmenu(int w, int h)
 
     defaultshader->set();
     glEnable(GL_TEXTURE_2D);
-    #ifdef NEWGUI
     UI::render();
-    #else
-    g3d_render();
-    #endif
 
     notextureshader->set();
     glDisable(GL_TEXTURE_2D);
@@ -2055,7 +2086,7 @@ void drawdamagescreen(int w, int h)
     glEnable(GL_TEXTURE_2D);
 
     static Texture *damagetex = NULL;
-    if(!damagetex) damagetex = textureload("packages/hud/damage.png", 3);
+    if(!damagetex) damagetex = textureload("data/fps/hud/damage.png", 3);
 
     glBlendFunc(GL_ONE, GL_ONE_MINUS_SRC_ALPHA);
     glBindTexture(GL_TEXTURE_2D, damagetex->id);
@@ -2079,7 +2110,7 @@ VAR(hidestats, 0, 0, 1);
 VAR(hidehud, 0, 0, 1);
 
 VARP(crosshairsize, 0, 15, 50);
-VARP(cursorsize, 0, 30, 50);
+VARP(cursorsize, 0, 20, 50);
 VARP(crosshaircol, 0, 1, 1);
 VARP(crosshairimg, 0, 1, 1);
 
@@ -2087,29 +2118,29 @@ vector<Texture *> crosshairs;
 
 void loadcrosshair(const char *name, int i)
 {
-	if(i < 0) return;
+    if(i < 0) return;
 
-	if(!crosshairs.inrange(i))
-	{
-		int len = crosshairs.length();
-		crosshairs.growbuf(i + 1);
-		crosshairs.advance(i + 1 - len);
+    if(!crosshairs.inrange(i))
+    {
+        int len = crosshairs.length();
+        crosshairs.growbuf(i + 1);
+        crosshairs.advance(i + 1 - len);
 
-		loopj(crosshairs.length() - len)
-			crosshairs[j + len] = NULL;
-	}
+        loopj(crosshairs.length() - len)
+            crosshairs[j + len] = NULL;
+    }
 
-	crosshairs[i] = name ? textureload(name, 3, true) : notexture;
-	if(crosshairs[i] == notexture)
-		crosshairs[i] = textureload(game::defaultcrosshair(i) ? game::defaultcrosshair(i) : "data/crosshair", 3, true);
+    crosshairs[i] = name ? textureload(name, 3, true) : notexture;
+    if(crosshairs[i] == notexture)
+        crosshairs[i] = textureload(game::defaultcrosshair(i) ? game::defaultcrosshair(i) : "data/crosshair", 3, true);
 }
 
 void loadcrosshair_(const char *name, int *i)
 {
-	loadcrosshair(name, *i);
+    loadcrosshair(name, *i);
 }
 
-SVARP(guicursortex, "hud/2.3/guicursor");
+SVARP(guicursortex, "newui/cursor");
 
 COMMANDN(loadcrosshair, loadcrosshair_, "si");
 
@@ -2118,7 +2149,7 @@ ICOMMAND(getcrosshair, "i", (int *i),
     if(crosshairs.inrange(*i))
     {
         name = crosshairs[*i] ? crosshairs[*i]->name : game::defaultcrosshair(*i);
-	if(!name) name = "packages/crosshair/default.png";
+        if(!name) name = "packages/crosshair/default.png";
     }
     result(name);
 )
@@ -2127,33 +2158,23 @@ ICOMMAND(getcrosshair, "i", (int *i),
 void writecrosshairs(stream *f)
 {
     loopv(crosshairs) if(crosshairs[i] && crosshairs[i]!=notexture)
-        f->printf("loadcrosshair \"%s\" %d\n", crosshairs[i]->name, i);
+        f->printf("loadcrosshair %s %d\n", escapestring(crosshairs[i]->name), i);
     f->printf("\n");
 }
 
 void drawcrosshair(int w, int h)
 {
     bool windowhit =
-    #ifdef NEWGUI
     UI::hascursor();
-    #else
-    g3d_windowhit(true, false);
-    #endif
 
     if(!windowhit && hidehud) return; //(hidehud || player->state==CS_SPECTATOR || player->state==CS_DEAD)) return;
 
-    #ifndef NEWGUI
-    if(!windowhit && mainmenu) return;
-    #else
     if(!windowhit && UI::mainmenu) return;
-    #endif
 
     float r = 1, g = 1, b = 1, cx = 0.5f, cy = 0.5f, chsize;
     Texture *crosshair;
 
-    #ifdef NEWGUI
     UI::getcursorpos(cx, cy);
-    #endif
 
     if(windowhit)
     {
@@ -2162,10 +2183,6 @@ void drawcrosshair(int w, int h)
         cursor = textureload(tname, 3, true);
         crosshair = cursor;
         chsize = cursorsize*w/900.0f;
-
-        #ifndef NEWGUI
-        g3d_cursorpos(cx, cy);
-        #endif
     }
     else
     {
@@ -2187,7 +2204,7 @@ void drawcrosshair(int w, int h)
         }
         chsize = crosshairsize*w/900.0f;
     }
-    if(crosshair->bpp==4) glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+    if(crosshair->type&Texture::ALPHA) glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
     else glBlendFunc(GL_ONE, GL_ONE);
     glColor3f(r, g, b);
     float x = cx*w - (windowhit ? 0 : chsize/2.0f);
@@ -2216,29 +2233,6 @@ FVARP(conscale, 1e-3f, 0.33f, 1e3f);
 
 void gl_drawhud(int w, int h)
 {
-    #ifdef NEWGUI
-    if(!hidehud && !UI::mainmenu)
-    #else
-    if(!hidehud && !mainmenu)
-    #endif
-    {
-        glEnable(GL_DEPTH_TEST);
-        glDepthMask(GL_FALSE);
-
-        if(editmode)renderblendbrush();
-
-        if (editmode) rendereditcursor();
-	glEnable(GL_BLEND);
-	glBlendFunc(GL_ONE, GL_ONE);
-	extern int showenthelpers;
-        extern void renderhelpers();
-        if((showenthelpers==1 && game::showenthelpers()) || showenthelpers == 2) renderhelpers();
-	glDisable(GL_BLEND);
-
-        glDepthMask(GL_TRUE);
-	glDisable(GL_DEPTH_TEST);
-    }
-
     gettextres(w, h);
 
     glMatrixMode(GL_PROJECTION);
@@ -2272,11 +2266,7 @@ void gl_drawhud(int w, int h)
 
     glEnable(GL_BLEND);
 
-    #ifndef NEWGUI
-    if(!mainmenu)
-    #else
     if(!UI::mainmenu)
-    #endif
     {
         drawdamagescreen(w, h);
         drawdamagecompass(w, h);
@@ -2285,24 +2275,21 @@ void gl_drawhud(int w, int h)
     glEnable(GL_TEXTURE_2D);
     defaultshader->set();
 
-    int conw = int(w/conscale), conh = int(h/conscale), abovehud = conh - FONTH, limitgui = abovehud;
-    #ifdef NEWGUI
+    int conw = int(w/conscale), conh = int(h/conscale), abovehud = conh - FONTH; // int limitgui = abovehud;
     if(!hidehud && !UI::mainmenu)
-    #else
-    if(!hidehud && !mainmenu)
-    #endif
     {
         if(hidestats || (!editmode && !showeditstats))
         {
             glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
             game::gameplayhud(w, h);
-            limitgui = abovehud = min(abovehud, int(conh*game::abovegameplayhud(w, h)));
+            abovehud = min(abovehud, int(conh*game::abovegameplayhud(w, h)));
+            //limitgui = abovehud = min(abovehud, int(conh*game::abovegameplayhud(w, h)));
         }
 
         if(!hidestats)
         {
-	    glPushMatrix();
-	    glScalef(conscale, conscale, 1);
+            glPushMatrix();
+            glScalef(conscale, conscale, 1);
 
             int roffset = 0;
             if(showfps)
@@ -2364,7 +2351,7 @@ void gl_drawhud(int w, int h)
                 abovehud -= 2*FONTH;
                 draw_textf("wtr:%dk(%d%%) wvt:%dk(%d%%) evt:%dk eva:%dk", FONTH/2, abovehud, wtris/1024, curstats[0], wverts/1024, curstats[1], curstats[2], curstats[3]);
                 draw_textf("ond:%d va:%d gl:%d(%d) oq:%d lm:%d rp:%d pvs:%d", FONTH/2, abovehud+FONTH, allocnodes*8, allocva, curstats[4], curstats[5], curstats[6], lightmaps.length(), curstats[7], getnumviewcells());
-                limitgui = abovehud;
+                //limitgui = abovehud;
             }
 
             if(editmode)
@@ -2372,7 +2359,7 @@ void gl_drawhud(int w, int h)
                 abovehud -= FONTH;
                 draw_textf("cube %s%d", FONTH/2, abovehud, selchildcount<0 ? "1/" : "", abs(selchildcount));
 
-                char *editinfo = executeret("edithud");
+                char *editinfo = executestr("edithud");
                 if(editinfo)
                 {
                     if(editinfo[0])
@@ -2388,7 +2375,7 @@ void gl_drawhud(int w, int h)
             }
             else if(identexists("gamehud"))
             {
-                char *gameinfo = executeret("gamehud");
+                char *gameinfo = executestr("gamehud");
                 if(gameinfo)
                 {
                     if(gameinfo[0])
@@ -2407,9 +2394,6 @@ void gl_drawhud(int w, int h)
 
         rendertexturepanel(w, h);
     }
-#ifndef NEWGUI
-    g3d_limitscale((2*limitgui - conh) / float(conh));
-#endif
 
     glPushMatrix();
     glScalef(conscale, conscale, 1);
@@ -2525,240 +2509,236 @@ void restorebackground()
 
 void renderbackground(const char *caption, Texture *mapshot, const char *mapname, const char *mapinfo, bool restore, bool force)
 {
-	if(!inbetweenframes && !force) return;
+    if(!inbetweenframes && !force) return;
 
-	stopsounds();
+    stopsounds();
 
-	int w = screen->w, h = screen->h;
-	getbackgroundres(w, h);
-	gettextres(w, h);
+    int w = screen->w, h = screen->h;
+    getbackgroundres(w, h);
+    gettextres(w, h);
 
-	glMatrixMode(GL_PROJECTION);
-	glLoadIdentity();
-	glOrtho(0, w, h, 0, -1, 1);
-	glMatrixMode(GL_MODELVIEW);
-	glLoadIdentity();
+    glMatrixMode(GL_PROJECTION);
+    glLoadIdentity();
+    glOrtho(0, w, h, 0, -1, 1);
+    glMatrixMode(GL_MODELVIEW);
+    glLoadIdentity();
 
-	defaultshader->set();
-	glEnable(GL_TEXTURE_2D);
+    defaultshader->set();
+    glEnable(GL_TEXTURE_2D);
 
-	static int lastupdate = -1, lastw = -1, lasth = -1;
+    static int lastupdate = -1, lastw = -1, lasth = -1;
 
-	#ifdef NEWGUI
-	if((renderedframe && !UI::mainmenu && lastupdate != lastmillis) || lastw != w || lasth != h)
-	#else
-	if((renderedframe && !mainmenu && lastupdate != lastmillis) || lastw != w || lasth != h)
-	#endif
-	{
-		lastupdate = lastmillis;
-		lastw = w;
-		lasth = h;
-	}
-	else if(lastupdate != lastmillis)
-		lastupdate = lastmillis;
+    if((renderedframe && !UI::mainmenu && lastupdate != lastmillis) || lastw != w || lasth != h)
+    {
+        lastupdate = lastmillis;
+        lastw = w;
+        lasth = h;
+    }
+    else if(lastupdate != lastmillis)
+        lastupdate = lastmillis;
 
-	loopi(restore ? 1 : 3)
-	{
-		glColor3f(1,1,1);
-		float ratio = 0;
+    loopi(restore ? 1 : 3)
+    {
+        glColor3f(1,1,1);
+        float ratio = 0;
 
-		/*loading screen background*/
-		glEnable(GL_BLEND);
-		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+        /*loading screen background*/
+        glEnable(GL_BLEND);
+        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 
-		Texture *background = textureload("data/loadingscreen/load_back.png", 3, false, false);
-		glBindTexture(GL_TEXTURE_2D, background->id);
+        Texture *background = textureload("data/loadingscreen/load_back.png", 3, false, false);
+        glBindTexture(GL_TEXTURE_2D, background->id);
 
-		ratio = max((float)h/background->ys, (float)w/background->xs);
+        ratio = max((float)h/background->ys, (float)w/background->xs);
 
-		int wborder = (w - background->xs*ratio) / 2;
-		int hborder = (h - background->ys*ratio) / 2;
+        int wborder = (w - background->xs*ratio) / 2;
+        int hborder = (h - background->ys*ratio) / 2;
 
-		glBegin(GL_TRIANGLE_STRIP);
-			glTexCoord2f(0,0); glVertex2f(wborder,   hborder);
-			glTexCoord2f(1,0); glVertex2f(w-wborder, hborder);
-			glTexCoord2f(0,1); glVertex2f(wborder,   h-hborder);
-			glTexCoord2f(1,1); glVertex2f(w-wborder, h-hborder);
-		glEnd();
+        glBegin(GL_TRIANGLE_STRIP);
+            glTexCoord2f(0,0); glVertex2f(wborder,   hborder);
+            glTexCoord2f(1,0); glVertex2f(w-wborder, hborder);
+            glTexCoord2f(0,1); glVertex2f(wborder,   h-hborder);
+            glTexCoord2f(1,1); glVertex2f(w-wborder, h-hborder);
+        glEnd();
 
-		/*module logo*/
-		float logoh = 0.30f*min(w, h), logow = logoh*0.90;
-		float logox = 0.18f*(w - logow), logoy = 1.35*(h*0.5f - logoh);
+        /*module logo*/
+        float logoh = 0.30f*min(w, h), logow = logoh*0.90;
+        float logox = 0.18f*(w - logow), logoy = 1.35*(h*0.5f - logoh);
 
-		settexture(imagelogo, 3);
+        settexture(imagelogo, 3);
 
-		glBegin(GL_TRIANGLE_STRIP);
-			glTexCoord2f(0,0); glVertex2f(logox,       logoy);
-			glTexCoord2f(1,0); glVertex2f(logox+logow, logoy);
-			glTexCoord2f(0,1); glVertex2f(logox,       logoy+logoh);
-			glTexCoord2f(1,1); glVertex2f(logox+logow, logoy+logoh);
-		glEnd();
+        glBegin(GL_TRIANGLE_STRIP);
+            glTexCoord2f(0,0); glVertex2f(logox,       logoy);
+            glTexCoord2f(1,0); glVertex2f(logox+logow, logoy);
+            glTexCoord2f(0,1); glVertex2f(logox,       logoy+logoh);
+            glTexCoord2f(1,1); glVertex2f(logox+logow, logoy+logoh);
+        glEnd();
 
-		/*loading screen title*/
-		float lh = 0.23f*min(w,h), lw = lh*5.67;
-		float lx = 0.03f*(w - lw), ly = 0.03f*(h*0.5f - lh);
+        /*loading screen title*/
+        float lh = 0.23f*min(w,h), lw = lh*5.67;
+        float lx = 0.03f*(w - lw), ly = 0.03f*(h*0.5f - lh);
 
-		settexture("data/loadingscreen/title.png", 3);
+        settexture("data/loadingscreen/title.png", 3);
 
-		glBegin(GL_TRIANGLE_STRIP);
-			glTexCoord2f(0,0); glVertex2f(lx,    ly);
-			glTexCoord2f(1,0); glVertex2f(lx+lw, ly);
-			glTexCoord2f(0,1); glVertex2f(lx,    ly+lh);
-			glTexCoord2f(1,1); glVertex2f(lx+lw, ly+lh);
-		glEnd();
+        glBegin(GL_TRIANGLE_STRIP);
+            glTexCoord2f(0,0); glVertex2f(lx,    ly);
+            glTexCoord2f(1,0); glVertex2f(lx+lw, ly);
+            glTexCoord2f(0,1); glVertex2f(lx,    ly+lh);
+            glTexCoord2f(1,1); glVertex2f(lx+lw, ly+lh);
+        glEnd();
 
-		/*engine badge*/
-		float badgeh = 0.12f*min(w, h), badgew = badgeh*2.00;
-		float badgex = 0.01f*(w - badgew), badgey = 2.3f*(h*0.5f - badgeh);
+        /*engine badge*/
+        float badgeh = 0.12f*min(w, h), badgew = badgeh*2.00;
+        float badgex = 0.01f*(w - badgew), badgey = 2.3f*(h*0.5f - badgeh);
 
-		settexture("data/loadingscreen/engine_badge.png", 3);
+        settexture("data/loadingscreen/engine_badge.png", 3);
 
-		glBegin(GL_TRIANGLE_STRIP);
-			glTexCoord2f(0, 0); glVertex2f(badgex,			badgey);
-			glTexCoord2f(1, 0); glVertex2f(badgex+badgew,	badgey);
-			glTexCoord2f(0, 1); glVertex2f(badgex,			badgey+badgeh);
-			glTexCoord2f(1, 1); glVertex2f(badgex+badgew,	badgey+badgeh);
-		glEnd();
+        glBegin(GL_TRIANGLE_STRIP);
+            glTexCoord2f(0, 0); glVertex2f(badgex,          badgey);
+            glTexCoord2f(1, 0); glVertex2f(badgex+badgew,   badgey);
+            glTexCoord2f(0, 1); glVertex2f(badgex,          badgey+badgeh);
+            glTexCoord2f(1, 1); glVertex2f(badgex+badgew,   badgey+badgeh);
+        glEnd();
 
-		/*the rest*/
-		if(caption)
-		{
-			int tw = text_width(caption);
+        /*the rest*/
+        if(caption)
+        {
+            int tw = text_width(caption);
 
-			float tsz = 0.04f*min(w, h)/FONTH;
-			float tx = 0.5f*(w - tw*tsz);
-			float ty = h - 0.075f*1.5f*min(w, h) - 1.25f*FONTH*tsz;
+            float tsz = 0.04f*min(w, h)/FONTH;
+            float tx = 0.5f*(w - tw*tsz);
+            float ty = h - 0.075f*1.5f*min(w, h) - 1.25f*FONTH*tsz;
 
-			glPushMatrix();
-			glTranslatef(tx, ty, 0);
-			glScalef(tsz, tsz, 1);
-			draw_text(caption, 0, 0);
-			glPopMatrix();
-		}
+            glPushMatrix();
+            glTranslatef(tx, ty, 0);
+            glScalef(tsz, tsz, 1);
+            draw_text(caption, 0, 0);
+            glPopMatrix();
+        }
 
-		if(mapshot || mapname)
-		{
-			int infowidth = 12*FONTH;
-			float sz = 0.35f*min(w, h);
-			float msz = (0.75f*min(w, h) - sz)/(infowidth + FONTH);
-			float x = 0.5f*(w-sz);
-			float y = ly+lh - sz/15;
+        if(mapshot || mapname)
+        {
+            int infowidth = 12*FONTH;
+            float sz = 0.35f*min(w, h);
+            float msz = (0.75f*min(w, h) - sz)/(infowidth + FONTH);
+            float x = 0.5f*(w-sz);
+            float y = ly+lh - sz/15;
 
-			if(mapinfo)
-			{
-				int miw, mih;
+            if(mapinfo)
+            {
+                int miw, mih;
 
-				text_bounds(mapinfo, miw, mih, infowidth);
-				x-= 0.5*(miw*msz + FONTH*msz);
-			}
-			if(mapshot && mapshot!=notexture)
-			{
-				glBindTexture(GL_TEXTURE_2D, mapshot->id);
-				glBegin(GL_TRIANGLE_STRIP);
-					glTexCoord2f(0, 0); glVertex2f(x,    y);
-					glTexCoord2f(1, 0); glVertex2f(x+sz, y);
-					glTexCoord2f(0, 1); glVertex2f(x,    y+sz);
-					glTexCoord2f(1, 1); glVertex2f(x+sz, y+sz);
-				glEnd();
-			}
-			else
-			{
-				int qw, qh;
-				text_bounds("?", qw, qh);
-				float qsz = sz*0.5f/max(qw, qh);
-				glPushMatrix();
-				glTranslatef(x + 0.5f*(sz - qw*qsz), y + 0.5f*(sz - qh*qsz), 0);
-				glScalef(qsz, qsz, 1);
-				draw_text("?", 0, 0);
-				glPopMatrix();
-				//glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
-			}
+                text_bounds(mapinfo, miw, mih, infowidth);
+                x-= 0.5*(miw*msz + FONTH*msz);
+            }
+            if(mapshot && mapshot!=notexture)
+            {
+                glBindTexture(GL_TEXTURE_2D, mapshot->id);
+                glBegin(GL_TRIANGLE_STRIP);
+                    glTexCoord2f(0, 0); glVertex2f(x,    y);
+                    glTexCoord2f(1, 0); glVertex2f(x+sz, y);
+                    glTexCoord2f(0, 1); glVertex2f(x,    y+sz);
+                    glTexCoord2f(1, 1); glVertex2f(x+sz, y+sz);
+                glEnd();
+            }
+            else
+            {
+                int qw, qh;
+                text_bounds("?", qw, qh);
+                float qsz = sz*0.5f/max(qw, qh);
+                glPushMatrix();
+                glTranslatef(x + 0.5f*(sz - qw*qsz), y + 0.5f*(sz - qh*qsz), 0);
+                glScalef(qsz, qsz, 1);
+                draw_text("?", 0, 0);
+                glPopMatrix();
+                //glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+            }
 
-			settexture("data/loadingscreen/mapshot_frame.png", 3);
-			glBegin(GL_TRIANGLE_STRIP);
-				glTexCoord2f(0, 0); glVertex2f(x,    y);
-				glTexCoord2f(1, 0); glVertex2f(x+sz, y);
-				glTexCoord2f(0, 1); glVertex2f(x,    y+sz);
-				glTexCoord2f(1, 1); glVertex2f(x+sz, y+sz);
-			glEnd();
+            settexture("data/loadingscreen/mapshot_frame.png", 3);
+            glBegin(GL_TRIANGLE_STRIP);
+                glTexCoord2f(0, 0); glVertex2f(x,    y);
+                glTexCoord2f(1, 0); glVertex2f(x+sz, y);
+                glTexCoord2f(0, 1); glVertex2f(x,    y+sz);
+                glTexCoord2f(1, 1); glVertex2f(x+sz, y+sz);
+            glEnd();
 
-			if(mapname)
-			{
-				int tw = text_width(mapname);
-				float tsz = sz/(8*FONTH);
-				float tx = 0.9f*sz - tw*tsz;
-				float ty = 0.9f*sz - FONTH*tsz;
+            if(mapname)
+            {
+                int tw = text_width(mapname);
+                float tsz = sz/(8*FONTH);
+                float tx = 0.9f*sz - tw*tsz;
+                float ty = 0.9f*sz - FONTH*tsz;
 
-				if(tx < 0.1f*sz)
-				{
-					tsz = 0.1f*sz/tw;
-					tx = 0.1f;
-				}
-				glPushMatrix();
+                if(tx < 0.1f*sz)
+                {
+                    tsz = 0.1f*sz/tw;
+                    tx = 0.1f;
+                }
+                glPushMatrix();
 
-				glTranslatef(x+tx, y+ty, 0);
-				glScalef(tsz, tsz, 1);
-				draw_text(mapname, 0, 0);
+                glTranslatef(x+tx, y+ty, 0);
+                glScalef(tsz, tsz, 1);
+                draw_text(mapname, 0, 0);
 
-				glPopMatrix();
-			}
-			if(mapinfo)
-			{
-				glPushMatrix();
+                glPopMatrix();
+            }
+            if(mapinfo)
+            {
+                glPushMatrix();
 
-				glTranslatef(x+sz+FONTH*msz, y, 0);
-				glScalef(msz, msz, 1);
-				draw_text(mapinfo, 0, 0, 0xFF, 0xFF, 0xFF, 0xFF, -1, infowidth);
+                glTranslatef(x+sz+FONTH*msz, y, 0);
+                glScalef(msz, msz, 1);
+                draw_text(mapinfo, 0, 0, 0xFF, 0xFF, 0xFF, 0xFF, -1, infowidth);
 
-				glPopMatrix();
-			}
-		}
-		if(!mapshot && !mapname && !caption && !mapinfo)
-		{
-			float scale = (min(w, h) * 0.100f) / max(1, text_width(version)),
-				ty = w - (min(w,h) * 0.15f),
-				tx = h - (1.5f * FONTH * scale);
+                glPopMatrix();
+            }
+        }
+        if(!mapshot && !mapname && !caption && !mapinfo)
+        {
+            float scale = (min(w, h) * 0.100f) / max(1, text_width(version)),
+                ty = w - (min(w,h) * 0.15f),
+                tx = h - (1.5f * FONTH * scale);
 
-			glPushMatrix();
+            glPushMatrix();
 
-			glTranslatef(ty, tx, 0);
-			glScalef(scale, scale, 1);
-			draw_text(version, 0, 0);
+            glTranslatef(ty, tx, 0);
+            glScalef(scale, scale, 1);
+            draw_text(version, 0, 0);
 
-			glPopMatrix();
-		}
-		glDisable(GL_BLEND);
-		if(!restore)
-			swapbuffers();
-	}
-	glDisable(GL_TEXTURE_2D);
+            glPopMatrix();
+        }
+        glDisable(GL_BLEND);
+        if(!restore)
+            swapbuffers();
+    }
+    glDisable(GL_TEXTURE_2D);
 
-	if(!restore)
-	{
-		renderedframe = false;
-		copystring(backgroundcaption, caption ? caption : "");
-		backgroundmapshot = mapshot;
-		copystring(backgroundmapname, mapname ? mapname : "");
-		if(mapinfo != backgroundmapinfo)
-		{
-			DELETEA(backgroundmapinfo);
-			if(mapinfo)
-				backgroundmapinfo = newstring(mapinfo);
-		}
-	}
+    if(!restore)
+    {
+        renderedframe = false;
+        copystring(backgroundcaption, caption ? caption : "");
+        backgroundmapshot = mapshot;
+        copystring(backgroundmapname, mapname ? mapname : "");
+        if(mapinfo != backgroundmapinfo)
+        {
+            DELETEA(backgroundmapinfo);
+            if(mapinfo)
+                backgroundmapinfo = newstring(mapinfo);
+        }
+    }
 }
 
 void setnotextureshader()
 {
-	notextureshader->set();
+    notextureshader->set();
 }
 
 void setdefaultshader()
 {
-	defaultshader->set();
+    defaultshader->set();
 }
 
 void setlineshader()
 {
-	lineshader->set();
+    lineshader->set();
 }
