@@ -210,8 +210,8 @@ COMMAND(mdlpitch, "f");
 
 void mdlroll(float *angle)
 {
-	checkmdl;
-	loadingmodel->offsetroll = *angle;
+    checkmdl;
+    loadingmodel->offsetroll = *angle;
 }
 
 COMMAND(mdlroll, "f");
@@ -351,7 +351,7 @@ void mapmodelreset(int *n)
     mapmodels.shrink(clamp(*n, 0, mapmodels.length()));
 }
 
-mapmodelinfo &getmminfo(int i) { return mapmodels.inrange(i) ? mapmodels[i] : *(mapmodelinfo *)0; }
+mapmodelinfo *getmminfo(int i) { return mapmodels.inrange(i) ? &mapmodels[i] : 0; }
 const char *mapmodelname(int i) { return mapmodels.inrange(i) ? mapmodels[i].name : NULL; }
 
 COMMAND(mmodel, "s");
@@ -396,10 +396,10 @@ void preloadusedmapmodels(bool msg, bool bih)
     {
         loadprogress = float(i+1)/mapmodels.length();
         int mmindex = mapmodels[i];
-        mapmodelinfo &mmi = getmminfo(mmindex);
-        if(!&mmi) { if(msg) conoutf(CON_WARN, "could not find map model: %d", mmindex); }
-        else if(!loadmodel(NULL, mmindex, msg)) { if(msg) conoutf(CON_WARN, "could not load model: %s", mmi.name); }
-        else if(mmi.m && bih) mmi.m->preloadBIH();
+        mapmodelinfo *mmi = getmminfo(mmindex);
+        if(!mmi) { if(msg) conoutf(CON_WARN, "could not find map model: %d", mmindex); }
+        else if(!loadmodel(NULL, mmindex, msg)) { if(msg) conoutf(CON_WARN, "could not load model: %s", mmi->name); }
+        else if(mmi->m && bih) mmi->m->preloadBIH();
     }
     loadprogress = 0;
 }
@@ -433,7 +433,11 @@ model *loadmodel(const char *name, int i, bool msg)
             DELETEP(m);
         }
         loadingmodel = NULL;
-        if(!m) return NULL;
+        if(!m)
+        {
+            conoutf(CON_WARN, "unable to load model: packages/models/%s", name);
+            return NULL;
+        }
         mdllookup.access(m->name(), m);
     }
     if(mapmodels.inrange(i) && !mapmodels[i].m) mapmodels[i].m = m;
@@ -513,9 +517,10 @@ void renderellipse(vec &o, float xradius, float yradius, float yaw)
     glDisable(GL_TEXTURE_2D);
     glColor3f(0.5f, 0.5f, 0.5f);
     glBegin(GL_LINE_LOOP);
-    loopi(16)
+    loopi(15)
     {
-        vec p(xradius*cosf(2*M_PI*i/16.0f), yradius*sinf(2*M_PI*i/16.0f), 0);
+        const vec2 &sc = sincos360[i*(360/15)];
+        vec p(xradius*sc.x, yradius*sc.y, 0);
         p.rotate_around_z((yaw+90)*RAD);
         p.add(o);
         glVertex3fv(p.v);
@@ -908,7 +913,7 @@ void rendermodel(entitylight *light, const char *mdl, int anim, const vec &o, fl
         b.anim = anim;
         b.yaw = yaw;
         b.pitch = pitch;
-	b.roll = roll;
+        b.roll = roll;
         b.basetime = basetime;
         b.basetime2 = basetime2;
         b.transparent = trans;
@@ -978,8 +983,8 @@ bool matchanim(const char *name, const char *pattern)
             if(!c || c=='|') break;
             else if(c=='*')
             {
-                if(!*s || isspace(*s)) break;
-                do s++; while(*s && !isspace(*s));
+                if(!*s || iscubespace(*s)) break;
+                do s++; while(*s && !iscubespace(*s));
             }
             else if(c!=*s) break;
             else s++;
@@ -993,7 +998,8 @@ bool matchanim(const char *name, const char *pattern)
 
 void findanims(const char *pattern, vector<int> &anims)
 {
-    loopi(sizeof(animnames)/sizeof(animnames[0])) if(matchanim(animnames[i], pattern)) anims.add(i);
+    loopi(game::numanims())
+        if(matchanim(game::animname(i), pattern)) anims.add(i);
 }
 
 ICOMMAND(findanims, "s", (char *name),
@@ -1031,7 +1037,8 @@ void loadskin(const char *dir, const char *altdir, Texture *&skin, Texture *&mas
 
 // convenient function that covers the usual anims for players/monsters/npcs
 
-VAR(animoverride, -1, 0, NUMANIMS-1);
+VARF(animoverride, -1, 0, 0x7F,
+    if(animoverride > 0) animoverride %= game::numanims());
 VAR(testanims, 0, 0, 1);
 VAR(testpitch, -90, 0, 90);
 
@@ -1090,7 +1097,7 @@ void renderclient(dynent *d, const char *mdlname, modelattach *attachments, int
     rendermodel(NULL, mdlname, anim, o, yaw, pitch, d->roll, flags, d, attachments, basetime, 0, fade, colour);
 }
 
-void setbbfrommodel(dynent *d, const char *mdl)
+void setbbfrommodel(physent *d, const char *mdl)
 {
     model *m = loadmodel(mdl);
     if(!m) return;
