@@ -158,17 +158,17 @@ static struct domevert
     vec pos;
     uchar color[4];
 
-	domevert() {}
-	domevert(const vec &pos, const bvec &fcolor, float alpha) : pos(pos)
-	{
-		memcpy(color, fcolor.v, 3);
-		color[3] = uchar(alpha*255);
-	}
-	domevert(const domevert &v0, const domevert &v1) : pos(vec(v0.pos).add(v1.pos).normalize())
-	{
+    domevert() {}
+    domevert(const vec &pos, const bvec &fcolor, float alpha) : pos(pos)
+    {
+        memcpy(color, fcolor.v, 3);
+        color[3] = uchar(alpha*255);
+    }
+    domevert(const domevert &v0, const domevert &v1) : pos(vec(v0.pos).add(v1.pos).normalize())
+    {
         memcpy(color, v0.color, 4);
         if(v0.pos.z != v1.pos.z) color[3] += uchar((v1.color[3] - v0.color[3]) * (pos.z - v0.pos.z) / (v1.pos.z - v0.pos.z));
-	}
+    }
 } *domeverts = NULL;
 static GLushort *domeindices = NULL;
 static int domenumverts = 0, domenumindices = 0, domecapindices = 0;
@@ -220,30 +220,23 @@ static void initdome(const bvec &color, float minalpha = 0.0f, float maxalpha =
     if(clipz >= 1)
     {
         domeverts[domenumverts++] = domevert(vec(0.0f, 0.0f, 1.0f), color, minalpha); //build initial 'hres' sided pyramid
-        loopi(hres)
-        {
-            float angle = 2*M_PI*float(i)/hres;
-            domeverts[domenumverts++] = domevert(vec(cosf(angle), sinf(angle), 0.0f), color, maxalpha);
-        }
+        loopi(hres) domeverts[domenumverts++] = domevert(vec(sincos360[(360*i)/hres], 0.0f), color, maxalpha);
         loopi(hres) genface(depth, 0, i+1, 1+(i+1)%hres);
     }
     else if(clipz <= 0)
     {
-        loopi(hres<<depth)
-        {
-            float angle = 2*M_PI*float(i)/(hres<<depth), x = cosf(angle), y = sinf(angle);
-            domeverts[domenumverts++] = domevert(vec(x, y, 0.0f), color, maxalpha);
-        }
+        loopi(hres<<depth) domeverts[domenumverts++] = domevert(vec(sincos360[(360*i)/(hres<<depth)], 0.0f), color, maxalpha);
     }
     else
     {
-        float clipxy = sqrtf(1 - clipz*clipz), xm = cosf(M_PI/hres), ym = sinf(M_PI/hres);
+        float clipxy = sqrtf(1 - clipz*clipz);
+        const vec2 &scm = sincos360[180/hres];
         loopi(hres)
         {
-            float angle = 2*M_PI*float(i)/hres, x = cosf(angle), y = sinf(angle);
-            domeverts[domenumverts++] = domevert(vec(x*clipxy, y*clipxy, clipz), color, minalpha);
-            domeverts[domenumverts++] = domevert(vec(x, y, 0.0f), color, maxalpha);
-            domeverts[domenumverts++] = domevert(vec(x*xm - y*ym, y*xm + x*ym, 0.0f), color, maxalpha);
+            const vec2 &sc = sincos360[(360*i)/hres];
+            domeverts[domenumverts++] = domevert(vec(sc.x*clipxy, sc.y*clipxy, clipz), color, minalpha);
+            domeverts[domenumverts++] = domevert(vec(sc.x, sc.y, 0.0f), color, maxalpha);
+            domeverts[domenumverts++] = domevert(vec(sc.x*scm.x - sc.y*scm.y, sc.y*scm.x + sc.x*scm.y, 0.0f), color, maxalpha);
         }
         loopi(hres)
         {
@@ -255,11 +248,11 @@ static void initdome(const bvec &color, float minalpha = 0.0f, float maxalpha =
 
     if(capsize >= 0)
     {
-		GLushort *domecap = &domeindices[domenumindices];
-		int domecapverts = 0;
-		loopi(domenumverts) if(!domeverts[i].pos.z) domecap[domecapverts++] = i;
+        GLushort *domecap = &domeindices[domenumindices];
+        int domecapverts = 0;
+        loopi(domenumverts) if(!domeverts[i].pos.z) domecap[domecapverts++] = i;
         domeverts[domenumverts++] = domevert(vec(0.0f, 0.0f, -capsize), color, maxalpha);
-		quicksort(domecap, domecapverts, sortdomecap);
+        quicksort(domecap, domecapverts, sortdomecap);
         loopi(domecapverts)
         {
             int n = domecapverts-1-i;
@@ -286,7 +279,7 @@ static void initdome(const bvec &color, float minalpha = 0.0f, float maxalpha =
 
 static void deletedome()
 {
-	domenumverts = domenumindices = 0;
+    domenumverts = domenumindices = 0;
     if(domevbuf) { glDeleteBuffers_(1, &domevbuf); domevbuf = 0; }
     if(domeebuf) { glDeleteBuffers_(1, &domeebuf); domeebuf = 0; }
     DELETEA(domeverts);
@@ -557,11 +550,11 @@ void drawskybox(int farplane, bool limited)
         glEnable(GL_CULL_FACE);
     }
 
-	if(!glaring && fogdomemax && fogdomeclouds)
-	{
+    if(!glaring && fogdomemax && fogdomeclouds)
+    {
         if(fading) glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_FALSE);
         drawfogdome(farplane);
-	}
+    }
 
     if(clampsky) glDepthRange(0, 1);
 
