@@ -307,26 +307,21 @@ void drawbb(const ivec &bo, const ivec &br, const vec &camera)
 {
     glBegin(GL_QUADS);
 
-    loopi(6)
-    {
-        int dim = dimension(i), coord = dimcoord(i);
-
-        if(coord)
-        {
-            if(camera[dim] < bo[dim] + br[dim]) continue;
-        }
-        else if(camera[dim] > bo[dim]) continue;
-
-        loopj(4)
-        {
-            const ivec &cc = facecoords[i][j];
-            glVertex3f(cc.x ? bo.x+br.x : bo.x,
-                       cc.y ? bo.y+br.y : bo.y,
-                       cc.z ? bo.z+br.z : bo.z);
-        }
-
-        xtraverts += 4;
-    }
+    #define GENFACEORIENT(orient, v0, v1, v2, v3) do { \
+        int dim = dimension(orient); \
+        if(dimcoord(orient)) \
+        { \
+            if(camera[dim] < bo[dim] + br[dim]) continue; \
+        } \
+        else if(camera[dim] > bo[dim]) continue; \
+        v0 v1 v2 v3 \
+        xtraverts += 4; \
+    } while(0);
+    #define GENFACEVERT(orient, vert, ox,oy,oz, rx,ry,rz) \
+        glVertex3f(ox rx, oy ry, oz rz);
+    GENFACEVERTS(bo.x, bo.x + br.x, bo.y, bo.y + br.y, bo.z, bo.z + br.z, , , , , , )
+    #undef GENFACEORIENT
+    #undef GENFACEVERTS
 
     glEnd();
 }
@@ -397,8 +392,8 @@ void rendermapmodel(extentity &e)
 {
     int anim = ANIM_MAPMODEL|ANIM_LOOP, basetime = 0;
     if(e.flags&extentity::F_ANIM) entities::animatemapmodel(e, anim, basetime);
-    mapmodelinfo &mmi = getmminfo(e.attr[1]);
-    if(&mmi) rendermodel(&e.light, mmi.name, anim, e.o, e.attr[0], 0, 0, MDL_CULL_VFC | MDL_CULL_DIST | MDL_DYNLIGHT, NULL, NULL, basetime, 0, 1, e.attr[5]? e.attr[5] : 0xFFFFFF);
+    mapmodelinfo *mmi = getmminfo(e.attr[1]);
+    if(mmi) rendermodel(&e.light, mmi->name, anim, e.o, e.attr[0], 0, 0, MDL_CULL_VFC | MDL_CULL_DIST | MDL_DYNLIGHT, NULL, NULL, basetime, 0, 1, e.attr[5]? e.attr[5] : 0xFFFFFF);
 }
 
 extern int reflectdist;
@@ -425,7 +420,7 @@ void renderreflectedmapmodels()
         }
         *lastmms = NULL;
     }
-    for(octaentities *oe = mms; oe; oe = reflecting ? oe->rnext : oe->next)
+    for(octaentities *oe = mms; oe; oe = reflecting ? oe->rnext : oe->next) if(reflecting || oe->distance >= 0)
     {
         if(reflecting || refracting>0 ? oe->bbmax.z <= reflectz : oe->bbmin.z >= reflectz) continue;
         if(isfoggedcube(oe->o, oe->size)) continue;
@@ -2337,7 +2332,7 @@ void rendergeom(float causticspass, bool fogpass)
 
     if(doZP)
     {
-		glFlush();
+        glFlush();
         if(shadowmap && hasFBO && mainpass)
         {
             glDisableClientState(GL_VERTEX_ARRAY);
