@@ -48,16 +48,16 @@ static void writelog(FILE *file, const char *fmt, va_list args)
         fwrite(ubuf, 1, numu, file);
     }
 }
- 
+
 #ifdef STANDALONE
 void fatal(const char *fmt, ...)
 {
     void cleanupserver();
     cleanupserver();
-	defvformatstring(msg,fmt,fmt);
-	if(logfile) logoutf("%s", msg);
+    defvformatstring(msg,fmt,fmt);
+    if(logfile) logoutf("%s", msg);
 #ifdef WIN32
-	MessageBox(NULL, msg, "Cube 2: Sauerbraten fatal error", MB_OK|MB_SYSTEMMODAL);
+    MessageBox(NULL, msg, "Sandbox fatal error", MB_OK|MB_SYSTEMMODAL);
 #else
     fprintf(stderr, "server error: %s\n", msg);
 #endif
@@ -199,7 +199,7 @@ void filtertext(char *dst, const char *src, bool whitespace, int len)
             if(!*++src) break;
             continue;
         }
-        if(iscubeprint(c) || (isspace(c) && whitespace))
+        if(iscubeprint(c) || (iscubespace(c) && whitespace))
         {
             *dst++ = c;
             if(!--len) break;
@@ -225,6 +225,53 @@ ENetHost *serverhost = NULL;
 int laststatus = 0;
 ENetSocket pongsock = ENET_SOCKET_NULL, lansock = ENET_SOCKET_NULL;
 
+int localclients = 0, nonlocalclients = 0;
+
+bool hasnonlocalclients() { return nonlocalclients!=0; }
+bool haslocalclients() { return localclients!=0; }
+
+client &addclient(int type)
+{
+    client *c = NULL;
+    loopv(clients) if(clients[i]->type==ST_EMPTY)
+    {
+        c = clients[i];
+        break;
+    }
+    if(!c)
+    {
+        c = new client;
+        c->num = clients.length();
+        clients.add(c);
+    }
+    c->info = server::newclientinfo();
+    c->type = type;
+    switch(type)
+    {
+        case ST_TCPIP: nonlocalclients++; break;
+        case ST_LOCAL: localclients++; break;
+    }
+    return *c;
+}
+
+void delclient(client *c)
+{
+    if(!c) return;
+    switch(c->type)
+    {
+        case ST_TCPIP: nonlocalclients--; if(c->peer) c->peer->data = NULL; break;
+        case ST_LOCAL: localclients--; break;
+        case ST_EMPTY: return;
+    }
+    c->type = ST_EMPTY;
+    c->peer = NULL;
+    if(c->info)
+    {
+        server::deleteclientinfo(c->info);
+        c->info = NULL;
+    }
+}
+
 void cleanupserver()
 {
     if(serverhost) enet_host_destroy(serverhost);
@@ -363,10 +410,7 @@ void disconnect_client(int n, int reason)
     if(!clients.inrange(n) || clients[n]->type!=ST_TCPIP) return;
     enet_peer_disconnect(clients[n]->peer, reason);
     server::clientdisconnect(n);
-    clients[n]->type = ST_EMPTY;
-    clients[n]->peer->data = NULL;
-    server::deleteclientinfo(clients[n]->info);
-    clients[n]->info = NULL;
+    delclient(clients[n]);
     defformatstring(s)("client (%s) disconnected because: %s", clients[n]->hostname, disc_reasons[reason]);
     logoutf("%s", s);
     server::sendservmsg(s);
@@ -391,25 +435,6 @@ void localclienttoserver(int chan, ENetPacket *packet)
     if(c) process(packet, c->num, chan);
 }
 
-client &addclient()
-{
-    loopv(clients) if(clients[i]->type==ST_EMPTY)
-    {
-        clients[i]->info = server::newclientinfo();
-        return *clients[i];
-    }
-    client *c = new client;
-    c->num = clients.length();
-    c->info = server::newclientinfo();
-    clients.add(c);
-    return *c;
-}
-
-int localclients = 0, nonlocalclients = 0;
-
-bool hasnonlocalclients() { return nonlocalclients!=0; }
-bool haslocalclients() { return localclients!=0; }
-
 #ifdef STANDALONE
 bool resolverwait(const char *name, ENetAddress *address)
 {
@@ -433,7 +458,7 @@ VARN(updatemaster, allowupdatemaster, 0, 1, 1);
 
 void disconnectmaster()
 {
-    if(mastersock != ENET_SOCKET_NULL) 
+    if(mastersock != ENET_SOCKET_NULL)
     {
         enet_socket_destroy(mastersock);
         mastersock = ENET_SOCKET_NULL;
@@ -510,9 +535,9 @@ void processmasterinput()
         *end++ = '\0';
 
         const char *args = input;
-        while(args < end && !isspace(*args)) args++;
+        while(args < end && !iscubespace(*args)) args++;
         int cmdlen = args - input;
-        while(args < end && isspace(*args)) args++;
+        while(args < end && iscubespace(*args)) args++;
 
         if(!strncmp(input, "failreg", cmdlen))
             conoutf(CON_ERROR, "master server registration failed: %s", args);
@@ -558,7 +583,7 @@ void flushmasterinput()
         masterin.reserve(4096);
 
     ENetBuffer buf;
-    buf.data = &masterin[masterin.length()];
+    buf.data = masterin.getbuf() + masterin.length();
     buf.dataLength = masterin.capacity() - masterin.length();
     int recv = enet_socket_receive(mastersock, NULL, &buf, 1);
     if(recv > 0)
@@ -638,7 +663,7 @@ uint totalsecs = 0;
 void updatetime()
 {
     static int lastsec = 0;
-    if(totalmillis - lastsec >= 1000) 
+    if(totalmillis - lastsec >= 1000)
     {
         int cursecs = (totalmillis - lastsec) / 1000;
         totalsecs += cursecs;
@@ -648,13 +673,6 @@ void updatetime()
 
 void serverslice(bool dedicated, uint timeout)   // main server update, called from main loop in sp, or from below in dedicated server
 {
-    localclients = nonlocalclients = 0;
-    loopv(clients) switch(clients[i]->type)
-    {
-        case ST_LOCAL: localclients++; break;
-        case ST_TCPIP: nonlocalclients++; break;
-    }
-
     if(!serverhost)
     {
         server::serverupdate();
@@ -700,16 +718,14 @@ void serverslice(bool dedicated, uint timeout)   // main server update, called f
         {
             case ENET_EVENT_TYPE_CONNECT:
             {
-                client &c = addclient();
-                c.type = ST_TCPIP;
+                client &c = addclient(ST_TCPIP);
                 c.peer = event.peer;
                 c.peer->data = &c;
                 char hn[1024];
                 copystring(c.hostname, (enet_address_get_host_ip(&c.peer->address, hn, sizeof(hn))==0) ? hn : "unknown");
                 logoutf("client connected (%s)", c.hostname);
                 int reason = server::clientconnect(c.num, c.peer->address.host);
-                if(!reason) nonlocalclients++;
-                else disconnect_client(c.num, reason);
+                if(reason) disconnect_client(c.num, reason);
                 break;
             }
             case ENET_EVENT_TYPE_RECEIVE:
@@ -725,11 +741,7 @@ void serverslice(bool dedicated, uint timeout)   // main server update, called f
                 if(!c) break;
                 logoutf("disconnected client (%s)", c->hostname);
                 server::clientdisconnect(c->num);
-                nonlocalclients--;
-                c->type = ST_EMPTY;
-                event.peer->data = NULL;
-                server::deleteclientinfo(c->info);
-                c->info = NULL;
+                delclient(c);
                 break;
             }
             default:
@@ -751,27 +763,18 @@ void localdisconnect(bool cleanup)
     loopv(clients) if(clients[i]->type==ST_LOCAL)
     {
         server::localdisconnect(i);
-        localclients--;
-        clients[i]->type = ST_EMPTY;
-        server::deleteclientinfo(clients[i]->info);
-        clients[i]->info = NULL;
+        delclient(clients[i]);
         disconnected = true;
     }
     if(!disconnected) return;
     game::gamedisconnect(cleanup);
-    #ifndef NEWGUI
-    mainmenu = 1;
-    #else
     UI::mainmenu = 1;
-    #endif
 }
 
 void localconnect()
 {
-    client &c = addclient();
-    c.type = ST_LOCAL;
+    client &c = addclient(ST_LOCAL);
     copystring(c.hostname, "local");
-    localclients++;
     game::gameconnect(false);
     server::localconnect(c.num);
 }
@@ -805,16 +808,16 @@ static void cleanupsystemtray()
 
 static bool setupsystemtray(UINT uCallbackMessage)
 {
-	NOTIFYICONDATA nid;
-	memset(&nid, 0, sizeof(nid));
-	nid.cbSize = sizeof(nid);
-	nid.hWnd = appwindow;
-	nid.uID = IDI_ICON1;
-	nid.uCallbackMessage = uCallbackMessage;
-	nid.uFlags = NIF_MESSAGE | NIF_ICON | NIF_TIP;
-	nid.hIcon = appicon;
-	strcpy(nid.szTip, apptip);
-	if(Shell_NotifyIcon(NIM_ADD, &nid) != TRUE)
+    NOTIFYICONDATA nid;
+    memset(&nid, 0, sizeof(nid));
+    nid.cbSize = sizeof(nid);
+    nid.hWnd = appwindow;
+    nid.uID = IDI_ICON1;
+    nid.uCallbackMessage = uCallbackMessage;
+    nid.uFlags = NIF_MESSAGE | NIF_ICON | NIF_TIP;
+    nid.hIcon = appicon;
+    strcpy(nid.szTip, apptip);
+    if(Shell_NotifyIcon(NIM_ADD, &nid) != TRUE)
         return false;
     atexit(cleanupsystemtray);
     return true;
@@ -823,30 +826,30 @@ static bool setupsystemtray(UINT uCallbackMessage)
 #if 0
 static bool modifysystemtray()
 {
-	NOTIFYICONDATA nid;
-	memset(&nid, 0, sizeof(nid));
-	nid.cbSize = sizeof(nid);
-	nid.hWnd = appwindow;
-	nid.uID = IDI_ICON1;
-	nid.uFlags = NIF_TIP;
-	strcpy(nid.szTip, apptip);
-	return Shell_NotifyIcon(NIM_MODIFY, &nid) == TRUE;
+    NOTIFYICONDATA nid;
+    memset(&nid, 0, sizeof(nid));
+    nid.cbSize = sizeof(nid);
+    nid.hWnd = appwindow;
+    nid.uID = IDI_ICON1;
+    nid.uFlags = NIF_TIP;
+    strcpy(nid.szTip, apptip);
+    return Shell_NotifyIcon(NIM_MODIFY, &nid) == TRUE;
 }
 #endif
 
 static void cleanupwindow()
 {
-	if(!appinstance) return;
-	if(appmenu)
-	{
-		DestroyMenu(appmenu);
-		appmenu = NULL;
-	}
-	if(wndclass)
-	{
-		UnregisterClass(MAKEINTATOM(wndclass), appinstance);
-		wndclass = 0;
-	}
+    if(!appinstance) return;
+    if(appmenu)
+    {
+        DestroyMenu(appmenu);
+        appmenu = NULL;
+    }
+    if(wndclass)
+    {
+        UnregisterClass(MAKEINTATOM(wndclass), appinstance);
+        wndclass = 0;
+    }
 }
 
 static BOOL WINAPI consolehandler(DWORD dwCtrlType)
@@ -871,18 +874,18 @@ static void writeline(logline &line)
         int numu = encodeutf8(ubuf, sizeof(ubuf), &((uchar *)line.buf)[carry], len - carry, &carry);
         DWORD written = 0;
         WriteConsole(outhandle, ubuf, numu, &written, NULL);
-    }     
+    }
 }
 
 static void setupconsole()
 {
-	if(conwindow) return;
+    if(conwindow) return;
     if(!AllocConsole()) return;
-	SetConsoleCtrlHandler(consolehandler, TRUE);
-	conwindow = GetConsoleWindow();
+    SetConsoleCtrlHandler(consolehandler, TRUE);
+    conwindow = GetConsoleWindow();
     SetConsoleTitle(apptip);
-	//SendMessage(conwindow, WM_SETICON, ICON_SMALL, (LPARAM)appicon);
-	SendMessage(conwindow, WM_SETICON, ICON_BIG, (LPARAM)appicon);
+    //SendMessage(conwindow, WM_SETICON, ICON_SMALL, (LPARAM)appicon);
+    SendMessage(conwindow, WM_SETICON, ICON_BIG, (LPARAM)appicon);
     outhandle = GetStdHandle(STD_OUTPUT_HANDLE);
     CONSOLE_SCREEN_BUFFER_INFO coninfo;
     GetConsoleScreenBufferInfo(outhandle, &coninfo);
@@ -895,93 +898,93 @@ static void setupconsole()
 
 enum
 {
-	MENU_OPENCONSOLE = 0,
-	MENU_SHOWCONSOLE,
-	MENU_HIDECONSOLE,
-	MENU_EXIT
+    MENU_OPENCONSOLE = 0,
+    MENU_SHOWCONSOLE,
+    MENU_HIDECONSOLE,
+    MENU_EXIT
 };
 
 static LRESULT CALLBACK handlemessages(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
 {
-	switch(uMsg)
-	{
-		case WM_APP:
-			SetForegroundWindow(hWnd);
-			switch(lParam)
-			{
-				//case WM_MOUSEMOVE:
-				//	break;
-				case WM_LBUTTONUP:
-				case WM_RBUTTONUP:
-				{
-					POINT pos;
-					GetCursorPos(&pos);
-					TrackPopupMenu(appmenu, TPM_CENTERALIGN|TPM_BOTTOMALIGN|TPM_RIGHTBUTTON, pos.x, pos.y, 0, hWnd, NULL);
-					PostMessage(hWnd, WM_NULL, 0, 0);
-					break;
-				}
-			}
-			return 0;
-		case WM_COMMAND:
-			switch(LOWORD(wParam))
-			{
-                case MENU_OPENCONSOLE:
-					setupconsole();
-					if(conwindow) ModifyMenu(appmenu, 0, MF_BYPOSITION|MF_STRING, MENU_HIDECONSOLE, "Hide Console");
+    switch(uMsg)
+    {
+        case WM_APP:
+            SetForegroundWindow(hWnd);
+            switch(lParam)
+            {
+                //case WM_MOUSEMOVE:
+                //break;
+                case WM_LBUTTONUP:
+                case WM_RBUTTONUP:
+                {
+                    POINT pos;
+                    GetCursorPos(&pos);
+                    TrackPopupMenu(appmenu, TPM_CENTERALIGN|TPM_BOTTOMALIGN|TPM_RIGHTBUTTON, pos.x, pos.y, 0, hWnd, NULL);
+                    PostMessage(hWnd, WM_NULL, 0, 0);
                     break;
-				case MENU_SHOWCONSOLE:
-					ShowWindow(conwindow, SW_SHOWNORMAL);
-					ModifyMenu(appmenu, 0, MF_BYPOSITION|MF_STRING, MENU_HIDECONSOLE, "Hide Console");
-					break;
-				case MENU_HIDECONSOLE:
-					ShowWindow(conwindow, SW_HIDE);
-					ModifyMenu(appmenu, 0, MF_BYPOSITION|MF_STRING, MENU_SHOWCONSOLE, "Show Console");
-					break;
-				case MENU_EXIT:
-					PostMessage(hWnd, WM_CLOSE, 0, 0);
-					break;
-			}
-			return 0;
-		case WM_CLOSE:
-			PostQuitMessage(0);
-			return 0;
-	}
-	return DefWindowProc(hWnd, uMsg, wParam, lParam);
+                }
+            }
+            return 0;
+        case WM_COMMAND:
+            switch(LOWORD(wParam))
+            {
+                case MENU_OPENCONSOLE:
+                    setupconsole();
+                    if(conwindow) ModifyMenu(appmenu, 0, MF_BYPOSITION|MF_STRING, MENU_HIDECONSOLE, "Hide Console");
+                    break;
+                case MENU_SHOWCONSOLE:
+                    ShowWindow(conwindow, SW_SHOWNORMAL);
+                    ModifyMenu(appmenu, 0, MF_BYPOSITION|MF_STRING, MENU_HIDECONSOLE, "Hide Console");
+                    break;
+                case MENU_HIDECONSOLE:
+                    ShowWindow(conwindow, SW_HIDE);
+                    ModifyMenu(appmenu, 0, MF_BYPOSITION|MF_STRING, MENU_SHOWCONSOLE, "Show Console");
+                    break;
+                case MENU_EXIT:
+                    PostMessage(hWnd, WM_CLOSE, 0, 0);
+                    break;
+            }
+            return 0;
+        case WM_CLOSE:
+            PostQuitMessage(0);
+            return 0;
+    }
+    return DefWindowProc(hWnd, uMsg, wParam, lParam);
 }
 
 static void setupwindow(const char *title)
 {
-	copystring(apptip, title);
-	//appinstance = GetModuleHandle(NULL);
-	if(!appinstance) fatal("failed getting application instance");
-	appicon = LoadIcon(appinstance, MAKEINTRESOURCE(IDI_ICON1));//(HICON)LoadImage(appinstance, MAKEINTRESOURCE(IDI_ICON1), IMAGE_ICON, 0, 0, LR_DEFAULTSIZE);
-	if(!appicon) fatal("failed loading icon");
+    copystring(apptip, title);
+    //appinstance = GetModuleHandle(NULL);
+    if(!appinstance) fatal("failed getting application instance");
+    appicon = LoadIcon(appinstance, MAKEINTRESOURCE(IDI_ICON1));//(HICON)LoadImage(appinstance, MAKEINTRESOURCE(IDI_ICON1), IMAGE_ICON, 0, 0, LR_DEFAULTSIZE);
+    if(!appicon) fatal("failed loading icon");
 
-	appmenu = CreatePopupMenu();
-	if(!appmenu) fatal("failed creating popup menu");
+    appmenu = CreatePopupMenu();
+    if(!appmenu) fatal("failed creating popup menu");
     AppendMenu(appmenu, MF_STRING, MENU_OPENCONSOLE, "Open Console");
     AppendMenu(appmenu, MF_SEPARATOR, 0, NULL);
-	AppendMenu(appmenu, MF_STRING, MENU_EXIT, "Exit");
-	//SetMenuDefaultItem(appmenu, 0, FALSE);
+    AppendMenu(appmenu, MF_STRING, MENU_EXIT, "Exit");
+    //SetMenuDefaultItem(appmenu, 0, FALSE);
 
-	WNDCLASS wc;
-	memset(&wc, 0, sizeof(wc));
-	wc.hCursor = NULL; //LoadCursor(NULL, IDC_ARROW);
-	wc.hIcon = appicon;
-	wc.lpszMenuName = NULL;
-	wc.lpszClassName = title;
-	wc.style = 0;
-	wc.hInstance = appinstance;
-	wc.lpfnWndProc = handlemessages;
-	wc.cbWndExtra = 0;
-	wc.cbClsExtra = 0;
-	wndclass = RegisterClass(&wc);
-	if(!wndclass) fatal("failed registering window class");
+    WNDCLASS wc;
+    memset(&wc, 0, sizeof(wc));
+    wc.hCursor = NULL; //LoadCursor(NULL, IDC_ARROW);
+    wc.hIcon = appicon;
+    wc.lpszMenuName = NULL;
+    wc.lpszClassName = title;
+    wc.style = 0;
+    wc.hInstance = appinstance;
+    wc.lpfnWndProc = handlemessages;
+    wc.cbWndExtra = 0;
+    wc.cbClsExtra = 0;
+    wndclass = RegisterClass(&wc);
+    if(!wndclass) fatal("failed registering window class");
 
-	appwindow = CreateWindow(MAKEINTATOM(wndclass), title, 0, CW_USEDEFAULT, CW_USEDEFAULT, 0, 0, HWND_MESSAGE, NULL, appinstance, NULL);
-	if(!appwindow) fatal("failed creating window");
+    appwindow = CreateWindow(MAKEINTATOM(wndclass), title, 0, CW_USEDEFAULT, CW_USEDEFAULT, 0, 0, HWND_MESSAGE, NULL, appinstance, NULL);
+    if(!appwindow) fatal("failed creating window");
 
-	atexit(cleanupwindow);
+    atexit(cleanupwindow);
 
     if(!setupsystemtray(WM_APP)) fatal("failed adding to system tray");
 }
@@ -994,13 +997,13 @@ static char *parsecommandline(const char *src, vector<char *> &args)
         while(isspace(*src)) src++;
         if(!*src) break;
         args.add(dst);
-		for(bool quoted = false; *src && (quoted || !isspace(*src)); src++)
+        for(bool quoted = false; *src && (quoted || !isspace(*src)); src++)
         {
             if(*src != '"') *dst++ = *src;
-			else if(dst > buf && src[-1] == '\\') dst[-1] = '"';
-			else quoted = !quoted;
-		}
-		*dst++ = '\0';
+            else if(dst > buf && src[-1] == '\\') dst[-1] = '"';
+            else quoted = !quoted;
+        }
+        *dst++ = '\0';
     }
     args.add(NULL);
     return buf;
@@ -1011,7 +1014,7 @@ int WINAPI WinMain(HINSTANCE hInst, HINSTANCE hPrev, LPSTR szCmdLine, int sw)
 {
     vector<char *> args;
     char *buf = parsecommandline(GetCommandLine(), args);
-	appinstance = hInst;
+    appinstance = hInst;
 #ifdef STANDALONE
     int standalonemain(int argc, char **argv);
     int status = standalonemain(args.length()-1, args.getbuf());
@@ -1050,23 +1053,20 @@ void logoutfv(const char *fmt, va_list args)
 
 void rundedicatedserver()
 {
-#ifdef WIN32
-    setupwindow("Cube 2: Sauerbraten server");
-#endif
     logoutf("dedicated server started, waiting for clients...");
 #ifdef WIN32
     SetPriorityClass(GetCurrentProcess(), HIGH_PRIORITY_CLASS);
-	for(;;)
-	{
-		MSG msg;
-		while(PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
-		{
-			if(msg.message == WM_QUIT) exit(EXIT_SUCCESS);
-			TranslateMessage(&msg);
-			DispatchMessage(&msg);
-		}
-		serverslice(true, 5);
-	}
+    for(;;)
+    {
+        MSG msg;
+        while(PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
+        {
+            if(msg.message == WM_QUIT) exit(EXIT_SUCCESS);
+            TranslateMessage(&msg);
+            DispatchMessage(&msg);
+        }
+        serverslice(true, 5);
+    }
 #else
     for(;;) serverslice(true, 5);
 #endif
@@ -1088,7 +1088,7 @@ bool servererror(bool dedicated, const char *desc)
 
 bool setuplistenserver(bool dedicated)
 {
-    ENetAddress address = { ENET_HOST_ANY, serverport <= 0 ? server::serverport() : serverport };
+    ENetAddress address = { ENET_HOST_ANY, enet_uint16(serverport <= 0 ? server::serverport() : serverport) };
     if(*serverip)
     {
         if(enet_address_set_host(&address, serverip)<0) conoutf(CON_WARN, "WARNING: server ip not resolved");
@@ -1120,7 +1120,13 @@ bool setuplistenserver(bool dedicated)
 
 void initserver(bool listen, bool dedicated)
 {
-    if(dedicated) execfile("server-init.cfg", false);
+    if(dedicated)
+    {
+#ifdef WIN32
+        setupwindow("Sandbox server");
+#endif
+        execfile("server-init.cfg", false);
+    }
 
     if(listen) setuplistenserver(dedicated);
 
