@@ -89,7 +89,7 @@ void resolverstop(resolverthread &rt)
     rt.query = NULL;
     rt.starttime = 0;
     SDL_UnlockMutex(resolvermutex);
-} 
+}
 
 void resolverclear()
 {
@@ -130,12 +130,12 @@ bool resolvercheck(const char **name, ENetAddress *address)
     else loopv(resolverthreads)
     {
         resolverthread &rt = resolverthreads[i];
-        if(rt.query && totalmillis - rt.starttime > RESOLVERLIMIT)        
+        if(rt.query && totalmillis - rt.starttime > RESOLVERLIMIT)
         {
             resolverstop(rt);
             *name = rt.query;
             resolved = true;
-        }    
+        }
     }
     SDL_UnlockMutex(resolvermutex);
     return resolved;
@@ -153,10 +153,10 @@ bool resolverwait(const char *name, ENetAddress *address)
     SDL_CondSignal(querycond);
     int starttime = SDL_GetTicks(), timeout = 0;
     bool resolved = false;
-    for(;;) 
+    for(;;)
     {
         SDL_CondWaitTimeout(resultcond, resolvermutex, 250);
-        loopv(resolverresults) if(resolverresults[i].query == name) 
+        loopv(resolverresults) if(resolverresults[i].query == name)
         {
             address->host = resolverresults[i].address.host;
             resolverresults.remove(i);
@@ -164,11 +164,11 @@ bool resolverwait(const char *name, ENetAddress *address)
             break;
         }
         if(resolved) break;
-    
+
         timeout = SDL_GetTicks() - starttime;
         renderprogress(min(float(timeout)/RESOLVERLIMIT, 1.0f), text);
         if(interceptkey(SDLK_ESCAPE)) timeout = RESOLVERLIMIT + 1;
-        if(timeout > RESOLVERLIMIT) break;    
+        if(timeout > RESOLVERLIMIT) break;
     }
     if(!resolved && timeout > RESOLVERLIMIT)
     {
@@ -242,7 +242,7 @@ int connectwithtimeout(ENetSocket sock, const char *hostname, const ENetAddress
         {
             if(cd.result<0) enet_socket_destroy(sock);
             break;
-        }      
+        }
         timeout = SDL_GetTicks() - starttime;
         renderprogress(min(float(timeout)/CONNLIMIT, 1.0f), text);
         if(interceptkey(SDLK_ESCAPE)) timeout = CONNLIMIT + 1;
@@ -250,23 +250,23 @@ int connectwithtimeout(ENetSocket sock, const char *hostname, const ENetAddress
     }
 
     /* thread will actually timeout eventually if its still trying to connect
-     * so just leave it (and let it destroy socket) instead of causing problems on some platforms by killing it 
+     * so just leave it (and let it destroy socket) instead of causing problems on some platforms by killing it
      */
     connthread = NULL;
     SDL_UnlockMutex(connmutex);
 
     return cd.result;
 }
- 
+
 enum { UNRESOLVED = 0, RESOLVING, RESOLVED };
 
 struct serverinfo
 {
-    enum 
-    { 
+    enum
+    {
         WAITING = INT_MAX,
 
-        MAXPINGS = 3 
+        MAXPINGS = 3
     };
 
     string name, map, sdesc;
@@ -397,14 +397,14 @@ void addserver(const char *name, int port, const char *password, bool keep)
     s->keep = keep;
 }
 
-VARP(searchlan, 0, 1, 1);
+VARP(searchlan, 0, 0, 1);
 VARP(servpingrate, 1000, 5000, 60000);
 VARP(servpingdecay, 1000, 15000, 60000);
 VARP(maxservpings, 0, 10, 1000);
 
 void pingservers()
 {
-    if(pingsock == ENET_SOCKET_NULL) 
+    if(pingsock == ENET_SOCKET_NULL)
     {
         pingsock = enet_socket_create(ENET_SOCKET_TYPE_DATAGRAM);
         if(pingsock == ENET_SOCKET_NULL)
@@ -430,7 +430,7 @@ void pingservers()
         buf.data = ping;
         buf.dataLength = p.length();
         enet_socket_send(pingsock, &si.address, &buf, 1);
-        
+
         si.checkdecay(servpingdecay);
     }
     if(searchlan)
@@ -444,7 +444,7 @@ void pingservers()
     }
     lastinfo = totalmillis;
 }
-  
+
 void checkresolver()
 {
     int resolving = 0;
@@ -470,7 +470,7 @@ void checkresolver()
             serverinfo &si = *servers[i];
             if(name == si.name)
             {
-                si.resolved = RESOLVED; 
+                si.resolved = RESOLVED;
                 si.address.host = addr.host;
                 break;
             }
@@ -488,15 +488,15 @@ void checkpings()
     ENetAddress addr;
     uchar ping[MAXTRANS];
     char text[MAXTRANS];
-    buf.data = ping; 
+    buf.data = ping;
     buf.dataLength = sizeof(ping);
     while(enet_socket_wait(pingsock, &events, 0) >= 0 && events)
     {
         int len = enet_socket_receive(pingsock, &addr, &buf, 1);
-        if(len <= 0) return;  
+        if(len <= 0) return;
         serverinfo *si = NULL;
         loopv(servers) if(addr.host == servers[i]->address.host && addr.port == servers[i]->address.port) { si = servers[i]; break; }
-        if(!si && searchlan) si = newserver(NULL, server::serverport(addr.port), addr.host); 
+        if(!si && searchlan) si = newserver(NULL, server::serverport(addr.port), addr.host);
         if(!si) continue;
         ucharbuf p(ping, len);
         int millis = getint(p), rtt = clamp(totalmillis - millis, 0, min(servpingdecay, totalmillis));
@@ -512,11 +512,20 @@ void checkpings()
     }
 }
 
+void sortservers()
+{
+    servers.sort(serverinfo::compare);
+}
+COMMAND(sortservers, "");
+
+VARP(autosortservers, 0, 1, 1);
+VARP(autoupdateservers, 0, 1, 1);
+
 void refreshservers()
 {
     static int lastrefresh = 0;
     if(lastrefresh==totalmillis) return;
-    if(totalmillis - lastrefresh > 1000) 
+    if(totalmillis - lastrefresh > 1000)
     {
         loopv(servers) servers[i]->reset();
         lastreset = totalmillis;
@@ -526,52 +535,67 @@ void refreshservers()
     checkresolver();
     checkpings();
     if(totalmillis - lastinfo >= servpingrate/(maxservpings ? max(1, (servers.length() + maxservpings - 1) / maxservpings) : 1)) pingservers();
-    servers.sort(serverinfo::compare);
+    if(autosortservers) sortservers();
 }
 
-#ifndef NEWGUI
-char *showservers(g3d_gui *cgui)
-{
-  refreshservers();
-  serverinfo *sc = NULL;
-  for(int start = 0; start < servers.length();)
-  {
-      if(start > 0) cgui->tab();
-      int end = servers.length();
-      cgui->pushlist();
-      loopi(10)
-      {
-          if(!game::serverinfostartcolumn(cgui, i)) break;
-          for(int j = start; j < end; j++)
-          {
-              if(!i && cgui->shouldtab()) { end = j; break; }
-              serverinfo &si = *servers[j];
-              const char *sdesc = si.sdesc;
-              if(si.address.host == ENET_HOST_ANY) sdesc = "[unknown host]";
-                else if(si.ping == serverinfo::WAITING) sdesc = "[waiting for response]";
-                if(game::serverinfoentry(cgui, i, si.name, si.port, sdesc, si.map, sdesc == si.sdesc ? si.ping : -1, si.attr, si.numplayers))
-                    sc = &si;
-            }
-            game::serverinfoendcolumn(cgui, i);
-        }
-        cgui->poplist();
-        start = end;
+serverinfo *selectedserver = NULL;
+
+ICOMMAND(numservers, "", (), intret(servers.length()))
+ICOMMAND(selectserver, "i", (int *i),
+    if(selectedserver) return;
+    if(servers.inrange(*i))
+        selectedserver = servers[*i];
+)
+ICOMMAND(serverinfo, "ii", (int *i, int *n),
+    refreshservers();
+    if(!servers.inrange(*i))
+    {
+        result("");
+        return;
     }
-    if(!sc) return NULL;
-    string command;
-    if(sc->password) formatstring(command)("connect %s %d \"%s\"", sc->name, sc->port, sc->password);
-    else formatstring(command)("connect %s %d", sc->name, sc->port);
-    return newstring(command);
+    serverinfo &si = *servers[*i];
+    switch(*n)
+    {
+        case 0: result(si.name); return;
+        case 1: intret(si.port); return;
+        case 2:
+            if(si.address.host == ENET_HOST_ANY) result("[unknown host]");
+            else if(si.ping == serverinfo::WAITING) result("[waiting for response]");
+            else result(si.sdesc);
+
+            return;
+        case 3: result(si.map); return;
+        case 4: intret(si.ping); return;
+        case 5: intret(si.numplayers); return;
+        default: result(""); return;
+    }
+)
+
+ICOMMAND(serverauxinfo, "ii", (int *i, int *n),
+    refreshservers();
+    if(servers.inrange(*i) && servers[*i]->attr.inrange(*n))
+        intret(servers[*i]->attr[*n]);
+    else intret(0);
+)
+
+void connectselected()
+{
+    if(!selectedserver) return;
+    connectserv(selectedserver->name, selectedserver->port, selectedserver->password);
+    selectedserver = NULL;
 }
-#endif
+
+COMMAND(connectselected, "");
 
 void clearservers(bool full = false)
 {
     resolverclear();
     if(full) servers.deletecontents();
     else loopvrev(servers) if(!servers[i]->keep) delete servers.remove(i);
+    selectedserver = NULL;
 }
 
+
 #define RETRIEVELIMIT 20000
 
 void retrieveservers(vector<char> &data)
@@ -590,7 +614,7 @@ void retrieveservers(vector<char> &data)
     while(reqlen > 0)
     {
         enet_uint32 events = ENET_SOCKET_WAIT_SEND;
-        if(enet_socket_wait(sock, &events, 250) >= 0 && events) 
+        if(enet_socket_wait(sock, &events, 250) >= 0 && events)
         {
             buf.data = (void *)req;
             buf.dataLength = reqlen;
@@ -628,6 +652,8 @@ void retrieveservers(vector<char> &data)
     enet_socket_destroy(sock);
 }
 
+bool updatedservers = false;
+
 void updatefrommaster()
 {
     vector<char> data;
@@ -639,17 +665,25 @@ void updatefrommaster()
         execute(data.getbuf());
     }
     refreshservers();
+    updatedservers = true;
+}
+
+void initservers()
+{
+    selectedserver = NULL;
+    if(autoupdateservers && !updatedservers) updatefrommaster();
 }
 
 ICOMMAND(addserver, "sis", (const char *name, int *port, const char *password), addserver(name, *port, password[0] ? password : NULL));
 ICOMMAND(keepserver, "sis", (const char *name, int *port, const char *password), addserver(name, *port, password[0] ? password : NULL, true));
 ICOMMAND(clearservers, "i", (int *full), clearservers(*full!=0));
 COMMAND(updatefrommaster, "");
+COMMAND(initservers, "");
 
 void writeservercfg()
 {
     if(!game::savedservers()) return;
-    stream *f = openutf8file(path(game::savedservers(), true), "wb");
+    stream *f = openutf8file(path(game::savedservers(), true), "w");
     if(!f) return;
     int kept = 0;
     loopv(servers)
@@ -658,20 +692,20 @@ void writeservercfg()
         if(s->keep)
         {
             if(!kept) f->printf("// servers that should never be cleared from the server list\n\n");
-            if(s->password) f->printf("keepserver %s %d \"%s\"\n", s->name, s->port, s->password);
-            else f->printf("keepserver %s %d\n", s->name, s->port);
+            if(s->password) f->printf("keepserver %s %d %s\n", escapeid(s->name), s->port, escapestring(s->password));
+            else f->printf("keepserver %s %d\n", escapeid(s->name), s->port);
             kept++;
         }
     }
     if(kept) f->printf("\n");
     f->printf("// servers connected to are added here automatically\n\n");
-    loopv(servers) 
+    loopv(servers)
     {
         serverinfo *s = servers[i];
-        if(!s->keep) 
+        if(!s->keep)
         {
-            if(s->password) f->printf("addserver %s %d \"%s\"\n", s->name, s->port, s->password);
-            else f->printf("addserver %s %d\n", s->name, s->port);
+            if(s->password) f->printf("addserver %s %d %s\n", escapeid(s->name), s->port, escapestring(s->password));
+            else f->printf("addserver %s %d\n", escapeid(s->name), s->port);
         }
     }
     delete f;
