@@ -1011,7 +1011,7 @@ static void gengenericvariant(Shader &s, const char *sname, const char *vs, cons
         memset(vspragma, ' ', len);
         vspragma += len;
         if(!strncmp(vspragma, "override", olen))
-        { 
+        {
             memset(vspragma, ' ', olen);
             vspragma += olen;
             char *end = vspragma + strcspn(vspragma, "\n\r");
@@ -1026,7 +1026,7 @@ static void gengenericvariant(Shader &s, const char *sname, const char *vs, cons
         memset(pspragma, ' ', len);
         pspragma += len;
         if(!strncmp(pspragma, "override", olen))
-        { 
+        {
             memset(pspragma, ' ', olen);
             pspragma += olen;
             char *end = pspragma + strcspn(pspragma, "\n\r");
@@ -1071,7 +1071,7 @@ static bool genwatervariant(Shader &s, const char *sname, vector<char> &vs, vect
         }
         if(fadetc>=0)
         {
-		defformatstring(fadedef)("MAD result.texcoord[%d].%c, vertex.position.z, program.env[8].x, program.env[8].y;\n",
+            defformatstring(fadedef)("MAD result.texcoord[%d].%c, vertex.position.z, program.env[8].x, program.env[8].y;\n",
                                 fadetc, fadecomp==3 ? 'w' : 'x'+fadecomp);
             vs.insert(vspragma-vs.getbuf(), fadedef, strlen(fadedef));
             defformatstring(fadeuse)("MOV result.color.a, fragment.texcoord[%d].%c;\n",
@@ -1326,15 +1326,15 @@ static void genshadowmapvariant(Shader &s, const char *sname, const char *vs, co
             smoothshadowmappeel ?
                 "TEMP smvals, smdiff, smambient;\n"
                 "TEX smvals, fragment.texcoord[%d], texture[7], 2D;\n"
-                "MAD_SAT smdiff.xz, -fragment.texcoord[%d].z, smvals.y, smvals;\n"
+                "MAD_SAT smdiff.xy, -fragment.texcoord[%d].z, smvals.y, smvals.xzzz;\n"
                 "CMP smvals.w, -smdiff.x, smvals.w, 0;\n"
-                "MAD_SAT smvals.w, -8, smdiff.z, smvals.w;\n" :
+                "MAD_SAT smvals.w, -8, smdiff.y, smvals.w;\n" :
 
                 "TEMP smvals, smtest, smambient;\n"
                 "TEX smvals, fragment.texcoord[%d], texture[7], 2D;\n"
-                "MUL smtest.z, fragment.texcoord[%d].z, smvals.y;\n"
-                "SLT smtest.xz, smtest.z, smvals;\n"
-                "MAD_SAT smvals.w, smvals.w, smtest.x, -smtest.z;\n",
+                "MUL smtest.y, fragment.texcoord[%d].z, smvals.y;\n"
+                "SLT smtest.xy, smtest.y, smvals.xzzz;\n"
+                "MAD_SAT smvals.w, smvals.w, smtest.x, -smtest.y;\n",
             smtc, smtc);
         pssm.put(sm, strlen(sm));
         formatstring(sm)(
@@ -1380,7 +1380,7 @@ static void genfogshader(vector<char> &vsbuf, vector<char> &psbuf, const char *v
         if(vspragma)
         {
             vspragma += pragmalen;
-            while(*vspragma && !isspace(*vspragma)) vspragma++;
+            while(*vspragma && !iscubespace(*vspragma)) vspragma++;
             vspragma += strspn(vspragma, " \t\v\f");
             clen = strcspn(vspragma, "\r\n");
         }
@@ -1403,8 +1403,8 @@ static void genfogshader(vector<char> &vsbuf, vector<char> &psbuf, const char *v
         if(pspragma)
         {
             pspragma += pragmalen;
-            while (isalpha(*pspragma)) pspragma++;
-            while(*pspragma && !isspace(*pspragma)) pspragma++;
+            while(iscubealpha(*pspragma)) pspragma++;
+            while(*pspragma && !iscubespace(*pspragma)) pspragma++;
             pspragma += strspn(pspragma, " \t\v\f");
             clen = strcspn(pspragma, "\r\n");
         }
@@ -1567,7 +1567,7 @@ void shader(int *type, char *name, char *vs, char *ps)
         if(*type & SHADER_GLSLANG)
         {
             GENSHADER(curparams.length(), genuniformdefs(vsbuf, psbuf, vs, ps));
-	    GENSHADER(strstr(vs, "#pragma CUBE2_fog") || strstr(ps, "#pragma CUBE2_fog"), genfogshader(vsbuf, psbuf, vs, ps));
+            GENSHADER(strstr(vs, "#pragma CUBE2_fog") || strstr(ps, "#pragma CUBE2_fog"), genfogshader(vsbuf, psbuf, vs, ps));
         }
     }
     Shader *s = newshader(*type, name, vs, ps);
@@ -1729,7 +1729,7 @@ void linkvslotshader(VSlot &s, bool load)
     {
         ShaderParam *cparam = findshaderparam(s, "glowcolor");
         if(cparam) loopk(3) s.glowcolor[k] = clamp(cparam->val[k], 0.0f, 1.0f);
-        ShaderParam *pulseparam = findshaderparam(s, "pulseglowcolor"), 
+        ShaderParam *pulseparam = findshaderparam(s, "pulseglowcolor"),
                     *speedparam = findshaderparam(s, "pulseglowspeed");
         if(pulseparam) loopk(3) s.pulseglowcolor[k] = clamp(pulseparam->val[k], 0.0f, 1.0f);
         if(speedparam) s.pulseglowspeed = speedparam->val[0]/1000.0f;
@@ -2032,8 +2032,8 @@ struct tmu
 { \
     0, \
     { -1, -1, -1, -1 }, \
-    { 0, { 0, 0, 0, ~0 }, { 0, 0, 0, 0 }, 0 }, \
-    { 0, { 0, 0, 0, ~0 }, { 0, 0, 0, 0 }, 0 } \
+    { 0, { 0, 0, 0, ~0U }, { 0, 0, 0, 0 }, 0 }, \
+    { 0, { 0, 0, 0, ~0U }, { 0, 0, 0, 0 }, 0 } \
 }
 
 #define INITTMU \
