@@ -21,7 +21,23 @@ struct soundsample
 struct soundslot
 {
     soundsample *sample;
-    int volume, maxuses;
+    int volume;
+};
+
+struct soundconfig
+{
+    int slots, numslots;
+    int maxuses;
+
+    bool hasslot(const soundslot *p, const vector<soundslot> &v) const
+    {
+        return p >= v.getbuf() + slots && p < v.getbuf() + slots+numslots && slots+numslots < v.length();
+    }
+
+    int chooseslot() const
+    {
+        return numslots > 1 ? slots + rnd(numslots) : slots;
+    }
 };
 
 struct soundchannel
@@ -141,11 +157,11 @@ void initsound()
 {
     if(Mix_OpenAudio(soundfreq, MIX_DEFAULT_FORMAT, 2, soundbufferlen)<0)
     {
-    	nosound = true;
+        nosound = true;
         conoutf(CON_ERROR, "sound init failed (SDL_mixer): %s", (size_t)Mix_GetError());
         return;
     }
-	Mix_AllocateChannels(soundchans);
+    Mix_AllocateChannels(soundchans);
     Mix_ChannelFinished(freechannel);
     maxchannels = soundchans;
     nosound = false;
@@ -182,64 +198,69 @@ Mix_Music *loadmusic(const char *name)
     return music;
 }
 
-const char *exts[] = { "", ".ogg", ".mp3", ".mid", ".mod", ".wav" };
+const char *exts[] = { "", ".ogg", ".mid", ".flac", ".wav" };
 
 VAR(dbgmusic, 0, 0, 1);
 
 void startmusic(char *name, char *cmd)
 {
-	if(nosound || !musicvol || !soundvol) { if(dbgmusic) conoutf("aborting, sound disabled"); return;}
-	stopmusic();
+    if(nosound || !musicvol || !soundvol) { if(dbgmusic) conoutf("aborting, sound disabled"); return;}
+    stopmusic();
 
-	const char *paths[] = { "packages/", "", "packages/music/" };
-	string sn;
+    const char *paths[] = { "packages/", "", "packages/music/" };
+    string sn;
 
-	if(*name)
-	{
-		loopi(sizeof(paths)/sizeof(paths[0]))
-		{
-			loopj(sizeof(exts)/sizeof(exts[0]))
-			{
-				formatstring(sn)("%s%s%s", paths[i], name, exts[j]);
-				if(dbgmusic) conoutf("trying path: %s", sn);
-				const char *file = path(sn);
-				if(loadmusic(file))
-				{
-					if(dbgmusic) conoutf("File found!");
-					DELETEA(musicfile);
-					DELETEA(musicdonecmd);
-					musicfile = newstring(file);
-					if(cmd[0]) musicdonecmd = newstring(cmd);
-					Mix_PlayMusic(music, cmd[0] ? 0 : -1);
-					Mix_VolumeMusic((musicvol*MAXVOL)/255);
-					intret(1);
-					return;
-				}
-			}
-		}
-		conoutf(CON_ERROR, "could not play music: %s", name);
-	}
-	else if(dbgmusic) conoutf("no file name provided");
+    if(*name)
+    {
+        loopi(sizeof(paths)/sizeof(paths[0]))
+        {
+            loopj(sizeof(exts)/sizeof(exts[0]))
+            {
+                formatstring(sn)("%s%s%s", paths[i], name, exts[j]);
+                if(dbgmusic) conoutf("trying path: %s", sn);
+                const char *file = path(sn);
+                if(loadmusic(file))
+                {
+                    if(dbgmusic) conoutf("File found!");
+                    DELETEA(musicfile);
+                    DELETEA(musicdonecmd);
+                    musicfile = newstring(file);
+                    if(cmd[0]) musicdonecmd = newstring(cmd);
+                    Mix_PlayMusic(music, cmd[0] ? 0 : -1);
+                    Mix_VolumeMusic((musicvol*MAXVOL)/255);
+                    intret(1);
+                    return;
+                }
+            }
+        }
+        conoutf(CON_ERROR, "could not play music: %s", name);
+    }
+    else if(dbgmusic) conoutf("no file name provided");
 
-
-	intret(0);
+    intret(0);
 }
 
 COMMANDN(music, startmusic, "ss");
 
-hashtable<const char *, soundsample> samples;
-vector<soundslot> gamesounds, mapsounds;
+static hashtable<const char *, soundsample> samples;
+static vector<soundslot> gameslots, mapslots;
+static vector<soundconfig> gamesounds, mapsounds;
 
-int findsound(const char *name, int vol, vector<soundslot> &sounds)
+static int findsound(const char *name, int vol, vector<soundconfig> &sounds, vector<soundslot> &slots)
 {
     loopv(sounds)
     {
-        if(!strcmp(sounds[i].sample->name, name) && (!vol || sounds[i].volume==vol)) return i;
+        soundconfig &s = sounds[i];
+        loopj(s.numslots)
+        {
+            soundslot &c = slots[s.slots+j];
+            if(!strcmp(c.sample->name, name) && (!vol || c.volume==vol)) return i;
+        }
     }
     return -1;
 }
 
-int addsound(const char *name, int vol, int maxuses, vector<soundslot> &sounds)
+static int addslot(const char *name, int vol, vector<soundslot> &slots)
 {
     soundsample *s = samples.access(name);
     if(!s)
@@ -249,28 +270,52 @@ int addsound(const char *name, int vol, int maxuses, vector<soundslot> &sounds)
         s->name = n;
         s->chunk = NULL;
     }
-    soundslot *oldsounds = sounds.getbuf();
-    int oldlen = sounds.length();
-    soundslot &slot = sounds.add();
-    // sounds.add() may relocate slot pointers
-    if(sounds.getbuf() != oldsounds) loopv(channels)
+    soundslot *oldslots = slots.getbuf();
+    int oldlen = slots.length();
+    soundslot &slot = slots.add();
+    // soundslots.add() may relocate slot pointers
+    if(slots.getbuf() != oldslots) loopv(channels)
     {
         soundchannel &chan = channels[i];
-        if(chan.inuse && chan.slot >= oldsounds && chan.slot < &oldsounds[oldlen])
-            chan.slot = &sounds[chan.slot - oldsounds];
+        if(chan.inuse && chan.slot >= oldslots && chan.slot < &oldslots[oldlen])
+            chan.slot = &slots[chan.slot - oldslots];
     }
     slot.sample = s;
     slot.volume = vol ? vol : 100;
-    slot.maxuses = maxuses;
     return oldlen;
 }
 
-void registersound(char *name, int *vol) { intret(addsound(name, *vol, 0, gamesounds)); }
+static int addsound(const char *name, int vol, int maxuses, vector<soundconfig> &sounds, vector<soundslot> &slots)
+{
+    soundconfig &s = sounds.add();
+    s.slots = addslot(name, vol, slots);
+    s.numslots = 1;
+    s.maxuses = maxuses;
+    return sounds.length()-1;
+}
+
+void registersound(char *name, int *vol) { intret(addsound(name, *vol, 0, gamesounds, gameslots)); }
 COMMAND(registersound, "si");
 
-void mapsound(char *name, int *vol, int *maxuses) { intret(addsound(name, *vol, *maxuses < 0 ? 0 : max(1, *maxuses), mapsounds)); }
+void mapsound(char *name, int *vol, int *maxuses) { intret(addsound(name, *vol, *maxuses < 0 ? 0 : max(1, *maxuses), mapsounds, mapslots)); }
 COMMAND(mapsound, "sii");
 
+void altsound(char *name, int *vol)
+{
+    if(gamesounds.empty()) return;
+    addslot(name, *vol, gameslots);
+    gamesounds.last().numslots++;
+}
+COMMAND(altsound, "si");
+
+void altmapsound(char *name, int *vol)
+{
+    if(mapsounds.empty()) return;
+    addslot(name, *vol, mapslots);
+    mapsounds.last().numslots++;
+}
+COMMAND(altmapsound, "si");
+
 void resetchannels()
 {
     loopv(channels) if(channels[i].inuse) freechannel(i);
@@ -285,7 +330,9 @@ void clear_sound()
     enumerate(samples, soundsample, s, s.cleanup());
     Mix_CloseAudio();
     resetchannels();
+    gameslots.setsize(0);
     gamesounds.setsize(0);
+    mapslots.setsize(0);
     mapsounds.setsize(0);
     samples.clear();
 }
@@ -297,6 +344,7 @@ void clearmapsounds()
         Mix_HaltChannel(i);
         freechannel(i);
     }
+    mapslots.setsize(0);
     mapsounds.setsize(0);
 }
 
@@ -423,9 +471,10 @@ int playsound(int n, const vec *loc, extentity *ent, int loops, int fade, int ch
 {
     if(nosound || !soundvol) return -1;
 
-    vector<soundslot> &sounds = ent ? mapsounds : gamesounds;
+    vector<soundslot> &slots = ent ? mapslots : gameslots;
+    vector<soundconfig> &sounds = ent ? mapsounds : gamesounds;
     if(!sounds.inrange(n)) { conoutf(CON_WARN, "unregistered sound: %d", n); return -1; }
-    soundslot &slot = sounds[n];
+    soundconfig &config = sounds[n];
 
     if(loc && (maxsoundradius || radius > 0))
     {
@@ -433,7 +482,7 @@ int playsound(int n, const vec *loc, extentity *ent, int loops, int fade, int ch
         int rad = radius > 0 ? (maxsoundradius ? min(maxsoundradius, radius) : radius) : maxsoundradius;
         if(camera1->o.dist(*loc) > 1.5f*rad)
         {
-            if(channels.inrange(chanid) && channels[chanid].inuse && channels[chanid].slot == &slot)
+            if(channels.inrange(chanid) && channels[chanid].inuse && config.hasslot(channels[chanid].slot, slots))
             {
                 Mix_HaltChannel(chanid);
                 freechannel(chanid);
@@ -444,24 +493,37 @@ int playsound(int n, const vec *loc, extentity *ent, int loops, int fade, int ch
 
     if(chanid < 0)
     {
-        if(slot.maxuses)
-      {
-          int uses = 0;
-          loopv(channels) if(channels[i].inuse && channels[i].slot == &slot && ++uses >= slot.maxuses) return -1;
-      }
+        if(config.maxuses)
+        {
+            int uses = 0;
+            loopv(channels) if(channels[i].inuse && config.hasslot(channels[i].slot, slots) && ++uses >= config.maxuses) return -1;
+        }
 
-      // avoid bursts of sounds with heavy packetloss and in sp
-      static int soundsatonce = 0, lastsoundmillis = 0;
-      if(totalmillis == lastsoundmillis) soundsatonce++; else soundsatonce = 1;
-      lastsoundmillis = totalmillis;
-      if(maxsoundsatonce && soundsatonce > maxsoundsatonce) return -1;
+        // avoid bursts of sounds with heavy packetloss and in sp
+        static int soundsatonce = 0, lastsoundmillis = 0;
+        if(totalmillis == lastsoundmillis) soundsatonce++; else soundsatonce = 1;
+        lastsoundmillis = totalmillis;
+        if(maxsoundsatonce && soundsatonce > maxsoundsatonce) return -1;
     }
 
+    if(channels.inrange(chanid))
+    {
+        soundchannel &chan = channels[chanid];
+        if(chan.inuse && config.hasslot(chan.slot, slots))
+        {
+            if(loc) chan.loc = *loc;
+            else if(chan.hasloc()) chan.clearloc();
+            return chanid;
+        }
+    }
+    if(fade < 0) return -1;
+
+    soundslot &slot = slots[config.chooseslot()];
     if(!slot.sample->chunk)
     {
         if(!slot.sample->name[0]) return -1;
 
-        const char *exts[] = { "", ".wav", ".ogg" };
+        const char *exts[] = { "", ".ogg", ".flac", ".wav" };
         string buf;
         loopi(sizeof(exts)/sizeof(exts[0]))
         {
@@ -474,18 +536,6 @@ int playsound(int n, const vec *loc, extentity *ent, int loops, int fade, int ch
         if(!slot.sample->chunk) { conoutf(CON_ERROR, "failed to load sample: %s", buf); return -1; }
     }
 
-    if(channels.inrange(chanid))
-    {
-        soundchannel &chan = channels[chanid];
-        if(chan.inuse && chan.slot == &slot)
-        {
-            if(loc) chan.loc = *loc;
-            else if(chan.hasloc()) chan.clearloc();
-            return chanid;
-        }
-    }
-    if(fade < 0) return -1;
-
     if(dbgsound) conoutf("sound: %s", slot.sample->name);
 
     chanid = -1;
@@ -526,7 +576,7 @@ void stopsounds()
 
 bool stopsound(int n, int chanid, int fade)
 {
-    if(!channels.inrange(chanid) || !channels[chanid].inuse || !gamesounds.inrange(n) || channels[chanid].slot != &gamesounds[n]) return false;
+    if(!channels.inrange(chanid) || !channels[chanid].inuse || !gamesounds.inrange(n) || !gamesounds[n].hasslot(channels[chanid].slot, gameslots)) return false;
     if(dbgsound) conoutf("stopsound: %s", channels[chanid].slot->sample->name);
     if(!fade || !Mix_FadeOutChannel(chanid, fade))
     {
@@ -539,8 +589,8 @@ bool stopsound(int n, int chanid, int fade)
 int playsoundname(const char *s, const vec *loc, int vol, int loops, int fade, int chanid, int radius, int expire)
 {
     if(!vol) vol = 100;
-    int id = findsound(s, vol, gamesounds);
-    if(id < 0) id = addsound(s, vol, 0, gamesounds);
+    int id = findsound(s, vol, gamesounds, gameslots);
+    if(id < 0) id = addsound(s, vol, 0, gamesounds, gameslots);
     return playsound(id, loc, NULL, loops, fade, chanid, radius, expire);
 }
 
@@ -549,16 +599,22 @@ void sound(int *n) { playsound(*n); }
 COMMAND(sound, "i");
 
 ICOMMAND(entsoundname, "i", (int *id),
-	if(mapsounds.inrange(*id))
-	{defformatstring(s)("%s", mapsounds[*id].sample->name); result(s);}
+    if(mapsounds.inrange(*id))
+    {defformatstring(s)("%s", mapslots[mapsounds[*id].slots].sample->name); result(s);}
 );
 
 void writemapsounds(stream *f)
 {
-	loopv(mapsounds)
-	{
-		f->printf("mapsound \"%s\" %d %d // %d\n", mapsounds[i].sample->name, mapsounds[i].volume, mapsounds[i].maxuses, i);
-	}
+    loopv(mapsounds)
+    {
+        soundconfig &c = mapsounds[i];
+        loopj(c.numslots)
+        {
+            soundslot &s = mapslots[c.slots + j];
+            if(!j) f->printf("mapsound %s %d %d // %d\n", escapestring(s.sample->name), s.volume, c.maxuses, i);
+            else f->printf("altmapsound %s %d\n", escapestring(s.sample->name), s.volume);
+        }
+    }
 }
 
 void resetsound()
