@@ -442,7 +442,8 @@ struct Texture
         STUB       = 1<<8,
         TRANSIENT  = 1<<9,
         COMPRESSED = 1<<10,
-        FLAGS      = 0xF0
+        ALPHA      = 1<<11,
+        FLAGS      = 0xFF00
     };
 
     char *name;
@@ -456,23 +457,23 @@ struct Texture
 
 struct frame
 {
-	GLuint num;
-	int slot; //solely for cfg generation
-	frame(int _s, GLuint _n) : num(_n), slot(_s) {}
+    GLuint num;
+    int slot; //solely for cfg generation
+    frame(int _s, GLuint _n) : num(_n), slot(_s) {}
 };
 
 struct TexAnim
 {
-  vector<frame>  frames;
-  int               pos;
-  int               delay;
-  int               lastupdate, pausemillis;
-  bool              on;
+    vector<frame>  frames;
+    int               pos;
+    int               delay;
+    int               lastupdate, pausemillis;
+    bool              on;
 
-  TexAnim(int _s, GLuint _if): pos(0), delay(500), lastupdate(0), pausemillis(0), on(false)
-  {
-  	frames.add(frame(_s, _if));
-  }
+    TexAnim(int _s, GLuint _if): pos(0), delay(500), lastupdate(0), pausemillis(0), on(false)
+    {
+        frames.add(frame(_s, _if));
+    }
 };
 
 enum
