@@ -508,7 +508,7 @@ struct vertmodel : animmodel
                 if(lasttcbuf!=lastvbuf)
                 {
                     glTexCoordPointer(2, GL_FLOAT, vertsize, &vverts->u);
-                    lasttcbuf = lastnbuf;
+                    lasttcbuf = lastvbuf;
                 }
             }
             if(enablebones) disablebones();
@@ -639,7 +639,7 @@ template<class MDL> struct vertcommands : modelcommands<MDL, struct MDL::vertmes
         if(!MDL::loading || MDL::loading->parts.empty()) { conoutf("not loading an %s", MDL::formatname()); return; }
         vector<int> anims;
         findanims(anim, anims);
-        if(anims.empty()) conoutf("could not find animation %s", anim);
+        if(anims.empty()) conoutf("%s: could not find animation %s", MDL::dir, anim);
         else loopv(anims)
         {
             MDL::loading->parts.last()->setanim(0, anims[i], *frame, *range, *speed, *priority);
@@ -648,9 +648,9 @@ template<class MDL> struct vertcommands : modelcommands<MDL, struct MDL::vertmes
 
     vertcommands()
     {
-        if(MDL::multiparted()) modelcommand(loadpart, "load", "sf");
-        modelcommand(setpitch, "pitch", "ffff");
-        if(MDL::animated()) modelcommand(setanim, "anim", "siiff");
+        if(MDL::multiparted()) this->modelcommand(loadpart, "load", "sf");
+        this->modelcommand(setpitch, "pitch", "ffff");
+        if(MDL::animated()) this->modelcommand(setanim, "anim", "siiff");
     }
 };
 
