@@ -9,6 +9,7 @@ SVARR(maptitle, "Untitled Map by Unknown");
 
 VAR(octaentsize, 0, 128, 1024);
 VAR(entselradius, 0, 2, 10);
+ICOMMAND(getcurmap, "", (), result(game::getclientmap()))
 
 bool getentboundingbox(extentity &e, ivec &o, ivec &r)
 {
@@ -70,7 +71,7 @@ void modifyoctaentity(int flags, int id, cube *c, const ivec &cor, int size, con
                         if(va)
                         {
                             va->bbmin.x = -1;
-			    if(oe.mapmodels.empty()) va->mapmodels.add(&oe);
+                            if(oe.mapmodels.empty()) va->mapmodels.add(&oe);
                         }
                         oe.mapmodels.add(id);
                         loopk(3)
@@ -311,8 +312,8 @@ undoblock *newundoent()
     loopv(entgroup)
     {
         e->i = entgroup[i];
-	e->e.attr.disown(); //points to random values; this causes problems
-	e->e = *entities::getents()[entgroup[i]];
+        e->e.attr.disown(); //points to random values; this causes problems
+        e->e = *entities::getents()[entgroup[i]];
         e++;
     }
     return u;
@@ -422,8 +423,8 @@ undoblock *copyundoents(undoblock *u)
     loopi(u->numents)
         entadd(e[i].i);
     undoblock *c = newundoent();
-   	loopi(u->numents) if(e[i].e.type==ET_EMPTY)
-		entgroup.removeobj(e[i].i);
+    loopi(u->numents) if(e[i].e.type==ET_EMPTY)
+        entgroup.removeobj(e[i].i);
     return c;
 }
 
@@ -537,11 +538,12 @@ void renderentring(const extentity &e, float radius, int axis)
 {
     if(radius <= 0 || !showentradius) return;
     glBegin(GL_LINE_LOOP);
-    loopi(16)
+    loopi(15)
     {
         vec p(e.o);
-        p[axis>=2 ? 1 : 0] += radius*cosf(2*M_PI*i/16.0f);
-        p[axis>=1 ? 2 : 1] += radius*sinf(2*M_PI*i/16.0f);
+        const vec2 &sc = sincos360[i*(360/15)];
+        p[axis>=2 ? 1 : 0] += radius*sc.x;
+        p[axis>=1 ? 2 : 1] += radius*sc.y;
         glVertex3fv(p.v);
     }
     glEnd();
@@ -630,7 +632,7 @@ void renderentradius(extentity &e, bool color)
                 float radius = e.attached->attr[0];
                 if(!radius) radius = 2*e.o.dist(e.attached->o);
                 vec dir = vec(e.o).sub(e.attached->o).normalize();
-                float angle = max(1, min(90, int(e.attr[0])));
+                float angle = clamp(int(e.attr[0]), 1, 89);
                 renderentattachment(e);
                 renderentcone(*e.attached, dir, radius, angle);
             }
@@ -639,7 +641,7 @@ void renderentradius(extentity &e, bool color)
         case ET_SOUND:
             if(color) glColor3f(0, 1, 1);
             renderentsphere(e, e.attr[1]);
-	    renderentsphere(e, min(e.attr[1], e.attr[2]));
+            renderentsphere(e, min(e.attr[1], e.attr[2]));
             break;
 
         case ET_ENVMAP:
@@ -673,73 +675,73 @@ void renderentradius(extentity &e, bool color)
 
 void renderhelpers()
 {
-	if(showentradius < 2 && showentdir < 2) return;
-	loopv(entities::getents())
-	{
-		extentity &e = *entities::getents()[i];
+    if(showentradius < 2 && showentdir < 2) return;
+    loopv(entities::getents())
+    {
+        extentity &e = *entities::getents()[i];
 
-		if((showentradius == 2 && entities::radiusent(e))|| (showentdir == 2 && entities::dirent(e)))
-		{
-			glDepthFunc(GL_GREATER);
-			glColor3f(0.25f, 0.25f, 0.25f);
-			renderentradius(e, false);
-			glDepthFunc(GL_LESS);
-			renderentradius(e, true);
-		}
-	}
+        if((showentradius == 2 && entities::radiusent(e))|| (showentdir == 2 && entities::dirent(e)))
+        {
+            glDepthFunc(GL_GREATER);
+            glColor3f(0.25f, 0.25f, 0.25f);
+            renderentradius(e, false);
+            glDepthFunc(GL_LESS);
+            renderentradius(e, true);
+        }
+    }
 }
 
 void renderentselection(const vec &o, const vec &ray, bool entmoving)
 {
-	if(!noentedit())
-	{
-		vec eo, es;
+    if(!noentedit())
+    {
+        vec eo, es;
 
-		glColor3ub(0, 40, 0);
-		loopv(entgroup) entfocus(entgroup[i],
-			entselectionbox(e, eo, es);
-			boxs3D(eo, es, 1);
-		);
+        glColor3ub(0, 40, 0);
+        loopv(entgroup) entfocus(entgroup[i],
+            entselectionbox(e, eo, es);
+            boxs3D(eo, es, 1);
+        );
 
-		if(enthover >= 0)
-		{
-			entfocus(enthover, entselectionbox(e, eo, es)); // also ensures enthover is back in focus
-			boxs3D(eo, es, 1);
+        if(enthover >= 0)
+        {
+            entfocus(enthover, entselectionbox(e, eo, es)); // also ensures enthover is back in focus
+            boxs3D(eo, es, 1);
 
-			if(entmoving && entmovingshadow==1)
-			{
-				vec a, b;
-				glColor3ub(20, 20, 20);
-				(a = eo).x = eo.x - fmod(eo.x, worldsize); (b = es).x = a.x + worldsize; boxs3D(a, b, 1);
-				(a = eo).y = eo.y - fmod(eo.y, worldsize); (b = es).y = a.x + worldsize; boxs3D(a, b, 1);
-				(a = eo).z = eo.z - fmod(eo.z, worldsize); (b = es).z = a.x + worldsize; boxs3D(a, b, 1);
-			}
-			glColor3ub(150,0,0);
-			glLineWidth(5);
-			boxs(entorient, eo, es);
-			glLineWidth(1);
-		}
-		if(showenthelpers>=1 && (entgroup.length() || enthover >= 0))
-		{
-			glDepthFunc(GL_GREATER);
-			glColor3f(0.25f, 0.25f, 0.25f);
-			loopv(entgroup)
-			{
-				extentity &e = *entities::getents()[entgroup[i]];
-				if((showentdir == 2 && entities::dirent(e)) || (showentradius == 2 && entities::radiusent(e))) continue;
-				renderentradius(e, false);
-			}
-			if(enthover>=0) entfocus(enthover, renderentradius(e, false));
-			glDepthFunc(GL_LESS);
-			loopv(entgroup)
-			{
-				extentity &e = *entities::getents()[entgroup[i]];
-				if((showentdir == 2 && entities::dirent(e)) || (showentradius == 2 && entities::radiusent(e))) continue;
-				renderentradius(e, true);
-			}
-			if(enthover>=0) entfocus(enthover, renderentradius(e, true));
-		}
-	}
+            if(entmoving && entmovingshadow==1)
+            {
+                vec a, b;
+                glColor3ub(20, 20, 20);
+                (a = eo).x = eo.x - fmod(eo.x, worldsize); (b = es).x = a.x + worldsize; boxs3D(a, b, 1);
+                (a = eo).y = eo.y - fmod(eo.y, worldsize); (b = es).y = a.x + worldsize; boxs3D(a, b, 1);
+                (a = eo).z = eo.z - fmod(eo.z, worldsize); (b = es).z = a.x + worldsize; boxs3D(a, b, 1);
+            }
+            glColor3ub(150,0,0);
+            glLineWidth(5);
+            boxs(entorient, eo, es);
+            glLineWidth(1);
+        }
+        if(showenthelpers>=1 && (entgroup.length() || enthover >= 0))
+        {
+            glDepthFunc(GL_GREATER);
+            glColor3f(0.25f, 0.25f, 0.25f);
+            loopv(entgroup)
+            {
+                extentity &e = *entities::getents()[entgroup[i]];
+                if((showentdir == 2 && entities::dirent(e)) || (showentradius == 2 && entities::radiusent(e))) continue;
+                renderentradius(e, false);
+            }
+            if(enthover>=0) entfocus(enthover, renderentradius(e, false));
+            glDepthFunc(GL_LESS);
+            loopv(entgroup)
+            {
+                extentity &e = *entities::getents()[entgroup[i]];
+                if((showentdir == 2 && entities::dirent(e)) || (showentradius == 2 && entities::radiusent(e))) continue;
+                renderentradius(e, true);
+            }
+            if(enthover>=0) entfocus(enthover, renderentradius(e, true));
+        }
+    }
 }
 
 bool enttoggle(int id)
@@ -895,12 +897,22 @@ void attachent()
 
 COMMAND(attachent, "");
 
-extentity *newentity(bool local, const vec &o, int type, int *attrs)
+static int keepents = 0;
+
+extentity *newentity(bool local, const vec &o, int type, int *attrs, int &idx)
 {
+    vector<extentity *> &ents = entities::getents();
+   if(local)
+    {
+        idx = -1;
+        for(int i = keepents; i < ents.length(); i++) if(ents[i]->type == ET_EMPTY) { idx = i; break; }
+        if(idx < 0 && ents.length() >= MAXENTS) { conoutf("too many entities"); return NULL; }
+    }
+    else while(ents.length() < idx) ents.add(entities::newentity())->type = ET_EMPTY;
     extentity &e = *entities::newentity();
     e.o = o;
     loopi(getattrnum(type))
-    	e.attr.add(attrs[i]);
+        e.attr.add(attrs[i]);
 
     e.type = type;
     e.spawned = false;
@@ -911,64 +923,65 @@ extentity *newentity(bool local, const vec &o, int type, int *attrs)
     {
         switch(type)
         {
-		case ET_PARTICLES:
-			switch(e.attr[0])
-			{
-				case 0: case 1: case 2: case 6: case 7: case 9: case 10: case 11: case 12: case 13: case 14: case 15:
-					if(!e.attr[3]) e.attr[3] |= 0x0F0F0F;
-					if(e.attr[0] != 6 && e.attr[0] != 7) break;
-				case 5: case 8:
-					if(!e.attr[2]) e.attr[2] |= 0x0F0F0F;
-					break;
-			}
-			break;
+        case ET_PARTICLES:
+            switch(e.attr[0])
+            {
+                case 0: case 1: case 2: case 6: case 7: case 9: case 10: case 11: case 12: case 13: case 14: case 15:
+                    if(!e.attr[3]) e.attr[3] |= 0x0F0F0F;
+                    if(e.attr[0] != 6 && e.attr[0] != 7) break;
+                case 5: case 8:
+                    if(!e.attr[2]) e.attr[2] |= 0x0F0F0F;
+                    break;
+            }
+            break;
                 case ET_MAPMODEL:
                 case ET_PLAYERSTART:
-			e.attr.pop();
-			e.attr.insert(0, camera1->yaw);
+                    e.attr.pop();
+                    e.attr.insert(0, camera1->yaw);
                     break;
         }
         entities::fixentity(e);
     }
+    if(ents.inrange(idx)) { entities::deleteentity(ents[idx]); ents[idx] = &e; }
+    else { idx = ents.length(); ents.add(&e); }
     return &e;
 }
 
 const int getattrnum(int type)
 {
-	if(type >= ET_GAMESPECIFIC)
-		return entities::numattrs(type);
+    if(type >= ET_GAMESPECIFIC)
+        return entities::numattrs(type);
 
-	static const int num[] =
-	{
-		0, //empty
-		4, //light
-		6, //mapmodel
-		2, //playerstart
-		3, //envmap
-		8, //particles
-		3, //sound
-		1  //spotlight
-	};
-	return type >= 0 && size_t(type) < sizeof(num)/sizeof(num[0]) ? num[type] : 0;
+    static const int num[] =
+    {
+        0, //empty
+        4, //light
+        6, //mapmodel
+        2, //playerstart
+        3, //envmap
+        8, //particles
+        3, //sound
+        1  //spotlight
+    };
+    return type >= 0 && size_t(type) < sizeof(num)/sizeof(num[0]) ? num[type] : 0;
 }
 
 ICOMMAND(getattrnum, "s", (char *s),
-	int type = findtype(s);
-	intret(getattrnum(type));
+    int type = findtype(s);
+    intret(getattrnum(type));
 )
 
 void newentity(int type, int *attrs)
 {
-    if(entities::getents().length() >= MAXENTS) { conoutf("too many entities"); return; }
-    extentity *t = newentity(true, player->o, type, attrs);
+    int idx;
+    extentity *t = newentity(true, player->o, type, attrs, idx);
+    if(!t) return;
 
     dropentity(*t);
-    entities::getents().add(t);
-    int i = entities::getents().length()-1;
     t->type = ET_EMPTY;
-    enttoggle(i);
+    enttoggle(idx);
     makeundoent();
-    entedit(i, e.type = type);
+    entedit(idx, e.type = type);
 }
 
 void newent(tagval *args, int num)
@@ -979,7 +992,7 @@ void newent(tagval *args, int num)
     int *attrs = new int[numattrs];
 
     loopi(numattrs)
-    	attrs[i] = (num - 1 >= i) ? args[i + 1].getint() : 0;
+        attrs[i] = (num - 1 >= i) ? args[i + 1].getint() : 0;
 
     if(type != ET_EMPTY)
         newentity(type, attrs);
@@ -1004,17 +1017,19 @@ void entpaste()
     if(noentedit()) return;
     if(entcopybuf.length()==0) return;
     entcancel();
-    int last = entities::getents().length()-1;
     float m = float(sel.grid)/float(entcopygrid);
     loopv(entcopybuf)
     {
         entity &c = entcopybuf[i];
         vec o(c.o);
         o.mul(m).add(sel.o.tovec());
-        extentity *e = newentity(true, o, ET_EMPTY, c.attr.getbuf());
-        entities::getents().add(e);
-        entadd(++last);
+        int idx;
+        extentity *e = newentity(true, o, ET_EMPTY, c.attr.getbuf(), idx);
+        if(!e) continue;
+        entadd(idx);
+        keepents = max(keepents, idx + 1);
     }
+    keepents = 0;
     int j = 0;
     groupeditundo(e.attr = entcopybuf[j].attr; e.type = entcopybuf[j++].type;);
 }
@@ -1025,40 +1040,40 @@ COMMAND(dropent, "");
 COMMAND(entcopy, "");
 COMMAND(entpaste, "");
 
-void entset(tagval *args, int num) //char *what, int *a1, int *a2, int *a3, int *a4, int *a5, int *a6, int *a7, int *a8)
+void entset(tagval *args, int num)
 {
-	if(noentedit()) return;
-	int type = num ? findtype(args[0].getstr()) : 0;
-	int numattrs = getattrnum(type);
-	int *attrs = new int[numattrs];
+    if(noentedit()) return;
+    int type = num ? findtype(args[0].getstr()) : 0;
+    int numattrs = getattrnum(type);
+    int *attrs = new int[numattrs];
 
-	loopi(numattrs)
-		attrs[i] = (num - 1 >= i) ? args[i + 1].getint() : 0;
+    loopi(numattrs)
+        attrs[i] = (num - 1 >= i) ? args[i + 1].getint() : 0;
 
-	groupedit(e.type=type;
-	          e.attr.setsize(0);
-	          e.attr.put(attrs, numattrs);
-	);
-	delete[] attrs;
+    if(type != ET_EMPTY)
+        groupedit(e.type=type;
+                  e.attr.setsize(0);
+                  e.attr.put(attrs, numattrs));
+    delete[] attrs;
 }
 
 void printent(extentity &e, char *buf)
 {
-	switch(e.type)
-	{
-		case ET_PARTICLES:
-			if(printparticles(e, buf)) return;
-			break;
-		default:
-			if(e.type >= ET_GAMESPECIFIC && entities::printent(e, buf)) return;
-			break;
-	}
-	static string tmp;
-	loopv(e.attr)
-	{
-		formatstring(tmp)("%s%d", i ? " " : "", e.attr[i]);
-		concatstring(buf, tmp);
-	}
+    switch(e.type)
+    {
+        case ET_PARTICLES:
+            if(printparticles(e, buf)) return;
+            break;
+        default:
+            if(e.type >= ET_GAMESPECIFIC && entities::printent(e, buf)) return;
+            break;
+    }
+    static string tmp;
+    loopv(e.attr)
+    {
+        formatstring(tmp)("%s%d", i ? " " : "", e.attr[i]);
+        concatstring(buf, tmp);
+    }
 }
 
 void nearestent()
@@ -1082,40 +1097,77 @@ void nearestent()
 }
 
 ICOMMAND(enthavesel,"",  (), addimplicit(intret(entgroup.length())));
-ICOMMAND(entselect, "e", (uint *body), if(!noentedit()) addgroup(e.type != ET_EMPTY && entgroup.find(n)<0 && execute(body)>0));
+ICOMMAND(entselect, "e", (uint *body), if(!noentedit()) addgroup(e.type != ET_EMPTY && entgroup.find(n)<0 && executebool(body)));
 ICOMMAND(entloop,   "e", (uint *body), if(!noentedit()) addimplicit(groupeditloop(((void)e, execute(body)))));
 ICOMMAND(insel,     "",  (), entfocus(efocus, intret(pointinsel(sel, e.o))));
 ICOMMAND(entget,    "",  (),
-	entfocus(efocus,
-		defformatstring(s)("%s ", entities::entname(e.type));
-		printent(e, s);
-		result(s)
-	)
+    entfocus(efocus,
+        defformatstring(s)("%s ", entities::entname(e.type));
+        printent(e, s);
+        result(s)
+    )
 );
 ICOMMAND(entindex,  "",  (), intret(efocus));
 ICOMMAND(entgetinfo,    "",  (), entfocus(efocus,result(entities::entnameinfo(e))));
 COMMAND(entset, "V");
 COMMAND(nearestent, "");
 
+void enttype(char *type, int *numargs)
+{
+    if(*numargs >= 1)
+    {
+        int typeidx = findtype(type);
+        if(typeidx != ET_EMPTY) groupedit(
+            e.type = typeidx;
+            int numattrs = getattrnum(typeidx);
+            while(e.attr.length() < numattrs) e.attr.add(0);
+            while(e.attr.length() > numattrs) e.attr.pop();
+        );
+    }
+    else entfocus(efocus,
+    {
+        result(entities::entname(e.type));
+    })
+}
+
+void entattr(int *attr, int *val, int *numargs)
+{
+    if(*numargs >= 2)
+    {
+        groupedit(
+            if(e.attr.inrange(*attr))
+                e.attr[*attr] = *val;
+        );
+    }
+    else entfocus(efocus,
+    {
+        if(e.attr.inrange(*attr))
+            intret(e.attr[*attr]);
+    });
+}
+
+COMMAND(enttype, "sN");
+COMMAND(entattr, "iiN");
+
 int findentity(int type, int index, int attr1, int attr2)
 {
     const vector<extentity *> &ents = entities::getents();
     for(int i = index; i<ents.length(); i++)
     {
         extentity &e = *ents[i];
-	if(e.type == type &&
-		(attr1 < 0 || e.attr.length() <= 0 || e.attr[0] == attr1 ) &&
-		(attr1 < 0 || e.attr.length() <= 1 || e.attr[1] == attr2 )
-	)
-		return i;
+    if(e.type == type &&
+        (attr1 < 0 || e.attr.length() <= 0 || e.attr[0] == attr1 ) &&
+        (attr1 < 0 || e.attr.length() <= 1 || e.attr[1] == attr2 )
+    )
+        return i;
     }
     loopj(min(index, ents.length()))
     {
         extentity &e = *ents[j];
-	if(e.type == type &&
-		(attr1 < 0 || e.attr.length() <= 0 || e.attr[0] == attr1 ) &&
-		(attr1 < 0 || e.attr.length() <= 1 || e.attr[1] == attr2 )
-	)
+    if(e.type == type &&
+        (attr1 < 0 || e.attr.length() <= 0 || e.attr[0] == attr1 ) &&
+        (attr1 < 0 || e.attr.length() <= 1 || e.attr[1] == attr2 )
+    )
             return j;
     }
     return -1;
@@ -1183,6 +1235,7 @@ void resetmap()
     clearsleep();
     cancelsel();
     pruneundos();
+    clearmapcrc();
 
     setvar("gamespeed", 100, false);
     setvar("paused", 0, false);
@@ -1216,11 +1269,7 @@ bool emptymap(int scale, bool force, const char *mname, bool usecfg)    // main
 
     if(worldsize > 0x1000) splitocta(worldroot, worldsize>>1);
 
-    #ifndef NEWGUI
-    clearmainmenu();
-    #else
     UI::clearmainmenu();
-    #endif
 
     if(usecfg)
     {
@@ -1322,24 +1371,24 @@ COMMAND(mapname, "");
 void mpeditent(int i, const vec &o, int type, int *attrs, bool local)
 {
     if(i < 0 || i >= MAXENTS) return;
-    if(entities::getents().length()<=i)
+    vector<extentity *> &ents = entities::getents();
+    if(ents.length()<=i)
     {
-        while(entities::getents().length()<i) entities::getents().add(entities::newentity())->type = ET_EMPTY;
-        extentity *e = newentity(local, o, type, attrs);
-        entities::getents().add(e);
+        extentity *e = newentity(local, o, type, attrs, i);
+        if(!e) return;
         addentity(i);
         attachentity(*e);
     }
     else
     {
-        extentity &e = *entities::getents()[i];
+        extentity &e = *ents[i];
         removeentity(i);
         int oldtype = e.type;
         if(oldtype!=type) detachentity(e);
         e.type = type;
         e.o = o;
-	e.attr.setsize(0);
-	e.attr.put(attrs, getattrnum(type));
+        e.attr.setsize(0);
+        e.attr.put(attrs, getattrnum(type));
         addentity(i);
         if(oldtype!=type) attachentity(e);
     }
