@@ -316,9 +316,9 @@ void setmapfilenames(const char *fname, bool fall = true)
 
 void mapcfgname()
 {
-	defformatstring(res)("packages/%s/%s.cfg", mpath, mname);
-	path(res);
-	result(res);
+    defformatstring(res)("packages/%s/%s.cfg", mpath, mname);
+    path(res);
+    result(res);
 }
 
 COMMAND(mapcfgname, "");
@@ -677,6 +677,19 @@ void loadc(stream *f, cube &c, const ivec &co, int size, bool &failed)
                 }
             }
         }
+        if(mapversion <= 8) edgespan2vectorcube(c);
+        if(mapversion <= 11)
+        {
+            swap(c.faces[0], c.faces[2]);
+            swap(c.texture[0], c.texture[4]);
+            swap(c.texture[1], c.texture[5]);
+            if(hassurfs&0x33)
+            {
+                swap(surfaces[0], surfaces[4]);
+                swap(surfaces[1], surfaces[5]);
+                hassurfs = (hassurfs&~0x33) | ((hassurfs&0x30)>>4) | ((hassurfs&0x03)<<4);
+            }
+        }
         if(mapversion >= 20)
         {
             if(octsav&0x80)
@@ -854,10 +867,7 @@ void saveslotconfig(stream *h, Slot &s, int index)
     }
     loopvj(s.sts)
     {
-        h->printf("texture");
-        if(index >= 0) h->printf(" %s ", textypename(s.sts[j].type));
-        else h->printf(" 1 ");
-        writeescapedstring(h, s.sts[j].name);
+        h->printf("texture %s %s", index >= 0 ? textypename(s.sts[j].type) : "1", escapestring(s.sts[j].name));
         if(!j && index >= 0) h->printf(" // %d", index);
         h->putchar('\n');
     }
@@ -897,7 +907,7 @@ void writemapcfg(const char *a)
         defformatstring(bak)("packages/%s/%s_%d.cfg.BAK", mpath, mname, totalmillis);
         backup(acfname, bak);
     }
-    stream *f = openutf8file(path(acfname, true), "wb");
+    stream *f = openutf8file(path(acfname, true), "w");
 
     f->printf("//Configuration generated by Platinum Arts Sandbox %s, modify with caution\n//This file contains map variables, and art definitions\n//to add anything, add them onto the end of the file or it's given section.\n//for further information, see our wiki or catch us on IRC\n//Part 1: Game module specific stuff\n//Part 2: World Variables\n//Part 3: Mapsounds\n//Part 4: Mapmodels\n//Part 5: Textures\n\n", version);
 
@@ -907,7 +917,7 @@ void writemapcfg(const char *a)
 
     f->printf("//world variables\n//uncomment to override\n\n");
 
-    extern int sortidents(ident *x, ident *y);
+    extern bool sortidents(ident *x, ident *y);
 
     vector<ident *> ids;
     enumerate(idents, ident, id, ids.add(&id));
@@ -917,9 +927,9 @@ void writemapcfg(const char *a)
         ident &id = *ids[i];
         if(!(id.flags&IDF_OVERRIDDEN) || id.flags&IDF_READONLY) continue; switch(id.type)
         {
-            case ID_VAR: f->printf(id.flags & IDF_HEX ? "//%s 0x%.6X\n" : "//%s %d\n", id.name, *id.storage.i); break;
-            case ID_FVAR: f->printf("//%s %s\n", id.name, floatstr(*id.storage.f)); break;
-            case ID_SVAR: f->printf("//%s ", id.name); writeescapedstring(f, *id.storage.s); f->putchar('\n'); break;
+            case ID_VAR: f->printf(id.flags & IDF_HEX ? "//%s 0x%.6X\n" : "//%s %d\n", escapeid(id.name), *id.storage.i); break;
+            case ID_FVAR: f->printf("//%s %s\n", escapeid(id.name), floatstr(*id.storage.f)); break;
+            case ID_SVAR: f->printf("//%s %s\n", escapeid(id.name), escapestring(*id.storage.s)); break;
         }
     }
 
@@ -931,9 +941,7 @@ void writemapcfg(const char *a)
     extern vector<mapmodelinfo> mapmodels;
 
     loopv(mapmodels)
-    {
-        f->printf("mmodel "); writeescapedstring(f, mapmodels[i].name); f->printf(" // %i\n", i);
-    }
+        f->printf("mmodel %s // %d\n", escapestring(mapmodels[i].name), i);
 
     f->printf("\n//Textures\n\ntexturereset\n\n");
 
@@ -1228,7 +1236,7 @@ bool save_world(const char *mname, bool nolms, bool octa)
                                 ent.attr1 -= 2; break;
 
                             //bars 5/6
-                            case 6:	case 7:
+                            case 6: case 7:
                                 ent.attr1--; break;
 
                             //text
@@ -1308,22 +1316,7 @@ bool save_world(const char *mname, bool nolms, bool octa)
 static uint mapcrc = 0;
 
 uint getmapcrc() { return mapcrc; }
-
-static void swapXZ(cube *c)
-{
-    loopi(8)
-    {
-        swap(c[i].faces[0],   c[i].faces[2]);
-        swap(c[i].texture[0], c[i].texture[4]);
-        swap(c[i].texture[1], c[i].texture[5]);
-        if(c[i].ext && c[i].ext->surfaces)
-        {
-            swap(c[i].ext->surfaces[0], c[i].ext->surfaces[4]);
-            swap(c[i].ext->surfaces[1], c[i].ext->surfaces[5]);
-        }
-        if(c[i].children) swapXZ(c[i].children);
-    }
-}
+void clearmapcrc() { mapcrc = 0; }
 
 bool load_world(const char *mname, const char *cname)        // still supports all map formats that have existed since the earliest cube betas!
 {
@@ -1391,7 +1384,7 @@ bool load_world(const char *mname, const char *cname)        // still supports a
     }
 
     resetmap();
-	Texture *mapshot = textureload(picname, 3, true, false);
+    Texture *mapshot = textureload(picname, 3, true, false);
     renderbackground("loading...", mapshot, mname, game::getmapinfo()); //checks are done not to render the map
 
     setvar("mapversion", octaversion, true, false);
@@ -1399,7 +1392,6 @@ bool load_world(const char *mname, const char *cname)        // still supports a
     if(octaversion <= 28)
     {
         lilswap(&chdr.lightprecision, 3);
-        if(octaversion<=20) conoutf(CON_WARN, "loading older / less efficient map format, may benefit from \"calclight\", then \"savecurrentmap\"");
         if(chdr.lightprecision) setvar("lightprecision", chdr.lightprecision);
         if(chdr.lighterror) setvar("lighterror", chdr.lighterror);
         if(chdr.bumperror) setvar("bumperror", chdr.bumperror);
@@ -1664,12 +1656,6 @@ bool load_world(const char *mname, const char *cname)        // still supports a
     worldroot = loadchildren(f, ivec(0, 0, 0), hdr.worldsize>>1, failed);
     if(failed) conoutf(CON_ERROR, "garbage in map");
 
-    if(octaversion <= 11)
-        swapXZ(worldroot);
-
-    if(octaversion <= 8)
-        converttovectorworld();
-
     renderprogress(0, "validating...");
     validatec(worldroot, hdr.worldsize>>1);
 
@@ -1704,11 +1690,7 @@ bool load_world(const char *mname, const char *cname)        // still supports a
 
     conoutf("read map %s (%.1f seconds)", ogzname, (SDL_GetTicks()-loadingstart)/1000.0f);
 
-    #ifndef NEWGUI
-    clearmainmenu();
-    #else
     UI::clearmainmenu();
-    #endif
 
     identflags |= IDF_OVERRIDDEN;
 
