@@ -326,7 +326,7 @@ struct captureclientmode : clientmode
 
     void preload()
     {
-        static const char *basemodels[3] = { "base/neutral", "base/red", "base/blue" };
+        static const char *basemodels[3] = { "banana", "banana/red", "banana/blue" };
         loopi(3) preloadmodel(basemodels[i]);
     }
 
@@ -344,7 +344,7 @@ struct captureclientmode : clientmode
         loopv(bases)
         {
             baseinfo &b = bases[i];
-            const char *flagname = b.owner[0] ? (strcmp(b.owner, player1->team) ? "base/red" : "base/blue") : "base/neutral";
+            const char *flagname = b.owner[0] ? (strcmp(b.owner, player1->team) ? "banana/red" : "banana/blue") : "banana";
             rendermodel(&b.light, flagname, ANIM_MAPMODEL|ANIM_LOOP, b.o, 0, 0, 0, MDL_SHADOW | MDL_CULL_VFC | MDL_CULL_OCCLUDED);
             float fradius = 1.0f, fheight = 0.5f;
             regular_particle_flame(PART_FLAME, vec(b.ammopos.x, b.ammopos.y, b.ammopos.z - 4.5f), fradius, fheight, b.owner[0] ? (strcmp(b.owner, player1->team) ? 0x802020 : 0x2020FF) : 0x208020, 3, 2.0f);
@@ -471,10 +471,10 @@ struct captureclientmode : clientmode
         if(minimapalpha >= 1) glEnable(GL_BLEND);
         glColor3f(1, 1, 1);
         float margin = 0.04f, roffset = s*margin, rsize = s + 2*roffset;
-        settexture("packages/hud/radar.png", 3);
+        settexture("data/fps/hud/radar.png", 3);
         drawradar(x - roffset, y - roffset, rsize);
         #if 0
-        settexture("packages/hud/compass.png", 3);
+        settexture("data/fps/hud/compass.png", 3);
         glPushMatrix();
         glTranslatef(x - roffset + 0.5f*rsize, y - roffset + 0.5f*rsize, 0);
         glRotatef(camera1->yaw + 180, 0, 0, -1);
@@ -489,17 +489,17 @@ struct captureclientmode : clientmode
             setfont("digit_blue");
             text_bounds(" ", fw, fh);
         }
-        else settexture("packages/hud/blip_blue.png", 3);
+        else settexture("data/fps/hud/blip_blue.png", 3);
         glPushMatrix();
         glTranslatef(x + 0.5f*s, y + 0.5f*s, 0);
         float blipsize = basenumbers ? 0.1f : 0.05f;
         glScalef((s*blipsize)/fw, (s*blipsize)/fh, 1.0f);
         drawblips(d, blipsize, fw, fh, 1, showenemies);
         if(basenumbers) setfont("digit_grey");
-        else settexture("packages/hud/blip_grey.png", 3);
+        else settexture("data/fps/hud/blip_grey.png", 3);
         drawblips(d, blipsize, fw, fh, 0, showenemies);
         if(basenumbers) setfont("digit_red");
-        else settexture("packages/hud/blip_red.png", 3);
+        else settexture("data/fps/hud/blip_red.png", 3);
         drawblips(d, blipsize, fw, fh, -1, showenemies);
         if(showenemies) drawblips(d, blipsize, fw, fh, -2);
         glPopMatrix();
@@ -528,7 +528,7 @@ struct captureclientmode : clientmode
             baseinfo &b = bases.add();
             b.o = e->o;
             b.ammopos = b.o;
-            abovemodel(b.ammopos, "base/neutral");
+            abovemodel(b.ammopos, "banana");
             b.ammopos.z += AMMOHEIGHT-2;
             b.ammotype = e->attr[0];
             defformatstring(alias)("base_%d", e->attr[1]);
@@ -667,93 +667,93 @@ struct captureclientmode : clientmode
     const char *prefixnextmap() { return "capture_"; }
 
 
-	bool aicheck(fpsent *d, ai::aistate &b)
-	{
-		return false;
-	}
+    bool aicheck(fpsent *d, ai::aistate &b)
+    {
+        return false;
+    }
 
-	void aifind(fpsent *d, ai::aistate &b, vector<ai::interest> &interests)
-	{
-		vec pos = d->feetpos();
-		loopvj(bases)
-		{
-			baseinfo &f = bases[j];
-			static vector<int> targets; // build a list of others who are interested in this
-			targets.setsize(0);
-			ai::checkothers(targets, d, ai::AI_S_DEFEND, ai::AI_T_AFFINITY, j, true);
-			fpsent *e = NULL;
-			int regen = !m_regencapture || d->health >= 100 ? 0 : 1;
-			if(m_regencapture)
-			{
-				int gun = f.ammotype-1+I_SHELLS;
-				if(f.ammo > 0 && f.ammotype > 0 && f.ammotype <= I_CARTRIDGES-I_SHELLS+1 && !d->hasmaxammo(gun))
-					regen = gun != d->ai->weappref ? 2 : 4;
-			}
-			loopi(numdynents()) if((e = (fpsent *)iterdynents(i)) && !e->ai && e->state == CS_ALIVE && isteam(d->team, e->team))
-			{ // try to guess what non ai are doing
-				vec ep = e->feetpos();
-				if(targets.find(e->clientnum) < 0 && ep.squaredist(f.o) <= (CAPTURERADIUS*CAPTURERADIUS))
-					targets.add(e->clientnum);
-			}
-			if((regen && f.owner[0] && !strcmp(f.owner, d->team)) || (targets.empty() && (!f.owner[0] || strcmp(f.owner, d->team) || f.enemy[0])))
-			{
-				ai::interest &n = interests.add();
-				n.state = ai::AI_S_DEFEND;
-				n.node = ai::closestwaypoint(f.o, ai::SIGHTMIN, false);
-				n.target = j;
-				n.targtype = ai::AI_T_AFFINITY;
-				n.score = pos.squaredist(f.o)/(regen ? float(100*regen) : 1.f);
-			}
-		}
-	}
+    void aifind(fpsent *d, ai::aistate &b, vector<ai::interest> &interests)
+    {
+        vec pos = d->feetpos();
+        loopvj(bases)
+        {
+            baseinfo &f = bases[j];
+            static vector<int> targets; // build a list of others who are interested in this
+            targets.setsize(0);
+            ai::checkothers(targets, d, ai::AI_S_DEFEND, ai::AI_T_AFFINITY, j, true);
+            fpsent *e = NULL;
+            int regen = !m_regencapture || d->health >= 100 ? 0 : 1;
+            if(m_regencapture)
+            {
+                int gun = f.ammotype-1+I_SHELLS;
+                if(f.ammo > 0 && f.ammotype > 0 && f.ammotype <= I_CARTRIDGES-I_SHELLS+1 && !d->hasmaxammo(gun))
+                    regen = gun != d->ai->weappref ? 2 : 4;
+            }
+            loopi(numdynents()) if((e = (fpsent *)iterdynents(i)) && !e->ai && e->state == CS_ALIVE && isteam(d->team, e->team))
+            { // try to guess what non ai are doing
+                vec ep = e->feetpos();
+                if(targets.find(e->clientnum) < 0 && ep.squaredist(f.o) <= (CAPTURERADIUS*CAPTURERADIUS))
+                    targets.add(e->clientnum);
+            }
+            if((regen && f.owner[0] && !strcmp(f.owner, d->team)) || (targets.empty() && (!f.owner[0] || strcmp(f.owner, d->team) || f.enemy[0])))
+            {
+                ai::interest &n = interests.add();
+                n.state = ai::AI_S_DEFEND;
+                n.node = ai::closestwaypoint(f.o, ai::SIGHTMIN, false);
+                n.target = j;
+                n.targtype = ai::AI_T_AFFINITY;
+                n.score = pos.squaredist(f.o)/(regen ? float(100*regen) : 1.f);
+            }
+        }
+    }
 
-	bool aidefend(fpsent *d, ai::aistate &b)
-	{
-		if(bases.inrange(b.target))
-		{
-			baseinfo &f = bases[b.target];
-			bool regen = !m_regencapture || d->health >= 100 ? false : true;
-			if(!regen && m_regencapture)
-			{
-				int gun = f.ammotype-1+I_SHELLS;
-				if(f.ammo > 0 && f.ammotype > 0 && f.ammotype <= I_CARTRIDGES-I_SHELLS+1 && !d->hasmaxammo(gun))
-					regen = true;
-			}
-			int walk = 0;
-			if(!regen && !f.enemy[0] && f.owner[0] && !strcmp(f.owner, d->team))
-			{
-				static vector<int> targets; // build a list of others who are interested in this
-				targets.setsize(0);
-				ai::checkothers(targets, d, ai::AI_S_DEFEND, ai::AI_T_AFFINITY, b.target, true);
-				fpsent *e = NULL;
-				loopi(numdynents()) if((e = (fpsent *)iterdynents(i)) && !e->ai && e->state == CS_ALIVE && isteam(d->team, e->team))
-				{ // try to guess what non ai are doing
-					vec ep = e->feetpos();
-					if(targets.find(e->clientnum) < 0 && (ep.squaredist(f.o) <= (CAPTURERADIUS*CAPTURERADIUS*4)))
-						targets.add(e->clientnum);
-				}
-				if(!targets.empty())
-				{
-					if(lastmillis-b.millis >= (201-d->skill)*33)
-					{
-						d->ai->trywipe = true; // re-evaluate so as not to herd
-						return true;
-					}
-					else walk = 2;
-				}
-				else walk = 1;
-				b.millis = lastmillis;
-			}
-			return ai::defend(d, b, f.o, float(CAPTURERADIUS), float(CAPTURERADIUS*(2+(walk*2))), walk); // less wander than ctf
-		}
-		return false;
-	}
+    bool aidefend(fpsent *d, ai::aistate &b)
+    {
+        if(bases.inrange(b.target))
+        {
+            baseinfo &f = bases[b.target];
+            bool regen = !m_regencapture || d->health >= 100 ? false : true;
+            if(!regen && m_regencapture)
+            {
+                int gun = f.ammotype-1+I_SHELLS;
+                if(f.ammo > 0 && f.ammotype > 0 && f.ammotype <= I_CARTRIDGES-I_SHELLS+1 && !d->hasmaxammo(gun))
+                    regen = true;
+            }
+            int walk = 0;
+            if(!regen && !f.enemy[0] && f.owner[0] && !strcmp(f.owner, d->team))
+            {
+                static vector<int> targets; // build a list of others who are interested in this
+                targets.setsize(0);
+                ai::checkothers(targets, d, ai::AI_S_DEFEND, ai::AI_T_AFFINITY, b.target, true);
+                fpsent *e = NULL;
+                loopi(numdynents()) if((e = (fpsent *)iterdynents(i)) && !e->ai && e->state == CS_ALIVE && isteam(d->team, e->team))
+                { // try to guess what non ai are doing
+                    vec ep = e->feetpos();
+                    if(targets.find(e->clientnum) < 0 && (ep.squaredist(f.o) <= (CAPTURERADIUS*CAPTURERADIUS*4)))
+                        targets.add(e->clientnum);
+                }
+                if(!targets.empty())
+                {
+                    if(lastmillis-b.millis >= (201-d->skill)*33)
+                    {
+                        d->ai->trywipe = true; // re-evaluate so as not to herd
+                        return true;
+                    }
+                    else walk = 2;
+                }
+                else walk = 1;
+                b.millis = lastmillis;
+            }
+            return ai::defend(d, b, f.o, float(CAPTURERADIUS), float(CAPTURERADIUS*(2+(walk*2))), walk); // less wander than ctf
+        }
+        return false;
+    }
 
-	bool aipursue(fpsent *d, ai::aistate &b)
-	{
-		b.type = ai::AI_S_DEFEND;
-		return aidefend(d, b);
-	}
+    bool aipursue(fpsent *d, ai::aistate &b)
+    {
+        b.type = ai::AI_S_DEFEND;
+        return aidefend(d, b);
+    }
 };
 
 extern captureclientmode capturemode;
@@ -766,7 +766,7 @@ ICOMMAND(insidebases, "", (),
         if(capturemode.insidebase(capturemode.bases[i], player1->feetpos()))
         {
             if(buf.length()) buf.add(' ');
-            defformatstring(basenum)("%d", i);
+            defformatstring(basenum)("%d", i+1);
             buf.put(basenum, strlen(basenum));
         }
     }
