@@ -39,15 +39,18 @@ namespace game
 
     #include "capture.h"
     #include "ctf.h"
+    #include "collect.h"
 
     clientmode *cmode = NULL;
     captureclientmode capturemode;
     ctfclientmode ctfmode;
+    collectclientmode collectmode;
 
     void setclientmode()
     {
         if(m_capture) cmode = &capturemode;
         else if(m_ctf) cmode = &ctfmode;
+        else if(m_collect) cmode = &collectmode;
         else cmode = NULL;
     }
 
@@ -153,7 +156,7 @@ namespace game
         loopv(authkeys)
         {
             authkey *a = authkeys[i];
-            f->printf("authkey \"%s\" \"%s\" \"%s\"\n", a->name, a->key, a->desc);
+            f->printf("authkey %s %s %s\n", escapestring(a->name), escapestring(a->key), escapestring(a->desc));
         }
         conoutf("saved authkeys to auth.cfg");
         delete f;
@@ -168,11 +171,12 @@ namespace game
 
     void writeclientinfo(stream *f)
     {
-        f->printf("name \"%s\"\n", player1->name);
+        f->printf("name %s\n", escapestring(player1->name));
     }
 
     bool allowedittoggle()
     {
+        if(editmode) return true;
         if(connected && multiplayer(false) && !m_edit)
         {
             conoutf(CON_ERROR, "editing in multiplayer requires coop edit mode (1)");
@@ -199,6 +203,41 @@ namespace game
     }
     ICOMMAND(getclientname, "i", (int *cn), result(getclientname(*cn)));
 
+    int getclientping(int cn)
+    {
+        fpsent *d = getclient(cn);
+        return d ? d->ping : 0;
+    }
+    ICOMMAND(getclientping, "i", (int *cn), intret(getclientping(*cn)));
+
+    int getclientpj(int cn)
+    {
+        fpsent *d = getclient(cn);
+        return d ? d->plag : 0;
+    }
+    ICOMMAND(getclientpj, "i", (int *cn), intret(getclientpj(*cn)));
+
+    int getclientfrags(int cn)
+    {
+        fpsent *d = getclient(cn);
+        return d ? d->frags : 0;
+    }
+    ICOMMAND(getclientfrags, "i", (int *cn), intret(getclientfrags(*cn)));
+
+    int getclientflags(int cn)
+    {
+        fpsent *d = getclient(cn);
+        return d ? d->flags : 0;
+    }
+    ICOMMAND(getclientflags, "i", (int *cn), intret(getclientflags(*cn)));
+
+    int getclientaccuracy(int cn)
+    {
+        fpsent *d = getclient(cn);
+        return d ? d->totaldamage * 100 / max(d->totalshots, 1) : 0;
+    }
+    ICOMMAND(getclientaccuracy, "i", (int *cn), intret(getclientaccuracy(*cn)));
+
     const char *getclientteam(int cn)
     {
         fpsent *d = getclient(cn);
@@ -222,6 +261,26 @@ namespace game
     }
     ICOMMAND(getclienticon, "i", (int *cn), result(getclienticon(*cn)));
 
+    int getteamscore(const char *team)
+    {
+        int score = 0;
+        if(cmode) score = cmode->getteamscore(team);
+        else if(m_teammode) loopv(players)
+        {
+            if(!strcmp(team, players[i]->team))
+                score += players[i]->frags;
+        }
+        return score;
+    }
+    ICOMMAND(getteamscore, "s", (const char *team), intret(getteamscore(team)));
+
+    bool isdead(int cn)
+    {
+        fpsent *d = getclient(cn);
+        return d && d->state == CS_DEAD;
+    }
+    ICOMMAND(isdead, "i", (int *cn), intret(isdead(*cn) ? 1 : 0));
+
     bool ismaster(int cn)
     {
         fpsent *d = getclient(cn);
@@ -446,6 +505,7 @@ namespace game
     ICOMMANDS("m_ctf", "i", (int *mode), { int gamemode = *mode; intret(m_ctf); });
     ICOMMANDS("m_protect", "i", (int *mode), { int gamemode = *mode; intret(m_protect); });
     ICOMMANDS("m_hold", "i", (int *mode), { int gamemode = *mode; intret(m_hold); });
+    ICOMMANDS("m_collect", "i", (int *mode), { int gamemode = *mode; intret(m_collect); });
     ICOMMANDS("m_teammode", "i", (int *mode), { int gamemode = *mode; intret(m_teammode); });
     ICOMMANDS("m_demo", "i", (int *mode), { int gamemode = *mode; intret(m_demo); });
     ICOMMANDS("m_edit", "i", (int *mode), { int gamemode = *mode; intret(m_edit); });
@@ -568,7 +628,9 @@ namespace game
             {
                 int val = *id->storage.i;
                 string str;
-                if(id->flags&IDF_HEX && id->maxval==0xFFFFFF)
+                if(val < 0)
+                    formatstring(str)("%d", val);
+                else if(id->flags&IDF_HEX && id->maxval==0xFFFFFF)
                     formatstring(str)("0x%.6X (%d, %d, %d)", val, (val>>16)&0xFF, (val>>8)&0xFF, val&0xFF);
                 else
                     formatstring(str)(id->flags&IDF_HEX ? "0x%X" : "%d", val);
@@ -729,11 +791,13 @@ namespace game
     void sayteam(char *text) { conoutf(CON_TEAMCHAT, "<%s>\f1 %s", colorname(player1), text); addmsg(N_TEXT, "rcis", player1, CHAT_TEAM, text); }
     COMMAND(sayteam, "C");
 
-	void emote(char *text) { conoutf(CON_CHAT, "\f6* %s %s", colorname(player1), text); addmsg(N_TEXT, "rcis", player1, CHAT_TALK|CHAT_EMOTE, text); }
-	COMMANDN(me, emote, "C");
+    void emote(char *text) { conoutf(CON_CHAT, "\f6* %s %s", colorname(player1), text); addmsg(N_TEXT, "rcis", player1, CHAT_TALK|CHAT_EMOTE, text); }
+    COMMANDN(me, emote, "C");
 
-	void teamemote(char *text) { conoutf(CON_TEAMCHAT, "\f5* %s %s", colorname(player1), text); addmsg(N_TEXT, "rcis", player1, CHAT_TEAM|CHAT_EMOTE, text); }
-	COMMANDN(meteam, teamemote, "C");
+    void teamemote(char *text) { conoutf(CON_TEAMCHAT, "\f5* %s %s", colorname(player1), text); addmsg(N_TEXT, "rcis", player1, CHAT_TEAM|CHAT_EMOTE, text); }
+    COMMANDN(meteam, teamemote, "C");
+
+    ICOMMAND(servcmd, "C", (char *cmd), addmsg(N_SERVCMD, "rs", cmd));
 
     static void sendposition(fpsent *d, packetbuf &q)
     {
@@ -944,7 +1008,7 @@ namespace game
                 int seqcolor = (physstate>>3)&1;
                 fpsent *d = getclient(cn);
                 if(!d || d->lifesequence < 0 || seqcolor!=(d->lifesequence&1) || d->state==CS_DEAD) continue;
-                float oldyaw = d->yaw, oldpitch = d->pitch;
+                float oldyaw = d->yaw, oldpitch = d->pitch, oldroll = d->roll;
                 d->yaw = yaw;
                 d->pitch = pitch;
                 d->roll = roll;
@@ -966,14 +1030,17 @@ namespace game
                     d->newpos = d->o;
                     d->newyaw = d->yaw;
                     d->newpitch = d->pitch;
+                    d->newroll = d->roll;
                     d->o = oldpos;
                     d->yaw = oldyaw;
                     d->pitch = oldpitch;
+                    d->roll = oldroll;
                     (d->deltapos = oldpos).sub(d->newpos);
                     d->deltayaw = oldyaw - d->newyaw;
                     if(d->deltayaw > 180) d->deltayaw -= 360;
                     else if(d->deltayaw < -180) d->deltayaw += 360;
                     d->deltapitch = oldpitch - d->newpitch;
+                    d->deltaroll = oldroll - d->newroll;
                     d->smoothmillis = lastmillis;
                 }
                 else d->smoothmillis = 0;
@@ -1029,7 +1096,8 @@ namespace game
         }
         else
         {
-            d->gunselect = getint(p);
+            int gun = getint(p);
+            d->gunselect = clamp(gun, int(GUN_FIST), int(GUN_PISTOL));
             loopi(GUN_PISTOL-GUN_SG+1) d->ammo[GUN_SG+i] = getint(p);
         }
     }
@@ -1095,10 +1163,10 @@ namespace game
                 int tcn = getint(p), mtype = getint(p);
                 getstring(text, p);
                 filtertext(text, text);
-                if(isignored(d->clientnum)) break;
                 fpsent *t = getclient(tcn);
+                if(isignored(t->clientnum)) break;
                 if(t->state!=CS_DEAD && t->state!=CS_SPECTATOR)
-                particle_textcopy(t->abovehead(), text, PART_TEXT, 2000, 0x32FF64, 4.0f, -8);
+                    particle_textcopy(t->abovehead(), text, PART_TEXT, 2000, 0x32FF64, 4.0f, -8);
 
                 if(mtype&CHAT_EMOTE)
                     conoutf(mtype&CHAT_TEAM ? CON_TEAMCHAT : CON_CHAT, "\f%i* %s %s",
@@ -1253,7 +1321,7 @@ namespace game
                     lasthit = 0;
                 }
                 if(cmode) cmode->respawned(s);
-				ai::spawned(s);
+                ai::spawned(s);
                 addmsg(N_SPAWN, "rcii", s, s->lifesequence, s->gunselect);
                 break;
             }
@@ -1269,10 +1337,10 @@ namespace game
                 if(gun>GUN_FIST && gun<=GUN_PISTOL && s->ammo[gun]) s->ammo[gun]--;
                 s->gunselect = clamp(gun, (int)GUN_FIST, (int)GUN_PISTOL);
                 s->gunwait = guns[s->gunselect].attackdelay;
-                //int prevaction = s->lastaction;
+                int prevaction = s->lastaction;
                 s->lastaction = lastmillis;
                 s->lastattackgun = s->gunselect;
-                //shoteffects(gun, from, to, s, false, id, prevaction);
+                shoteffects(gun, from, to, s, false, id, prevaction);
                 break;
             }
 
@@ -1281,7 +1349,7 @@ namespace game
                 int ecn = getint(p), gun = getint(p), id = getint(p);
                 fpsent *e = getclient(ecn);
                 if(!e) break;
-                //explodeeffects(gun, e, false, id);
+                explodeeffects(gun, e, false, id);
                 break;
             }
 
@@ -1411,15 +1479,15 @@ namespace game
                 ivec moveo;
                 switch(type)
                 {
-                    case N_EDITF: dir = getint(p); mode = getint(p); mpeditface(dir, mode, sel, false); break;
-                    case N_EDITT: tex = getint(p); allfaces = getint(p); mpedittex(tex, allfaces, sel, false); break;
-                    case N_EDITM: mat = getint(p); filter = getint(p); mpeditmat(mat, filter, sel, false); break;
-                    case N_FLIP: mpflip(sel, false); break;
-                    case N_COPY: if(d) mpcopy(d->edit, sel, false); break;
-                    case N_PASTE: if(d) mppaste(d->edit, sel, false); break;
-                    case N_ROTATE: dir = getint(p); mprotate(dir, sel, false); break;
-                    case N_REPLACE: tex = getint(p); newtex = getint(p); insel = getint(p); mpreplacetex(tex, newtex, insel>0, sel, false); break;
-                    case N_DELCUBE: mpdelcube(sel, false); break;
+                    case N_EDITF: dir = getint(p); mode = getint(p); if(sel.validate()) mpeditface(dir, mode, sel, false); break;
+                    case N_EDITT: tex = getint(p); allfaces = getint(p); if(sel.validate()) mpedittex(tex, allfaces, sel, false); break;
+                    case N_EDITM: mat = getint(p); filter = getint(p); if(sel.validate()) mpeditmat(mat, filter, sel, false); break;
+                    case N_FLIP: if(sel.validate()) mpflip(sel, false); break;
+                    case N_COPY: if(d && sel.validate()) mpcopy(d->edit, sel, false); break;
+                    case N_PASTE: if(d && sel.validate()) mppaste(d->edit, sel, false); break;
+                    case N_ROTATE: dir = getint(p); if(sel.validate()) mprotate(dir, sel, false); break;
+                    case N_REPLACE: tex = getint(p); newtex = getint(p); insel = getint(p); if(sel.validate()) mpreplacetex(tex, newtex, insel>0, sel, false); break;
+                    case N_DELCUBE: if(sel.validate()) mpdelcube(sel, false); break;
                 }
                 break;
             }
@@ -1458,19 +1526,19 @@ namespace game
                     case ID_VAR:
                     {
                         int val = getint(p);
-                        if(id && !(id->flags&IDF_READONLY)) setvar(name, val);
+                        if(id && id->flags&IDF_OVERRIDE && !(id->flags&IDF_READONLY)) setvar(name, val);
                         break;
                     }
                     case ID_FVAR:
                     {
                         float val = getfloat(p);
-                        if(id && !(id->flags&IDF_READONLY)) setfvar(name, val);
+                        if(id && id->flags&IDF_OVERRIDE && !(id->flags&IDF_READONLY)) setfvar(name, val);
                         break;
                     }
                     case ID_SVAR:
                     {
                         getstring(text, p);
-                        if(id && !(id->flags&IDF_READONLY)) setsvar(name, text);
+                        if(id && id->flags&IDF_OVERRIDE && !(id->flags&IDF_READONLY)) setsvar(name, text);
                         break;
                     }
                 }
@@ -1608,6 +1676,7 @@ namespace game
             #define PARSEMESSAGES 1
             #include "capture.h"
             #include "ctf.h"
+            #include "collect.h"
             #undef PARSEMESSAGES
 
             case N_ANNOUNCE:
@@ -1692,6 +1761,10 @@ namespace game
                 break;
             }
 
+            case N_SERVCMD:
+                getstring(text, p);
+                break;
+
             default:
                 neterr("type", cn < 0);
                 return;
@@ -1735,57 +1808,57 @@ namespace game
         }
     }
 
-	void receivefile(uchar *data, int len)
-	{
-		string fname = "packages/base/fps/getmap.ogz";
-		string cname = "packages/base/fps/getmap-art.cfg";
+    void receivefile(uchar *data, int len)
+    {
+        string fname = "packages/base/fps/getmap.ogz";
+        string cname = "packages/base/fps/getmap-art.cfg";
 
-		ucharbuf p(data, len);
-		int type = getint(p);
-		data += p.len;
-		len -= p.len;
+        ucharbuf p(data, len);
+        int type = getint(p);
+        data += p.length();
+        len -= p.length();
 
-		switch(type)
-		{
-			case N_SENDDEMO:
-			{
-				defformatstring(fname)("%d.dmo", lastmillis);
-				stream *demo = openrawfile(fname, "wb");
-				if(!demo) return;
-				conoutf("received demo \"%s\"", fname);
-				demo->write(data, len);
-				delete demo;
-				break;
-			}
+        switch(type)
+        {
+            case N_SENDDEMO:
+            {
+                defformatstring(fname)("%d.dmo", lastmillis);
+                stream *demo = openrawfile(fname, "wb");
+                if(!demo) return;
+                conoutf("received demo \"%s\"", fname);
+                demo->write(data, len);
+                delete demo;
+                break;
+            }
 
-			case N_SENDCFG:
-			{
-				if(!m_edit) return;
-				stream *cfg = openrawfile(path(cname), "wb");
-				if(!cfg) return;
-				cfg->write(data, len);
-				delete cfg;
-				break;
-			}
+            case N_SENDCFG:
+            {
+                if(!m_edit) return;
+                stream *cfg = openrawfile(path(cname), "wb");
+                if(!cfg) return;
+                cfg->write(data, len);
+                delete cfg;
+                break;
+            }
 
-			case N_SENDMAP:
-			{
-				if(!m_edit) return;
-				string oldname;
-				copystring(oldname, getclientmap());
-				stream *map = openrawfile(path(fname), "wb");
-				if(!map) return;
-				conoutf("received map");
-				map->write(data, len);
-				delete map;
-				load_world("getmap", oldname[0] ? oldname : NULL);
-				remove(findfile(fname, "rb"));
-				remove(findfile(cname, "rb"));
-				entities::spawnitems(true);
-				break;
-			}
-		}
-	}
+            case N_SENDMAP:
+            {
+                if(!m_edit) return;
+                string oldname;
+                copystring(oldname, getclientmap());
+                stream *map = openrawfile(path(fname), "wb");
+                if(!map) return;
+                conoutf("received map");
+                map->write(data, len);
+                delete map;
+                if(load_world("getmap", oldname[0] ? oldname : NULL))
+                    entities::spawnitems(true);
+                remove(findfile(fname, "rb"));
+                remove(findfile(cname, "rb"));
+                break;
+            }
+        }
+    }
 
 /* THIS IS ONLY A PORTION OF IT
     void receivefile(uchar *data, int len)
@@ -1950,31 +2023,31 @@ namespace game
      * TODO: move to a better place :)
      */
     const char* attachtag(int i) {
-    	if(i < ATTACH_PARTS || i >= 0)
-    		return body[i].tag;
-    	else
-    		return NULL;
+        if(i < ATTACH_PARTS || i >= 0)
+            return body[i].tag;
+        else
+            return NULL;
     }
 
     int attachnumbypart(char* part) {
-    	for( int i = 0; i < ATTACH_PARTS; i++)
-    	{
-    		if (strcmp(body[i].part, part) == 0)
-    			return i;
-    	}
-    	return -1;
+        for( int i = 0; i < ATTACH_PARTS; i++)
+        {
+            if (strcmp(body[i].part, part) == 0)
+                return i;
+        }
+        return -1;
     }
 
     void playerattach(int p, char* m, int r)
     {
-    	player1->attached.add(p,m,r);
-    	addmsg(N_ATTACH, "risi", p, m, r);
+        player1->attached.add(p,m,r);
+        addmsg(N_ATTACH, "risi", p, m, r);
     }
 
     void playerdetach(int p)
     {
-    	player1->attached.remove(p);
-    	addmsg(N_DETACH, "ri", p);
+        player1->attached.remove(p);
+        addmsg(N_DETACH, "ri", p);
     }
 
     ICOMMAND(getattachnum, "s", (char *p), intret(attachnumbypart(p)));
