@@ -62,9 +62,9 @@ struct ctfclientmode : clientmode
 #ifndef SERVMODE
         const vec &pos()
         {
-        	if(owner) return vec(owner->o).sub(owner->eyeheight);
-        	if(droptime) return droploc;
-        	return spawnloc;
+            if(owner) return vec(owner->o).sub(owner->eyeheight);
+            if(droptime) return droploc;
+            return spawnloc;
         }
 #endif
     };
@@ -214,7 +214,7 @@ struct ctfclientmode : clientmode
 
 #ifdef SERVMODE
     static const int RESETFLAGTIME = 10000;
-    static const int INVISFLAGTIME = 15000;
+    static const int INVISFLAGTIME = 20000;
 
     bool notgotflags;
 
@@ -470,10 +470,10 @@ struct ctfclientmode : clientmode
     void drawblip(fpsent *d, float x, float y, float s, int i, bool flagblip)
     {
         flag &f = flags[i];
-        settexture(m_hold && (!flagblip || !f.owner || lastmillis%1000 < 500) ? (flagblip ? "packages/hud/blip_neutral_flag.png" : "packages/hud/blip_neutral.png") :
+        settexture(m_hold && (!flagblip || !f.owner || lastmillis%1000 < 500) ? (flagblip ? "data/fps/hud/blip_neutral_flag.png" : "data/fps/hud/blip_neutral.png") :
                     ((m_hold ? ctfteamflag(f.owner->team) : f.team)==ctfteamflag(player1->team) ?
-                        (flagblip ? "packages/hud/blip_blue_flag.png" : "packages/hud/blip_blue.png") :
-                        (flagblip ? "packages/hud/blip_red_flag.png" : "packages/hud/blip_red.png")), 3);
+                        (flagblip ? "data/fps/hud/blip_blue_flag.png" : "data/fps/hud/blip_blue.png") :
+                        (flagblip ? "data/fps/hud/blip_red_flag.png" : "data/fps/hud/blip_red.png")), 3);
         drawblip(d, x, y, s, flagblip ? (f.owner ? f.owner->o : (f.droptime ? f.droploc : f.spawnloc)) : f.spawnloc, flagblip);
     }
 
@@ -510,10 +510,10 @@ struct ctfclientmode : clientmode
         if(minimapalpha >= 1) glEnable(GL_BLEND);
         glColor3f(1, 1, 1);
         float margin = 0.04f, roffset = s*margin, rsize = s + 2*roffset;
-        settexture("packages/hud/radar.png", 3);
+        settexture("data/fps/hud/radar.png", 3);
         drawradar(x - roffset, y - roffset, rsize);
         #if 0
-        settexture("packages/hud/compass.png", 3);
+        settexture("data/fps/hud/compass.png", 3);
         glPushMatrix();
         glTranslatef(x - roffset + 0.5f*rsize, y - roffset + 0.5f*rsize, 0);
         glRotatef(camera1->yaw + 180, 0, 0, -1);
@@ -522,7 +522,7 @@ struct ctfclientmode : clientmode
         #endif
         if(m_hold)
         {
-            settexture("packages/hud/blip_neutral.png", 3);
+            settexture("data/fps/hud/blip_neutral.png", 3);
             loopv(holdspawns) drawblip(d, x, y, s, holdspawns[i].o, false);
         }
         loopv(flags)
@@ -634,7 +634,6 @@ struct ctfclientmode : clientmode
     void setup()
     {
         resetflags();
-        vector<extentity *> radarents;
         if(m_hold)
         {
             loopv(entities::ents)
@@ -643,7 +642,6 @@ struct ctfclientmode : clientmode
                 if(e->type!=BASE) continue;
                 if(!addholdspawn(e->o)) continue;
                 holdspawns.last().light = e->light;
-                radarents.add(e);
             }
             if(holdspawns.length()) while(flags.length() < HOLDFLAGS) addflag(flags.length(), vec(0, 0, 0), 0, -1000);
         }
@@ -961,214 +959,214 @@ struct ctfclientmode : clientmode
 
     const char *prefixnextmap() { return m_hold ? "capture_" : "ctf_"; }
 
-	bool aihomerun(fpsent *d, ai::aistate &b)
-	{
-		if(m_protect || m_hold)
-		{
-			static vector<ai::interest> interests;
-			loopk(2)
-			{
-				interests.setsize(0);
-				ai::assist(d, b, interests, k != 0);
-				if(ai::parseinterests(d, b, interests, false, true)) return true;
-			}
-		}
-		else
-		{
-			vec pos = d->feetpos();
-			loopk(2)
-			{
-				int goal = -1;
-				loopv(flags)
-				{
-					flag &g = flags[i];
-					if(g.team == ctfteamflag(d->team) && (k || (!g.owner && !g.droptime)) &&
-						(!flags.inrange(goal) || g.spawnloc.squaredist(pos) < flags[goal].spawnloc.squaredist(pos)))
-					{
-						goal = i;
-					}
-				}
-				if(flags.inrange(goal) && ai::makeroute(d, b, flags[goal].spawnloc))
-				{
-					d->ai->switchstate(b, ai::AI_S_PURSUE, ai::AI_T_AFFINITY, goal);
-					return true;
-				}
-			}
-		}
+    bool aihomerun(fpsent *d, ai::aistate &b)
+    {
+        if(m_protect || m_hold)
+        {
+            static vector<ai::interest> interests;
+            loopk(2)
+            {
+                interests.setsize(0);
+                ai::assist(d, b, interests, k != 0);
+                if(ai::parseinterests(d, b, interests, false, true)) return true;
+            }
+        }
+        else
+        {
+            vec pos = d->feetpos();
+            loopk(2)
+            {
+                int goal = -1;
+                loopv(flags)
+                {
+                    flag &g = flags[i];
+                    if(g.team == ctfteamflag(d->team) && (k || (!g.owner && !g.droptime)) &&
+                        (!flags.inrange(goal) || g.spawnloc.squaredist(pos) < flags[goal].spawnloc.squaredist(pos)))
+                    {
+                        goal = i;
+                    }
+                }
+                if(flags.inrange(goal) && ai::makeroute(d, b, flags[goal].spawnloc))
+                {
+                    d->ai->switchstate(b, ai::AI_S_PURSUE, ai::AI_T_AFFINITY, goal);
+                    return true;
+                }
+            }
+        }
         if(b.type == ai::AI_S_INTEREST && b.targtype == ai::AI_T_NODE) return true; // we already did this..
         if(randomnode(d, b, ai::SIGHTMIN, 1e16f))
         {
              d->ai->switchstate(b, ai::AI_S_INTEREST, ai::AI_T_NODE, d->ai->route[0]);
              return true;
         }
-		return false;
-	}
+        return false;
+    }
 
-	bool aicheck(fpsent *d, ai::aistate &b)
-	{
-		static vector<int> takenflags;
-		takenflags.setsize(0);
-		loopv(flags)
-		{
-			flag &g = flags[i];
-			if(g.owner == d) return aihomerun(d, b);
-			else if(g.team == ctfteamflag(d->team) && ((g.owner && g.team != ctfteamflag(g.owner->team)) || g.droptime))
-				takenflags.add(i);
-		}
-		if(!ai::badhealth(d) && !takenflags.empty())
-		{
-			int flag = takenflags.length() > 2 ? rnd(takenflags.length()) : 0;
-			d->ai->switchstate(b, ai::AI_S_PURSUE, ai::AI_T_AFFINITY, takenflags[flag]);
-			return true;
-		}
-		return false;
-	}
+    bool aicheck(fpsent *d, ai::aistate &b)
+    {
+        static vector<int> takenflags;
+        takenflags.setsize(0);
+        loopv(flags)
+        {
+            flag &g = flags[i];
+            if(g.owner == d) return aihomerun(d, b);
+            else if(g.team == ctfteamflag(d->team) && ((g.owner && g.team != ctfteamflag(g.owner->team)) || g.droptime))
+                takenflags.add(i);
+        }
+        if(!ai::badhealth(d) && !takenflags.empty())
+        {
+            int flag = takenflags.length() > 2 ? rnd(takenflags.length()) : 0;
+            d->ai->switchstate(b, ai::AI_S_PURSUE, ai::AI_T_AFFINITY, takenflags[flag]);
+            return true;
+        }
+        return false;
+    }
 
-	void aifind(fpsent *d, ai::aistate &b, vector<ai::interest> &interests)
-	{
-		vec pos = d->feetpos();
-		loopvj(flags)
-		{
-			flag &f = flags[j];
-			if((!m_protect && !m_hold) || f.owner != d)
-			{
-				static vector<int> targets; // build a list of others who are interested in this
-				targets.setsize(0);
-				bool home = !m_hold && f.team == ctfteamflag(d->team);
-				ai::checkothers(targets, d, home ? ai::AI_S_DEFEND : ai::AI_S_PURSUE, ai::AI_T_AFFINITY, j, true);
-				fpsent *e = NULL;
-				loopi(numdynents()) if((e = (fpsent *)iterdynents(i)) && !e->ai && e->state == CS_ALIVE && isteam(d->team, e->team))
-				{ // try to guess what non ai are doing
-					vec ep = e->feetpos();
-					if(targets.find(e->clientnum) < 0 && (ep.squaredist(f.pos()) <= (FLAGRADIUS*FLAGRADIUS*4) || f.owner == e))
-						targets.add(e->clientnum);
-				}
-				if(home)
-				{
-					bool guard = false;
-					if((f.owner && f.team != ctfteamflag(f.owner->team)) || f.droptime || targets.empty()) guard = true;
-					else if(d->hasammo(d->ai->weappref))
-					{ // see if we can relieve someone who only has a piece of crap
-						fpsent *t;
-						loopvk(targets) if((t = getclient(targets[k])))
-						{
-							if((t->ai && !t->hasammo(t->ai->weappref)) || (!t->ai && (t->gunselect == GUN_FIST || t->gunselect == GUN_PISTOL)))
-							{
-								guard = true;
-								break;
-							}
-						}
-					}
-					if(guard)
-					{ // defend the flag
-						ai::interest &n = interests.add();
-						n.state = ai::AI_S_DEFEND;
-						n.node = ai::closestwaypoint(f.pos(), ai::SIGHTMIN, true);
-						n.target = j;
-						n.targtype = ai::AI_T_AFFINITY;
-						n.score = pos.squaredist(f.pos())/100.f;
-					}
-				}
-				else
-				{
-					if(targets.empty())
-					{ // attack the flag
-						ai::interest &n = interests.add();
-						n.state = ai::AI_S_PURSUE;
-						n.node = ai::closestwaypoint(f.pos(), ai::SIGHTMIN, true);
-						n.target = j;
-						n.targtype = ai::AI_T_AFFINITY;
-						n.score = pos.squaredist(f.pos());
-					}
-					else
-					{ // help by defending the attacker
-						fpsent *t;
-						loopvk(targets) if((t = getclient(targets[k])))
-						{
-							ai::interest &n = interests.add();
-							n.state = ai::AI_S_DEFEND;
-							n.node = t->lastnode;
-							n.target = t->clientnum;
-							n.targtype = ai::AI_T_PLAYER;
-							n.score = d->o.squaredist(t->o);
-						}
-					}
-				}
-			}
-		}
-	}
+    void aifind(fpsent *d, ai::aistate &b, vector<ai::interest> &interests)
+    {
+        vec pos = d->feetpos();
+        loopvj(flags)
+        {
+            flag &f = flags[j];
+            if((!m_protect && !m_hold) || f.owner != d)
+            {
+                static vector<int> targets; // build a list of others who are interested in this
+                targets.setsize(0);
+                bool home = !m_hold && f.team == ctfteamflag(d->team);
+                ai::checkothers(targets, d, home ? ai::AI_S_DEFEND : ai::AI_S_PURSUE, ai::AI_T_AFFINITY, j, true);
+                fpsent *e = NULL;
+                loopi(numdynents()) if((e = (fpsent *)iterdynents(i)) && !e->ai && e->state == CS_ALIVE && isteam(d->team, e->team))
+                { // try to guess what non ai are doing
+                    vec ep = e->feetpos();
+                    if(targets.find(e->clientnum) < 0 && (ep.squaredist(f.pos()) <= (FLAGRADIUS*FLAGRADIUS*4) || f.owner == e))
+                        targets.add(e->clientnum);
+                }
+                if(home)
+                {
+                    bool guard = false;
+                    if((f.owner && f.team != ctfteamflag(f.owner->team)) || f.droptime || targets.empty()) guard = true;
+                    else if(d->hasammo(d->ai->weappref))
+                    { // see if we can relieve someone who only has a piece of crap
+                        fpsent *t;
+                        loopvk(targets) if((t = getclient(targets[k])))
+                        {
+                            if((t->ai && !t->hasammo(t->ai->weappref)) || (!t->ai && (t->gunselect == GUN_FIST || t->gunselect == GUN_PISTOL)))
+                            {
+                                guard = true;
+                                break;
+                            }
+                        }
+                    }
+                    if(guard)
+                    { // defend the flag
+                        ai::interest &n = interests.add();
+                        n.state = ai::AI_S_DEFEND;
+                        n.node = ai::closestwaypoint(f.pos(), ai::SIGHTMIN, true);
+                        n.target = j;
+                        n.targtype = ai::AI_T_AFFINITY;
+                        n.score = pos.squaredist(f.pos())/100.f;
+                    }
+                }
+                else
+                {
+                    if(targets.empty())
+                    { // attack the flag
+                        ai::interest &n = interests.add();
+                        n.state = ai::AI_S_PURSUE;
+                        n.node = ai::closestwaypoint(f.pos(), ai::SIGHTMIN, true);
+                        n.target = j;
+                        n.targtype = ai::AI_T_AFFINITY;
+                        n.score = pos.squaredist(f.pos());
+                    }
+                    else
+                    { // help by defending the attacker
+                        fpsent *t;
+                        loopvk(targets) if((t = getclient(targets[k])))
+                        {
+                            ai::interest &n = interests.add();
+                            n.state = ai::AI_S_DEFEND;
+                            n.node = t->lastnode;
+                            n.target = t->clientnum;
+                            n.targtype = ai::AI_T_PLAYER;
+                            n.score = d->o.squaredist(t->o);
+                        }
+                    }
+                }
+            }
+        }
+    }
 
-	bool aidefend(fpsent *d, ai::aistate &b)
-	{
+    bool aidefend(fpsent *d, ai::aistate &b)
+    {
         loopv(flags)
         {
             flag &g = flags[i];
             if(g.owner == d) return aihomerun(d, b);
         }
-		if(flags.inrange(b.target))
-		{
-			flag &f = flags[b.target];
-			if(f.droptime && ai::makeroute(d, b, f.pos())) return true;
-			if(f.owner && ai::violence(d, b, f.owner, true)) return true;
-			int walk = 0;
-			if(lastmillis-b.millis >= (201-d->skill)*33)
-			{
-				static vector<int> targets; // build a list of others who are interested in this
-				targets.setsize(0);
-				ai::checkothers(targets, d, ai::AI_S_DEFEND, ai::AI_T_AFFINITY, b.target, true);
-				fpsent *e = NULL;
-				loopi(numdynents()) if((e = (fpsent *)iterdynents(i)) && !e->ai && e->state == CS_ALIVE && isteam(d->team, e->team))
-				{ // try to guess what non ai are doing
-					vec ep = e->feetpos();
-					if(targets.find(e->clientnum) < 0 && (ep.squaredist(f.pos()) <= (FLAGRADIUS*FLAGRADIUS*4) || f.owner == e))
-						targets.add(e->clientnum);
-				}
-				if(!targets.empty())
-				{
-					d->ai->trywipe = true; // re-evaluate so as not to herd
-					return true;
-				}
-				else
-				{
-					walk = 2;
-					b.millis = lastmillis;
-				}
-			}
-			vec pos = d->feetpos();
-			float mindist = float(FLAGRADIUS*FLAGRADIUS*8);
-			loopv(flags)
-			{ // get out of the way of the returnee!
-				flag &g = flags[i];
-				if(pos.squaredist(g.pos()) <= mindist)
-				{
-					if(!m_protect && !m_hold && g.owner && !strcmp(g.owner->team, d->team)) walk = 1;
-					if(g.droptime && ai::makeroute(d, b, g.pos())) return true;
-				}
-			}
-			return ai::defend(d, b, f.pos(), float(FLAGRADIUS*2), float(FLAGRADIUS*(2+(walk*2))), walk);
-		}
-		return false;
-	}
+        if(flags.inrange(b.target))
+        {
+            flag &f = flags[b.target];
+            if(f.droptime) return ai::makeroute(d, b, f.pos());
+            if(f.owner) return ai::violence(d, b, f.owner, 4);
+            int walk = 0;
+            if(lastmillis-b.millis >= (201-d->skill)*33)
+            {
+                static vector<int> targets; // build a list of others who are interested in this
+                targets.setsize(0);
+                ai::checkothers(targets, d, ai::AI_S_DEFEND, ai::AI_T_AFFINITY, b.target, true);
+                fpsent *e = NULL;
+                loopi(numdynents()) if((e = (fpsent *)iterdynents(i)) && !e->ai && e->state == CS_ALIVE && isteam(d->team, e->team))
+                { // try to guess what non ai are doing
+                    vec ep = e->feetpos();
+                    if(targets.find(e->clientnum) < 0 && (ep.squaredist(f.pos()) <= (FLAGRADIUS*FLAGRADIUS*4) || f.owner == e))
+                        targets.add(e->clientnum);
+                }
+                if(!targets.empty())
+                {
+                    d->ai->trywipe = true; // re-evaluate so as not to herd
+                    return true;
+                }
+                else
+                {
+                    walk = 2;
+                    b.millis = lastmillis;
+                }
+            }
+            vec pos = d->feetpos();
+            float mindist = float(FLAGRADIUS*FLAGRADIUS*8);
+            loopv(flags)
+            { // get out of the way of the returnee!
+                flag &g = flags[i];
+                if(pos.squaredist(g.pos()) <= mindist)
+                {
+                    if(!m_protect && !m_hold && g.owner && !strcmp(g.owner->team, d->team)) walk = 1;
+                    if(g.droptime && ai::makeroute(d, b, g.pos())) return true;
+                }
+            }
+            return ai::defend(d, b, f.pos(), float(FLAGRADIUS*2), float(FLAGRADIUS*(2+(walk*2))), walk);
+        }
+        return false;
+    }
 
-	bool aipursue(fpsent *d, ai::aistate &b)
-	{
-		if(flags.inrange(b.target))
-		{
-			flag &f = flags[b.target];
+    bool aipursue(fpsent *d, ai::aistate &b)
+    {
+        if(flags.inrange(b.target))
+        {
+            flag &f = flags[b.target];
             if(f.owner == d) return aihomerun(d, b);
-			if(!m_hold && f.team == ctfteamflag(d->team))
-			{
-				if(f.droptime && ai::makeroute(d, b, f.pos())) return true;
-				if(f.owner && ai::violence(d, b, f.owner, true)) return true;
-			}
-			else
-			{
-				if(f.owner && ai::violence(d, b, f.owner, true)) return true;
-				return ai::makeroute(d, b, f.pos());
-			}
-		}
-		return false;
-	}
+            if(!m_hold && f.team == ctfteamflag(d->team))
+            {
+                if(f.droptime) return ai::makeroute(d, b, f.pos());
+                if(f.owner) return ai::violence(d, b, f.owner, 4);
+            }
+            else
+            {
+                if(f.owner) return ai::violence(d, b, f.owner, 4);
+                return ai::makeroute(d, b, f.pos());
+            }
+        }
+        return false;
+    }
 };
 
 extern ctfclientmode ctfmode;
