@@ -92,7 +92,7 @@ namespace entities
 
     void preloadentities()
     {
-        /*loopi(MAXENTTYPES)
+        loopi(MAXENTTYPES)
         {
             switch(i)
             {
@@ -109,7 +109,7 @@ namespace entities
             const char *mdl = entmdlname(i);
             if(!mdl) continue;
             preloadmodel(mdl);
-        } */
+        }
     }
 
     void renderentities()
@@ -155,8 +155,8 @@ namespace entities
     {
         loopv(ents)
         {
-            if((ents[i]->attr[4] > -1 && ents[i]->type == DYNLIGHT && dynlightsactive[ents[i]->attr[4]] && (!editmode || showdynlights)) //dynlights
-                || ((entgroup.find(i) >= 0 || enthover == i) &&    (ents[i]->type == LIGHT) && (editmode && showlighting))) //normal lights
+            if((ents[i]->type == DYNLIGHT && ents[i]->attr[4] > -1 && dynlightsactive[ents[i]->attr[4]] && (!editmode || showdynlights)) //dynlights
+                || ((entgroup.find(i) >= 0 || enthover == i) && (ents[i]->type == LIGHT) && (editmode && showlighting))) //normal lights
             {
                 renderfocus(i, renderentlight(e));
             }
@@ -316,10 +316,10 @@ namespace entities
                 if(d->lastpickup==ents[n]->type && lastmillis-d->lastpickupmillis<300) break;
                 d->lastpickup = ents[n]->type;
                 d->lastpickupmillis = lastmillis;
-		jumppadeffects(d, n, true);
+                jumppadeffects(d, n, true);
                 vec v((int)(char)ents[n]->attr[2]*10.0f, (int)(char)ents[n]->attr[1]*10.0f, ents[n]->attr[0]*12.5f);
                 d->timeinair = 0;
-		if(d->ai) d->ai->becareful = true;
+                if(d->ai) d->ai->becareful = true;
                 d->falling = vec(0, 0, 0);
 //                d->vel = v;
                 d->vel.z = 0;
@@ -403,7 +403,7 @@ namespace entities
 
     static const int triggertypes[NUMTRIGGERTYPES] =
     {
-        0,
+        -1,
         TRIG_ONCE,                    // 1
         TRIG_RUMBLE,                  // 2
         TRIG_TOGGLE,                  // 3
@@ -418,26 +418,26 @@ namespace entities
         TRIG_DISAPPEAR,               // 12
         TRIG_DISAPPEAR | TRIG_RUMBLE, // 13
         TRIG_DISAPPEAR | TRIG_COLLIDE | TRIG_LOCKED, // 14
-        0 /* reserved 15 */,
-        0 /* reserved 16 */,
-        0 /* reserved 17 */,
-        0 /* reserved 18 */,
-        0 /* reserved 19 */,
-        0 /* reserved 20 */,
-        0 /* reserved 21 */,
-        0 /* reserved 22 */,
-        0 /* reserved 23 */,
-        0 /* reserved 24 */,
-        0 /* reserved 25 */,
-        0 /* reserved 26 */,
-        0 /* reserved 27 */,
-        0 /* reserved 28 */,
+        -1 /* reserved 15 */,
+        -1 /* reserved 16 */,
+        -1 /* reserved 17 */,
+        -1 /* reserved 18 */,
+        -1 /* reserved 19 */,
+        -1 /* reserved 20 */,
+        -1 /* reserved 21 */,
+        -1 /* reserved 22 */,
+        -1 /* reserved 23 */,
+        -1 /* reserved 24 */,
+        -1 /* reserved 25 */,
+        -1 /* reserved 26 */,
+        -1 /* reserved 27 */,
+        -1 /* reserved 28 */,
         TRIG_DISAPPEAR | TRIG_RUMBLE | TRIG_ENDSP, // 29
-        0 /* reserved 30 */,
-        0 /* reserved 31 */,
+        -1 /* reserved 30 */,
+        -1 /* reserved 31 */,
     };
 
-    #define validtrigger(type) (triggertypes[(type) & (NUMTRIGGERTYPES-1)]!=0)
+    #define validtrigger(type) (triggertypes[(type) & (NUMTRIGGERTYPES-1)]>=0)
     #define checktriggertype(type, flag) (triggertypes[(type) & (NUMTRIGGERTYPES-1)] & (flag))
 
     static inline void setuptriggerflags(fpsentity &e)
@@ -479,7 +479,7 @@ namespace entities
             if(e.type != ET_MAPMODEL || !validtrigger(e.attr[2])) continue;
             if(e.attr[3] == tag && e.triggerstate == oldstate && checktriggertype(e.attr[2], TRIG_LOCKED))
             {
-				if(newstate == TRIGGER_RESETTING && checktriggertype(e.attr[2], TRIG_COLLIDE) && overlapsdynent(e.o, e.attr[4] ? e.attr[4] : 20)) continue;
+                if(newstate == TRIGGER_RESETTING && checktriggertype(e.attr[2], TRIG_COLLIDE) && overlapsdynent(e.o, e.attr[4] ? e.attr[4] : 20)) continue;
                 e.triggerstate = newstate;
                 e.lasttrigger = lastmillis;
                 if(checktriggertype(e.attr[2], TRIG_RUMBLE)) playsound(S_RUMBLE, &e.o);
@@ -533,7 +533,7 @@ namespace entities
                 case TRIGGER_RESET:
                     if(e.lasttrigger)
                     {
-						if(checktriggertype(e.attr[2], TRIG_AUTO_RESET|TRIG_MANY|TRIG_LOCKED) && e.o.dist(o)-player1->radius>= (e.attr[4] ? e.attr[4] : (checktriggertype(e.attr[2], TRIG_COLLIDE)  ? 20 : 12)))
+                        if(checktriggertype(e.attr[2], TRIG_AUTO_RESET|TRIG_MANY|TRIG_LOCKED) && e.o.dist(o)-player1->radius>= (e.attr[4] ? e.attr[4] : (checktriggertype(e.attr[2], TRIG_COLLIDE)  ? 20 : 12)))
                             e.lasttrigger = 0;
                         break;
                     }
@@ -553,7 +553,7 @@ namespace entities
                     if(e.attr[3]) doleveltrigger(e.attr[3], 1);
                     break;
                 case TRIGGERED:
-					if(e.o.dist(o)-player1->radius< (e.attr[4] ? e.attr[4] : (checktriggertype(e.attr[2], TRIG_COLLIDE) ? 20 : 12)))
+                    if(e.o.dist(o)-player1->radius< (e.attr[4] ? e.attr[4] : (checktriggertype(e.attr[2], TRIG_COLLIDE) ? 20 : 12)))
                     {
                         if(e.lasttrigger) break;
                     }
@@ -603,8 +603,8 @@ namespace entities
             case MONSTER:
             case TELEDEST:
             case RESPAWNPOINT:
-		e.attr.pop();
-		e.attr.insert(0, player1->yaw);
+                e.attr.pop();
+                e.attr.insert(0, player1->yaw);
                 break;
         }
     }
@@ -651,43 +651,43 @@ namespace entities
 
     bool radiusent(extentity &e)
     {
-    	switch(e.type)
-    	{
-    		case LIGHT:
-    		case ENVMAP:
-    		case MAPSOUND:
-    		case DYNLIGHT:
-    			return true;
-    			break;
-    		default:
-    			return false;
-    			break;
-    	}
+        switch(e.type)
+        {
+            case LIGHT:
+            case ENVMAP:
+            case MAPSOUND:
+            case DYNLIGHT:
+                return true;
+                break;
+            default:
+                return false;
+                break;
+        }
     }
 
     bool dirent(extentity &e)
     {
-    	switch(e.type)
-    	{
-    		case MAPMODEL:
-    		case PLAYERSTART:
-    		case SPOTLIGHT:
-    		case TELEDEST:
-    		case TELEPORT:
-    		case MONSTER:
-    		case JUMPPAD:
-    		case BOX:
-    		case BARREL:
-    		case PLATFORM:
-    		case ELEVATOR:
-    		case RESPAWNPOINT:
-    		case FLAG:
-    			return true;
-    			break;
-    		default:
-    			return false;
-    			break;
-    	}
+        switch(e.type)
+        {
+            case MAPMODEL:
+            case PLAYERSTART:
+            case SPOTLIGHT:
+            case TELEDEST:
+            case TELEPORT:
+            case MONSTER:
+            case JUMPPAD:
+            case BOX:
+            case BARREL:
+            case PLATFORM:
+            case ELEVATOR:
+            case RESPAWNPOINT:
+            case FLAG:
+                return true;
+                break;
+            default:
+                return false;
+                break;
+        }
     }
 
     bool printent(extentity &e, char *buf)
@@ -705,7 +705,7 @@ namespace entities
             "health", "healthboost", "greenarmour", "yellowarmour", "quaddamage",
             "teleport", "teledest",
             "creature", "carrot", "jumppad",
-            "banana", "respawnpoint",
+            "base", "respawnpoint",
             "box", "barrel",
             "platform", "elevator",
             "flag",
@@ -716,128 +716,128 @@ namespace entities
 
     const int numattrs(int type)
     {
-	    static const int num[] =
-	    {
-		    0, //shells
-		    0, //bullets
-		    0, //rockets
-		    0, //riflerounds
-		    0, //grenades
-		    0, //cartridges
-		    0, //health
-		    0, //healthboost
-		    0, //greenarmour
-		    0, //yellowarmour
-		    0, //quaddamage
-		    3, //teleport
-		    3, //teledest
-		    3, //creature
-		    1, //carrot
-		    3, //jumppad
-		    2, //banana/base
-		    1, //respawnpoint
-		    4, //box
-		    4, //barrel
-		    4, //platform
-		    4, //elevator
-		    2, //flag
-		    0, //null x 4
-		    0,
-		    0,
-		    0
-	    };
+        static const int num[] =
+        {
+            0, //shells
+            0, //bullets
+            0, //rockets
+            0, //riflerounds
+            0, //grenades
+            0, //cartridges
+            0, //health
+            0, //healthboost
+            0, //greenarmour
+            0, //yellowarmour
+            0, //quaddamage
+            3, //teleport
+            3, //teledest
+            3, //creature
+            1, //carrot
+            3, //jumppad
+            2, //banana/base
+            1, //respawnpoint
+            4, //box
+            4, //barrel
+            4, //platform
+            4, //elevator
+            2, //flag
+            0, //null x 4
+            0,
+            0,
+            0
+        };
 
-	    type -= ET_GAMESPECIFIC;
-	    return type >= 0 && size_t(type) < sizeof(num)/sizeof(num[0]) ? num[type] : 0;
+        type -= ET_GAMESPECIFIC;
+        return type >= 0 && size_t(type) < sizeof(num)/sizeof(num[0]) ? num[type] : 0;
     }
 
     void renderhelpertext(extentity &e, int &colour, vec &pos, string &tmp)
     {
-	switch(e.type)
-	{
-		case TELEPORT:
-			pos.z += 3.0;
-			formatstring(tmp)("Teledest Tag: %i\nModel: %s (%i)\nTag: %i",
-				e.attr[0],
-				mapmodelname(e.attr[1]),
-				e.attr[1],
-				e.attr[2]
-			);
-			return;
-		case TELEDEST:
-			pos.z += 3;
-			formatstring(tmp)("Yaw: %i\nTeleport Tag: %i\n",
-				e.attr[0],
-				e.attr[1]
-			);
-			return;
-		case MONSTER:
-			pos.z += 4.5;
-			formatstring(tmp)("Yaw: %i\nMonster: %s (%i)\nTag: monster_dead_%i",
-				e.attr[0],
-				((e.attr[1] >= 0 && e.attr[1] < game::NUMMONSTERTYPES) ? game::monstertypes[e.attr[1]].name : "(null)"), e.attr[1],
-				e.attr[2]
-			);
-			return;
-		case JUMPPAD:
-			pos.z += 4.5;
-			formatstring(tmp)("Z: %i\nY: %i\nX: %i",
-				e.attr[0],
-				e.attr[1],
-				e.attr[2]
-			);
-			return;
-		case BASE:
-			pos.z += 3.0f;
-			formatstring(tmp)("Ammo: %i\nTag: base_%i",
-				e.attr[0],
-				e.attr[1]
-			);
-			return;
-		case RESPAWNPOINT:
-			pos.z += 1.5;
-			formatstring(tmp)("Yaw: %i",
-				e.attr[0]
-			);
-			return;
-		case BOX:
-		case BARREL:
-			pos.z += 6.0;
-			formatstring(tmp)("Yaw: %i\nModel: %s (%i)\nWeight: %i\nHealth: %i",
-				e.attr[0],
-				mapmodelname(e.attr[1]), e.attr[1],
-				e.attr[2],
-				e.attr[3]
-			);
-			return;
-		case PLATFORM:
-		case ELEVATOR:
-			pos.z += 6.0;
-			formatstring(tmp)("Yaw: %i\nModel: %s (%i)\nTag: %i\nSpeed: %i",
-				e.attr[0],
-				mapmodelname(e.attr[1]), e.attr[1],
-				e.attr[2],
-				e.attr[3]
-			);
-			return;
-		case FLAG:
-			pos.z += 3.0f;
-			formatstring(tmp)("Yaw: %i\nTeam: %i",
-				e.attr[0],
-				e.attr[1]
-			);
-			return;
-		case DYNLIGHT:
-			pos.z += 7.5;
-			formatstring(tmp)("Radius %i\n\fs\fRRed: %i\n\fJGreen: %i\n\fDBlue: %i\fr\nTag: %i",
-				e.attr[0],
-				e.attr[1],
-				e.attr[2],
-				e.attr[3],
-				e.attr[4]
-			);
-			return;
-	}
+        switch(e.type)
+        {
+            case TELEPORT:
+                pos.z += 3.0;
+                formatstring(tmp)("Teledest Tag: %i\nModel: %s (%i)\nTag: %i",
+                    e.attr[0],
+                    mapmodelname(e.attr[1]),
+                    e.attr[1],
+                    e.attr[2]
+                );
+                return;
+            case TELEDEST:
+                pos.z += 3;
+                formatstring(tmp)("Yaw: %i\nTeleport Tag: %i\n",
+                    e.attr[0],
+                    e.attr[1]
+                );
+                return;
+            case MONSTER:
+                pos.z += 4.5;
+                formatstring(tmp)("Yaw: %i\nMonster: %s (%i)\nTag: monster_dead_%i",
+                    e.attr[0],
+                    ((e.attr[1] >= 0 && e.attr[1] < game::NUMMONSTERTYPES) ? game::monstertypes[e.attr[1]].name : "(null)"), e.attr[1],
+                    e.attr[2]
+                );
+                return;
+            case JUMPPAD:
+                pos.z += 4.5;
+                formatstring(tmp)("Z: %i\nY: %i\nX: %i",
+                    e.attr[0],
+                    e.attr[1],
+                    e.attr[2]
+                );
+                return;
+            case BASE:
+                pos.z += 3.0f;
+                formatstring(tmp)("Ammo: %i\nTag: base_%i",
+                    e.attr[0],
+                    e.attr[1]
+                );
+                return;
+            case RESPAWNPOINT:
+                pos.z += 1.5;
+                formatstring(tmp)("Yaw: %i",
+                    e.attr[0]
+                );
+                return;
+            case BOX:
+            case BARREL:
+                pos.z += 6.0;
+                formatstring(tmp)("Yaw: %i\nModel: %s (%i)\nWeight: %i\nHealth: %i",
+                    e.attr[0],
+                    mapmodelname(e.attr[1]), e.attr[1],
+                    e.attr[2],
+                    e.attr[3]
+                );
+                return;
+            case PLATFORM:
+            case ELEVATOR:
+                pos.z += 6.0;
+                formatstring(tmp)("Yaw: %i\nModel: %s (%i)\nTag: %i\nSpeed: %i",
+                    e.attr[0],
+                    mapmodelname(e.attr[1]), e.attr[1],
+                    e.attr[2],
+                    e.attr[3]
+                );
+                return;
+            case FLAG:
+                pos.z += 3.0f;
+                formatstring(tmp)("Yaw: %i\nTeam: %i",
+                    e.attr[0],
+                    e.attr[1]
+                );
+                return;
+            case DYNLIGHT:
+                pos.z += 7.5;
+                formatstring(tmp)("Radius %i\n\fs\fRRed: %i\n\fJGreen: %i\n\fDBlue: %i\fr\nTag: %i",
+                    e.attr[0],
+                    e.attr[1],
+                    e.attr[2],
+                    e.attr[3],
+                    e.attr[4]
+                );
+                return;
+        }
     }
 
     void editent(int i, bool local)
