@@ -1,5 +1,4 @@
 #include "game.h"
-//#include "sb.h"
 
 namespace game
 {
@@ -27,17 +26,12 @@ namespace game
     }
     COMMAND(setdynlightactivity, "ii");
 
-    void adddynlights()
-    {
-    	entities::adddynlights();
-    }
-
     void taunt()
     {
         if(player1->state!=CS_ALIVE || player1->physstate<PHYS_SLOPE) return;
         if(lastmillis-player1->lasttaunt<1000) return;
         player1->lasttaunt = lastmillis;
-	addmsg(N_TAUNT, "rc", player1);
+        addmsg(N_TAUNT, "rc", player1);
     }
     COMMAND(taunt, "");
 
@@ -47,17 +41,17 @@ namespace game
         intret(f ? f->clientnum : -1);
     });
 
-	void follow(char *arg)
+    void follow(char *arg)
     {
         if(arg[0] ? player1->state==CS_SPECTATOR : following>=0)
         {
-	    following = arg[0] ? parseplayer(arg) : -1;
+        following = arg[0] ? parseplayer(arg) : -1;
             if(following==player1->clientnum) following = -1;
             followdir = 0;
             conoutf("follow %s", following>=0 ? "on" : "off");
         }
-	}
-	COMMAND(follow, "s");
+    }
+    COMMAND(follow, "s");
 
     void nextfollow(int dir)
     {
@@ -88,10 +82,12 @@ namespace game
     {
         if(m_classicsp)
         {
-		clearmovables();
+            clearmovables();
             clearmonsters();                 // all monsters back at their spawns for editing
             entities::resettriggers();
         }
+        clearprojectiles();
+        clearbouncers();
     }
 
     fpsent *spawnstate(fpsent *d)              // reset player state not persistent accross spawns
@@ -115,45 +111,12 @@ namespace game
         else
         {
             spawnplayer(player1);
-            #ifndef NEWGUI
-	    if (deathscore) showscores(false);
-            #endif
-            lasthit = 0;
-	    if(cmode) cmode->respawned(player1);
+            showscores(false);
+//             lasthit = 0;
+            if(cmode) cmode->respawned(player1);
         }
     }
 
-    float intersectdist = 1e16f;
-
-    bool intersect(dynent *d, const vec &from, const vec &to, float &dist)   // if lineseg hits entity bounding box
-    {
-        vec bottom(d->o), top(d->o);
-        bottom.z -= d->eyeheight;
-        top.z += d->aboveeye;
-        return linecylinderintersect(from, to, bottom, top, d->radius, dist);
-    }
-
-    dynent *intersectclosest(const vec &from, const vec &to, fpsent *at, float &bestdist)
-    {
-        dynent *best = NULL;
-        bestdist = 1e16f;
-        loopi(numdynents())
-        {
-            dynent *o = iterdynents(i);
-            if(o==at || o->state!=CS_ALIVE) continue;
-            float dist;
-            if(!intersect(o, from, to, dist)) continue;
-            if(dist<bestdist)
-            {
-                best = o;
-                bestdist = dist;
-            }
-        }
-        return best;
-    }
-    void gibeffect(int damage, const vec &vel, fpsent *d) {}
-    void bounced(physent *d, const vec &surface) {}
-
     fpsent *pointatplayer()
     {
         loopv(players) if(players[i] != player1 && intersect(players[i], player1->o, worldpos)) return players[i];
@@ -219,6 +182,7 @@ namespace game
         d->o = d->newpos;
         d->yaw = d->newyaw;
         d->pitch = d->newpitch;
+        d->roll = d->newroll;
         if(move)
         {
             moveplayer(d, 1, false);
@@ -232,6 +196,7 @@ namespace game
             if(d->yaw<0) d->yaw += 360;
             else if(d->yaw>=360) d->yaw -= 360;
             d->pitch += d->deltapitch*k;
+            d->roll += d->deltaroll*k;
         }
     }
 
@@ -242,12 +207,12 @@ namespace game
             fpsent *d = players[i];
             if(d == player1 || d->ai) continue;
 
-            if(d->state==CS_ALIVE)
+            if(d->state==CS_DEAD && d->ragdoll) moveragdoll(d);
+            else if(!intermission)
             {
                 if(lastmillis - d->lastaction >= d->gunwait) d->gunwait = 0;
                 if(d->quadmillis) entities::checkquad(curtime, d);
             }
-            else if(d->state==CS_DEAD && d->ragdoll) moveragdoll(d);
 
             const int lagtime = totalmillis-d->lastupdate;
             if(!lagtime || intermission) continue;
@@ -288,14 +253,18 @@ namespace game
 
         physicsframe();
         ai::navigate();
-        entities::checkquad(curtime, player1);
+        if(player1->state != CS_DEAD && !intermission)
+        {
+            if(player1->quadmillis) entities::checkquad(curtime, player1);
+        }
+        updateweapons(curtime);
         otherplayers(curtime);
         ai::update();
         moveragdolls();
         gets2c();
         updatemovables(curtime);
         updatemonsters(curtime);
-        if(player1->state==CS_DEAD)
+        if(player1->state == CS_DEAD)
         {
             if(player1->ragdoll) moveragdoll(player1);
             else if(lastmillis-player1->lastpain<2000)
@@ -358,7 +327,7 @@ namespace game
             }
             if(lastmillis < player1->lastpain + spawnwait) return;
             if(m_dmsp) { changemap(clientmap, gamemode); return; }    // if we die in SP we try the same map again
-	    respawnself();
+            respawnself();
         }
     }
 
@@ -366,8 +335,10 @@ namespace game
 
     void doattack(bool on)
     {
-    	if(!intermission) respawn();
+        if(intermission) return;
+        if((player1->attacking = on)) respawn();
     }
+    ICOMMAND(attack, "D", (int *down), { doattack(*down!=0); });
 
     bool canjump()
     {
@@ -401,14 +372,15 @@ namespace game
             damageblend(damage);
             damagecompass(damage, actor->o);
         }
+        damageeffect(damage, d, d!=h);
 
-		ai::damaged(d, actor);
+        ai::damaged(d, actor);
 
         if(m_sp && slowmosp && d==player1 && d->health < 1) d->health = 1;
 
         if(d->health<=0) { if(local) killed(d, actor); }
-        else if(d==h) playsound(S_PAIN6);
-        else playsound(S_PAIN1+rnd(5), &d->o);
+        else if(d==h) playsound(S_PAIN);
+        else playsound(S_PAIN, &d->o);
     }
 
     VARP(deathscore, 0, 1, 1);
@@ -420,23 +392,21 @@ namespace game
         if(!restore) gibeffect(max(-d->health, 0), d->vel, d);
         if(d==player1)
         {
-            #ifndef NEWGUI
-            showscores(true);
-            #endif
+            if(deathscore) showscores(true);
             disablezoom();
             if(!restore) loopi(NUMGUNS) savedammo[i] = player1->ammo[i];
             d->attacking = false;
             if(!restore) d->deaths++;
             //d->pitch = 0;
             d->roll = 0;
-            playsound(S_DIE1+rnd(2));
+            playsound(S_DIE);
         }
         else
         {
             d->move = d->strafe = 0;
             d->resetinterp();
             d->smoothmillis = 0;
-            playsound(S_DIE1+rnd(2), &d->o);
+            playsound(S_DIE, &d->o);
         }
     }
 
@@ -461,7 +431,7 @@ namespace game
             conoutf(contype, "\f2%s did something very dangerous%s", dname, d==player1 ? "!" : "");
 
         deathstate(d);
-	ai::killed(d, actor);
+        ai::killed(d, actor);
     }
 
     void timeupdate(int secs)
@@ -474,12 +444,10 @@ namespace game
         {
             intermission = true;
             player1->attacking = false;
-	    if(cmode) cmode->gameover();
+            if(cmode) cmode->gameover();
             conoutf(CON_GAMEINFO, "\f2intermission:");
             conoutf(CON_GAMEINFO, "\f2game has ended!");
-	    #ifndef NEWGUI
             showscores(true);
-            #endif
             disablezoom();
 
             if(identexists("intermission")) execute("intermission");
@@ -534,6 +502,7 @@ namespace game
         fpsent *d = clients[cn];
         if(!d) return;
         if(notify && d->name[0]) conoutf("player %s disconnected", colorname(d));
+        removeweapons(d);
         removetrackedparticles(d);
         removetrackeddynlights(d);
         if(cmode) cmode->removeplayer(d);
@@ -550,7 +519,7 @@ namespace game
     void initclient()
     {
         player1 = spawnstate(new fpsent);
-	players.add(player1);
+        players.add(player1);
     }
 
     VARP(showmodeinfo, 0, 1, 1);
@@ -564,10 +533,11 @@ namespace game
     {
         clearmovables();
         clearmonsters();
-
+        clearprojectiles();
+        clearbouncers();
         clearragdolls();
 
-	          // reset perma-state
+        // reset perma-state
         loopv(players)
         {
             fpsent *d = players[i];
@@ -619,7 +589,7 @@ namespace game
     {
         ai::savewaypoints();
         ai::clearwaypoints(true);
-	for(int i = 0; i != 32767; i++)
+        for(int i = 0; i != 32767; i++)
         {
             entities::dynlightsactive[i] = true;
         }
@@ -647,12 +617,12 @@ namespace game
                 if (waterlevel==0) break;
                 playsound(S_BURN, d==player1 ? NULL : &d->o);
                 loopi(60)
-		{
-			vec o = d->o;
-			o.z -= d->eyeheight *i/60.0f;
-			regular_particle_flame(PART_FLAME, o, 6, 2, 0x903020, 3, 2.0f);
-			regular_particle_flame(PART_SMOKE, vec(o.x, o.y, o.z + 8.0f), 6, 2, 0x303020, 1, 4.0f, 100.0f, 2000.0f, -20);
-		}
+                {
+                    vec o = d->o;
+                    o.z -= d->eyeheight *i/60.0f;
+                    regular_particle_flame(PART_FLAME, o, 6, 2, 0x903020, 3, 2.0f);
+                    regular_particle_flame(PART_SMOKE, vec(o.x, o.y, o.z + 8.0f), 6, 2, 0x303020, 1, 4.0f, 100.0f, 2000.0f, -20);
+                }
                 break;
             case MAT_WATER:
                 if (waterlevel==0) break;
@@ -733,16 +703,16 @@ namespace game
                 pl->suicided = pl->lifesequence;
             }
         }
-	else if(d->type==ENT_AI) suicidemonster((monster *)d);
+        else if(d->type==ENT_AI) suicidemonster((monster *)d);
         else if(d->type==ENT_INANIMATE) suicidemovable((movable *)d);
     }
     ICOMMAND(nap, "", (), suicide(player1));
 
-    bool needminimap() { return m_ctf || m_protect || m_hold || m_capture; }
+    bool needminimap() { return m_ctf || m_protect || m_hold || m_capture || m_collect; }
 
     void drawicon(int icon, float x, float y, float sz)
     {
-        settexture("packages/hud/items.png");
+        settexture("data/fps/hud/items");
         glBegin(GL_TRIANGLE_STRIP);
         float tsz = 0.25f, tx = tsz*(icon%4), ty = tsz*(icon/4);
         glTexCoord2f(tx,     ty);     glVertex2f(x,    y);
@@ -768,35 +738,19 @@ namespace game
         ammohuddown[3] = { GUN_RIFLE, GUN_SG, GUN_PISTOL },
         ammohudcycle[7] = { -1, -1, -1, -1, -1, -1, -1 };
 
-    ICOMMAND(ammohudup, "sss", (char *w1, char *w2, char *w3),
+    ICOMMAND(ammohudup, "V", (tagval *args, int numargs),
     {
-        int i = 0;
-        if(w1[0]) ammohudup[i++] = parseint(w1);
-        if(w2[0]) ammohudup[i++] = parseint(w2);
-        if(w3[0]) ammohudup[i++] = parseint(w3);
-        while(i < 3) ammohudup[i++] = -1;
+        loopi(3) ammohudup[i] = i < numargs ? getweapon(args[i].getstr()) : -1;
     });
 
-    ICOMMAND(ammohuddown, "sss", (char *w1, char *w2, char *w3),
+    ICOMMAND(ammohuddown, "V", (tagval *args, int numargs),
     {
-        int i = 0;
-        if(w1[0]) ammohuddown[i++] = parseint(w1);
-        if(w2[0]) ammohuddown[i++] = parseint(w2);
-        if(w3[0]) ammohuddown[i++] = parseint(w3);
-        while(i < 3) ammohuddown[i++] = -1;
+        loopi(3) ammohuddown[i] = i < numargs ? getweapon(args[i].getstr()) : -1;
     });
 
-    ICOMMAND(ammohudcycle, "sssssss", (char *w1, char *w2, char *w3, char *w4, char *w5, char *w6, char *w7),
+    ICOMMAND(ammohudcycle, "V", (tagval *args, int numargs),
     {
-        int i = 0;
-        if(w1[0]) ammohudcycle[i++] = parseint(w1);
-        if(w2[0]) ammohudcycle[i++] = parseint(w2);
-        if(w3[0]) ammohudcycle[i++] = parseint(w3);
-        if(w4[0]) ammohudcycle[i++] = parseint(w4);
-        if(w5[0]) ammohudcycle[i++] = parseint(w5);
-        if(w6[0]) ammohudcycle[i++] = parseint(w6);
-        if(w7[0]) ammohudcycle[i++] = parseint(w7);
-        while(i < 7) ammohudcycle[i++] = -1;
+        loopi(7) ammohudcycle[i] = i < numargs ? getweapon(args[i].getstr()) : -1;
     });
 
     VARP(ammohud, 0, 1, 1);
@@ -868,11 +822,11 @@ namespace game
 
     void gameplayhud(int w, int h)
     {
+        glPushMatrix();
+        glScalef(h/1800.0f, h/1800.0f, 1);
+
         if(player1->state==CS_SPECTATOR)
         {
-            glPushMatrix();
-            glScalef(h/1800.0f, h/1800.0f, 1);
-
             int pw, ph, tw, th, fw, fh;
             text_bounds("  ", pw, ph);
             text_bounds("SPECTATOR", tw, th);
@@ -882,20 +836,16 @@ namespace game
             fh = max(fh, ph);
             draw_text("SPECTATOR", w*1800/h - tw - pw, 1650 - th - fh);
             if(f) draw_text(colorname(f), w*1800/h - fw - pw, 1650 - fh);
-
-            /*fpsent *d = hudplayer();
-            if(d->state!=CS_EDITING)
-            {
-                if(d->state!=CS_SPECTATOR) drawhudicons(d);
-                if(cmode) cmode->drawhud(d, w, h);
-            }*/
-
-            glPopMatrix();
         }
-        else
+
+        fpsent *d = hudplayer();
+        if(d->state!=CS_EDITING)
         {
-           // SB::RenderHud(w,h);
+            if(d->state!=CS_SPECTATOR) drawhudicons(d);
+            if(cmode) cmode->drawhud(d, w, h);
         }
+
+        glPopMatrix();
     }
 
     int clipconsole(int w, int h)
@@ -911,26 +861,34 @@ namespace game
     {
         switch(index)
         {
-	    case 3: return "packages/crosshairs/edit.png";
-	    case 2: return "packages/crosshairs/hit.png";
-	    case 1: return "packages/crosshairs/teammate.png";
-	    default: return "packages/crosshairs/default.png";
+            case 3: return "packages/crosshairs/edit.png";
+            case 2: return "packages/crosshairs/hit.png";
+            case 1: return "packages/crosshairs/teammate.png";
+            default: return "packages/crosshairs/default.png";
         }
     }
 
     int selectcrosshair(float &r, float &g, float &b)
     {
         fpsent *d = hudplayer();
-	if(editmode) { r = g = 0.5f; b = 1; return 3;
-	}
+        if(editmode) { r = g = 0.5f; b = 1; return 3; }
         if(d->state==CS_SPECTATOR || d->state==CS_DEAD) return -1;
 
         if(d->state!=CS_ALIVE) return 0;
 
         int crosshair = 0;
         if(lasthit && lastmillis - lasthit < hitcrosshair) crosshair = 2;
+        else if(teamcrosshair)
+        {
+            dynent *o = intersectclosest(d->o, worldpos, d);
+            if(o && o->type==ENT_PLAYER && isteam(((fpsent *)o)->team, d->team))
+            {
+                crosshair = 1;
+                r = g = 0;
+            }
+        }
 
-        if(crosshair!=1 && !m_insta)
+        if(crosshair!=1 && !editmode && !m_insta)
         {
             if(d->health<=25) { r = 1.0f; g = b = 0; }
             else if(d->health<=50) { r = 1.0f; g = 0.5f; b = 0; }
@@ -952,148 +910,13 @@ namespace game
 #endif
     }
 
-    void particletrack(physent *owner, vec &o, vec &d)
-    {
-          if(owner->type!=ENT_PLAYER && owner->type!=ENT_AI) return;
-         fpsent *pl = (fpsent *)owner;
-         if(pl->muzzle.x < 0 || pl->lastattackgun != pl->gunselect) return;
-          float dist = o.dist(d);
-         o = pl->muzzle;
-         if(dist <= 0) d = o;
-         else
-         {
-             vecfromyawpitch(owner->yaw, owner->pitch, 1, 0, d);
-             float newdist = raycube(owner->o, d, dist, RAY_CLIPMAT|RAY_ALPHAPOLY);
-             d.mul(min(newdist, dist)).add(owner->o);
-         }
-    }
-
-        void dynlighttrack(physent *owner, vec &o, vec &hud)
-    {
-        if(owner->type!=ENT_PLAYER && owner->type!=ENT_AI) return;
-        fpsent *pl = (fpsent *)owner;
-        if(pl->muzzle.x < 0 || pl->lastattackgun != pl->gunselect) return;
-        o = pl->muzzle;
-	hud = owner == hudplayer() ? vec(pl->o).add(vec(0, 0, 2)) : pl->muzzle;
-    }
-
-    #ifndef NEWGUI
-    bool serverinfostartcolumn(g3d_gui *g, int i)
-    {
-        static const char *names[] = { "ping ", "players ", "map ", "mode ", "master ", "host ", "port ", "description " };
-        static const int struts[] =  { 7,       7,          14,     13,      8,         14,      7,       24 };
-        if(size_t(i) >= sizeof(names)/sizeof(names[0])) return false;
-        g->pushlist();
-        g->text(names[i], 0xFFFF80, !i ? " " : NULL);
-        if(struts[i]) g->strut(struts[i]);
-        g->mergehits(true);
-        return true;
-    }
-
-    void serverinfoendcolumn(g3d_gui *g, int i)
-    {
-        g->mergehits(false);
-        g->poplist();
-    }
-
-    const char *mastermodecolor(int n, const char *unknown)
-    {
-        return (n>=MM_START && size_t(n-MM_START)<sizeof(mastermodecolors)/sizeof(mastermodecolors[0])) ? mastermodecolors[n-MM_START] : unknown;
-    }
-
-    const char *mastermodeicon(int n, const char *unknown)
-    {
-        return (n>=MM_START && size_t(n-MM_START)<sizeof(mastermodeicons)/sizeof(mastermodeicons[0])) ? mastermodeicons[n-MM_START] : unknown;
-    }
-
-    bool serverinfoentry(g3d_gui *g, int i, const char *name, int port, const char *sdesc, const char *map, int ping, const vector<int> &attr, int np)
-    {
-        if(ping < 0 || attr.empty() || attr[0]!=PROTOCOL_VERSION)
-        {
-            switch(i)
-            {
-                case 0:
-                    if(g->button(" ", 0xFFFFDD, "serverunk")&G3D_UP) return true;
-                    break;
-
-                case 1:
-                case 2:
-                case 3:
-                case 4:
-                    if(g->button(" ", 0xFFFFDD)&G3D_UP) return true;
-                    break;
-
-                case 5:
-                    if(g->buttonf("%s ", 0xFFFFDD, NULL, name)&G3D_UP) return true;
-                    break;
-
-                case 6:
-                    if(g->buttonf("%d ", 0xFFFFDD, NULL, port)&G3D_UP) return true;
-                    break;
-
-                case 7:
-                    if(ping < 0)
-                    {
-                        if(g->button(sdesc, 0xFFFFDD)&G3D_UP) return true;
-                    }
-                    else if(g->buttonf("[%s protocol] ", 0xFFFFDD, NULL, attr.empty() ? "unknown" : (attr[0] < PROTOCOL_VERSION ? "older" : "newer"))&G3D_UP) return true;
-                    break;
-            }
-            return false;
-        }
-
-        switch(i)
-        {
-            case 0:
-            {
-                const char *icon = attr.inrange(3) && np >= attr[3] ? "serverfull" : (attr.inrange(4) ? mastermodeicon(attr[4], "serverunk") : "serverunk");
-                if(g->buttonf("%d ", 0xFFFFDD, icon, ping)&G3D_UP) return true;
-                break;
-            }
-
-            case 1:
-                if(attr.length()>=4)
-                {
-                    if(g->buttonf(np >= attr[3] ? "\f3%d/%d " : "%d/%d ", 0xFFFFDD, NULL, np, attr[3])&G3D_UP) return true;
-                }
-                else if(g->buttonf("%d ", 0xFFFFDD, NULL, np)&G3D_UP) return true;
-                break;
-
-            case 2:
-                if(g->buttonf("%.25s ", 0xFFFFDD, NULL, map)&G3D_UP) return true;
-                break;
-
-            case 3:
-                if(g->buttonf("%s ", 0xFFFFDD, NULL, attr.length()>=2 ? server::modename(attr[1], "") : "")&G3D_UP) return true;
-                break;
-
-            case 4:
-                if(g->buttonf("%s%s ", 0xFFFFDD, NULL, attr.length()>=5 ? mastermodecolor(attr[4], "") : "", attr.length()>=5 ? server::mastermodename(attr[4], "") : "")&G3D_UP) return true;
-                break;
-
-            case 5:
-                if(g->buttonf("%s ", 0xFFFFDD, NULL, name)&G3D_UP) return true;
-                break;
-
-            case 6:
-                if(g->buttonf("%d ", 0xFFFFDD, NULL, port)&G3D_UP) return true;
-                break;
-
-            case 7:
-                if(g->buttonf("%.25s", 0xFFFFDD, NULL, sdesc)&G3D_UP) return true;
-                break;
-        }
-        return false;
-    }
-    #endif
-
     bool showenthelpers()
     {
-    	if(m_edit || editmode) return true;
-    	return false;
+        if(m_edit || editmode) return true;
+        return false;
     }
 
-    bool allowdoublejump() {return false;}
+    bool allowdoublejump(physent *d) {return false;}
     void updatecamera() {}
     bool mousemove(int &dx, int &dy, float &cursens)
     {
