@@ -21,9 +21,9 @@ enum
 
 enum
 {
-	CHAT_TALK = 1<<0,
-	CHAT_EMOTE = 1<<1,
-	CHAT_TEAM = 1<<2,
+    CHAT_TALK = 1<<0,
+    CHAT_EMOTE = 1<<1,
+    CHAT_TEAM = 1<<2,
 };
 
 // network quantization scale
@@ -101,7 +101,8 @@ enum
     M_LOBBY      = 1<<15,
     M_DMSP       = 1<<16,
     M_CLASSICSP  = 1<<17,
-    M_SLOWMO     = 1<<18
+    M_SLOWMO     = 1<<18,
+    M_COLLECT    = 1<<19
 };
 
 static struct gamemodeinfo
@@ -111,15 +112,19 @@ static struct gamemodeinfo
     const char *info;
 } gamemodes[] =
 {
-    { "SP", M_LOCAL | M_CLASSICSP, "enable single player options" },
+    { "DMSP", M_LOCAL | M_DMSP, NULL },
+	{ "SP", M_LOCAL | M_CLASSICSP, "enable single player options" },
     { "demo", M_DEMO | M_LOCAL, "playback recorded demos" },
     { "default", M_LOBBY, "Run around freely without a care in the world" },
     { "coop edit", M_EDIT, "Edit cooperatively with friends over the LAN or internet" },
     { "banana relay", M_NOAMMO | M_TACTICS | M_CAPTURE | M_TEAM, "Capture bananas, robochimp is hungry" },
-    { "capture the banana (ctf)", M_CTF | M_TEAM, "Capture the other team's banana, and add it to your own"}
+    { "capture the banana (ctf)", M_CTF | M_TEAM, "Capture the other team's banana, and add it to your own"},
+	{ "team drenchmatch", M_TEAM | M_OVERTIME, "Teamplay: Collect items for ammo. Drench \fs\f3the enemy team\fr to score points for \fs\f1your team\fr." },
+	{ "instadrench", M_NOITEMS | M_INSTA, "Instadrench: You spawn with full rifle ammo and drench instantly from one shot. There are no items. Drench everyone to score points." },
+    { "instadrench team", M_NOITEMS | M_INSTA | M_TEAM | M_OVERTIME, "Instadrench Team: You spawn with full rifle ammo and drench instantly from one shot. There are no items. Drench \fs\f3the enemy team\fr to score points for \fs\f1your team\fr." }
 };
 
-#define STARTGAMEMODE (-2)
+#define STARTGAMEMODE (-3)
 #define NUMGAMEMODES ((int)(sizeof(gamemodes)/sizeof(gamemodes[0])))
 
 #define m_valid(mode)          ((mode) >= STARTGAMEMODE && (mode) < STARTGAMEMODE + NUMGAMEMODES)
@@ -137,6 +142,7 @@ static struct gamemodeinfo
 #define m_ctf          (m_check(gamemode, M_CTF))
 #define m_protect      (m_checkall(gamemode, M_CTF | M_PROTECT))
 #define m_hold         (m_checkall(gamemode, M_CTF | M_HOLD))
+#define m_collect      (m_check(gamemode, M_COLLECT))
 #define m_teammode     (m_check(gamemode, M_TEAM))
 #define m_overtime     (m_check(gamemode, M_OVERTIME))
 #define isteam(a,b)    (m_teammode && strcmp(a, b)==0)
@@ -164,8 +170,8 @@ enum
     S_JUMP = 0, S_LAND, S_RIFLE, S_TELEPORT, S_SPLASH1, S_SPLASH2, S_CG,
     S_RLFIRE, S_RUMBLE, S_JUMPPAD, S_WEAPLOAD, S_ITEMAMMO, S_ITEMHEALTH,
     S_ITEMARMOUR, S_ITEMPUP, S_ITEMSPAWN,  S_NOAMMO, S_PUPOUT,
-    S_PAIN1, S_PAIN2, S_PAIN3, S_PAIN4, S_PAIN5, S_PAIN6,
-    S_DIE1, S_DIE2,
+    S_PAIN,
+    S_DIE,
     S_FLAUNCH, S_FEXPLODE,
     S_SG, S_PUNCH1,
     S_GRUNT1, S_GRUNT2, S_RLHIT,
@@ -227,6 +233,8 @@ enum
     N_ADDBOT, N_DELBOT, N_INITAI, N_FROMAI, N_BOTLIMIT, N_BOTBALANCE,
     N_MAPCRC, N_CHECKMAPS,
     N_SWITCHNAME, N_SWITCHMODEL, N_SWITCHTEAM,
+    N_INITTOKENS, N_TAKETOKEN, N_EXPIRETOKENS, N_DROPTOKENS, N_DEPOSITTOKENS,
+	N_SERVCMD,
     N_ATTACH, N_DETACH,
     NUMSV
 };
@@ -256,6 +264,8 @@ static const int msgsizes[] =               // size inclusive message token, 0 f
     N_ADDBOT, 2, N_DELBOT, 1, N_INITAI, 0, N_FROMAI, 2, N_BOTLIMIT, 2, N_BOTBALANCE, 2,
     N_MAPCRC, 0, N_CHECKMAPS, 1,
     N_SWITCHNAME, 0, N_SWITCHMODEL, 2, N_SWITCHTEAM, 0,
+    N_INITTOKENS, 0, N_TAKETOKEN, 2, N_EXPIRETOKENS, 0, N_DROPTOKENS, 0, N_DEPOSITTOKENS, 2,
+	N_SERVCMD, 0,
     N_ATTACH, 0, N_DETACH, 0,
     -1
 };
@@ -264,7 +274,7 @@ static const int msgsizes[] =               // size inclusive message token, 0 f
 #define SANDBOX_SERVER_PORT 28785
 #define SANDBOX_SERVINFO_PORT 28786
 #define SANDBOX_MASTER_PORT 28787
-#define PROTOCOL_VERSION 7            // bump when protocol changes
+#define PROTOCOL_VERSION 9              // bump when protocol changes
 #define DEMO_VERSION 1                  // bump when demo format changes
 #define DEMO_MAGIC "SAUERBRATEN_DEMO"
 
@@ -326,6 +336,8 @@ enum
     HICON_BLUE_FLAG,
     HICON_NEUTRAL_FLAG,
 
+    HICON_TOKEN,
+
     HICON_X       = 20,
     HICON_Y       = 1650,
     HICON_TEXTY   = 1644,
@@ -335,6 +347,22 @@ enum
 };
 
 static struct itemstat { int add, max, sound; const char *name; int icon, info; } itemstats[] =
+
+{
+    {10,    30,    S_ITEMAMMO,   "SG", HICON_SG, GUN_SG},
+    {40,    100,    S_ITEMAMMO,   "CG", HICON_CG, GUN_CG},
+    {5,     15,    S_ITEMAMMO,   "RL", HICON_RL, GUN_RL},
+    {5,     15,    S_ITEMAMMO,   "RI", HICON_RIFLE, GUN_RIFLE},
+    {4,    8,    S_ITEMAMMO,   "GL", HICON_GL, GUN_GL},
+    {30,    120,   S_ITEMAMMO,   "PI", HICON_PISTOL, GUN_PISTOL},
+    {25,    100,   S_ITEMHEALTH, "H", HICON_HEALTH},
+    {10,    1000,  S_ITEMHEALTH, "MH", HICON_HEALTH},
+    {100,   100,   S_ITEMARMOUR, "GA", HICON_GREEN_ARMOUR, A_GREEN},
+    {200,   200,   S_ITEMARMOUR, "YA", HICON_YELLOW_ARMOUR, A_YELLOW},
+    {20000, 30000, S_ITEMPUP,    "Q", HICON_QUAD},
+};
+
+/* original values cal edit
 {
     {10,    30,    S_ITEMAMMO,   "SG", HICON_SG, GUN_SG},
     {20,    60,    S_ITEMAMMO,   "CG", HICON_CG, GUN_CG},
@@ -348,29 +376,48 @@ static struct itemstat { int add, max, sound; const char *name; int icon, info;
     {200,   200,   S_ITEMARMOUR, "YA", HICON_YELLOW_ARMOUR, A_YELLOW},
     {20000, 30000, S_ITEMPUP,    "Q", HICON_QUAD},
 };
+*/
+
 
 #define SGRAYS 20
-#define SGSPREAD 4
 #define RL_DAMRAD 40
 #define RL_SELFDAMDIV 2
 #define RL_DISTSCALE 1.5f
 
-static const struct guninfo { short sound, attackdelay, damage, projspeed, part, kickamount, range; const char *name, *file; } guns[NUMGUNS] =
+static const struct guninfo { short sound, attackdelay, damage, spread, projspeed, part, kickamount, range; const char *name, *file; } guns[NUMGUNS] =
+
 {
-    { S_PUNCH1,    250,  50, 0,   0, 0,   14,  "fist",            "fist"  },
-    { S_SG,       1400,  10, 0,   0, 20, 1024, "shotgun",         "shotg" },  // *SGRAYS
-    { S_CG,        100,  30, 0,   0, 7, 1024,  "chaingun",        "chaing"},
-    { S_RLFIRE,    800, 120, 80,  0, 10, 1024, "rocketlauncher",  "rocket"},
-    { S_RIFLE,    1500, 100, 0,   0, 30, 2048, "rifle",           "rifle" },
-    { S_FLAUNCH,   500,  75, 80,  0, 10, 1024, "grenadelauncher", "gl" },
-    { S_PISTOL,    500,  25, 0,   0,  7, 1024, "pistol",          "pistol" },
-    { S_FLAUNCH,   200,  20, 50,  PART_FIREBALL1,  1, 1024, "fireball",  NULL },
-    { S_ICEBALL,   200,  40, 30,  PART_FIREBALL2,  1, 1024, "iceball",   NULL },
-    { S_SLIMEBALL, 200,  30, 160, PART_FIREBALL3,  1, 1024, "slimeball", NULL },
-    { S_PIGR1,     250,  50, 0,   0,  1,   12, "bite",            NULL },
-    { -1,            0, 120, 0,   0,  0,    0, "barrel",          NULL }
+    { S_PUNCH1,    250,  50, 0,   0,   0, 0,   14,  "fist",            "fist"  },
+    { S_SG,       1400,  15, 400, 0,   0, 20, 1024, "shotgun",         "shotg" },  // *SGRAYS
+    { S_CG,        100,  20, 100, 0,   0, 7, 1024,  "chaingun",        "chaing"},
+    { S_RLFIRE,    1800, 120, 0,   180,  0, 10, 1024, "rocketlauncher",  "rocket"},
+    { S_RIFLE,    1500, 75, 0,   0,   0, 30, 2048, "rifle",           "rifle" },
+    { S_FLAUNCH,  2000,  175, 0,   300, 0, 10, 25, "grenadelauncher", "gl" },
+    { S_PISTOL,    500,  20, 50,  0,   0,  7, 1024, "pistol",          "pistol" },
+    { S_FLAUNCH,   200,  20, 0,   50,  PART_FIREBALL1,  1, 1024, "fireball",  NULL },
+    { S_ICEBALL,   200,  40, 0,   30,  PART_FIREBALL2,  1, 1024, "iceball",   NULL },
+    { S_SLIMEBALL, 200,  30, 0,   160, PART_FIREBALL3,  1, 1024, "slimeball", NULL },
+    { S_PIGR1,     250,  50, 0,   0,   0,  1,   12, "bite",            NULL },
+    { -1,            0, 120, 0,   0,   0,  0,    0, "barrel",          NULL }
 };
 
+/* Original weapon values cal edit
+{
+    { S_PUNCH1,    250,  50, 0,   0,   0, 0,   14,  "fist",            "fist"  },
+    { S_SG,       1400,  10, 400, 0,   0, 20, 1024, "shotgun",         "shotg" },  // *SGRAYS
+    { S_CG,        100,  30, 100, 0,   0, 7, 1024,  "chaingun",        "chaing"},
+    { S_RLFIRE,    800, 120, 0,   80,  0, 10, 1024, "rocketlauncher",  "rocket"},
+    { S_RIFLE,    1500, 100, 0,   0,   0, 30, 2048, "rifle",           "rifle" },
+    { S_FLAUNCH,   500,  75, 0,   270, 0, 10, 1024, "grenadelauncher", "gl" },
+    { S_PISTOL,    500,  35, 50,  0,   0,  7, 1024, "pistol",          "pistol" },
+    { S_FLAUNCH,   200,  20, 0,   50,  PART_FIREBALL1,  1, 1024, "fireball",  NULL },
+    { S_ICEBALL,   200,  40, 0,   30,  PART_FIREBALL2,  1, 1024, "iceball",   NULL },
+    { S_SLIMEBALL, 200,  30, 0,   160, PART_FIREBALL3,  1, 1024, "slimeball", NULL },
+    { S_PIGR1,     250,  50, 0,   0,   0,  1,   12, "bite",            NULL },
+    { -1,            0, 120, 0,   0,   0,  0,    0, "barrel",          NULL }
+};
+*/
+
 #include "ai.h"
 
 // inherited by fpsent and server clients
@@ -497,7 +544,7 @@ struct fpsstate
             gunselect = GUN_CG;
             ammo[GUN_CG] /= 2;
         }
-        else if(m_ctf)
+        else if(m_ctf || m_collect)
         {
             armourtype = A_BLUE;
             armour = 50;
@@ -535,37 +582,37 @@ struct fpsstate
  */
 
 enum { ATTACH_HEAD = 0, ATTACH_NECK,
-	ATTACH_LEFTSHOULDER, ATTACH_RIGHTSHOULDER,
-	ATTACH_LEFTARM, ATTACH_RIGHTARM,
-	ATTACH_LEFTFOREARM, ATTACH_RIGHTFOREARM,
-	ATTACH_LEFTHAND, ATTACH_RIGHTHAND,
-	ATTACH_BELLY, ATTACH_HIP,
-	ATTACH_LEFTTHIGH, ATTACH_RIGHTTHIGH,
-	ATTACH_LEFTSHIN, ATTACH_RIGHTSHIN,
-	ATTACH_LEFTFOOT, ATTACH_RIGHTFOOT,
-	ATTACH_PARTS
+    ATTACH_LEFTSHOULDER, ATTACH_RIGHTSHOULDER,
+    ATTACH_LEFTARM, ATTACH_RIGHTARM,
+    ATTACH_LEFTFOREARM, ATTACH_RIGHTFOREARM,
+    ATTACH_LEFTHAND, ATTACH_RIGHTHAND,
+    ATTACH_BELLY, ATTACH_HIP,
+    ATTACH_LEFTTHIGH, ATTACH_RIGHTTHIGH,
+    ATTACH_LEFTSHIN, ATTACH_RIGHTSHIN,
+    ATTACH_LEFTFOOT, ATTACH_RIGHTFOOT,
+    ATTACH_PARTS
 };
 
 static const struct attachinfo { const char* part; const char* tag; } body[ATTACH_PARTS] =
 {
-		{ "Head", "tag_head" },
-		{ "Neck", "tag_neck" },
-		{ "LeftShoulder", "tag_lshoulder" },
-		{ "RightShoulder", "tag_rshoulder" },
-		{ "LeftArm", "tag_larm" },
-		{ "RightArm", "tag_rarm" },
-		{ "LeftForearm", "tag_lforearm" },
-		{ "RightForearm", "tag_rforearm" },
-		{ "LeftHand", "tag_lhand" },
-		{ "RightHand", "tag_rhand" },
-		{ "Belly", "tag_belly" },
-		{ "Hip", "tag_hip" },
-		{ "LeftThigh", "tag_lthigh" },
-		{ "RightThigh", "tag_rthigh" },
-		{ "LeftShin", "tag_lshin" },
-		{ "RightShin", "tag_rshin" },
-		{ "LeftFoot", "tag_lfoot" },
-		{ "RightFoot", "tag_rfoot" }
+    { "Head", "tag_head" },
+    { "Neck", "tag_neck" },
+    { "LeftShoulder", "tag_lshoulder" },
+    { "RightShoulder", "tag_rshoulder" },
+    { "LeftArm", "tag_larm" },
+    { "RightArm", "tag_rarm" },
+    { "LeftForearm", "tag_lforearm" },
+    { "RightForearm", "tag_rforearm" },
+    { "LeftHand", "tag_lhand" },
+    { "RightHand", "tag_rhand" },
+    { "Belly", "tag_belly" },
+    { "Hip", "tag_hip" },
+    { "LeftThigh", "tag_lthigh" },
+    { "RightThigh", "tag_rthigh" },
+    { "LeftShin", "tag_lshin" },
+    { "RightShin", "tag_rshin" },
+    { "LeftFoot", "tag_lfoot" },
+    { "RightFoot", "tag_rfoot" }
 };
 
 /*
@@ -589,64 +636,64 @@ enum { ATTACH_ANIM_NONE = 0, ATTACH_ANIM_IDLE, ATTACH_ANIM_FULL, ATTACH_ANIM_NUM
  */
 struct attachlist
 {
-	struct attachment
-	{
-		int part;
-		int animrule;
-		int ao;
-		string model;
-
-		attachment() : part(-1), animrule(ATTACH_ANIM_NONE), ao(-1)
-		{
-			model[0] = 0;
-		}
-	};
-
-	vector<attachment*> list;
-
-	bool hasattachment(int part)
+    struct attachment
     {
-    	loopv(list)
-    	{
-    		if(list[i]->part == part)
-    			return true;
-    	}
-    	return false;
+        int part;
+        int animrule;
+        int ao;
+        string model;
+
+        attachment() : part(-1), animrule(ATTACH_ANIM_NONE), ao(-1)
+        {
+            model[0] = 0;
+        }
+    };
+
+    vector<attachment*> list;
+
+    bool hasattachment(int part)
+    {
+        loopv(list)
+        {
+            if(list[i]->part == part)
+                return true;
+        }
+        return false;
     }
 
-	int numattachments()
-	{
-		return list.length();
-	}
+    int numattachments()
+    {
+        return list.length();
+    }
 
     const char* gettag(int i) {
-    	if(i < ATTACH_PARTS || i >= 0)
-    		return body[i].tag;
-    	else
-    		return NULL;
+        if(i < ATTACH_PARTS || i >= 0)
+            return body[i].tag;
+        else
+            return NULL;
     }
 
-	void add(int part, char* model, int animrule)
-	{
-    	if (part < 0 || part > ATTACH_PARTS) return;
+    void add(int part, char* model, int animrule)
+    {
+        if (part < 0 || part > ATTACH_PARTS) return;
 
-    	if (hasattachment(part)) remove(part);
+        if (hasattachment(part)) remove(part);
 
-    	attachment* a = list.add(new attachment);
-    	if(animrule > ATTACH_ANIM_NONE || animrule <= ATTACH_ANIM_NUMRULES) a->animrule = animrule;
-    	copystring(a->model, model);
-    	a->part = part;
-	}
+        attachment* a = list.add(new attachment);
+        if(animrule > ATTACH_ANIM_NONE || animrule <= ATTACH_ANIM_NUMRULES) a->animrule = animrule;
+        copystring(a->model, model);
+        a->part = part;
+    }
 
     void remove(int part)
     {
-    	loopv(list)
-    	{
-    		if(list[i]->part == part) {
-    			list.remove(i);
-    			return;
-    		}
-    	}
+        loopv(list)
+        {
+            if(list[i]->part == part) {
+                list.remove(i);
+                return;
+            }
+        }
     }
 };
 
@@ -661,10 +708,11 @@ struct fpsent : dynent, fpsstate
     bool attacking;
     int attacksound, attackchan, idlesound, idlechan;
     int lasttaunt;
-    int lastpickup, lastpickupmillis, lastbase, lastrepammo, flagpickup;
+    int lastpickup, lastpickupmillis, lastbase, lastrepammo, flagpickup, tokens;
+    vec lastcollect;
     int frags, flags, deaths, totaldamage, totalshots;
     editinfo *edit;
-    float deltayaw, deltapitch, newyaw, newpitch;
+    float deltayaw, deltapitch, deltaroll, newyaw, newpitch, newroll;
     int smoothmillis;
 
     string name, team, info;
@@ -722,6 +770,8 @@ struct fpsent : dynent, fpsstate
         lastpickupmillis = 0;
         lastbase = lastrepammo = -1;
         flagpickup = 0;
+        tokens = 0;
+        lastcollect = vec(-1e10f, -1e10f, -1e10f);
         stopattacksound();
         lastnode = -1;
     }
@@ -742,6 +792,26 @@ struct teamscore
     }
 };
 
+enum
+{
+    ANIM_HOLD1 = ANIM_GAMESPECIFIC, ANIM_HOLD2, ANIM_HOLD3, ANIM_HOLD4, ANIM_HOLD5, ANIM_HOLD6, ANIM_HOLD7,
+    ANIM_ATTACK1, ANIM_ATTACK2, ANIM_ATTACK3, ANIM_ATTACK4, ANIM_ATTACK5, ANIM_ATTACK6, ANIM_ATTACK7,
+    ANIM_TAUNT, ANIM_WIN, ANIM_LOSE,
+    ANIM_GUN_IDLE, ANIM_GUN_SHOOT,
+    ANIM_VWEP_IDLE, ANIM_VWEP_SHOOT, ANIM_SHIELD, ANIM_POWERUP,
+    NUMANIMS
+};
+
+static const char * const animnames[] =
+{
+    ANIMNAMES,
+    "hold 1", "hold 2", "hold 3", "hold 4", "hold 5", "hold 6", "hold 7",
+    "attack 1", "attack 2", "attack 3", "attack 4", "attack 5", "attack 6", "attack 7",
+    "taunt", "win", "lose",
+    "gun idle", "gun shoot",
+    "vwep idle", "vwep shoot", "shield", "powerup",
+};
+
 namespace entities
 {
     extern vector<extentity *> ents;
@@ -893,16 +963,17 @@ namespace game
     extern void hitmovable(int damage, movable *m, fpsent *at, const vec &vel, int gun);
 
     // weapon
-    /*extern void shoot(fpsent *d, const vec &targ);
+    extern int getweapon(const char *name);
+    extern void shoot(fpsent *d, const vec &targ);
     extern void shoteffects(int gun, const vec &from, const vec &to, fpsent *d, bool local, int id, int prevaction);
     extern void explode(bool local, fpsent *owner, const vec &v, dynent *safe, int dam, int gun);
     extern void explodeeffects(int gun, fpsent *d, bool local, int id = 0);
     extern void damageeffect(int damage, fpsent *d, bool thirdperson = true);
-    extern void gibeffect(int damage, const vec &vel, fpsent *d); */
+    extern void gibeffect(int damage, const vec &vel, fpsent *d);
     extern float intersectdist;
     extern bool intersect(dynent *d, const vec &from, const vec &to, float &dist = intersectdist);
     extern dynent *intersectclosest(const vec &from, const vec &to, fpsent *at, float &dist = intersectdist);
-    /*extern void clearbouncers();
+    extern void clearbouncers();
     extern void updatebouncers(int curtime);
     extern void removebouncers(fpsent *owner);
     extern void renderbouncers();
@@ -915,7 +986,7 @@ namespace game
     extern void updateweapons(int curtime);
     extern void gunselect(int gun, fpsent *d);
     extern void weaponswitch(fpsent *d);
-    extern void avoidweapons(ai::avoidset &obstacles, float radius);*/
+    extern void avoidweapons(ai::avoidset &obstacles, float radius);
 
     // scoreboard
     extern void showscores(bool on);
@@ -945,8 +1016,8 @@ namespace game
 
 namespace server
 {
-	//~ extern bool allowupload; //offtools: server option
-	extern const char *modename(int n, const char *unknown = "unknown");
+    //~ extern bool allowupload; //offtools: server option
+    extern const char *modename(int n, const char *unknown = "unknown");
     extern const char *mastermodename(int n, const char *unknown = "unknown");
     extern void startintermission();
     extern void stopdemo();
