@@ -12,13 +12,13 @@ namespace game
 
     const monstertype monstertypes[NUMMONSTERTYPES] =
     {
-        { GUN_FIST,  4, 100, 3, 0,   100, 800, 50, 0,  5, S_PAINO, S_DIE1,   "A Butterfly",     "butterfly",       "butterfly"},
-        { GUN_BITE,  20, 800, 5, 0,   400, 1600, 100, 60,  4000, S_PAINO, S_DIE1,   "A Dragon",     "rpg/characters/dragon",       "Dragon"},
-        { GUN_FIST,  8, 800, 5, 0,   400, 1600, 100, 16,  4000, S_PAINO, S_DIE1,   "A Golem",     "rpg/characters/golem",       "golem"},
-        { GUN_FIST,  12, 200, 5, 0,   400, 1600, 100, 16,  800, S_PAINO, S_DIE1,   "A Grizzly",     "rpg/characters/grizzly",       "grizzly"},
-        { GUN_BITE,  25, 50, 5, 0,   400, 1600, 100, 2,  30, S_PAINO, S_DIE1,   "A Rat",     "rpg/characters/rat",       "rat"},
-        { GUN_FIST,  7, 150, 5, 0,   400, 1600, 100, 16,  200, S_PAINO, S_DIE1,   "A Snagon",     "rpg/characters/snagon",       "snagon"},
-        { GUN_BITE,  28, 80, 5, 0,   400, 1600, 100, 6,  150, S_PAINO, S_DIE1,   "A Wolf",     "rpg/characters/wolf",       "wolf"},
+        { GUN_FIST,  4, 100, 3, 0,   100, 800, 50, 0,  5, S_PAINO, S_DIE,   "A Butterfly",     "butterfly",       "butterfly"},
+        { GUN_BITE,  20, 800, 5, 0,   400, 1600, 100, 60,  4000, S_PAINO, S_DIE,   "A Dragon",     "rpg/characters/dragon",       "Dragon"},
+        { GUN_FIST,  8, 800, 5, 0,   400, 1600, 100, 16,  4000, S_PAINO, S_DIE,   "A Golem",     "rpg/characters/golem",       "golem"},
+        { GUN_FIST,  12, 200, 5, 0,   400, 1600, 100, 16,  800, S_PAINO, S_DIE,   "A Grizzly",     "rpg/characters/grizzly",       "grizzly"},
+        { GUN_BITE,  25, 50, 5, 0,   400, 1600, 100, 2,  30, S_PAINO, S_DIE,   "A Rat",     "rpg/characters/rat",       "rat"},
+        { GUN_FIST,  7, 150, 5, 0,   400, 1600, 100, 16,  200, S_PAINO, S_DIE,   "A Snagon",     "rpg/characters/snagon",       "snagon"},
+        { GUN_BITE,  28, 80, 5, 0,   400, 1600, 100, 6,  150, S_PAINO, S_DIE,   "A Wolf",     "rpg/characters/wolf",       "wolf"},
     };
 
     VAR(skill, 1, 3, 10);
@@ -166,7 +166,7 @@ namespace game
                     {
                         lastaction = 0;
                         attacking = true;
-                        //shoot(this, attacktarget);
+                        shoot(this, attacktarget);
                         transition(M_ATTACKING, 0, 600, 0);
                     }
                     break;
@@ -238,15 +238,14 @@ namespace game
                 monsterhurt = true;
                 monsterhurtpos = o;
             }
-            //damageeffect(damage, this);
+            damageeffect(damage, this);
             if((health -= damage)<=0)
             {
                 state = CS_DEAD;
                 lastpain = lastmillis;
                 playsound(monstertypes[mtype].diesound, &o);
                 monsterkilled();
-                //superdamage = -health;
-                //superdamageeffect(vel, this);
+                gibeffect(max(-health, 0), vel, this);
 
                 defformatstring(id)("monster_dead_%d", tag);
                 if(identexists(id)) execute(id);
