@@ -20,7 +20,7 @@ namespace game
             else return false;
         }
         else if(b->state==CS_SPECTATOR) return true;
-        if(m_ctf)
+        if(m_ctf || m_collect)
         {
             if(a->flags > b->flags) return true;
             if(a->flags < b->flags) return false;
@@ -120,7 +120,7 @@ namespace game
         return numgroups;
     }
 
-#ifndef NEWGUI
+    #if 0
     void renderscoreboard(g3d_gui &g, bool firstpass)
     {
         const ENetAddress *address = connectedpeer();
@@ -134,7 +134,8 @@ namespace game
             }
         }
 
-        g.pushlist(0);
+        g.pushlist();
+        g.spring();
         g.text(server::modename(gamemode), 0xFFFF80);
         g.separator();
         const char *mname = getclientmap();
@@ -143,7 +144,7 @@ namespace game
         {
             g.separator();
             if(intermission) g.text("intermission", 0xFFFF80);
-            else 
+            else
             {
                 int secs = max(maplimit-lastmillis, 0)/1000, mins = secs/60;
                 secs %= 60;
@@ -154,6 +155,7 @@ namespace game
             }
         }
         if(paused || ispaused()) { g.separator(); g.text("paused", 0xFFFF80); }
+        g.spring();
         g.poplist();
 
         g.separator();
@@ -384,14 +386,11 @@ namespace game
 
     VARFN(scoreboard, showscoreboard, 0, 0, 1, scoreboard.show(showscoreboard!=0));
 
-	#endif
+    #endif
     void showscores(bool on)
     {
-    	#ifndef NEWGUI
-        showscoreboard = on ? 1 : 0;
-        scoreboard.show(on);
-        #endif
+		if(on) execute("showscores");
+		else execute("hideui scores");
     }
-    ICOMMAND(showscores, "D", (int *down), showscores(*down!=0))
 }
 
