@@ -172,6 +172,8 @@ void PASL::Init()
 void PASL::CreateControls()
 {
 ////@begin PASL content construction
+    // Generated by DialogBlocks, Fri 15 Jan 2010 16:18:25 EST (unregistered)
+
     PASL* itemFrame1 = this;
 
     wxPanel* itemPanel2 = new wxPanel( itemFrame1, ID_PANEL, wxDefaultPosition, wxDefaultSize, wxSUNKEN_BORDER|wxTAB_TRAVERSAL );
@@ -192,7 +194,7 @@ void PASL::CreateControls()
     ResBoxStrings.Add(_("1024x768"));
     ResBoxStrings.Add(_("1280x960"));
     ResBoxStrings.Add(_("1600x1200"));
-    ResBox = new wxComboBox( itemPanel2, ID_COMBOBOX, _("1024x768"), wxDefaultPosition, wxDefaultSize, ResBoxStrings, wxCB_READONLY );
+    ResBox = new wxComboBox( itemPanel2, ID_COMBOBOX, _("1024x768"), wxDefaultPosition, wxSize(110, -1), ResBoxStrings, wxCB_READONLY );
     ResBox->SetStringSelection(_("1024x768"));
     itemStaticBoxSizer5->Add(ResBox, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
 
@@ -204,7 +206,7 @@ void PASL::CreateControls()
     ShaderBoxStrings.Add(_("Off"));
     ShaderBoxStrings.Add(_("Low"));
     ShaderBoxStrings.Add(_("High"));
-    ShaderBox = new wxComboBox( itemPanel2, ID_COMBOBOX1, _("Low"), wxDefaultPosition, wxDefaultSize, ShaderBoxStrings, wxCB_READONLY );
+    ShaderBox = new wxComboBox( itemPanel2, ID_COMBOBOX1, _("Low"), wxDefaultPosition, wxSize(70, -1), ShaderBoxStrings, wxCB_READONLY );
     ShaderBox->SetStringSelection(_("Low"));
     itemStaticBoxSizer7->Add(ShaderBox, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
 
@@ -215,7 +217,7 @@ void PASL::CreateControls()
     wxArrayString VSyncBoxStrings;
     VSyncBoxStrings.Add(_("Off"));
     VSyncBoxStrings.Add(_("On"));
-    VSyncBox = new wxComboBox( itemPanel2, ID_COMBOBOX2, _("Off"), wxDefaultPosition, wxDefaultSize, VSyncBoxStrings, wxCB_READONLY );
+    VSyncBox = new wxComboBox( itemPanel2, ID_COMBOBOX2, _("Off"), wxDefaultPosition, wxSize(60, -1), VSyncBoxStrings, wxCB_READONLY );
     VSyncBox->SetStringSelection(_("Off"));
     itemStaticBoxSizer9->Add(VSyncBox, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
 
@@ -227,7 +229,7 @@ void PASL::CreateControls()
     itemFlexGridSizer11->Add(itemStaticBoxSizer12, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 5);
 
     wxArrayString MapBoxStrings;
-    MapBox = new wxComboBox( itemPanel2, ID_COMBOBOX6, wxEmptyString, wxDefaultPosition, wxDefaultSize, MapBoxStrings, wxCB_READONLY );
+    MapBox = new wxComboBox( itemPanel2, ID_COMBOBOX6, wxEmptyString, wxDefaultPosition, wxSize(180, -1), MapBoxStrings, wxCB_READONLY );
     itemStaticBoxSizer12->Add(MapBox, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
 
     wxStaticBox* itemStaticBoxSizer14Static = new wxStaticBox(itemPanel2, wxID_ANY, _("Map Image"));
@@ -240,7 +242,7 @@ void PASL::CreateControls()
     wxFlexGridSizer* itemFlexGridSizer16 = new wxFlexGridSizer(1, 1, 0, 0);
     itemFlexGridSizer3->Add(itemFlexGridSizer16, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 5);
 
-    wxNotebook* itemNotebook17 = new wxNotebook( itemPanel2, ID_NOTEBOOK, wxDefaultPosition, wxDefaultSize, wxBK_DEFAULT );
+    wxNotebook* itemNotebook17 = new wxNotebook( itemPanel2, ID_NOTEBOOK, wxDefaultPosition, wxSize(320, -1), wxBK_DEFAULT );
 
     wxPanel* itemPanel18 = new wxPanel( itemNotebook17, ID_PANEL1, wxDefaultPosition, wxDefaultSize, wxSUNKEN_BORDER|wxTAB_TRAVERSAL );
     itemPanel18->SetBackgroundColour(wxColour(224, 223, 227));
@@ -285,7 +287,7 @@ void PASL::CreateControls()
     wxStaticBox* itemStaticBoxSizer30Static = new wxStaticBox(itemPanel27, wxID_ANY, _("Server Name"));
     wxStaticBoxSizer* itemStaticBoxSizer30 = new wxStaticBoxSizer(itemStaticBoxSizer30Static, wxHORIZONTAL);
     itemFlexGridSizer29->Add(itemStaticBoxSizer30, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 5);
-    ServerName = new wxTextCtrl( itemPanel27, ID_TEXTCTRL, _("Sandbox Server"), wxDefaultPosition, wxDefaultSize, 0 );
+    ServerName = new wxTextCtrl( itemPanel27, ID_TEXTCTRL, _("Sandbox Server"), wxDefaultPosition, wxSize(100, -1), 0 );
     itemStaticBoxSizer30->Add(ServerName, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
 
     wxStaticBox* itemStaticBoxSizer32Static = new wxStaticBox(itemPanel27, wxID_ANY, _("Players"));
@@ -325,7 +327,7 @@ void PASL::CreateControls()
     PlayerBoxStrings.Add(_("30"));
     PlayerBoxStrings.Add(_("31"));
     PlayerBoxStrings.Add(_("32"));
-    PlayerBox = new wxComboBox( itemPanel27, ID_COMBOBOX3, _("12"), wxDefaultPosition, wxDefaultSize, PlayerBoxStrings, wxCB_READONLY );
+    PlayerBox = new wxComboBox( itemPanel27, ID_COMBOBOX3, _("12"), wxDefaultPosition, wxSize(60, -1), PlayerBoxStrings, wxCB_READONLY );
     PlayerBox->SetStringSelection(_("12"));
     itemStaticBoxSizer32->Add(PlayerBox, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
 
@@ -335,7 +337,7 @@ void PASL::CreateControls()
     wxArrayString MasterBoxStrings;
     MasterBoxStrings.Add(_("N"));
     MasterBoxStrings.Add(_("Y"));
-    MasterBox = new wxComboBox( itemPanel27, ID_COMBOBOX4, _("N"), wxDefaultPosition, wxDefaultSize, MasterBoxStrings, wxCB_READONLY );
+    MasterBox = new wxComboBox( itemPanel27, ID_COMBOBOX4, _("N"), wxDefaultPosition, wxSize(50, -1), MasterBoxStrings, wxCB_READONLY );
     MasterBox->SetStringSelection(_("N"));
     itemStaticBoxSizer34->Add(MasterBox, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
 
@@ -344,7 +346,7 @@ void PASL::CreateControls()
     wxStaticBox* itemStaticBoxSizer37Static = new wxStaticBox(itemPanel27, wxID_ANY, _("Server Password"));
     wxStaticBoxSizer* itemStaticBoxSizer37 = new wxStaticBoxSizer(itemStaticBoxSizer37Static, wxHORIZONTAL);
     itemFlexGridSizer36->Add(itemStaticBoxSizer37, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 5);
-    ServerPassword = new wxTextCtrl( itemPanel27, ID_TEXTCTRL1, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0 );
+    ServerPassword = new wxTextCtrl( itemPanel27, ID_TEXTCTRL1, wxEmptyString, wxDefaultPosition, wxSize(100, -1), 0 );
     itemStaticBoxSizer37->Add(ServerPassword, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
 
     wxStaticBox* itemStaticBoxSizer39Static = new wxStaticBox(itemPanel27, wxID_ANY, _("Dedicated"));
@@ -353,7 +355,7 @@ void PASL::CreateControls()
     wxArrayString DedBoxStrings;
     DedBoxStrings.Add(_("N"));
     DedBoxStrings.Add(_("Y"));
-    DedBox = new wxComboBox( itemPanel27, ID_COMBOBOX5, _("N"), wxDefaultPosition, wxDefaultSize, DedBoxStrings, wxCB_READONLY );
+    DedBox = new wxComboBox( itemPanel27, ID_COMBOBOX5, _("N"), wxDefaultPosition, wxSize(60, -1), DedBoxStrings, wxCB_READONLY );
     DedBox->SetStringSelection(_("N"));
     itemStaticBoxSizer39->Add(DedBox, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
 
@@ -364,7 +366,7 @@ void PASL::CreateControls()
     ModeBoxStrings.Add(_("FPS"));
     ModeBoxStrings.Add(_("SSP"));
     ModeBoxStrings.Add(_("RPG"));
-    ModeBox = new wxComboBox( itemPanel27, ID_COMBOBOX7, _("FPS"), wxDefaultPosition, wxDefaultSize, ModeBoxStrings, wxCB_READONLY );
+    ModeBox = new wxComboBox( itemPanel27, ID_COMBOBOX7, _("FPS"), wxDefaultPosition, wxSize(65, -1), ModeBoxStrings, wxCB_READONLY );
     ModeBox->SetStringSelection(_("FPS"));
     itemStaticBoxSizer41->Add(ModeBox, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
 
@@ -390,13 +392,13 @@ void PASL::CreateControls()
     wxStaticBox* itemStaticBoxSizer50Static = new wxStaticBox(itemPanel47, wxID_ANY, _("IP Address"));
     wxStaticBoxSizer* itemStaticBoxSizer50 = new wxStaticBoxSizer(itemStaticBoxSizer50Static, wxHORIZONTAL);
     itemFlexGridSizer49->Add(itemStaticBoxSizer50, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 5);
-    ClientIP = new wxTextCtrl( itemPanel47, ID_TEXTCTRL2, _("192.168.1.100"), wxDefaultPosition, wxDefaultSize, 0 );
+    ClientIP = new wxTextCtrl( itemPanel47, ID_TEXTCTRL2, _("192.168.1.100"), wxDefaultPosition, wxSize(120, -1), 0 );
     itemStaticBoxSizer50->Add(ClientIP, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
 
     wxStaticBox* itemStaticBoxSizer52Static = new wxStaticBox(itemPanel47, wxID_ANY, _("Password"));
     wxStaticBoxSizer* itemStaticBoxSizer52 = new wxStaticBoxSizer(itemStaticBoxSizer52Static, wxHORIZONTAL);
     itemFlexGridSizer49->Add(itemStaticBoxSizer52, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 5);
-    ClientPassword = new wxTextCtrl( itemPanel47, ID_TEXTCTRL3, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0 );
+    ClientPassword = new wxTextCtrl( itemPanel47, ID_TEXTCTRL3, wxEmptyString, wxDefaultPosition, wxSize(100, -1), 0 );
     itemStaticBoxSizer52->Add(ClientPassword, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
 
     wxFlexGridSizer* itemFlexGridSizer54 = new wxFlexGridSizer(1, 2, 0, 0);
@@ -408,7 +410,7 @@ void PASL::CreateControls()
     CModeBoxStrings.Add(_("FPS"));
     CModeBoxStrings.Add(_("SSP"));
     CModeBoxStrings.Add(_("RPG"));
-    CModeBox = new wxComboBox( itemPanel47, ID_COMBOBOX8, _("FPS"), wxDefaultPosition, wxDefaultSize, CModeBoxStrings, wxCB_DROPDOWN );
+    CModeBox = new wxComboBox( itemPanel47, ID_COMBOBOX8, _("FPS"), wxDefaultPosition, wxSize(70, -1), CModeBoxStrings, wxCB_DROPDOWN );
     CModeBox->SetStringSelection(_("FPS"));
     itemStaticBoxSizer55->Add(CModeBox, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
 
