@@ -977,7 +977,7 @@ void CommandProxy::addcmd(char **args, int *numargs)
 	}
 	buf.add('\0');
 	commands.add(newstring(buf.getbuf()));
-	ret = executeret(commands.last());
+	ret = executestr(commands.last());
 	if (ret) {
 		result(ret);
 	}
