@@ -14,14 +14,15 @@
 struct ActionLib;
 struct ActionInstance;
 struct Control;
+struct ControlCharacter;
 
 //----------------Action Types-----------------
 struct GenericAction
 {
     GenericAction();
     virtual ~GenericAction() = 0;
-	virtual void process(Control& target) = 0;
-    void finished(Control& target);
+	virtual void process(ControlCharacter& target) = 0;
+    void finished(ControlCharacter& target);
 };
 
 struct ActionSpawn : GenericAction
@@ -30,7 +31,7 @@ struct ActionSpawn : GenericAction
 
     ActionSpawn(int uid);
     ~ActionSpawn();
-	void process(Control& target);
+	void process(ControlCharacter& target);
 };
 
 struct ActionWait : GenericAction
@@ -39,7 +40,7 @@ struct ActionWait : GenericAction
 
 	ActionWait(int _millis);
 	~ActionWait();
-	void process(Control& target);
+	void process(ControlCharacter& target);
 };
 
 struct ActionMove : GenericAction
@@ -48,7 +49,7 @@ struct ActionMove : GenericAction
 
 	ActionMove(int uid);
 	~ActionMove();
-	void process(Control& target);
+	void process(ControlCharacter& target);
 };
 
 //added by Protagoras
@@ -58,7 +59,7 @@ struct ActionJump : GenericAction
 
 	ActionJump();
 	~ActionJump();
-	void process(Control& target);
+	void process(ControlCharacter& target);
 };
 
 //added by Protagoras
@@ -68,7 +69,7 @@ struct ActionFace : GenericAction
 
 	ActionFace(int _cuid);
 	~ActionFace();
-	void process(Control& target);
+	void process(ControlCharacter& target);
 };
 
 //added by Protagoras
@@ -78,7 +79,7 @@ struct ActionExpect : GenericAction
 
 	ActionExpect(int _cuid, int _radius);
 	~ActionExpect();
-	void process(Control& target);
+	void process(ControlCharacter& target);
 };
 
 //added by Protagoras
@@ -88,7 +89,7 @@ struct ActionFollow : GenericAction
 
 	ActionFollow(int _cuid, int uid);
 	~ActionFollow();
-	void process(Control& target);
+	void process(ControlCharacter& target);
 };
 
 struct ActionSpeed : GenericAction
@@ -97,14 +98,14 @@ struct ActionSpeed : GenericAction
 
 	ActionSpeed(int _speed);
 	~ActionSpeed();
-	void process(Control& target);
+	void process(ControlCharacter& target);
 };
 
 struct ActionDestroy : GenericAction
 {
 	ActionDestroy();
 	~ActionDestroy();
-	void process(Control& target);
+	void process(ControlCharacter& target);
 };
 
 struct ActionModel : GenericAction
@@ -113,7 +114,7 @@ struct ActionModel : GenericAction
 
 	ActionModel(int model);
 	~ActionModel();
-	void process(Control& target);
+	void process(ControlCharacter& target);
 };
 
 struct ActionSetAO : GenericAction
@@ -123,7 +124,7 @@ struct ActionSetAO : GenericAction
 
 	ActionSetAO(int _ao, int _anim);
 	~ActionSetAO();
-	void process(Control& target);
+	void process(ControlCharacter& target);
 };
 
 struct ActionSetGesture : GenericAction
@@ -132,7 +133,7 @@ struct ActionSetGesture : GenericAction
 
 	ActionSetGesture(int anim);
 	~ActionSetGesture();
-	void process(Control& target);
+	void process(ControlCharacter& target);
 };
 
 struct ActionDoGesture : GenericAction
@@ -141,7 +142,7 @@ struct ActionDoGesture : GenericAction
 
 	ActionDoGesture(int millis);
 	~ActionDoGesture();
-	void process(Control& target);
+	void process(ControlCharacter& target);
 };
 
 struct ActionAttach : GenericAction
@@ -152,7 +153,7 @@ struct ActionAttach : GenericAction
 
 	ActionAttach(int part, char* model, int rule);
 	~ActionAttach();
-	void process(Control& target);
+	void process(ControlCharacter& target);
 };
 
 struct ActionDetach : GenericAction
@@ -161,7 +162,7 @@ struct ActionDetach : GenericAction
 
 	ActionDetach(int part);
 	~ActionDetach();
-	void process(Control& target);
+	void process(ControlCharacter& target);
 };
 
 struct ActionCommand : GenericAction
@@ -170,7 +171,7 @@ struct ActionCommand : GenericAction
 
 	ActionCommand(const char* cmd);
 	~ActionCommand();
-	void process(Control& target);
+	void process(ControlCharacter& target);
 };
 
 struct ActionCond : GenericAction
@@ -180,7 +181,7 @@ struct ActionCond : GenericAction
 
 	ActionCond(const char* test, const char* body);
 	~ActionCond();
-	void process(Control& target);
+	void process(ControlCharacter& target);
 };
 
 struct ActionCondWait : GenericAction
@@ -189,7 +190,7 @@ struct ActionCondWait : GenericAction
 
 	ActionCondWait(const char* test);
 	~ActionCondWait();
-	void process(Control& target);
+	void process(ControlCharacter& target);
 };
 //-------------Action Managment----------------
 struct ActionState
@@ -314,11 +315,11 @@ public:
 
 	void next();
 
-	void process(Control& target);
+	void process(ControlCharacter& target);
 
-	void nextstep(Control& target);
+	void nextstep(ControlCharacter& target);
 
-	void laststep(Control& target);
+	void laststep(ControlCharacter& target);
 };
 
 void clearactions();
