@@ -43,6 +43,11 @@ namespace game
         {
             CharacterInfo& charinfo = player1->getcharinfo();
             charinfo.setplayermodel(i);
+            // Protagoras: bounding box support
+            player1->o.z -= player1->eyeheight;
+            setbbfrommodel(player1, getplayermodelinfo(player1).name);
+            player1->o.z += player1->eyeheight;
+            player1->resetinterp();
             addmsg(SV_SWITCHMODEL, "rci", player1, charinfo.getplayermodel());
         }
     }
@@ -150,8 +155,8 @@ namespace game
         //disconnect all controls
         loopv(controls)
         {
-            ControlCharacter* ca = dynamic_cast<ControlCharacter*>(controls[i]);
-            if(ca) { ca->disconnect(); }
+            ControlCharacter* ca = (ControlCharacter*)controls[i];
+            if(ca->type() == CONTROL_CHARACTER) { ca->disconnect(); }
         }
 
         addmsg(SV_EDITMODE, "ri", on ? 1 : 0);
@@ -988,6 +993,11 @@ namespace game
                 {
                 	CharacterInfo& charinfo = d->getcharinfo();
                 	charinfo.setplayermodel(model);
+                	// Protagoras: bounding box support
+                	d->o.z -= d->eyeheight;
+                	setbbfrommodel(d, getplayermodelinfo(d).name);
+                 	d->o.z += d->eyeheight;
+                 	d->resetinterp();
 //                	conoutf("DEBUG game::parsemessages SV_SWITCHMODEL cn %d model %d", d->clientnum, model);
                 }
                 break;
@@ -1015,6 +1025,7 @@ namespace game
                 if(d)
                 {
                     if(d->state==CS_DEAD) saveragdoll(d);
+                    setbbfrommodel(d, getplayermodelinfo(d).name); // Protagoras: bounding box support
                     d->respawn();
                 }
                 parsestate(d, p);
@@ -1364,8 +1375,8 @@ namespace game
                 	//find uninitialized control
                 	loopv(controls)
                 	{
-                	    ControlCharacter* ca = dynamic_cast<ControlCharacter*>(controls[i]);
-                		if(ca && !controls[i]->getstate())
+                	    ControlCharacter* ca = (ControlCharacter*)game::controls[i];
+			    if(ca->type() == CONTROL_CHARACTER && !controls[i]->getstate())
                 		{
 //                            conoutf("DEBUG: game::parsemessages SV_INITBOT init ControlCharacter");
                         	ca->init(b);
