@@ -27,7 +27,7 @@ struct Control
 	virtual ~Control() {}
 
     //check type of an control
-	static bool checktype(Control* c) { return false; }
+    virtual int type()=0;
 
     //init client, has dynent (client) now
 	virtual void init(DynamicEntity* de) = 0;
@@ -58,7 +58,7 @@ struct ControlPlayer : Control
 
 	~ControlPlayer() {}
 
-	static bool checktype(Control* c) { return dynamic_cast<ControlPlayer*>(c) ? true : false; }
+	int type() {return CONTROL_PLAYER;}
 	void init(DynamicEntity* de);
 	void spawn();
 	void update();
@@ -74,7 +74,7 @@ struct ControlRemote : Control
 
 	~ControlRemote();
 
-	static bool checktype(Control* c) { return dynamic_cast<ControlRemote*>(c) ? true : false; }
+	int type() {return CONTROL_REMOTE;}
 	void init(DynamicEntity* de);
 	void spawn();
 	void update();
@@ -90,7 +90,7 @@ struct ControlAi : Control
 
 	~ControlAi() {}
 
-	static bool checktype(Control* c) { return dynamic_cast<ControlAi*>(c) ? true : false; }
+	int type() {return CONTROL_AI;}
 	void init(DynamicEntity* de);
 	void spawn();
 	void update();
@@ -107,7 +107,7 @@ struct ControlCharacter : Control, ActionInstance
 
 	~ControlCharacter();
 
-	static bool checktype(Control* c) { return dynamic_cast<ControlCharacter*>(c) ? true : false; }
+	int type() {return CONTROL_CHARACTER;}
 	void init(DynamicEntity* de);
 	void spawn();
 	void update();
