@@ -85,41 +85,41 @@ static const char * const mastermodenames[] = { "auth", "open", "veto", "locked"
 // hardcoded sounds, defined in sounds.cfg
 enum
 {
-    S_JUMP = 0, S_LAND, S_RIFLE, S_TELEPORT, S_SPLASH1, S_SPLASH2, S_CG,
-    S_RLFIRE, S_RUMBLE, S_JUMPPAD, S_WEAPLOAD, S_ITEMAMMO, S_ITEMHEALTH,
-    S_ITEMARMOUR, S_ITEMPUP, S_ITEMSPAWN,  S_NOAMMO, S_PUPOUT,
-    S_PAIN1, S_PAIN2, S_PAIN3, S_PAIN4, S_PAIN5, S_PAIN6,
-    S_DIE1, S_DIE2,
-    S_FLAUNCH, S_FEXPLODE,
-    S_SG, S_PUNCH1,
-    S_GRUNT1, S_GRUNT2, S_RLHIT,
-    S_PAINO,
-    S_PAINR, S_DEATHR,
-    S_PAINE, S_DEATHE,
-    S_PAINS, S_DEATHS,
-    S_PAINB, S_DEATHB,
-    S_PAINP, S_PIGGR2,
-    S_PAINH, S_DEATHH,
-    S_PAIND, S_DEATHD,
-    S_PIGR1, S_ICEBALL, S_SLIMEBALL, S_PISTOL,
+	S_JUMP = 0, S_LAND, S_RIFLE, S_TELEPORT, S_SPLASH1, S_SPLASH2, S_CG,
+	S_RLFIRE, S_RUMBLE, S_JUMPPAD, S_WEAPLOAD, S_ITEMAMMO, S_ITEMHEALTH,
+	S_ITEMARMOUR, S_ITEMPUP, S_ITEMSPAWN,  S_NOAMMO, S_PUPOUT,
+	S_PAIN,
+	S_DIE,
+	S_FLAUNCH, S_FEXPLODE,
+	S_SG, S_PUNCH1,
+	S_GRUNT1, S_GRUNT2, S_RLHIT,
+	S_PAINO,
+	S_PAINR, S_DEATHR,
+	S_PAINE, S_DEATHE,
+	S_PAINS, S_DEATHS,
+	S_PAINB, S_DEATHB,
+	S_PAINP, S_PIGGR2,
+	S_PAINH, S_DEATHH,
+	S_PAIND, S_DEATHD,
+	S_PIGR1, S_ICEBALL, S_SLIMEBALL, S_PISTOL,
 
-    S_V_BASECAP, S_V_BASELOST,
-    S_V_FIGHT,
-    S_V_BOOST, S_V_BOOST10,
-    S_V_QUAD, S_V_QUAD10,
-    S_V_RESPAWNPOINT,
+	S_V_BASECAP, S_V_BASELOST,
+	S_V_FIGHT,
+	S_V_BOOST, S_V_BOOST10,
+	S_V_QUAD, S_V_QUAD10,
+	S_V_RESPAWNPOINT,
 
-    S_FLAGPICKUP,
-    S_FLAGDROP,
-    S_FLAGRETURN,
-    S_FLAGSCORE,
-    S_FLAGRESET,
+	S_FLAGPICKUP,
+	S_FLAGDROP,
+	S_FLAGRETURN,
+	S_FLAGSCORE,
+	S_FLAGRESET,
 
-    S_BURN,
-    S_CHAINSAW_ATTACK,
-    S_CHAINSAW_IDLE,
+	S_BURN,
+	S_CHAINSAW_ATTACK,
+	S_CHAINSAW_IDLE,
 
-    S_HIT
+	S_HIT
 };
 
 // network messages codes, c2s, c2c, s2c
@@ -594,6 +594,28 @@ namespace CutScene
 #include "clientmode.h"
 #include "control.h"
 
+enum
+{
+	ANIM_HOLD1 = ANIM_GAMESPECIFIC, ANIM_HOLD2, ANIM_HOLD3, ANIM_HOLD4, ANIM_HOLD5, ANIM_HOLD6, ANIM_HOLD7,
+	ANIM_ATTACK1, ANIM_ATTACK2, ANIM_ATTACK3, ANIM_ATTACK4, ANIM_ATTACK5, ANIM_ATTACK6, ANIM_ATTACK7,
+	ANIM_TAUNT, ANIM_WIN, ANIM_LOSE,
+	ANIM_GUN_IDLE, ANIM_GUN_SHOOT,
+	ANIM_VWEP_IDLE, ANIM_VWEP_SHOOT, ANIM_SHIELD, ANIM_POWERUP,
+	ANIM_WAVE, ANIM_BOW,
+	NUMANIMS
+};
+
+static const char * const animnames[] =
+{
+	ANIMNAMES,
+	"hold 1", "hold 2", "hold 3", "hold 4", "hold 5", "hold 6", "hold 7",
+	"attack 1", "attack 2", "attack 3", "attack 4", "attack 5", "attack 6", "attack 7",
+	"taunt", "win", "lose",
+	"gun idle", "gun shoot",
+	"vwep idle", "vwep shoot", "shield", "powerup",
+	"wave", "bow"
+};
+
 namespace game
 {
     extern clientmode *cmode;
