@@ -8,7 +8,7 @@
 
 #include "cube.h"
 
-#define ENTSPAWNTIME 30000
+#define ENTSPAWNTIME 30000 // 30 seconds
 
 enum
 {
@@ -21,14 +21,11 @@ enum
 	MAPSOUND    = ET_SOUND,
 	SPOTLIGHT   = ET_SPOTLIGHT,
 
-
-	PLAYERFINISH,
 	MAXENTTYPES
 };
 
 struct pzlentity : extentity{};
-
-struct pzlchar : dynent
+struct pzlchar   : dynent
 {
 	const char *name;
 	int playerindex;
@@ -50,6 +47,37 @@ struct pzlchar : dynent
 	}
 };
 
+enum pzlportaltype
+{
+    PORTAL_BLUE,   // in
+    PORTAL_YELLOW  // out
+};
+
+struct pzlportal
+{
+    pzlportal() :
+        dist(1024),
+        size(0030),
+        dir (0, 0, 0),
+        pos (0, 0, 0),
+        o   (0, 0, 0)
+    {}
+
+    void calc(pzlchar *d)
+    {
+        // calculate placement for decal
+        dir = vec(d->yaw * RAD, d->pitch * RAD);
+        o   = d->o;
+        raycubepos(o, vec(dir).normalize(), pos, dist, RAY_CLIPMAT|RAY_ALPHAPOLY);
+    }
+
+    float dist; // max distance for portal
+    float size; // portal size
+    vec   dir;  // direction
+    vec   pos;  // position
+    vec   o;    // start point
+};
+
 namespace entities
 {
 	extern vector<extentity *> ents;
