@@ -3,28 +3,24 @@
 namespace game
 {
 	VARP(debug, 0, 0, DEBUG_MAX);
+	VAR(forceverbose, 1, 0, -1);
 	rpgchar *player1 = new rpgchar();
 
 	// GAME DEFINITIONS
 	vector<script *> scripts; ///scripts, includes dialogue
 	vector<effect *> effects; ///pretty particle effects for spells and stuff
 	vector<statusgroup *> statuses; ///status effect definitions to transfer onto victims
-	vector<item_base *> items;
-	vector<ammotype *> ammotypes;
-	vector<char_base *> chars;
 	vector<faction *> factions;
-	vector<obstacle_base *> obstacles;
-	vector<container_base *> containers;
-	vector<platform_base *> platforms;
-	vector<trigger_base *> triggers;
+	vector<ammotype *> ammotypes;
 	vector<mapscript *> mapscripts;
 	vector<recipe *> recipes;
+	vector<merchant *> merchants;
 
-	vector<int> variables; //global variables
-	vector<const char *> tips;
+	vector<const char *> variables, categories, tips;
+	hashset<journal> journals;
 
 	string data; //data folder
-	hashtable<const char *, mapinfo> *mapdata = NULL;
+	hashset<mapinfo> *mapdata = NULL;
 	mapinfo *curmap = NULL;
 	bool connected = false;
 	bool transfer = false;
@@ -45,50 +41,56 @@ namespace game
 		player1->secondary = *down != 0 && ! player1->primary;
 	)
 
-	vector<equipment> hotkeys;
-	equipment pending(-1, -1);
-
+//TODO redo hotkeys
+// 	vector<equipment> hotkeys;
+// 	equipment pending(-1, -1);
+//
 	ICOMMAND(hotkey, "i", (int *n),
 		if(*n < 0) return;
 
-		if(rpggui::open())
-			rpggui::hotkey(*n);
-		else if(pending.base >= 0)
-		{
-			while(!hotkeys.inrange(*n))
-				hotkeys.add(equipment(-1, -1));
-
-			hotkeys[*n] = pending;
-			pending = equipment(-1, -1);
-		}
-		else if(hotkeys.inrange(*n) && hotkeys[*n].base >= 0 && hotkeys[*n].use >= 0)
-		{
-			int i = player1->getitemcount(hotkeys[*n].base);
-			if(!i) return;
-			if(!player1->dequip(hotkeys[*n].base, -1))
-				player1->equip(hotkeys[*n].base, hotkeys[*n].use);
-		}
-	)
-
-	ICOMMAND(hotkey_get, "i", (int *idx),
-		if(!hotkeys.inrange(*idx))
-		{
-			result("-1 -1");
+		if(rpggui::open() && rpggui::hotkey(*n))
 			return;
-		}
-		defformatstring(ret)("%i %i", hotkeys[*idx].base, hotkeys[*idx].use);
-		result(ret);
-	)
-
-	ICOMMAND(hotkey_set_pending, "ii", (int *it, int *u),
-		pending.base = *it;
-		pending.use = *u;
-	)
-
-	ICOMMAND(hotkey_clear, "i", (int *idx),
-		if(hotkeys.inrange(*idx))
-			hotkeys[*idx] = equipment(-1, -1);
+// 		else if(pending.base >= 0)
+// 		{
+// 			while(!hotkeys.inrange(*n))
+// 				hotkeys.add(equipment(-1, -1));
+//
+// 			hotkeys[*n] = pending;
+// 			pending = equipment(-1, -1);
+// 		}
+// 		else if(!rpggui::open() && hotkeys.inrange(*n) && hotkeys[*n].base >= 0 && hotkeys[*n].use >= 0) //only allow with GUIs closed
+// 		{
+// 			int i = player1->getitemcount(hotkeys[*n].base);
+// 			if(!i) return;
+// 			if(!player1->dequip(hotkeys[*n].base, -1))
+// 				player1->equip(hotkeys[*n].base, hotkeys[*n].use);
+// 		}
 	)
+//
+// 	ICOMMAND(hotkey_get, "i", (int *idx),
+// 		if(!hotkeys.inrange(*idx))
+// 		{
+// 			result("-1 -1");
+// 			return;
+// 		}
+// 		defformatstring(ret)("%i %i", hotkeys[*idx].base, hotkeys[*idx].use);
+// 		result(ret);
+// 	)
+//
+// 	ICOMMAND(hotkey_set_pending, "ii", (int *it, int *u),
+// 		pending.base = *it;
+// 		pending.use = *u;
+// 	)
+//
+// 	ICOMMAND(hotkey_get_pending, "ii", (int *it, int *u),
+// 		defformatstring(str)("%i %i", pending.base, pending.use);
+// 		result(str);
+// 	)
+//
+// 	ICOMMAND(hotkey_clear, "i", (int *idx),
+// 		if(hotkeys.inrange(*idx))
+// 			hotkeys[*idx] = equipment(-1, -1);
+// 	)
 
 	ICOMMAND(include, "s", (const char *pth),
 		static string file;
@@ -102,9 +104,12 @@ namespace game
 		vec dir;
 
 		collision(rpgent *de, rpgent *oe, const vec &ddir) : d(de), o(oe), dir(ddir) {}
+		collision(rpgent *de, rpgent *oe) : d(de), o(oe) {}
+
 		static void add(physent *d, physent *o, const vec &dir);
+		static void add(physent *d, physent *o);
 	};
-	vector<collision> colcache;
+	vector<collision> colcache, mergecache;
 
 	void collision::add(physent *d, physent *o, const vec &dir)
 	{
@@ -115,12 +120,27 @@ namespace game
 		}
 
 		colcache.add(collision((rpgent *) d, (rpgent *) o, dir));
-		return;
+	}
+
+	void collision::add(physent *d, physent *o)
+	{
+		loopv(mergecache)
+		{
+			if(mergecache[i].d == d || mergecache[i].o == d || mergecache[i].d == o || mergecache[i].o == o)
+				return;
+		}
+		mergecache.add(collision((rpgent *) d, (rpgent *) o));
 	}
 
 	void dynentcollide(physent *d, physent *o, const vec &dir)
 	{
-		collision::add(d, o, dir);
+		rpgent *a = (rpgent *) d,
+			*b = (rpgent *) o;
+
+		collision::add(a, b, dir);
+		if(a->type() == ENT_ITEM && b->type() == ENT_ITEM)
+			collision::add(a, b);
+
 	}
 
 	void cleangame()
@@ -129,11 +149,12 @@ namespace game
 
 		rpgscript::clean();
 		colcache.setsize(0);
+		mergecache.setsize(0);
 
 		if(mapdata)
 		{
 			if(DEBUG_WORLD)
-				conoutf(CON_DEBUG, "DEBUG: clearing map data: %p %p", mapdata, curmap);
+				conoutf(CON_DEBUG, "\fs\f2DEBUG:\fr clearing map data: %p %p", mapdata, curmap);
 
 			if(curmap)
 				curmap->objs.removeobj(player1);
@@ -143,33 +164,28 @@ namespace game
 		}
 
 		if(DEBUG_WORLD)
-			conoutf(CON_DEBUG, "DEBUG: clearing %i scripts, %i effects, %i status effects, %i items,"
-				" %i ammotypes, %i characters, %i factions, %i obstacles, %i containers, %i platforms,"
-				" %i triggers, %i mapscripts, %i recipes, %i variables and %i tips",
-				scripts.length(), effects.length(), statuses.length(), items.length(),
-				ammotypes.length(), chars.length(), factions.length(), obstacles.length(), containers.length(), platforms.length(),
-				triggers.length(), mapscripts.length(), recipes.length(), variables.length(), tips.length());
+			conoutf(CON_DEBUG, "\fs\f2DEBUG:\fr clearing %i scripts, %i effects, %i status effects, %i ammotypes, %i mapscripts, %i recipes, %i merchants, %i variables, %i item categories and %i tips",
+				scripts.length(), effects.length(), statuses.length(),
+				ammotypes.length(), factions.length(), mapscripts.length(),
+				recipes.length(), merchants.length(), variables.length(), categories.length(), tips.length());
 
 		scripts.deletecontents();
 		effects.deletecontents();
 		statuses.deletecontents();
-		items.deletecontents();
 		ammotypes.deletecontents();
-		chars.deletecontents();
 		factions.deletecontents();
-		obstacles.deletecontents();
-		containers.deletecontents();
-		platforms.deletecontents();
-		triggers.deletecontents();
 		mapscripts.deletecontents();
 		recipes.deletecontents();
-		variables.setsize(0);
+		merchants.deletecontents();
+		variables.deletearrays();
+		categories.deletearrays();
 		tips.deletearrays();
+		journals.clear();
 		camera::cleanup(true);
 
-		char_base tmp;
-		tmp.transfer(*player1);
-		player1->equipped.setsize(0);
+		//We reset the player here so he has a clean slate on a new game.
+		player1->~rpgchar();
+		new (player1) rpgchar();
 	}
 
 	template<class T>
@@ -186,23 +202,23 @@ namespace game
 			int idx = strtol(files[i], &end, 10);
 			if(end == files[i] || *end != '\0' || idx < 0)
 			{
-				conoutf(CON_ERROR, "ERROR: can't load %s - invalid index or file name; skipping", files[i]);
+				conoutf(CON_ERROR, "\fs\f3ERROR:\fr can't load \"%s/%s.cfg\" - invalid index or file name; skipping", dir, files[i]);
 				continue;
 			}
 			if(DEBUG_WORLD)
-				conoutf(CON_DEBUG, "DEBUG: idx = %i, vector is %i; expanding if necessary", idx, objects.length());
+				conoutf(CON_DEBUG, "\fs\f2DEBUG:\fr idx = %i, vector has %i; expanding if necessary", idx, objects.length());
 			while(!objects.inrange(idx)) objects.add(NULL);
 
 			if(objects[idx])
 			{
-				conoutf(CON_ERROR, "ERROR: object with index %i already defined in %s; skipping", idx, dir);
+				conoutf(CON_ERROR, "\fs\f3ERROR:\fr object with index %i already defined in %s; skipping", idx, dir);
 				continue;
 			}
 
 			objects[idx] = var = new T();
 
 			formatstring(file)("%s/%s.cfg", dir, files[i]);
-			if(DEBUG_WORLD) conoutf("DEBUG: executing %s for index %i", file, idx);
+			if(DEBUG_WORLD) conoutf("\fs\f2DEBUG:\fr executing %s for index %i", file, idx);
 			execfile(file);
 		}
 		files.deletearrays();
@@ -212,7 +228,7 @@ namespace game
 		{
 			if(!objects[i])
 			{
-				conoutf(CON_ERROR, "ERROR: index %i not defined in %s; problems WILL occur if referenced.", i, dir);
+				conoutf(CON_ERROR, "\fs\f3ERROR:\fr index %i not defined in %s; problems WILL occur if referenced.", i, dir);
 				abort = true;
 			}
 		}
@@ -220,6 +236,7 @@ namespace game
 
 	void loadassets(const char *dir, bool definitions = false)
 	{
+		forceverbose++;
 		string pth;
 
 		if(!definitions)
@@ -230,40 +247,50 @@ namespace game
 			execfile(pth);
 		}
 
+		//Note that items, characters, triggers, obstacles, containers and platforms are not initialised here.
+		//They are templates and are called when the entity is spawned.
+		//The one exception is the player, see below.
+
+		#define RANGECHECK(x) \
+		if(!(x).length()) \
+		{ \
+			conoutf(CON_ERROR, "\fs\f3ERROR:\fr " #x " must contain at least one item"); \
+			abort = true; \
+		}
+
+		categories.deletearrays();
+		formatstring(pth)("%s/categories.cfg", dir);
+		execfile(pth);
+		RANGECHECK(categories);
+		formatstring(pth)("%s/merchants", dir);
+		loadassets(pth, loadingmerchant, merchants);
 		formatstring(pth)("%s/scripts", dir);
 		loadassets(pth, loadingscript, scripts);
+		RANGECHECK(scripts);
 		formatstring(pth)("%s/effects", dir);
 		loadassets(pth, loadingeffect, effects);
 		formatstring(pth)("%s/statuses", dir);
 		loadassets(pth, loadingstatusgroup, statuses);
-		formatstring(pth)("%s/items", dir);
-		loadassets(pth, loadingitem_base, items);
 		formatstring(pth)("%s/ammo", dir);
 		loadassets(pth, loadingammotype, ammotypes);
-		formatstring(pth)("%s/critters", dir);
-		loadassets(pth, loadingchar_base, chars);
 		formatstring(pth)("%s/factions", dir);
 		loadassets(pth, loadingfaction, factions);
-		formatstring(pth)("%s/obstacles", dir);
-		loadassets(pth, loadingobstacle_base, obstacles);
-		formatstring(pth)("%s/containers", dir);
-		loadassets(pth, loadingcontainer_base, containers);
-		formatstring(pth)("%s/platforms", dir);
-		loadassets(pth, loadingplatform_base, platforms);
-		formatstring(pth)("%s/triggers", dir);
-		loadassets(pth, loadingtrigger_base, triggers);
+		RANGECHECK(factions);
 		formatstring(pth)("%s/mapscripts", dir);
 		loadassets(pth, loadingmapscript, mapscripts);
+		RANGECHECK(mapscripts);
 		formatstring(pth)("%s/recipes", dir);
 		loadassets(pth, loadingrecipe, recipes);
 
+
 		if(!definitions)
 		{
-			loadingchar_base = player1;
+			loadingrpgchar = player1;
 			formatstring(pth)("%s/player.cfg", dir);
 			execfile(pth);
-			loadingchar_base = NULL;
+			loadingrpgchar = NULL;
 		}
+		forceverbose--;
 	}
 
 	ICOMMAND(r_rehash, "", (),
@@ -273,7 +300,7 @@ namespace game
 		loadassets(dir, true);
 		if(abort)
 		{
-			conoutf(CON_ERROR, "ERROR: game breaking errors were encountered while parsing files; aborting current game");
+			conoutf(CON_ERROR, "\fs\f3ERROR:\fr game breaking errors were encountered while parsing files; aborting current game");
 			abort = false;
 			localdisconnect();
 		}
@@ -281,41 +308,52 @@ namespace game
 
 	void newgame(const char *game, bool restore)
 	{
+		static bool nonewgame = false;
+		if(nonewgame) return;
+		forceverbose++;
+
+		if(!connected)
+		{
+			if(DEBUG_WORLD)
+				conoutf(CON_DEBUG, "\fs\f2DEBUG:\fr Trying to start game but not 'connected' - connecting and temporarily disabling newgame");
+			nonewgame = true;
+			localconnect();
+			nonewgame = false;
+		}
 		if(!game || !*game) game = "default";
 		if(DEBUG_WORLD)
-			conoutf(CON_DEBUG, "DEBUG: starting new game: %s", game);
+			conoutf(CON_DEBUG, "\fs\f2DEBUG:\fr starting new game: %s", game);
 
 		cleangame();
 		rpgscript::init();
 
-		mapdata = new hashtable<const char *, mapinfo>;
-		emptymap(0, true, NULL, false);
+		mapdata = new hashset<mapinfo>();
 		copystring(data, game);
 
 		defformatstring(dir)("data/rpg/games/%s", game);
-		{
-			defformatstring(pth)("%s/variables.cfg", dir);
-			execfile(pth);
-			formatstring(pth)("%s/tips.cfg", dir);
-			execfile(pth);
-		}
-		loadassets(dir);
+		loadassets(dir, false);
 		if(abort)
 		{
-			abort = false; localdisconnect(); return;
+			conoutf(CON_ERROR, "\fs\f3ERROR:\fr Critical errors were encountered while initialising the game; aborting");
+			abort = false; localdisconnect();
+			forceverbose--;
+			return;
 		}
+		emptymap(0, true, NULL, false);
 		concatstring(dir, ".cfg");
 		if(!execfile(dir))
-			conoutf(CON_WARN, "WARNING: game properties not defined");
+			conoutf(CON_WARN, "\fs\f6WARNING:\fr game properties not defined");
 		else if(DEBUG_WORLD)
-			conoutf(CON_DEBUG, "DEBUG: Loaded game properties");
+			conoutf(CON_DEBUG, "\fs\f2DEBUG:\fr Loaded game properties");
 
 		player1->resetmdl();
-		if(restore) return;
+		if(restore) { forceverbose--; return; }
 
 		player1->health = player1->base.getmaxhp();
 		player1->mana = player1->base.getmaxmp();
 		player1->getsignal("spawn", false);
+
+		forceverbose--;
 	}
 	ICOMMAND(newgame, "s", (const char *s), newgame(s));
 
@@ -344,11 +382,14 @@ namespace game
 	mapinfo *accessmap(const char *name)
 	{
 		if(!mapdata)
+		{
+			conoutf(CON_ERROR, "\fs\f2ERROR:\fr no map data; cannot access data for \"%s\"", name);
 			return NULL;
+		}
 
 		mapinfo *dummy = mapdata->access(name);
 		if(DEBUG_WORLD)
-			conoutf(CON_DEBUG, "DEBUG: finding map data (%p)", dummy);
+			conoutf(CON_DEBUG, "\fs\f2DEBUG:\fr finding map data (%p)", dummy);
 
 		if(!dummy)
 		{
@@ -357,7 +398,7 @@ namespace game
 			dummy->name = newname;
 
 			if(DEBUG_WORLD)
-				conoutf(CON_DEBUG, "DEBUG: map data not found - creating (%p)", dummy);
+				conoutf(CON_DEBUG, "\fs\f2DEBUG:\fr map data not found - creating (%p)", dummy);
 		}
 
 		return dummy;
@@ -368,9 +409,11 @@ namespace game
 	void clearmap(mapinfo *map)
 	{
 		colcache.setsize(0);
+		mergecache.setsize(0);
 		loopvrev(map->objs)
 		{
 			rpgent *ent = map->objs[i];
+// 			conoutf("freeing %p", ent);
 			rpgscript::removereferences(ent);
 			delete ent;
 		}
@@ -392,17 +435,19 @@ namespace game
 		if(!name)
 			return;
 
+		forceverbose++;
+
 		ai::loadwaypoints(name);
 
 		if(curmap)
 		{
 			if(DEBUG_WORLD)
-				conoutf(CON_DEBUG, "DEBUG: removing player from curmap vector");
+				conoutf(CON_DEBUG, "\fs\f2DEBUG:\fr removing player from curmap vector");
 			curmap->objs.removeobj(player1);
 			if(curmap->flags & mapinfo::F_VOLATILE)
 			{
 				if(DEBUG_WORLD)
-					conoutf(CON_DEBUG, "DEBUG: curmap volatile, clearing before map  change");
+					conoutf(CON_DEBUG, "\fs\f2DEBUG:\fr curmap volatile, clearing before map  change");
 				clearmap(curmap);
 				curmap->loaded = false;
 			}
@@ -412,13 +457,13 @@ namespace game
 		rpgscript::changemap();
 
 		if(DEBUG_WORLD)
-			conoutf(CON_DEBUG, "DEBUG: adding player to curmap vector");
+			conoutf(CON_DEBUG, "\fs\f2DEBUG:\fr adding player to curmap vector");
 		curmap->objs.add(player1);
 
 		if(!curmap->loaded)
 		{
 			if(DEBUG_WORLD)
-				conoutf(CON_DEBUG, "DEBUG: map not loaded - initialising");
+				conoutf(CON_DEBUG, "\fs\f2DEBUG:\fr map not loaded - initialising");
 
 			entities::startmap();
 			curmap->getsignal("load", false);
@@ -439,26 +484,22 @@ namespace game
 
 		if(DEBUG_WORLD)
 		{
-			enumeratekt(*mapdata, const char *, name, mapinfo, info,
-				conoutf(CON_DEBUG, "DEBUG: map %s", name);
-				conoutf(CON_DEBUG, "DEBUG: map objects %i", info.objs.length());
-				conoutf(CON_DEBUG, "DEBUG: loaded %i", info.loaded);
-				conoutf(CON_DEBUG, "DEBUG: deferred actions %i", info.loadactions.length());
+			enumerate(*mapdata, mapinfo, info,
+				conoutf(CON_DEBUG, "\fs\f2DEBUG:\fr map %s", info.name);
+				conoutf(CON_DEBUG, "\fs\f2DEBUG:\fr map objects %i", info.objs.length());
+				conoutf(CON_DEBUG, "\fs\f2DEBUG:\fr loaded %i", info.loaded);
+				conoutf(CON_DEBUG, "\fs\f2DEBUG:\fr deferred actions %i", info.loadactions.length());
 			);
 		}
 
 		if(tips.inrange(lasttip))
 			conoutf("\f2%s", tips[lasttip]);
+		forceverbose--;
 	}
 
 	void openworld(const char *name, bool fall)
 	{
-		if(!mapdata)
-		{
-			if(DEBUG_WORLD) conoutf(CON_DEBUG, "DEBUG: no game in progress, falling back to default");
-			newgame(NULL);
-		}
-		if(!connected) localconnect();
+		if(!connected) {conoutf("No game in progress"); return;}
 
 		if(name && *name && load_world(name))
 			return;
@@ -470,8 +511,17 @@ namespace game
 
 	ICOMMAND(map, "s", (char *s), openworld(s, false))
 
-	void changemap(const char *name) { openworld(name, true); }
-	void forceedit(const char *name) { openworld(name, false); }
+	void changemap(const char *name)
+	{
+		if(!connected) newgame(NULL);
+		openworld(name, true);
+	}
+
+	void forceedit(const char *name)
+	{
+		if(!connected) newgame(NULL);
+		//openworld(name, false);
+	}
 
 	void updateworld()
 	{
@@ -486,7 +536,7 @@ namespace game
 		physicsframe();
 		if(firstupdate)
 		{
-			if(DEBUG_WORLD) conoutf("DEBUG: curmap was recently opened, skipping update");
+			if(DEBUG_WORLD) conoutf("\fs\f2DEBUG:\fr curmap was recently opened; skipping update so entities don't take a %ims step", curtime);
 			firstupdate = false;
 			return;
 		}
@@ -501,6 +551,9 @@ namespace game
 			rpgent *d = curmap->objs[i];
 			float eye = d->eyeheight;
 
+			d->resetmdl();
+			d->temp.alpha = 1;
+			d->temp.light = vec4(0, 0, 0, 0);
 			d->update();
 			loopvj(d->seffects)
 			{
@@ -517,7 +570,7 @@ namespace game
 			if(eye)
 			{
 				if(DEBUG_ENT)
-					conoutf("DEBUG: ent has new eyeheight of %f, applying positional delta of %f", d->eyeheight, eye);
+					conoutf("\fs\f2DEBUG:\fr ent has new eyeheight of %f, applying positional delta of %f", d->eyeheight, eye);
 				d->o.z += eye;
 				d->newpos.z += eye;
 			}
@@ -549,8 +602,8 @@ namespace game
 				vecfromyawpitch(second->yaw, 0, second->move, second->strafe, vel1);
 				vecfromyawpitch(first->yaw, 0, first->move, first->strafe, vel2);
 
-				vel1.mul(second->maxspeed * curtime / 1000.f).add(first->vel);
-				vel2.mul(first->maxspeed * curtime / 1000.f).add(second->vel);
+				vel1.mul(second->maxspeed * curtime / 700.f).add(first->vel);
+				vel2.mul(first->maxspeed * curtime / 700.f).add(second->vel);
 
 				first->vel = vel1;
 				second->vel = vel2;
@@ -567,6 +620,22 @@ namespace game
 		}
 		colcache.setsize(0);
 
+		loopv(mergecache)
+		{
+			rpgitem *a = (rpgitem *) mergecache[i].d;
+			rpgitem *b = (rpgitem *) mergecache[i].o;
+
+// 			conoutf("attempting to merge items %p and %p; %i %i --> %i", a, b, a->quantity, b->quantity, a->quantity + b->quantity);
+
+			if(a->additem(b))
+			{
+// 				conoutf("Merge successful! %i %i", a->quantity, b->quantity);
+				b->quantity = 0;
+				rpgscript::obits.add(b);
+			}
+		}
+		mergecache.setsize(0);
+
 		ai::trydrop();
 
 		loopvrev(curmap->projs)
@@ -577,9 +646,9 @@ namespace game
 			if(!curmap->aeffects[i]->update())
 				delete curmap->aeffects.remove(i);
 
-		camera::update();
 		curmap->getsignal("update");
 		rpgscript::update();
+		camera::update();
 	}
 
 	void setwindowcaption()
@@ -590,46 +659,8 @@ namespace game
 
 	void preload()
 	{
-		loopv(effects)
-		{
-			if(effects[i]->mdl)
-			{
-				preloadmodel(effects[i]->mdl);
-				if(DEBUG_WORLD)
-					conoutf(CON_DEBUG, "DEBUG: preloading %s", effects[i]->mdl);
-			}
-		}
-		loopv(chars)
-		{
-			if(chars[i]->mdl)
-			{
-				preloadmodel(chars[i]->mdl);
-				if(DEBUG_WORLD)
-					conoutf(CON_DEBUG, "DEBUG: preloading %s", chars[i]->mdl);
-			}
-		}
-		loopv(items)
-		{
-			if(items[i]->mdl)
-			{
-				preloadmodel(items[i]->mdl);
-				if(DEBUG_WORLD)
-					conoutf(CON_DEBUG, "DEBUG: preloading %s", items[i]->mdl);
-			}
-		}
-		loopv(statuses)
-		{
-			loopvj(statuses[i]->effects)
-			{
-				status_polymorph *p = (status_polymorph *) statuses[i]->effects[j];
-				if(p->type == STATUS_POLYMORPH)
-				{
-					preloadmodel(p->mdl);
-					if(DEBUG_WORLD)
-						conoutf(CON_DEBUG, "DEBUG: preloading %s", p->mdl);
-				}
-			}
-		}
+		///rpgconfig queues models for preloading as they are assigned - so just make sure the defualt model is loaded.
+		preloadmodel(DEFAULTMODEL);
 	}
 
 	void suicide(physent *d)
@@ -668,9 +699,11 @@ namespace game
 	///NOTE: dist is a multiplier for the delta between from and to, if there's 10 units difference and you need to travel 5, dist is 0.5
 	bool intersect(rpgent *d, const vec &from, const vec &to, float &dist)   // if lineseg hits entity bounding box
 	{
-		vec bottom(d->o), top(d->o);
-		bottom.z -= d->eyeheight;
+		vec bottom(d->feetpos()), top(d->o);
+		//crop the bounding box to 1/3rd of its height when dead
+		if(d->state == CS_DEAD) top.z -= 2 * (d->aboveeye + d->eyeheight) / 3;
 		top.z += d->aboveeye;
+
 		return linecylinderintersect(from, to, bottom, top, d->radius, dist);
 	}
 
@@ -722,14 +755,13 @@ namespace game
 
 	void newmap(int size)
 	{
+		if(!connected) {conoutf("No game in progress"); return;}
 		if(size < 0) return;
-		if(!mapdata) newgame(NULL);
-		if(!connected) localconnect();
 		emptymap(size, true);
 	}
 	ICOMMAND(newmap, "i", (int *size), newmap(max(*size, 0));) ///overrides built in variant - we want to reset the game state before creating entities
 
-	void gameconnect(bool _remote) {if(!mapdata) newgame(NULL); connected = true;}
+	void gameconnect(bool _remote) { connected = true; if(!mapdata) newgame(NULL); }
 
 	void gamedisconnect(bool cleanup)
 	{
@@ -759,7 +791,7 @@ namespace game
 			if(edittogglereset)
 			{
 				if(DEBUG_WORLD)
-					conoutf(CON_DEBUG, "DEBUG: resetting game state for current map");
+					conoutf(CON_DEBUG, "\fs\f2DEBUG:\fr resetting game state for current map");
 
 				curmap->objs.removeobj(player1);
 				clearmap(curmap);
@@ -794,7 +826,7 @@ namespace game
 		else
 		{
 			if(DEBUG_WORLD)
-				conoutf(CON_DEBUG, "DEBUG: dynent requested while not in a game, returning player");
+				conoutf(CON_DEBUG, "\fs\f2DEBUG:\fr dynent requested while not in a game, returning player");
 			return player1;
 		}
 
@@ -818,7 +850,7 @@ namespace game
 	void texturefailed(char* name, int slot) {}
 	void mmodelfailed(const char *name, int idx) {}
 	void mapfailed(const char *name) {}
-	bool allowdoublejump() { return false; }
+	bool allowdoublejump(physent *d) {return false;}
 	bool detachcamera() { return player1->state == CS_DEAD; }
 	void toserver(char *text) { execute(text); } //since we don't talk, just execute if the / is forgotten
 }
