@@ -10,7 +10,7 @@ namespace camera
 		if(delta) elapsed = min(curtime, duration + startmillis - lastmillis);
 		else elapsed = lastmillis - startmillis;
 
-		return clamp<float>(1, 0, (float) elapsed / duration);
+		return clamp((float)elapsed / duration, 0.f, 1.0f);
 	}
 
 	static float dlinear(int startmillis, int duration, bool delta)
@@ -33,7 +33,7 @@ namespace camera
 		else
 		{
 			int elapsed = min(lastmillis - startmillis, duration);
-			return clamp<float>(1, 0, sinf(PI * elapsed / duration / 2));
+			return clamp(sinf(PI * elapsed / duration / 2), 0.f, 1.0f);
 		}
 	}
 
@@ -65,7 +65,7 @@ namespace camera
 		else
 		{
 			int elapsed = min(lastmillis - startmillis, duration);
-			return clamp<float>(1, 0, cosf(PI * elapsed / 2 / duration));
+			return clamp(cosf(PI * elapsed / 2 / duration), 0.f, 1.0f);
 		}
 	}
 
@@ -94,7 +94,7 @@ namespace camera
 
 	bool cutscene = false;
 	bool cancel = false;
-	const uint *post = NULL;
+	uint *post = NULL;
 	physent camera;
 	physent *attached = NULL;
 	float (*interpfunc)(int, int, bool) = linear;
@@ -108,7 +108,7 @@ namespace camera
 
 		cutscene = false;
         cancel = false;
-		DELETEA(post);
+		freecode(post); post = NULL;
 		attached = NULL;
 		interpfunc = linear;
 		pending.deletecontents();
@@ -118,11 +118,11 @@ namespace camera
 	struct action
 	{
 		int startmillis, duration;
-		const uint *successors;
+		uint *successors;
 		float (*interp)(int, int, bool);
 
-		action(int d, const char *s) : startmillis(lastmillis), duration(d), successors((s && *s) ? compilecode(s) : NULL), interp(interpfunc) {}
-		virtual ~action() { delete[] successors; }
+		action(int d, uint *s) : startmillis(lastmillis), duration(d), successors(s), interp(interpfunc) { keepcode(successors); }
+		virtual ~action() { freecode(successors); }
 
 		virtual bool update()
 		{
@@ -144,7 +144,7 @@ namespace camera
 		{
 			if(type)
 			{
-				draw_text("Type: Action/Generic", 100, 0 + hoffset);
+				draw_text("Type: Delay/Generic", 100, 0 + hoffset);
 				hoffset += 64;
 			}
 
@@ -155,6 +155,48 @@ namespace camera
 		virtual void getsignal(const char *sig) {}
 	};
 
+	struct signal : action
+	{
+		const char *name;
+		bool received;
+
+		signal(const char *n, uint *s) : action(0, s), name(newstring(n)), received(false) {}
+		~signal() {delete[] name;}
+
+		bool update()
+		{
+			if(received)
+			{
+				if(successors)
+				{
+					float (*ib)(int, int, bool) = interpfunc;
+					interpfunc = interp;
+					execute(successors);
+					interpfunc = ib;
+				}
+			}
+			return true;
+		}
+
+		void debug(int &hoffset, bool type)
+		{
+			if(type)
+			{
+				draw_text("Type: Delay/Signal", 100, 0 + hoffset);
+				hoffset += 64;
+			}
+
+			draw_textf("Successors: %p", 100, 64 + hoffset, successors);
+			hoffset += 128;
+		}
+
+		void getsignal(const char *sig)
+		{
+			if(!strcmp(name, sig))
+				received = true;
+		}
+	};
+
 	void update()
 	{
 		if(!cutscene) return;
@@ -176,16 +218,16 @@ namespace camera
 		loopv(pending)
 			pending[i]->render(w, h);
 
-		glPushMatrix();
-		glScalef(.3, .3, 1);
-
 		int offset = 128;
-		if(DEBUG_CAMERA)
+		if(DEBUG_CAMERA) loopv(pending)
 		{
-			loopv(pending)
-				pending[i]->debug(offset, true);
+			glPushMatrix();
+			glScalef(.3, .3, 1);
+
+			pending[i]->debug(offset, true);
+
+			glPopMatrix();
 		}
-		glPopMatrix();
 	}
 
 	void getsignal(const char *signal)
@@ -198,7 +240,7 @@ namespace camera
 	{
 		float dx, dy, dz;
 
-		move(float x, float y, float z, int d, const char *s) : action(d, s), dx(x), dy(y), dz(z) {}
+		move(float x, float y, float z, int d, uint *s) : action(d, s), dx(x), dy(y), dz(z) {}
 		~move() {}
 
 		bool update()
@@ -223,7 +265,7 @@ namespace camera
 	{
 		float dyaw, dpitch, droll;
 
-		view(float y, float p, float r, int d, const char *s) : action(d, s), dyaw(y), dpitch(p), droll(r) {}
+		view(float y, float p, float r, int d, uint *s) : action(d, s), dyaw(y), dpitch(p), droll(r) {}
 		~view() {}
 
 		bool update()
@@ -254,7 +296,7 @@ namespace camera
 		dynent *ent;
 		float interp, lead;
 
-		focus(dynent *e, float i, float l, int d, const char *s) : action(d, s), ent(e), interp(max<float>(0, i)), lead(l) {}
+		focus(dynent *e, float i, float l, int d, uint *s) : action(d, s), ent(e), interp(max<float>(0, i)), lead(l) {}
 		~focus() {}
 
 		bool update()
@@ -291,7 +333,7 @@ namespace camera
 		dynent *ent;
 		float interp, tail, height;
 
-		follow(dynent *e, float i, float t, float h, int d, const char *s) : action(d, s), ent(e), interp(max<float>(0, i)), tail(t), height(h) {}
+		follow(dynent *e, float i, float t, float h, int d, uint *s) : action(d, s), ent(e), interp(max<float>(0, i)), tail(t), height(h) {}
 		~follow() {}
 
 		bool update()
@@ -313,8 +355,8 @@ namespace camera
 				hoffset += 64;
 			}
 			draw_textf("Target: %p", 100, 0 + hoffset, ent);
-			draw_textf("Interp: %f", 100, 0 + hoffset, interp);
-			draw_textf("Delta: <-x, -y> == %f; z = %f", 100, 0 + hoffset, tail, height);
+			draw_textf("Interp: %f", 100, 64 + hoffset, interp);
+			draw_textf("Delta: <-x, -y> == %f; z = %f", 100, 128 + hoffset, tail, height);
 			hoffset += 192;
 
 			action::debug(hoffset, false);
@@ -325,7 +367,7 @@ namespace camera
 	{
 		dynent *ent;
 
-		viewport(dynent *e, int d, const char *s) : action(d, s), ent(e) {}
+		viewport(dynent *e, int d, uint *s) : action(d, s), ent(e) {}
 		~viewport() {}
 
 		bool update()
@@ -354,7 +396,7 @@ namespace camera
 	{
 		vector<action *> pending;
 
-		container(const char *c, int d, const char *s) : action(d, s)
+		container(uint *c, int d, uint *s) : action(d, s)
 		{
 			vector<action *> *vbak = actionvec;
 			float (*fbak)(int, int, bool) = interpfunc;
@@ -440,7 +482,7 @@ namespace camera
 	{
 		float dx, dy;
 
-		translate(float x, float y, const char *c, int d, const char *s) : container(c, d, s), dx(x), dy(y) {}
+		translate(float x, float y, uint *c, int d, uint *s) : container(c, d, s), dx(x), dy(y) {}
 		~translate() {}
 
 		void render(int w, int h)
@@ -471,7 +513,7 @@ namespace camera
 	{
 		float dx, dy;
 
-		scale(float x, float y, const char *c, int d, const char *s) : container(c, d, s), dx(x), dy(y) {}
+		scale(float x, float y, uint *c, int d, uint *s) : container(c, d, s), dx(x), dy(y) {}
 		~scale() {}
 
 		void render(int w, int h)
@@ -504,7 +546,7 @@ namespace camera
 	{
 		float angle;
 
-		rotate(float a, const char *c, int d, const char *s) : container(c, d, s), angle(a) {}
+		rotate(float a, uint *c, int d, uint *s) : container(c, d, s), angle(a) {}
 		~rotate() {}
 
 		void render(int w, int h)
@@ -519,7 +561,7 @@ namespace camera
 		{
 			if(type)
 			{
-				draw_text("Type: Container/Translate", 100, 0 + hoffset);
+				draw_text("Type: Container/Rotation", 100, 0 + hoffset);
 				hoffset += 64;
 			}
 			draw_textf("Angle: %f", 100, 0 + hoffset, angle);
@@ -535,7 +577,7 @@ namespace camera
 		int colour; //this should probably be unsigned...
 		const char *str;
 
-		text(float x, float y, float w, int c, const char *t, int d, const char *s) : action(d, s), x(x), y(y), width(w), colour(c), str((t && *t) ? newstring(t) : NULL) {}
+		text(float x, float y, float w, int c, const char *t, int d, uint *s) : action(d, s), x(x), y(y), width(w), colour(c), str((t && *t) ? newstring(t) : NULL) {}
 		~text() {delete[] str;}
 
 		void render(int w, int h)
@@ -571,7 +613,7 @@ namespace camera
 		int colour;
 		const char *img;
 
-		image(float x, float y, float dx, float dy, int c, const char *i, int d, const char *s) : action(d, s), x(x), y(y), dx(dx), dy(dy), colour(c), img((i && *i) ? newstring(i) : NULL) {}
+		image(float x, float y, float dx, float dy, int c, const char *i, int d, uint *s) : action(d, s), x(x), y(y), dx(dx), dy(dy), colour(c), img((i && *i) ? newstring(i) : NULL) {}
 		~image() {delete[] img;}
 
 		void render(int w, int h)
@@ -618,7 +660,7 @@ namespace camera
 		int colour;
 		bool modulate;
 
-		solid(float x, float y, float dx, float dy, int c, bool m, int d, const char *s) : action(d, s), x(x), y(y), dx(dx), dy(dy), colour(c), modulate(m) {}
+		solid(float x, float y, float dx, float dy, int c, bool m, int d, uint *s) : action(d, s), x(x), y(y), dx(dx), dy(dy), colour(c), modulate(m) {}
 		~solid() {}
 
 		void render(int w, int h)
@@ -665,7 +707,7 @@ namespace camera
 	};
 
 	#define INTERPCMD(name, func) \
-		ICOMMAND(cs_interp_ ## name, "e", (const uint *body), \
+		ICOMMAND(cs_interp_ ## name, "e", (uint *body), \
 			if(!cutscene) return; \
 			 \
 			float (*ib)(int, int, bool) = interpfunc; \
@@ -694,13 +736,21 @@ namespace camera
 
 	#define GETREF(var, ref) \
 		reference *var = NULL; \
+		int var ## idx; \
+		rpgscript::parseref(ref, var ## idx); \
 		if(*ref) var = rpgscript::searchstack(ref);
 
 	ICOMMAND(cs_start, "s", (const char *s),
+		//reuse camera state
 		if(cutscene)
 		{
 			if(post) execute(post);
-			 DELETEA(post);
+			freecode(post);
+			post = NULL;
+		}
+		else
+		{
+			camera = *game::player1;
 		}
 
 		static string file;
@@ -716,30 +766,35 @@ namespace camera
 		intret(cancel);
 	)
 
-	ICOMMAND(cs_post, "s", (const char *body),
-		DELETEA(post);
-		post = compilecode(body);
+	ICOMMAND(cs_post, "e", (uint *body),
+		freecode(post);
+		keepcode((post = body));
 	)
 
-	ICOMMAND(cs_action_generic, "is", (int *d, const char *s),
+	ICOMMAND(cs_action_generic, "ie", (int *d, uint *s),
 		if(!cutscene) return;
 
 		actionvec->add(new action(*d, s));
 	)
 
-	ICOMMAND(cs_action_movedelta, "fffis", (float *x, float *y, float *z, int *d, const char *s),
+	ICOMMAND(cs_action_signal, "se", (const char *sig, uint *s),
+		if(!cutscene) return;
+		actionvec->add(new signal(sig, s));
+	)
+
+	ICOMMAND(cs_action_movedelta, "fffie", (float *x, float *y, float *z, int *d, uint *s),
 		if(!cutscene) return;
 
 		actionvec->add(new move(*x, *y, *z, *d, s));
 	)
 
-	ICOMMAND(cs_action_moveset, "fffis", (float *x, float *y, float *z, int *d, const char *s),
+	ICOMMAND(cs_action_moveset, "fffie", (float *x, float *y, float *z, int *d, uint *s),
 		if(!cutscene) return;
 
 		actionvec->add(new move(*x - camera.o.x, *y - camera.o.y, *z - camera.o.z, *d, s));
 	)
 
-	ICOMMAND(cs_action_movecamera, "iis", (int *tag, int *d, const char *s),
+	ICOMMAND(cs_action_movecamera, "iie", (int *tag, int *d, uint *s),
 		if(!cutscene) return;
 
 		GETCAM
@@ -750,40 +805,40 @@ namespace camera
 		}
 		else
 		{
-			conoutf(CON_ERROR, "ERROR: camera entity %i does not exist, using dummy action instead of move", *tag);
+			conoutf(CON_ERROR, "\fs\f3ERROR:\fr camera entity %i does not exist, using dummy action instead of move", *tag);
 			actionvec->add(new action(*d, s));
 		}
 	)
 
-	ICOMMAND(cs_action_moveent, "sis", (const char *r, int *d, const char *s),
+	ICOMMAND(cs_action_moveent, "sie", (const char *r, int *d, uint *s),
 		if(!cutscene) return;
 
 		GETREF(ent, r)
-		if(ent && ent->getent())
+		if(ent && ent->getent(entidx))
 		{
-			vec delta = vec(ent->getent()->o).sub(camera.o);
+			vec delta = vec(ent->getent(entidx)->o).sub(camera.o);
 			actionvec->add(new move(delta.x, delta.y, delta.z, *d, s));
 		}
 		else
 		{
-			conoutf(CON_ERROR, "ERROR: reference \'%s\' does not exist or is not an entity, using dummy action instead of move", r);
+			conoutf(CON_ERROR, "\fs\f3ERROR:\fr reference \'%s\' does not exist or is not an entity, using dummy action instead of move", r);
 			actionvec->add(new action(*d, s));
 		}
 	)
 
-	ICOMMAND(cs_action_viewdelta, "fffis", (float *y, float *p, float *r, int *d, const char *s),
+	ICOMMAND(cs_action_viewdelta, "fffie", (float *y, float *p, float *r, int *d, uint *s),
 		if(!cutscene) return;
 
 		actionvec->add(new view(*y, *p, *r, *d, s));
 	)
 
-	ICOMMAND(cs_action_viewset, "fffis", (float *y, float *p, float *r, int *d, const char *s),
+	ICOMMAND(cs_action_viewset, "fffie", (float *y, float *p, float *r, int *d, uint *s),
 		if(!cutscene) return;
 
 		actionvec->add(new view(*y - camera.yaw, *p - camera.pitch, *r - camera.roll, *d, s));
 	)
 
-	ICOMMAND(cs_action_viewcamera, "iis", (int *tag, int *d, const char *s),
+	ICOMMAND(cs_action_viewcamera, "iie", (int *tag, int *d, uint *s),
 		if(!cutscene) return;
 
 		GETCAM
@@ -794,109 +849,109 @@ namespace camera
 		}
 		else
 		{
-			conoutf(CON_ERROR, "ERROR: camera entity %i does not exist, using dummy action instead of view", *tag);
+			conoutf(CON_ERROR, "\fs\f3ERROR:\fr camera entity %i does not exist, using dummy action instead of view", *tag);
 			actionvec->add(new action(*d, s));
 		}
 	)
 
-	ICOMMAND(cs_action_viewent, "sis", (const char *r, int *d, const char *s),
+	ICOMMAND(cs_action_viewent, "sie", (const char *r, int *d, uint *s),
 		if(!cutscene) return;
 
 		GETREF(ent, r)
-		if(ent && ent->getent())
+		if(ent && ent->getent(entidx))
 		{
-			vec delta = vec(ent->getent()->yaw, ent->getent()->pitch, ent->getent()->roll).sub(vec(camera.yaw, camera.pitch, camera.roll));
+			vec delta = vec(ent->getent(entidx)->yaw, ent->getent(entidx)->pitch, ent->getent(entidx)->roll).sub(vec(camera.yaw, camera.pitch, camera.roll));
 			actionvec->add(new view(delta.x, delta.y, delta.z, *d, s));
 		}
 		else
 		{
-			conoutf(CON_ERROR, "ERROR: reference \'%s\' does not exist or is not an entity, using dummy action instead of view", r);
+			conoutf(CON_ERROR, "\fs\f3ERROR:\fr reference \'%s\' does not exist or is not an entity, using dummy action instead of view", r);
 			actionvec->add(new action(*d, s));
 		}
 	)
 
-	ICOMMAND(cs_action_focus, "sffis", (const char *r, float *i, float *l, int *d, const char *s),
+	ICOMMAND(cs_action_focus, "sffie", (const char *r, float *i, float *l, int *d, uint *s),
 		if(!cutscene) return;
 
 		GETREF(ent, r)
-		if(ent && ent->getent())
+		if(ent && ent->getent(entidx))
 		{
-			actionvec->add(new focus(ent->getent(), *i, *l, *d, s));
+			actionvec->add(new focus(ent->getent(entidx), *i, *l, *d, s));
 		}
 		else
 		{
-			conoutf(CON_ERROR, "ERROR: reference \'%s\' does not exist or is not an entity, using dummy action instead of focus", r);
+			conoutf(CON_ERROR, "\fs\f3ERROR:\fr reference \'%s\' does not exist or is not an entity, using dummy action instead of focus", r);
 			actionvec->add(new action(*d, s));
 		}
 	)
 
-	ICOMMAND(cs_action_follow, "sfffis", (const char *r, float *i, float *t, float *h, int *d, const char *s),
+	ICOMMAND(cs_action_follow, "sfffie", (const char *r, float *i, float *t, float *h, int *d, uint *s),
 		if(!cutscene) return;
 
 		GETREF(ent, r)
-		if(ent && ent->getent())
+		if(ent && ent->getent(entidx))
 		{
-			actionvec->add(new follow(ent->getent(), *i, *t, *h, *d, s));
+			actionvec->add(new follow(ent->getent(entidx), *i, *t, *h, *d, s));
 		}
 		else
 		{
-			conoutf(CON_ERROR, "ERROR: reference \'%s\' does not exist or is not an entity, using dummy action instead of follow", r);
+			conoutf(CON_ERROR, "\fs\f3ERROR:\fr reference \'%s\' does not exist or is not an entity, using dummy action instead of follow", r);
 			actionvec->add(new action(*d, s));
 		}
 	)
 
-	ICOMMAND(cs_action_viewport, "sis", (const char *r, int *d, const char *s),
+	ICOMMAND(cs_action_viewport, "sie", (const char *r, int *d, uint *s),
 		if(!cutscene) return;
 
 		GETREF(ent, r)
-		if(ent && ent->getent())
+		if(ent && ent->getent(entidx))
 		{
-			actionvec->add(new viewport(ent->getent(), *d, s));
+			actionvec->add(new viewport(ent->getent(entidx), *d, s));
 		}
 		else
 		{
-			conoutf(CON_ERROR, "ERROR: reference \'%s\' does not exist or is not an entity, using dummy action instead of viewport", r);
+			conoutf(CON_ERROR, "\fs\f3ERROR:\fr reference \'%s\' does not exist or is not an entity, using dummy action instead of viewport", r);
 			actionvec->add(new action(*d, s));
 		}
 	)
 
-	ICOMMAND(cs_container_generic, "sis", (const char *c, int *d, const char *s),
+	ICOMMAND(cs_container_generic, "eie", (uint *c, int *d, uint *s),
 		if(!cutscene) return;
 
 		actionvec->add(new container(c, *d, s)); //useful for grouping things into layers, eg, a background layer and a subtitle layer
 	)
 
-	ICOMMAND(cs_container_translate, "ffsis", (float *x, float *y, const char *c, int *d, const char *s),
+	ICOMMAND(cs_container_translate, "ffeie", (float *x, float *y, uint *c, int *d, uint *s),
 		if(!cutscene) return;
 
 		actionvec->add(new translate(*x, *y, c, *d, s));
 	)
 
-	ICOMMAND(cs_container_scale, "ffsis", (float *x, float *y, const char *c, int *d, const char *s),
+	ICOMMAND(cs_container_scale, "ffeie", (float *x, float *y, uint *c, int *d, uint *s),
 		if(!cutscene) return;
 
 		actionvec->add(new scale(*x, *y, c, *d, s));
 	)
 
-	ICOMMAND(cs_container_rotate, "fsis", (float *a, const char *c, int *d, const char *s),
+	ICOMMAND(cs_container_rotate, "feie", (float *a, uint *c, int *d, uint *s),
 		if(!cutscene) return;
 
 		actionvec->add(new rotate(*a, c, *d, s));
 	)
 
-	ICOMMAND(cs_element_text, "fffisis", (float *x, float *y, float *w, int *c, const char *t, int *d, const char *s),
+	ICOMMAND(cs_element_text, "fffisie", (float *x, float *y, float *w, int *c, const char *t, int *d, uint *s),
 		if(!cutscene) return;
 
 		actionvec->add(new text(*x, *y, *w, *c, t, *d, s));
 	)
 
-	ICOMMAND(cs_element_image, "ffffisis", (float *x, float *y, float *dx, float *dy, int *c, const char *p, int *d, const char *s),
+	ICOMMAND(cs_element_image, "ffffisie", (float *x, float *y, float *dx, float *dy, int *c, const char *p, int *d, uint *s),
 		if(!cutscene) return;
 
 		actionvec->add(new image(*x, *y, *dx, *dy, *c, p, *d, s));
 	)
 
-	ICOMMAND(cs_element_solid, "ffffiiis", (float *x, float *y, float *dx, float *dy, int *c, int *m, int *d, const char *s),
+	ICOMMAND(cs_element_solid, "ffffiiie", (float *x, float *y, float *dx, float *dy, int *c, int *m, int *d, uint *s),
 		if(!cutscene) return;
 
 		actionvec->add(new solid(*x, *y, *dx, *dy, *c, *m, *d, s));
@@ -920,118 +975,6 @@ namespace game
 }
 
 #if 0
-	struct cond : action
-	{
-		const uint *test;
-
-		cond(const char *t, const char *s) : action(0, s), test(t && *t ? compilecode(t) : NULL) {}
-		~cond() { delete[] test;}
-
-		bool update()
-		{
-			if(!test || execute(test))
-			{
-				if(successors) execute(successors);
-				return false;
-			}
-
-			return true;
-		}
-
-		void debug(int w, int h, int &hoffset, bool type)
-		{
-			if(type)
-			{
-				draw_text("Type: Test", 100, 0 + hoffset);
-				hoffset += 64;
-			}
-
-			draw_textf("Test: %p", 100, 0 + hoffset, test);
-			draw_textf("Children: %p", 100, 64 + hoffset, successors);
-			hoffset += 192;
-		}
-	};
-
-	struct iterate : cond
-	{
-		int limit;
-		int i;
-
-		iterate(const char *t, int l, int d, const char *s) : cond(t, s), limit(l), i(0) { duration = d;}
-		~iterate() {}
-
-		bool update()
-		{
-			if(lastmillis - startmillis >= duration)
-			{
-				if(limit)
-					i++;
-
-				bool res = cond::update();
-
-				if(res || (limit && i == limit))
-					return false;
-
-				startmillis += duration;
-			}
-
-			return true;
-		}
-
-		void debug(int w, int h, int &hoffset, bool type)
-		{
-			if(type)
-			{
-				draw_text("Type: Loop", 100, 0 + hoffset);
-				hoffset += 64;
-			}
-
-			draw_textf("Test: %p", 100, 0 + hoffset);
-			draw_textf("Loop: %i/%i", 100, 64 + hoffset, i, limit);
-			hoffset += 128;
-
-			action::debug(w, h, hoffset);
-		}
-	};
-
-	struct signal : action
-	{
-		const char *name;
-		bool received;
-
-		signal(const char *n, const char *s) : action(0, s), name(n && *n ? newstring(n) : NULL), received(false) {}
-		~signal() {delete[] name;}
-
-		bool update()
-		{
-			if(!name || received)
-			{
-				if(successors) execute(successors);
-				return false;
-			}
-			return true;
-		}
-
-		void getsignal(const char *signal)
-		{
-			if(!strcmp(signal, name))
-				received = true;
-		}
-
-		void debug(int w, int h, int &hoffset, bool type)
-		{
-			if(type)
-			{
-				draw_text("Type: Signal", 100, 0 + hoffset);
-				hoffset += 64;
-			}
-
-			draw_textf("Signal: %s", 100, 0 + hoffset, name);
-			draw_textf("Children: %p", 100, 64 + hoffset, successors);
-			hoffset += 192;
-		}
-	};
-
 	struct sound : action
 	{
 		int ind;
