@@ -28,7 +28,7 @@ void use_armour::apply(rpgchar *user)
 		loopvj(sg->effects)
 		{
 			if(sg->effects[j]->duration >= 0)
-				conoutf(CON_ERROR, "ERROR: statuses[%i]->effect[%i] has a non negative duration, these will not work properly and some will be ridiculously overpowered", effects[i]->status, j);
+				conoutf(CON_ERROR, "\fs\f3ERROR:\fr statuses[%i]->effect[%i] has a non negative duration, these will not work properly and some will be ridiculously overpowered", effects[i]->status, j);
 			sg->effects[j]->update(user, user, resist, thresh, base, extra);
 		}
 	}
@@ -40,14 +40,14 @@ void use_weapon::apply(rpgchar *user) {} // does nothing, do it in the character
 bool rpgchar::checkammo(equipment &eq, equipment *quiver, bool remove)
 {
 	//it is assumed it's valid
-	use_weapon *wep = (use_weapon *) game::items[eq.base]->uses[eq.use];
+	use_weapon *wep = (use_weapon *) eq.it->uses[eq.use];
 	switch(wep->ammo)
 	{
 		case -3:
 			if(base.experience < wep->cost)
 			{
-				if(this == game::player1) conoutf("You lack sufficient exp to perform this action");
-				else if (DEBUG_AI) conoutf(CON_DEBUG, "DEBUG: AI attack interrupted for %p, too little exp", this);
+				if(this == game::player1) game::hudline("\f3You lack sufficient exp to perform this action");
+				else if (DEBUG_AI) conoutf(CON_DEBUG, "\fs\f2DEBUG:\fr AI attack interrupted for %p, too little exp", this);
 				return false;
 			}
 			if(remove) base.experience -= wep->cost;
@@ -56,8 +56,8 @@ bool rpgchar::checkammo(equipment &eq, equipment *quiver, bool remove)
 		{
 			if((wep->ammo == -1 ? mana : health) < wep->cost)
 			{
-				if(this == game::player1) conoutf(wep->ammo == -1 ? "You lack the mana to cast the full spell" : "Doing this would be suicide");
-				else if (DEBUG_AI) conoutf(CON_DEBUG, "DEBUG: AI attack interrupted for %p, too little %s", this, wep->ammo == -1 ? "mana" :"health");
+				if(this == game::player1) game::hudline(wep->ammo == -1 ? "\f3You lack the mana to cast the full spell" : "\f3Doing this would be suicide");
+				else if (DEBUG_AI) conoutf(CON_DEBUG, "\fs\f2DEBUG:\fr AI attack interrupted for %p, too little %s", this, wep->ammo == -1 ? "mana" :"health");
 				return false;
 			}
 			if(remove) (wep->ammo == -1 ? mana : health) -= wep->cost;
@@ -67,32 +67,33 @@ bool rpgchar::checkammo(equipment &eq, equipment *quiver, bool remove)
 		{
 			if(!game::ammotypes.inrange(wep->ammo))
 			{
-				conoutf(CON_ERROR, "ERROR: entity %p trying to attack with weapon using invalid ammotype %i; out of range", wep->ammo);
+				conoutf(CON_ERROR, "\fs\f3ERROR:\fr entity %p trying to attack with weapon using invalid ammotype %i; out of range", wep->ammo);
 				return false;
 			}
 			ammotype *at = game::ammotypes[wep->ammo];
-			int base = -1;
+			item *it = NULL;
 
-			if(quiver)     base = at->items.find(quiver->base);
-			if(base == -1) base = at->items.find(eq.base);
-			if(base == -1)
+			if(quiver && at->items.find(quiver->it->base) >= 0) it = quiver->it;
+			else if (at->items.find(eq.it->base) >= 0) it = eq.it;
+			else
 			{
-				if(this == game::player1) conoutf("You have the wrong ammo equipped for the current weapon");
-				else if (DEBUG_AI) conoutf(CON_DEBUG, "DEBUG: AI attack interrupted for %p, wrong ammo equipped", this);
+				if(this == game::player1) game::hudline("You have the wrong ammo equipped for the current weapon");
+				else if (DEBUG_AI) conoutf(CON_DEBUG, "\fs\f2DEBUG:\fr AI attack interrupted for %p, wrong ammo equipped", this);
 				return false;
 			}
 
-			if(getitemcount(at->items[base]) < wep->cost)
+			if(wep->cost > 0 && getcount(it) < wep->cost)
 			{
-				if(this == game::player1) conoutf("You have too little ammo remaining to attack again");
-				else if (DEBUG_AI) conoutf(CON_DEBUG, "DEBUG: AI attack interrupted for %p, too little ammo", this);
+				if(this == game::player1) game::hudline("You have too little ammo remaining to attack again");
+				else if (DEBUG_AI) conoutf(CON_DEBUG, "\fs\f2DEBUG:\fr AI attack interrupted for %p, too little ammo", this);
 				return false;
 			}
 
 			if(remove)
 			{
-				modinv(at->items[base], -wep->cost, false);
-				if(getitemcount(at->items[base]) == 0) return false; //this is the remove ammo pass - let it be known the quiver is no longer valid
+				drop(it, wep->cost, false);
+				//this is the remove ammo pass - let it be known the quiver is no longer valid
+				if(getcount(it) == 0) return false;
 			}
 			return true;
 		}
@@ -101,10 +102,11 @@ bool rpgchar::checkammo(equipment &eq, equipment *quiver, bool remove)
 
 VAR(lefthand, 1, 0, -1);
 
+//TODO implement alternative actions, eg block and strike with 2 handed weapons.
 void rpgchar::doattack(equipment *eleft, equipment *eright, equipment *quiver)
 {
-	use_weapon *left = eleft ? (use_weapon *) game::items[eleft->base]->uses[eleft->use] : NULL,
-	           *right = eright ? (use_weapon *) game::items[eright->base]->uses[eright->use] : NULL,
+	use_weapon *left = eleft ? (use_weapon *) eleft->it->uses[eleft->use] : NULL,
+	           *right = eright ? (use_weapon *) eright->it->uses[eright->use] : NULL,
 	           *ammo = NULL;
 
 	if(charge && !primary && !secondary)
@@ -134,9 +136,9 @@ void rpgchar::doattack(equipment *eleft, equipment *eright, equipment *quiver)
 				primary = secondary = 0;
 				return; //we lack the catalysts, don't attack
 			}
-			if(quiver && (!game::ammotypes.inrange(attack->ammo) || -1 == game::ammotypes[attack->ammo]->items.find(quiver->base))) quiver = NULL;
+			if(quiver && (!game::ammotypes.inrange(attack->ammo) || -1 == game::ammotypes[attack->ammo]->items.find(quiver->it->base))) quiver = NULL;
 		}
-		ammo = (use_weapon *) ((quiver && attack->ammo >= 0) ? game::items[quiver->base]->uses[quiver->use] : NULL);
+		ammo = (use_weapon *) ((quiver && attack->ammo >= 0) ? quiver->it->uses[quiver->use] : NULL);
 
 		float mult = attack->maxcharge;
 		if(attack->charge)
@@ -157,18 +159,11 @@ void rpgchar::doattack(equipment *eleft, equipment *eright, equipment *quiver)
 		//HACK
 		//propogate an attack signal to self and all items within instead of this
 		{
-			invstack *wep = NULL, *amm = NULL;
-			loopv(inventory)
-			{
-				if((attack == left && inventory[i]->base == eleft->base) || (right && inventory[i]->base == eright->base))
-					wep = inventory[i];
-				else if (ammo && inventory[i]->base == quiver->base)
-					amm = inventory[i];
-			}
-
-			if(wep) wep->getsignal("attacksound", false, this, (attack == left ? eleft : eright)->use);
-			if(amm) amm->getsignal("attacksound", false, this, quiver->use);
-			getsignal("attacksound", false, this); //mainly for grunts
+			if(attack == left || attack == right)
+				(attack == left ? eleft : eright)->it->getsignal("attacksound", false, this, (attack == left ? eleft : eright)->use);
+			if(ammo)
+				quiver->it->getsignal("attacksound", false, this, quiver->use);
+			getsignal("attacksound", false, this);
 		}
 
 		{
@@ -198,16 +193,14 @@ void rpgchar::doattack(equipment *eleft, equipment *eright, equipment *quiver)
 				break;
 
 			case T_SMULTI:
-			{
-				//UNIMPLEMENTED
-				break;
-			}
 			case T_SAREA:
 			{
-				//UNIMPLEMENTED
+				projectile *p = game::curmap->projs.add(new projectile());
+				p->init(this, attack == left ? eleft : eright, attack->ammo >= 0 ? quiver : NULL, 0, mult);
+				p->pflags = P_VOLATILE|P_STATIONARY;
+
 				break;
 			}
-
 			case T_HORIZ:
 			case T_VERT:
 			{
@@ -257,7 +250,30 @@ void rpgchar::doattack(equipment *eleft, equipment *eright, equipment *quiver)
 
 			case T_CONE:
 			{
-				//UNIMPLEMENTED
+				//use dot product to determine if inside cone; allow small error?
+				//draw particles by using a random value up to angle and applying a random rotation around view vector...; should probably do this elsewhere anyway...
+				vec view = vec(yaw * RAD, pitch *RAD);
+				int reach = (attack->range + (ammo ? ammo->range : 0)) * (flags & CHARGE_TRAVEL ? mult : 1) + radius;
+
+				loopv(game::curmap->objs)
+				{
+					rpgent *d = game::curmap->objs[i];
+					if(d == this) continue;
+
+					vec other = vec(d->o).sub(o).normalize();
+					if(view.dot(other) > sin(attack->angle * RAD) && o.dist(d->o) <= reach)
+					{
+						d->hit(this, attack, ammo, mult, flags, other);
+
+						if(game::effects.inrange(attack->deatheffect))
+						{
+							game::effects[attack->deatheffect]->drawsplash(vec(d->o).sub(vec(0, 0, d->eyeheight / 2)), vec(0, 0, 1), 5, 1, effect::DEATH, 1);
+						}
+					}
+				}
+
+				if(game::effects.inrange(attack->traileffect))
+					game::effects[attack->traileffect]->drawcone(this, attack, mult, effect::TRAIL_SINGLE, 0);
 				break;
 			}
 		}
@@ -273,7 +289,7 @@ void rpgchar::doattack(equipment *eleft, equipment *eright, equipment *quiver)
 
 void rpgchar::resetmdl()
 {
-	temp.mdl = mdl;
+	temp.mdl = (mdl && mdl[0]) ? mdl : DEFAULTMODEL;
 }
 
 ///REMEMBER route IS REVERSED
@@ -303,13 +319,13 @@ void rpgchar::doai(equipment *eleft, equipment *eright, equipment *quiver)
 			{
 				waypoint &second = ai::waypoints[route[route.length() - 2]];
 				const vec feet = feetpos();
-				if(first.o.dist(second.o) >= feet.dist(second.o) || feet.dist(second.o) < feet.dist(second.o))
+				if(second.o.dist(feet) <= first.o.dist(second.o) && first.o.dist(feet) < second.o.dist(feet))
 					route.pop();
 			}
 		}
 	}
 
-	if(target && aiflags & (AI_ATTACK | AI_ALERT))
+	if(target && target != this && aiflags & (AI_ATTACK | AI_ALERT))
 	{
 		vec dir = vec(lastknown).sub(feetpos());
 		dir.z += target->eyeheight - eyeheight;
@@ -329,19 +345,19 @@ void rpgchar::doai(equipment *eleft, equipment *eright, equipment *quiver)
 	}
 
 	primary = secondary = false;
-	if(target && aiflags & AI_ALERT)
+	if(target && target != this && aiflags & AI_ALERT)
 	{
 		bool left = true;
 		if(eleft)
 		{
-			attack = (use_weapon *) game::items[eleft->base]->uses[eleft->use];
+			attack = (use_weapon *) eleft->it->uses[eleft->use];
 			if(attack->type < USE_WEAPON || !attack->effects.length() || game::statuses[attack->effects[0]->status]->friendly)
 				attack = NULL;
 		}
 		if(!attack && eright)
 		{
 			left = false;
-			attack = (use_weapon *) game::items[eright->base]->uses[eright->use];
+			attack = (use_weapon *) eright->it->uses[eright->use];
 			if(attack->type < USE_WEAPON || !attack->effects.length() || game::statuses[attack->effects[0]->status]->friendly)
 				attack = NULL;
 		}
@@ -379,11 +395,10 @@ void rpgchar::doai(equipment *eleft, equipment *eright, equipment *quiver)
 	}
 }
 
+VARP(r_aiperiod, 0, 100, 1000);
+
 void rpgchar::update()
 {
-	resetmdl();
-	temp.alpha = 1;
-	temp.light = vec4(0, 0, 0, 0);
 	attack = NULL;
 
 	if(state == CS_DEAD)
@@ -396,6 +411,11 @@ void rpgchar::update()
 		mana =   min<float>(base.getmaxmp(),   mana + (base.getmpregen() * curtime / 1000.0f));
 		health = min<float>(base.getmaxhp(), health + (base.gethpregen() * curtime / 1000.0f));
 
+		static status_generic painmat(STATUS_HEALTH, -1, -1, -25, 0);
+		int mat = lookupmaterial(o);
+		if(mat & MAT_PAIN)
+			painmat.update(this, NULL, 0, 0);
+
 		if(health < 0)
 			die(NULL);
 	}
@@ -411,13 +431,21 @@ void rpgchar::update()
 		return;
 	}
 
-		//handle equipment
+	if(lastai <= lastmillis)
+	{
+		getsignal("ai update", false, this);
+		lastai = lastmillis + r_aiperiod / 2 + rnd(max(1, r_aiperiod));
+	}
+
+	//handle equipment
 	equipment *eleft = NULL, //primary
 	          *eright = NULL, //secondary
 	          *quiver = NULL;
+
+	base.resetdeltas(); //reset attribute changes
 	loopv(equipped)
 	{
-		use *usable = game::items[equipped[i]->base]->uses[equipped[i]->use];
+		use *usable = equipped[i]->it->uses[equipped[i]->use];
 		usable->apply(this);
 		if(usable->type == USE_WEAPON)
 		{
@@ -458,23 +486,21 @@ void rpgchar::update()
 
 	if(lastmillis >= lastaction)
 		doattack(eleft, eright, quiver);
-
-	base.resetdeltas();
 }
 
 void rpgchar::render(bool mainpass)
 {
 	int lastaction = 0,
-	anim = ANIM_ATTACK1,
+	anim = ANIM_MSTRIKE,
 	delay = 300,
-	hold = ANIM_HOLD1;
+	hold = ANIM_MHOLD;
 
 	vector<modelattach> attachments;
 	vec *emitter = emitters;
 
 	loopv(equipped)
 	{
-		use_armour *use = (use_armour *) game::items[equipped[i]->base]->uses[equipped[i]->use];
+		use_armour *use = (use_armour *) equipped[i]->it->uses[equipped[i]->use];
 		if(use->type < USE_ARMOUR || !use->vwepmdl || !use->slots) continue;
 
 		const char *tag = NULL;
@@ -502,7 +528,7 @@ void rpgchar::render(bool mainpass)
 	emitter = emitters;
 	if(mainpass) loopv(equipped)
 	{
-		use_armour *use = (use_armour *) game::items[equipped[i]->base]->uses[equipped[i]->use];
+		use_armour *use = (use_armour *) equipped[i]->it->uses[equipped[i]->use];
 		if(use->type < USE_ARMOUR || !use->vwepmdl || use->idlefx < 0 || !use->slots)
 			continue;
 
@@ -523,97 +549,70 @@ void rpgchar::givexp(int xp)
 		getsignal("level");
 }
 
-void rpgchar::equip(int i, int u)
+void rpgchar::equip(item *it, int u)
 {
 	if(primary || secondary || lastprimary || lastsecondary)
 	{
-		conoutf("You can't equip items while attacking!");
-		return;
-	}
-	else if(!game::items.inrange(i) || !game::items[i]->uses.inrange(u))
-	{
-		conoutf(CON_ERROR, "ERROR: unable to equip item %i with use %i; itself or the use does not exist", i, u);
+		game::hudline("You can't equip items while attacking!");
 		return;
 	}
 
-	invstack *item = NULL;
-	loopvj(inventory)
-	{
-		if(inventory[j]->base == i)
-		{
-			item = inventory[j];
-			break;
-		}
-	}
-	if(!item || !item->quantity)
-	{
-		conoutf(CON_ERROR, "ERROR: user contains no instances of item %i", i);
-		return;
-	}
-	else if(game::items[i]->uses[u]->type < USE_ARMOUR)
-	{
-		conoutf(CON_ERROR, "ERROR: invalid use type, needs to be of type weapon or armour to equip");
-		return;
-	}
+	vector<item *> &inv = inventory.access(it->base, vector<item *>());
 
-	use_armour *usecase = (use_armour *) game::items[i]->uses[u];
+	if(inv.find(it) < 0 || it->quantity <= 0 || !it->uses.inrange(u))
+		return;
 
+	use_armour *usecase = (use_armour *) it->uses[u];
 	if(!usecase->reqs.meet(base))
 	{
-		conoutf("You cannot wield this item! You do not meet the requirements!");
+		game::hudline("You cannot wield this item! You do not meet the requirements!");
 		return;
 	}
 
-	if(usecase->slots)
-		dequip(-1, usecase->slots);
-
-	equipped.add(new equipment(i, u));
-	item->getsignal("equip", false, this, u);
-	item->quantity--;
+	if(!usecase->slots || dequip(-1, usecase->slots))
+	{
+		equipped.add(new equipment(it, u));
+		it->quantity--;
+		it->getsignal("equip", false, this, u);
+	}
+	else
+		game::hudline("Unable to equip: requires slot unavailable");
 }
 
-bool rpgchar::dequip(int i, int slots)
+bool rpgchar::dequip(int base, int slots)
 {
 	if(primary || secondary || lastprimary || lastsecondary)
 	{
-		conoutf("You can't dequip items while attacking!");
+		game::hudline("You can't dequip items while attacking!");
 		return false;
 	}
+
+
+
 	int rem = 0;
-	loopvj(equipped)
+	loopv(equipped)
 	{
-		invstack *item = NULL;
-		loopvk(inventory)
+		use_armour *arm = (use_armour *) equipped[i]->it->uses[equipped[i]->use];
+
+		if((base == -1 ? true : equipped[i]->it->base == base) && (slots ? arm->slots & slots : true))
 		{
-			if(inventory[k]->base == equipped[j]->base)
+			if(equipped[i]->it->flags & item::F_CURSED)
+				rem++;
+			else
 			{
-				item = inventory[k];
-				break;
+				equipped[i]->it->quantity++;
+				rpgscript::removeminorrefs(equipped[i]);
+				delete equipped.remove(i--);
 			}
 		}
-		//this should not happen
-		if(!item)
-		{
-			item = new invstack(equipped[j]->base, 0);
-			inventory.add(item);
-		}
-
-
-		use_armour *arm = (use_armour *) game::items[equipped[j]->base]->uses[equipped[j]->use];
-		if((i == -1 ? true : equipped[j]->base == i) && (slots ? arm->slots & slots : true))
-		{
-			item->quantity++;
-			delete equipped.remove(j--);
-			rem++;
-		}
 	}
-	return rem != 0;
+	return !rem;
 }
 
 void rpgchar::die(rpgent *killer)
 {
 	if(DEBUG_ENT)
-		conoutf(CON_DEBUG, "DEBUG: ent %p killed by %p%s", this, killer, state != CS_ALIVE ? "; already dead?" : "");
+		conoutf(CON_DEBUG, "\fs\f2DEBUG:\fr ent %p killed by %p%s", this, killer, state != CS_ALIVE ? "; already dead?" : "");
 	if(state != CS_ALIVE)
 		return;
 
@@ -623,7 +622,7 @@ void rpgchar::die(rpgent *killer)
 	state = CS_DEAD;
 	health = 0;
 	route.setsize(0);
-	getsignal("death", true, killer); //in case the player has an item in his inventory that will revive him
+	getsignal("death", true, killer); //in case the player has an item in his inventory that will revive him or something
 }
 
 void rpgchar::revive(bool spawn)
@@ -655,18 +654,16 @@ void rpgchar::hit(rpgent *attacker, use_weapon *weapon, use_weapon *ammo, float
 	{
 		if(!game::statuses.inrange(weapon->effects[i]->status)) continue;
 		if(!game::statuses[weapon->effects[i]->status]->friendly) hit_friendly = 0;
-		//TODO friendly fire checks here
-		seffects.add(new victimeffect(attacker, weapon->effects[i], weapon->chargeflags, mul));
-		hit_total += game::statuses[weapon->effects[i]->status]->strength() * mul;
+		victimeffect &v = *seffects.add(new victimeffect(attacker, weapon->effects[i], weapon->chargeflags, mul));
+		loopvj(v.effects) hit_total += v.effects[j]->value();
 	}
 
 	if(ammo) loopv(ammo->effects)
 	{
 		if(!game::statuses.inrange(ammo->effects[i]->status)) continue;
 		if(!game::statuses[ammo->effects[i]->status]->friendly) hit_friendly = 0;
-		//TODO friendly fire checks here
-		seffects.add(new victimeffect(attacker, ammo->effects[i], weapon->chargeflags, mul));
-		hit_total += game::statuses[ammo->effects[i]->status]->strength() * mul;
+		victimeffect &v = *seffects.add(new victimeffect(attacker, ammo->effects[i], weapon->chargeflags, mul));
+		loopvj(v.effects) hit_total += v.effects[j]->value();
 	}
 
 	vel.add(dir.mul(weapon->kickback + (ammo ? ammo->kickback : 0)));
@@ -675,96 +672,150 @@ void rpgchar::hit(rpgent *attacker, use_weapon *weapon, use_weapon *ammo, float
 }
 
 ///Inventory
-int rpgchar::modinv(int b, int q, bool spawn)
+void rpgchar::init(int base)
 {
-	invstack *item = NULL;
+	game::loadingrpgchar = this;
 
-	loopv(inventory)
-	{
-		if(inventory[i]->base == b)
-		{
-			item = inventory[i];
-			break;
-		}
-	}
+	defformatstring(file)("data/rpg/games/%s/critters/%i.cfg", game::data, base);
+	execfile(file);
 
-	if(!item)
-		item = inventory.add(new invstack(b, 0));
+	game::loadingitem = NULL;
 
-	if(q > 0)
-	{
-		//factor weight into this?
-		//int added = q;
-		//int weight = getweight();
-
-		item->quantity += q;
-		return q;
-	}
-	else if (q < 0)
-	{
-		int removed = 0;
-		if(item->quantity >= -q)
-		{
-			removed = -q;
-			item->quantity += q;
-		}
-		else
-		{
-			removed = item->quantity;
-			item->quantity = 0;
-
-			loopvrev(equipped)
-			{
-				if(equipped[i]->base == b)
-				{
-					delete equipped.remove(i);
-					if((++removed) == -q) break;
-				}
-			}
-		}
-
-		if(spawn)
-		{
-			rpgitem *drop = new rpgitem();
-			game::curmap->objs.add(drop);
-
-			drop->base = b;
-			drop->quantity = removed;
-			drop->o = drop->newpos = vec(yaw * RAD, pitch * RAD).mul(radius * 2).add(o);
-			entinmap(drop);
-		}
-
-		return removed;
-	}
-
-	return 0;
+	health = this->base.getmaxhp();
+	mana = this->base.getmaxmp();
 }
 
-int rpgchar::pickup(rpgitem *item)
+item *rpgchar::additem(item *it)
 {
-	int add = item->quantity;
-	if(game::items.inrange(item->base) && game::items[item->base]->weight)
-		add = clamp<float>(add, 0, (base.getmaxcarry() * 2 - getweight()) / game::items[item->base]->weight);
+	vector<item *> &inv = inventory.access(it->base, vector<item *>());
 
-	modinv(item->base, add, false);
-	item->quantity -= add;
+	loopv(inv)
+	{
+		if(inv[i]->compare(it))
+		{
+			inv[i]->quantity += it->quantity;
+			return inv[i];
+		}
+	}
+
+	item *newit = inv.add(new item());
+	it->transfer(*newit);
+	return newit;
+}
+
+item *rpgchar::additem(int base, int q)
+{
+	item *it = new item();
+	it->init(base);
+	it->quantity = q;
+	item *ret = additem(it);
+	delete it;
+	return ret;
+}
+
+int rpgchar::drop(item *it, int q, bool spawn)
+{
+	vector<item *> &inv = inventory.access(it->base, vector<item *>());
+	int rem = 0;
+	if(inv.find(it) == -1) return 0;
+
+	rem = min(q, it->quantity);
+	it->quantity -= rem;
+
+	loopv(equipped)
+	{
+		if(rem >= q) break;
+		if(equipped[i]->it == it)
+		{
+			rem++;
+			rpgscript::removeminorrefs(equipped[i]);
+			delete equipped.remove(i--);
+		}
+	}
+
+	if(rem && spawn)
+	{
+		rpgitem *drop = new rpgitem();
+		it->transfer(*drop);
+		game::curmap->objs.add(drop);
+
+		drop->quantity = rem;
+		drop->o = vec(yaw * RAD, pitch * RAD).mul(radius * 2).add(o);
+		entinmap(drop);
+	}
+
+	return rem;
+}
+
+int rpgchar::drop(int base, int q, bool spawn)
+{
+	vector<item *> &inv = inventory.access(base, vector<item *>());
+	int rem = 0;
+
+	loopv(inv)
+		rem += min(q - rem, inv[i]->quantity);
+
+	loopv(equipped)
+	{
+		if(rem >= q) break;
+		if(equipped[i]->it->base == base)
+		{
+			equipped[i]->it->quantity++;
+			rem++;
+			rpgscript::removeminorrefs(equipped[i]);
+			delete equipped.remove(i--);
+		}
+	}
+
+	q = rem;
+
+	loopv(inv)
+		rem -= drop(inv[i], min(inv[i]->quantity, rem), spawn);
+
+	return q;
+}
+
+int rpgchar::pickup(rpgitem *it)
+{
+	int add = it->quantity;
+	if(it->weight)
+		add = clamp(add, 0.f, (base.getmaxcarry() * 2 - getweight()) / it->weight);
+
+	vector<item *> &inv = inventory.access(it->base, vector<item *>());
+	loopv(inv) if(inv[i]->compare(it))
+	{
+		it->quantity -= add;
+		inv[i]->quantity += add;
+		return add;
+	}
+
+	item *n = inv.add(new item());
+	it->transfer(*n);
+
+	it->quantity -= add;
+	n->quantity = add;
 
 	return add;
 }
 
-int rpgchar::getitemcount(int i)
+int rpgchar::getitemcount(int base)
 {
+	vector<item *> &inv = inventory.access(base, vector<item *>());
+
 	int count = 0;
-	loopvj(inventory)
-	{
-		if(inventory[j]->base == i)
-			count += inventory[j]->quantity;
-	}
-	loopvj(equipped)
-	{
-		if(equipped[j]->base == i)
-			count++;
-	}
+	loopv(inv) count += inv[i]->quantity;
+	loopv(equipped) if(equipped[i]->it->base == base) count++;
+
+	return count;
+}
+
+int rpgchar::getcount(item *it)
+{
+	vector<item *> &inv = inventory.access(it->base, vector<item *>());
+
+	int count = 0;
+	if(inv.find(it) >= 0) count += it->quantity;
+	loopv(equipped) if(equipped[i]->it == it) count++;
 
 	return count;
 }
@@ -772,16 +823,13 @@ int rpgchar::getitemcount(int i)
 float rpgchar::getweight()
 {
 	float ret = 0;
-	loopv(inventory)
-	{
-		if(game::items.inrange(inventory[i]->base))
-			ret += inventory[i]->quantity * game::items[inventory[i]->base]->weight;
-	}
+
+	enumerate(inventory, vector<item *>, stack,
+		loopvj(stack) ret += stack[j]->quantity * stack[j]->weight;
+	)
 	loopv(equipped)
-	{
-		if(game::items.inrange(equipped[i]->base))
-			ret += game::items[equipped[i]->base]->weight;
-	}
+		ret += equipped[i]->it->weight;
+
 	return ret;
 }
 
