@@ -17,10 +17,12 @@ This file is configures the following items
 * platforms
 * triggers
 * mapscripts
+* merchants
 
 This file is also used to declare the following items (but disallows subsequent changes)
 
 * Global Variables
+* Item Categories
 
 */
 
@@ -31,19 +33,23 @@ namespace game
 		static x *check ## x () \
 		{ \
 			if(! loading ## x) \
-				conoutf(CON_ERROR, "ERROR: " #c " not defined or being loaded"); \
+				conoutf(CON_ERROR, "\fs\f3ERROR:\fr " #c " not defined or being loaded"); \
 			return loading ## x; \
 		} \
 		ICOMMAND(r_select_ ## c, "se", (const char *ref, uint *contents), \
 			x *old = loading ## x; \
 			loading ## x = NULL; \
 			\
+			int objidx; \
+			rpgscript::parseref(ref, objidx);\
 			reference *obj = rpgscript::searchstack(ref, false); \
 			if(obj) \
 			{ \
-				loading ## x = select; \
+				int idx = select; \
+				if((vec).inrange(idx)) \
+					loading ## x = (vec)[idx]; \
 				if(loading ## x && DEBUG_VCONF) \
-					conoutf("DEBUG: successfully selected \"" #c "\" from reference %s", ref); \
+					conoutf("\fs\f2DEBUG:\fr successfully selected \"" #c "\" from reference %s", ref); \
 			} \
 			\
 			if(!loading ##x) \
@@ -52,46 +58,85 @@ namespace game
 				if(vec.inrange(idx)) \
 				{ \
 					if(DEBUG_VCONF) \
-						conoutf("DEBUG: successfully selected \"" #c "\" using index %i", idx); \
+						conoutf("\fs\f2DEBUG:\fr successfully selected \"" #c "\" using index %i", idx); \
 					loading ## x = vec[idx]; \
 				} \
 			} \
 			if(loading ## x) \
 				execute(contents); \
 			else \
-				conoutf(CON_ERROR, "ERROR: unable to select reference %s as type " #c, ref); \
+				conoutf(CON_ERROR, "\fs\f3ERROR:\fr unable to select reference %s as type " #c, ref); \
 			loading ## x = old; \
 		) \
 		ICOMMAND(r_num_ ## c, "", (), \
 			if(DEBUG_VCONF) \
-				conoutf("DEBUG: r_num_" #c " requested, returning %i", vec.length()); \
+				conoutf("\fs\f2DEBUG:\fr r_num_" #c " requested, returning %i", vec.length()); \
 			intret(vec.length()); \
 		)
 
-	CHECK(script, script, scripts, NULL)
-	CHECK(effect, effect, effects, NULL)
+	CHECK(script, script, scripts, -1)
+	CHECK(effect, effect, effects, -1)
 	CHECK(statusgroup, status, statuses,
-		obj->getveffect() ? statuses[obj->getveffect()->group] :
-		obj->getaeffect() ? statuses[obj->getaeffect()->group] :
-		NULL
+		obj->getveffect(objidx) ? obj->getveffect(objidx)->group :
+		obj->getaeffect(objidx) ? obj->getaeffect(objidx)->group :
+		-1
 	)
-	CHECK(item_base, item, items,
-		obj->getinv() ? items[obj->getinv()->base] :
-		obj->getequip() ? items[obj->getequip()->base] :
-		NULL
-	)
-	CHECK(recipe, recipe, recipes, NULL)
-	CHECK(ammotype, ammo, ammotypes, NULL)
-	CHECK(char_base, char, chars, obj->getchar())
 	CHECK(faction, faction, factions,
-		  obj->getchar() ? factions[obj->getchar()->faction] :
-		  NULL
+		  obj->getchar(objidx) ? obj->getchar(objidx)->faction :
+		  obj->getcontainer(objidx) ? obj->getcontainer(objidx)->faction :
+		  -1
 	)
-	CHECK(obstacle_base, obstacle, obstacles, obj->getobstacle())
-	CHECK(container_base, container, containers, obj->getcontainer())
-	CHECK(platform_base, platform, platforms, obj->getplatform())
-	CHECK(trigger_base, trigger, triggers, obj->gettrigger())
-	CHECK(mapscript, mapscript, mapscripts, NULL)
+	CHECK(recipe, recipe, recipes, -1)
+	CHECK(ammotype, ammo, ammotypes, -1)
+	CHECK(mapscript, mapscript, mapscripts, -1)
+	CHECK(merchant, merchant, merchants,
+		obj->getchar(objidx) ? obj->getchar(objidx)->merchant :
+		obj->getcontainer(objidx) ? obj->getcontainer(objidx)->merchant :
+		-1
+	)
+
+	#undef CHECK
+
+	//These don't have a vector counter part
+
+	#define CHECK(x, c, select) \
+		x *loading ## x = NULL; \
+		static x *check ##x () \
+		{ \
+			if(! loading ## x) \
+				conoutf(CON_ERROR, "\fs\f3ERROR:\fr " #c " not being loaded"); \
+			return loading ## x; \
+		} \
+		ICOMMAND(r_select_ ## c, "se", (const char *ref, uint *body),  \
+			x *old = loading ## x; \
+			loading ## x = NULL; \
+			 \
+			int objidx; \
+			rpgscript::parseref(ref, objidx); \
+			reference *obj = rpgscript::searchstack(ref, false); \
+			if(obj) \
+			{ \
+				loading ## x = select; \
+				if(loading ## x && DEBUG_VCONF) \
+					conoutf("\fs\f2DEBUG:\fr successfully selected \"" #c "\" from reference %s", ref); \
+			} \
+			if(loading ## x) \
+				execute(body); \
+			else \
+				conoutf(CON_ERROR, "\fs\f3ERROR:\fr unable to select reference %s as type " #c, ref); \
+			loading ## x = old; \
+		)
+
+	CHECK(item, item,
+		obj->getinv(objidx) ? obj->getinv(objidx) :
+		obj->getequip(objidx) ? obj->getequip(objidx)->it :
+		NULL
+	)
+	CHECK(rpgchar, char, obj->getchar(objidx))
+	CHECK(rpgobstacle, obstacle, obj->getobstacle(objidx))
+	CHECK(rpgcontainer, container, obj->getcontainer(objidx))
+	CHECK(rpgplatform, platform, obj->getplatform(objidx))
+	CHECK(rpgtrigger, trigger, obj->gettrigger(objidx))
 
 	#undef CHECK
 
@@ -99,9 +144,9 @@ namespace game
 	static use *checkuse(int type)
 	{
 		if(!loadinguse)
-			conoutf(CON_ERROR, "ERROR: use not defined or being loaded");
+			conoutf(CON_ERROR, "\fs\f3ERROR:\fr use not defined or being loaded");
 		else if(loadinguse->type < type)
-			conoutf(CON_ERROR, "ERROR: trying to set an unavailable use property");
+			conoutf(CON_ERROR, "\fs\f3ERROR:\fr trying to set an unavailable use property");
 		else
 			return loadinguse;
 		return NULL;
@@ -109,32 +154,34 @@ namespace game
 	ICOMMAND(r_select_item_use, "se", (const char *ref, uint *contents),
 		use *old = loadinguse;
 		loadinguse = NULL;
+		int objidx;
+		rpgscript::parseref(ref, objidx);
 		reference *obj = rpgscript::searchstack(ref, false);
 
-		if(obj && obj->getequip())
+		if(obj && obj->getequip(objidx))
 		{
 			if(DEBUG_VCONF)
-				conoutf("DEBUG: successfully selected \"use\" from reference %s", ref);
-			loadinguse = items[obj->getequip()->base]->uses[obj->getequip()->use];
+				conoutf("\fs\f2DEBUG:\fr successfully selected \"use\" from reference %s", ref);
+			loadinguse = obj->getequip(objidx)->it->uses[obj->getequip(objidx)->use];
 		}
-		else if(loadingitem_base)
+		else if(loadingitem)
 		{
 			int idx = parseint(ref);
-			if(loadingitem_base->uses.inrange(idx))
+			if(loadingitem->uses.inrange(idx))
 			{
 				if(DEBUG_VCONF)
-					conoutf("DEBUG: successfully selected \"use\" using index %i", idx);
-				loadinguse = loadingitem_base->uses[idx];
+					conoutf("\fs\f2DEBUG:\fr successfully selected \"use\" using index %i", idx);
+				loadinguse = loadingitem->uses[idx];
 			}
 		}
 
 		if(loadinguse) execute(contents);
-		else conoutf(CON_ERROR, "ERROR: unable to select reference %s as type use", ref);
+		else conoutf(CON_ERROR, "\fs\f3ERROR:\fr unable to select reference %s as type use", ref);
 		loadinguse = old;
 	)
 	ICOMMAND(r_num_item_use, "", (),
-		if(loadingitem_base)
-			intret(loadingitem_base->uses.length());
+		if(loadingitem)
+			intret(loadingitem->uses.length());
 		else intret(0);
 	)
 
@@ -145,50 +192,85 @@ namespace game
 	/// #define DEBUG "scripts[%i]"
 	/// #define DEBUG_IND scripts.length() - 1
 
-	#define INTN(name, var, min, max) \
+	#define INTSET(name, var, min, max) \
 		START(name, "i", (int *i), \
 			INIT \
-			if(!e) return; \
+			if(!e) \
+			{ \
+				conoutf(CON_ERROR, "\fs\f3ERROR:\fr " #name " requires you to have a valid object selected first"); \
+				return; \
+			} \
 			e->var = clamp((int) max, (int) min, *i); \
 			if(e->var != *i) \
-				conoutf("WARNING: value provided for " DEBUG "->" #var " exceeded limits", DEBUG_IND); \
+				conoutf("\fs\f6WARNING:\fr value provided for " DEBUG "->" #var " exceeded limits", DEBUG_IND); \
 			if(DEBUG_CONF || e->var != *i) \
 				conoutf(DEBUG "->" #var " = %i (%.8X)", DEBUG_IND, e->var, e->var); \
-		) \
+		)
+
+	#define INTGET(name, var) \
 		START(name ## _get, "", (), \
 			INIT \
-			if(!e) return; \
+			if(!e) \
+			{ \
+				conoutf(CON_ERROR, "\fs\f3ERROR:\fr " #name " requires you to have a valid object selected first"); \
+				return; \
+			} \
 			if(DEBUG_VCONF) \
-				conoutf("DEBUG: request for " #name " on %p, returning %i", e, e->var); \
+				conoutf("\fs\f2DEBUG:\fr request for " #name " on %p, returning %i", e, e->var); \
 			intret(e->var); \
 		)
+
+	#define INTN(name, var, min, max) \
+		INTSET(name, var, min, max) \
+		INTGET(name, var)
+
 	#define INT(var, min, max) INTN(var, var, min, max)
 
-	#define FLOATN(name, var, min, max) \
+	#define FLOATSET(name, var, min, max) \
 		START(name, "f", (float *f), \
 			INIT \
-			if(!e) return; \
+			if(!e) \
+			{ \
+				conoutf(CON_ERROR, "\fs\f3ERROR:\fr " #name " requires you to have a valid object selected first"); \
+				return; \
+			} \
 			e->var = clamp((float) max, (float) min, *f); \
 			if(e->var != *f) \
-				conoutf("WARNING: value provided for " DEBUG "->" #var " exceeded limits", DEBUG_IND); \
+				conoutf("\fs\f6WARNING:\fr value provided for " DEBUG "->" #var " exceeded limits", DEBUG_IND); \
 			if(DEBUG_CONF || e->var != *f) \
 				conoutf(DEBUG "->" #var " = %g", DEBUG_IND, e->var); \
-		) \
+		)
+
+	#define FLOATGET(name, var) \
 		START(name ## _get, "", (), \
 			INIT \
-			if(!e) return; \
+			if(!e) \
+			{ \
+				conoutf(CON_ERROR, "\fs\f3ERROR:\fr " #name " requires you to have a valid object selected first"); \
+				return; \
+			} \
 			if(DEBUG_VCONF) \
-				conoutf("DEBUG: request for " #name " on %p, returning %f", e, e->var); \
+				conoutf("\fs\f2DEBUG:\fr request for " #name " on %p, returning %f", e, e->var); \
 			floatret(e->var); \
 		)
+
+	#define FLOATN(name, var, min, max) \
+		FLOATSET(name, var, min, max) \
+		FLOATGET(name, var)
+
 	#define FLOAT(var, min, max) FLOATN(var, var, min, max)
 
-	#define STRINGN(name, var) \
+	#define STRINGN(name, var, body) \
 		START(name, "C", (const char *s), \
 			INIT \
-			if(!e) return; \
+			if(!e) \
+			{ \
+				conoutf(CON_ERROR, "\fs\f3ERROR:\fr " #name " requires you to have a valid object selected first"); \
+				return; \
+			} \
 			if(e->var) delete[] e->var; \
 			e->var = newstring(s); \
+			body; \
 			if(DEBUG_CONF) \
 				conoutf(CON_DEBUG, DEBUG "->" #var " = %s", DEBUG_IND, e->var); \
 		) \
@@ -196,10 +278,11 @@ namespace game
 			INIT \
 			if(!e) return; \
 			if(DEBUG_VCONF) \
-				conoutf("DEBUG: request for " #name " on %p, returning %s", e, e->var); \
+				conoutf("\fs\f2DEBUG:\fr request for " #name " on %p, returning %s", e, e->var); \
 			result(e->var ? e->var : ""); \
 		)
-	#define STRING(var) STRINGN(var, var)
+	#define STRING(var) STRINGN(var, var, )
+	#define MODEL(var) STRINGN(var, var, preloadmodel(e->var))
 
 	#define VECN(name, var, l1, l2, l3, h1, h2, h3) \
 		START(name, "fff", (float *x, float *y, float *z), \
@@ -209,7 +292,7 @@ namespace game
 			e->var.y = clamp((float)h2, (float)l2, *y); \
 			e->var.z = clamp((float)h3, (float)l3, *z); \
 			if(e->var.x != *x || e->var.y != *y || e->var.z != *z) \
-				conoutf("WARNING: value provided for " DEBUG "->" #var " exceeded limits", DEBUG_IND); \
+				conoutf("\fs\f6WARNING:\fr value provided for " DEBUG "->" #var " exceeded limits", DEBUG_IND); \
 			if(DEBUG_CONF || e->var.x != *x || e->var.y != *y || e->var.z != *z) \
 				conoutf(DEBUG "->" #var " = (%g, %g, %g)", DEBUG_IND, e->var.x, e->var.y, e->var.z); \
 		) \
@@ -218,7 +301,7 @@ namespace game
 			if(!e) return; \
 			defformatstring(res)("%g %g %g", e->var.x, e->var.y, e->var.z); \
 			if(DEBUG_VCONF) \
-				conoutf("DEBUG: request for " #name " on %p, returning %s", e, res); \
+				conoutf("\fs\f2DEBUG:\fr request for " #name " on %p, returning %s", e, res); \
 			result(res); \
 		)
 	#define VEC(var, l1, l2, l3, h1, h2, h3) VECN(var, var, l1, l2, l3, h1, h2, h3)
@@ -235,19 +318,19 @@ namespace game
 			INIT \
 			if(!e) return; \
 			if(DEBUG_VCONF) \
-				conoutf("DEBUG: request for " #name " on %p, returning %i", e, e->var); \
+				conoutf("\fs\f2DEBUG:\fr request for " #name " on %p, returning %i", e, e->var); \
 			intret(e->var); \
 		)
 	#define BOOL(var) BOOLN(var, var)
 
 	#define STATREQ(var) \
-		INTN(var ## _strength, var.attrs[STAT_STRENGTH], 1, 100) \
-		INTN(var ## _endurance, var.attrs[STAT_ENDURANCE], 1, 100) \
-		INTN(var ## _agility, var.attrs[STAT_AGILITY], 1, 100) \
-		INTN(var ## _charisma, var.attrs[STAT_CHARISMA], 1, 100) \
-		INTN(var ## _wisdom, var.attrs[STAT_WISDOM], 1, 100) \
-		INTN(var ## _intelligence, var.attrs[STAT_INTELLIGENCE], 1, 100) \
-		INTN(var ## _luck, var.attrs[STAT_LUCK], 1, 100) \
+		INTN(var ## _strength, var.attrs[STAT_STRENGTH], 0, 100) \
+		INTN(var ## _endurance, var.attrs[STAT_ENDURANCE], 0, 100) \
+		INTN(var ## _agility, var.attrs[STAT_AGILITY], 0, 100) \
+		INTN(var ## _charisma, var.attrs[STAT_CHARISMA], 0, 100) \
+		INTN(var ## _wisdom, var.attrs[STAT_WISDOM], 0, 100) \
+		INTN(var ## _intelligence, var.attrs[STAT_INTELLIGENCE], 0, 100) \
+		INTN(var ## _luck, var.attrs[STAT_LUCK], 0, 100) \
 		\
 		INTN(var ## _armour, var.skills[SKILL_ARMOUR], 0, 100) \
 		INTN(var ## _diplomacy, var.skills[SKILL_DIPLOMACY], 0, 100) \
@@ -260,7 +343,8 @@ namespace game
 	#define STATS(var) \
 		INTN(var ## _level, var.level, 1, 1000) \
 		INTN(var ## _experience, var.experience, 0, 0x7FFFFFFF) \
-		INTN(var ## _points, var.points, 0, 500) \
+		INTN(var ## _statpoints, var.statpoints, 0, 500) \
+		INTN(var ## _skillpoints, var.skillpoints, 0, 500) \
 		INTN(var ## _strength, var.baseattrs[STAT_STRENGTH], 1, 100) \
 		INTN(var ## _endurance, var.baseattrs[STAT_ENDURANCE], 1, 100) \
 		INTN(var ## _agility, var.baseattrs[STAT_AGILITY], 1, 100) \
@@ -269,6 +353,14 @@ namespace game
 		INTN(var ## _intelligence, var.baseattrs[STAT_INTELLIGENCE], 1, 100) \
 		INTN(var ## _luck, var.baseattrs[STAT_LUCK], 1, 100) \
 		\
+		INTGET(var ## _delta_strength, var.deltaattrs[STAT_STRENGTH]) \
+		INTGET(var ## _delta_endurance, var.deltaattrs[STAT_ENDURANCE]) \
+		INTGET(var ## _delta_agility, var.deltaattrs[STAT_AGILITY]) \
+		INTGET(var ## _delta_charisma, var.deltaattrs[STAT_CHARISMA]) \
+		INTGET(var ## _delta_wisdom, var.deltaattrs[STAT_WISDOM]) \
+		INTGET(var ## _delta_intelligence, var.deltaattrs[STAT_INTELLIGENCE]) \
+		INTGET(var ## _delta_luck, var.deltaattrs[STAT_LUCK]) \
+		\
 		INTN(var ## _armour, var.baseskills[SKILL_ARMOUR], 0, 100) \
 		INTN(var ## _diplomacy, var.baseskills[SKILL_DIPLOMACY], 0, 100) \
 		INTN(var ## _magic, var.baseskills[SKILL_MAGIC], 0, 100) \
@@ -277,24 +369,62 @@ namespace game
 		INTN(var ## _stealth, var.baseskills[SKILL_STEALTH], 0, 100) \
 		INTN(var ## _craft, var.baseskills[SKILL_CRAFT], 0, 100) \
 		\
+		INTGET(var ## _delta_armour, var.deltaskills[SKILL_ARMOUR]) \
+		INTGET(var ## _delta_diplomacy, var.deltaskills[SKILL_DIPLOMACY]) \
+		INTGET(var ## _delta_magic, var.deltaskills[SKILL_MAGIC]) \
+		INTGET(var ## _delta_marksman, var.deltaskills[SKILL_MARKSMAN]) \
+		INTGET(var ## _delta_melee, var.deltaskills[SKILL_MELEE]) \
+		INTGET(var ## _delta_stealth, var.deltaskills[SKILL_STEALTH]) \
+		INTGET(var ## _delta_craft, var.deltaskills[SKILL_CRAFT]) \
+		\
 		INTN(var ## _fire_thresh, var.bonusthresh[ATTACK_FIRE], -500, 500) \
 		INTN(var ## _water_thresh, var.bonusthresh[ATTACK_WATER], -500, 500) \
 		INTN(var ## _air_thresh, var.bonusthresh[ATTACK_AIR], -500, 500) \
 		INTN(var ## _earth_thresh, var.bonusthresh[ATTACK_EARTH], -500, 500) \
-		INTN(var ## _magic_thresh, var.bonusthresh[ATTACK_MAGIC], -500, 500) \
+		INTN(var ## _arcane_thresh, var.bonusthresh[ATTACK_ARCANE], -500, 500) \
+		INTN(var ## _mind_thresh, var.bonusthresh[ATTACK_MIND], -500, 500) \
+		INTN(var ## _holy_thresh, var.bonusthresh[ATTACK_HOLY], -500, 500) \
+		INTN(var ## _darkness_thresh, var.bonusthresh[ATTACK_DARKNESS], -500, 500) \
 		INTN(var ## _slash_thresh, var.bonusthresh[ATTACK_SLASH], -500, 500) \
 		INTN(var ## _blunt_thresh, var.bonusthresh[ATTACK_BLUNT], -500, 500) \
 		INTN(var ## _pierce_thresh, var.bonusthresh[ATTACK_PIERCE], -500, 500) \
 		\
+		INTGET(var ## _delta_fire_thresh, var.deltathresh[ATTACK_FIRE]) \
+		INTGET(var ## _delta_water_thresh, var.deltathresh[ATTACK_WATER]) \
+		INTGET(var ## _delta_air_thresh, var.deltathresh[ATTACK_AIR]) \
+		INTGET(var ## _delta_earth_thresh, var.deltathresh[ATTACK_EARTH]) \
+		INTGET(var ## _delta_arcane_thresh, var.deltathresh[ATTACK_ARCANE]) \
+		INTGET(var ## _delta_mind_thresh, var.deltathresh[ATTACK_MIND]) \
+		INTGET(var ## _delta_holy_thresh, var.deltathresh[ATTACK_HOLY]) \
+		INTGET(var ## _delta_darkness_thresh, var.deltathresh[ATTACK_DARKNESS]) \
+		INTGET(var ## _delta_slash_thresh, var.deltathresh[ATTACK_SLASH]) \
+		INTGET(var ## _delta_blunt_thresh, var.deltathresh[ATTACK_BLUNT]) \
+		INTGET(var ## _delta_pierce_thresh, var.deltathresh[ATTACK_PIERCE]) \
+		\
 		INTN(var ## _fire_resist, var.bonusresist[ATTACK_FIRE], -200, 100) \
 		INTN(var ## _water_resist, var.bonusresist[ATTACK_WATER], -200, 100) \
 		INTN(var ## _air_resist, var.bonusresist[ATTACK_AIR], -200, 100) \
 		INTN(var ## _earth_resist, var.bonusresist[ATTACK_EARTH], -200, 100) \
-		INTN(var ## _magic_resist, var.bonusresist[ATTACK_MAGIC], -200, 100) \
+		INTN(var ## _arcane_resist, var.bonusresist[ATTACK_ARCANE], -200, 100) \
+		INTN(var ## _mind_resist, var.bonusresist[ATTACK_MIND], -200, 100) \
+		INTN(var ## _holy_resist, var.bonusresist[ATTACK_HOLY], -200, 100) \
+		INTN(var ## _darkness_resist, var.bonusresist[ATTACK_DARKNESS], -200, 100) \
 		INTN(var ## _slash_resist, var.bonusresist[ATTACK_SLASH], -200, 100) \
 		INTN(var ## _blunt_resist, var.bonusresist[ATTACK_BLUNT], -200, 100) \
 		INTN(var ## _pierce_resist, var.bonusresist[ATTACK_PIERCE], -200, 100) \
 		\
+		INTGET(var ## _delta_fire_resist, var.deltaresist[ATTACK_FIRE]) \
+		INTGET(var ## _delta_water_resist, var.deltaresist[ATTACK_WATER]) \
+		INTGET(var ## _delta_air_resist, var.deltaresist[ATTACK_AIR]) \
+		INTGET(var ## _delta_earth_resist, var.deltaresist[ATTACK_EARTH]) \
+		INTGET(var ## _delta_arcane_resist, var.deltaresist[ATTACK_ARCANE]) \
+		INTGET(var ## _delta_mind_resist, var.deltaresist[ATTACK_MIND]) \
+		INTGET(var ## _delta_holy_resist, var.deltaresist[ATTACK_HOLY]) \
+		INTGET(var ## _delta_darkness_resist, var.deltaresist[ATTACK_DARKNESS]) \
+		INTGET(var ## _delta_slash_resist, var.deltaresist[ATTACK_SLASH]) \
+		INTGET(var ## _delta_blunt_resist, var.deltaresist[ATTACK_BLUNT]) \
+		INTGET(var ## _delta_pierce_resist, var.deltaresist[ATTACK_PIERCE]) \
+		\
 		INTN(var ## _maxspeed, var.bonusmovespeed, 0, 100) \
 		INTN(var ## _jumpvel, var.bonusjumpvel, 0, 100) \
 		INTN(var ## _maxhealth, var.bonushealth, 0, 100000) \
@@ -302,59 +432,79 @@ namespace game
 		INTN(var ## _crit, var.bonuscrit, -100, 100) \
 		FLOATN(var ## _healthregen, var.bonushregen, 0, 1000) \
 		FLOATN(var ## _manaregen, var.bonusmregen, 0, 1000) \
+		\
+		INTGET(var ## _delta_maxspeed, var.deltamovespeed) \
+		INTGET(var ## _delta_jumpvel, var.deltajumpvel) \
+		INTGET(var ## _delta_maxhealth, var.deltahealth) \
+		INTGET(var ## _delta_maxmana, var.deltamana) \
+		INTGET(var ## _delta_crit, var.deltacrit) \
+		FLOATGET(var ## _delta_healthregen, var.deltahregen) \
+		FLOATGET(var ## _delta_manaregen, var.deltamregen) \
 
-	ICOMMAND(r_script_say, "ss", (char *t, char *s),
+	ICOMMAND(r_script_node, "see", (const char *n, uint *txt, uint *scr),
+		script *s = checkscript();
+		if(!s) return;
+
+		dialogue *node = s->chat.access(n);
+		if(!node)
+		{
+			static dialogue dummy;
+			const char *name = newstring(n);
+
+			node = &s->chat.access(name, dummy);
+			node->node = name;
+		}
+
+		freecode(node->text);
+		node->text = txt;
+		keepcode(txt);
+
+		freecode(node->script);
+		node->script = scr;
+		keepcode(scr);
+	)
+
+	signal *getsignal(const char *sig, hashtable<const char *, signal> &table)
+	{
+		signal *listen = table.access(sig);
+		if(listen) return listen;
+
+		const char *name = newstring(sig);
+		listen = &table.access(name, signal());
+		listen->name = name;
+		return listen;
+	}
+
+	ICOMMAND(r_script_signal, "se", (const char *sig, uint *code),
 		script *scr = checkscript();
 		if(!scr) return;
 
-		scr->dialogue.add(new rpgchat(newstring(t), compilecode(s)));
-
-		if(DEBUG_CONF)
-		{
-			conoutf(CON_DEBUG, "script[%i]->dialogue[%i]->talk = \"%s\"", scripts.length() - 1, scr->dialogue.length() - 1, t);
-			conoutf(CON_DEBUG, "script[%i]->dialogue[%i]->script = \"%s\"", scripts.length() - 1, scr->dialogue.length() - 1, s);
-		}
+		signal *listen = getsignal(sig, scr->listeners);
+		listen->setcode(code);
 	)
 
-	ICOMMAND(r_script_signal, "ss", (const char *sig, const char *code),
+	ICOMMAND(r_script_signal_append, "se", (const char *sig, uint *code),
 		script *scr = checkscript();
 		if(!scr) return;
 
-		signal *listen = scr->listeners.access(sig);
-		if(!listen)
-		{
-			const char *name = newstring(sig);
-			listen = &scr->listeners.access(name, signal());
-			listen->name = name;
-			if(DEBUG_CONF)
-				conoutf(CON_DEBUG, "DEBUG: scripts[%i], creating signal slot %p (%s)", scripts.find(scr), listen, sig);
-		}
-		else if(DEBUG_CONF)
-			conoutf(CON_DEBUG, "DEBUG: scripts[%i], reusing signal slot %p (%s)", scripts.find(scr), listen, sig);
-
-		if(DEBUG_CONF) conoutf(CON_DEBUG, "DEBUG: scripts[%i] = [\n%s\n]", scripts.find(scr), code);
-		delete[] listen->code; listen->code = compilecode(code);
+		signal *listen = getsignal(sig, scr->listeners);
+		listen->appendcode(code);
 	)
 
-
-	ICOMMAND(r_mapscript_signal, "ss", (const char *sig, const char *code),
+	ICOMMAND(r_mapscript_signal, "se", (const char *sig, uint *code),
 		mapscript *scr = checkmapscript();
 		if(!scr) return;
 
-		signal *listen = scr->listeners.access(sig);
-		if(!listen)
-		{
-			const char *name = newstring(sig);
-			listen = &scr->listeners.access(name, signal());
-			listen->name = name;
-			if(DEBUG_CONF)
-				conoutf(CON_DEBUG, "DEBUG: mapscripts[%i], creating signal slot %p (%s)", mapscripts.find(scr), listen, sig);
-		}
-		else if(DEBUG_CONF)
-			conoutf(CON_DEBUG, "DEBUG: mapscripts[%i], reusing signal slot %p (%s)", mapscripts.find(scr), listen, sig);
+		signal *listen = getsignal(sig, scr->listeners);
+		listen->setcode(code);
+	)
 
-		if(DEBUG_CONF) conoutf(CON_DEBUG, "DEBUG: mapscripts[%i] = [\n%s\n]", mapscripts.find(scr), code);
-		delete[] listen->code; listen->code = compilecode(code);
+	ICOMMAND(r_mapscript_signal_append, "se", (const char *sig, uint *code),
+		mapscript *scr = checkmapscript();
+		if(!scr) return;
+
+		signal *listen = getsignal(sig, scr->listeners);
+		listen->appendcode(code);
 	)
 
 	#define START(n, f, a, b) ICOMMAND(r_effect_ ##n, f, a, b)
@@ -362,10 +512,11 @@ namespace game
 	#define DEBUG "effect[%i]"
 	#define DEBUG_IND effects.find(e)
 
-	INT(flags, 0, FX_MAX - 1)
+	INT(flags, 0, FX_MAX)
 	INT(decal, -1, DECAL_MAX - 1)
 
-	STRING(mdl)
+	MODEL(mdl)
+	VEC(spin, -100, -100, -100, 100, 100, 100)
 	INT(particle, 0, PART_LENS_FLARE - 1)
 	INT(colour, 0, 0xFFFFFF)
 	INT(fade, 1, 120000)
@@ -390,6 +541,11 @@ namespace game
 	#define DEBUG_IND statuses.find(e)
 
 	ICOMMAND(r_status_addgeneric, "iiif", (int *t, int *s, int *d, float *v),
+		if(STATUS_INVALID_GENERIC(*t))
+		{
+			conoutf(CON_ERROR, "\fs\f3ERROR:\fr trying to create a generic status effect of non-generic or invalid type, '%i'", *t);
+			return;
+		}
 		statusgroup *e = checkstatusgroup();
 		if(!e) return;
 
@@ -399,13 +555,13 @@ namespace game
 		g->type = *t;
 		g->duration = *d;
 		g->strength = *s;
-		g->variance = clamp(1.f, 0.f, *v);
+		g->variance = clamp(*v, 0.f, 1.0f);
 
 		if(DEBUG_CONF)
-			conoutf(CON_DEBUG, "DEBUG: adding generic effect (%i %i %i), to statusgroup %i", *t, *d, *s, DEBUG_IND);
+			conoutf(CON_DEBUG, "\fs\f2DEBUG:\fr adding generic effect (%i %i %i), to statusgroup %i", *t, *d, *s, DEBUG_IND);
 	)
 
-	ICOMMAND(r_status_addpolymorph, "siif", (char *m, int *st, int *d, float *v),
+	ICOMMAND(r_status_addpolymorph, "siif", (const char *m, int *st, int *d, float *v),
 		statusgroup *e = checkstatusgroup();
 		if(!e) return;
 
@@ -415,10 +571,11 @@ namespace game
 		p->strength = *st;
 		p->duration = *d;
 		p->mdl = newstring(m);
-		p->variance = clamp(1.f, 0.f, *v);
+		preloadmodel(p->mdl);
+		p->variance = clamp(*v, 0.f, 1.0f);
 
 		if(DEBUG_CONF)
-			conoutf(CON_DEBUG, "DEBUG: adding polymorph effect (%s), to statusgroup %i", m, DEBUG_IND);
+			conoutf(CON_DEBUG, "\fs\f2DEBUG:\fr adding polymorph effect (%s), to statusgroup %i", m, DEBUG_IND);
 	)
 
 	ICOMMAND(r_status_addlight, "fffiif", (float *r, float *g, float *b, int *str, int *d, float *v),
@@ -434,10 +591,10 @@ namespace game
 		l->colour.z = clamp((float) 1, (float) -1, *b);
 		l->strength = clamp((int) 512, (int) 8, *str);
 		l->duration = *d;
-		l->variance = clamp(1.f, 0.f, *v);
+		l->variance = clamp(*v, 0.f, 1.0f);
 
 		if(DEBUG_CONF)
-			conoutf(CON_DEBUG, "DEBUG: adding light effect (%f %f %f --> %i), to statusgroup %i", l->colour.x, l->colour.y, l->colour.z, l->strength, DEBUG_IND);
+			conoutf(CON_DEBUG, "\fs\f2DEBUG:\fr adding light effect (%f %f %f --> %i), to statusgroup %i", l->colour.x, l->colour.y, l->colour.z, l->strength, DEBUG_IND);
 	)
 
 	ICOMMAND(r_status_addsignal, "siif", (char *s, int *str, int *d, float *v),
@@ -447,7 +604,7 @@ namespace game
 		e->effects.add(sig);
 		sig->type = STATUS_SIGNAL;
 		sig->strength = *str;
-		sig->variance = clamp(1.f, 0.f, *v);
+		sig->variance = clamp(*v, 0.f, 1.0f);
 		sig->signal = newstring(s);
 		sig->duration = *d;
 	)
@@ -462,7 +619,7 @@ namespace game
 		scr->script = newstring(s);
 		scr->strength = *str;
 		scr->duration = *d;
-		scr->variance = clamp(1.f, 0.f, *v);
+		scr->variance = clamp(*v, 0.f, 1.0f);
 	)
 
 	ICOMMAND(r_status_get_effect, "i", (int *idx),
@@ -515,48 +672,52 @@ namespace game
 	#undef DEBUG_IND
 
 	#define START(n, f, a, b) ICOMMAND(r_item_ ##n, f, a, b)
-	#define INIT item_base *e = checkitem_base();
-	#define DEBUG "item_base[%i]"
-	#define DEBUG_IND items.find(e)
+	#define INIT item *e = checkitem();
+	#define DEBUG "item[%p]"
+	#define DEBUG_IND loadingitem
 
 	ICOMMAND(r_item_use_new_consumable, "", (),
-		item_base *e = checkitem_base();
+		item *e = checkitem();
 		if(!e) return;
 
 		loadinguse = (e->uses.add(new use(e->script)));
 		if(DEBUG_CONF)
-			conoutf(CON_DEBUG, "DEBUG: created new consumable use (%i) for item %i", e->uses.length() - 1, DEBUG_IND);
+			conoutf(CON_DEBUG, "\fs\f2DEBUG:\fr created new consumable use (%i) for " DEBUG, e->uses.length() - 1, DEBUG_IND);
 	)
 
 	ICOMMAND(r_item_use_new_armour, "", (),
-		item_base *e = checkitem_base();
+		item *e = checkitem();
 		if(!e) return;
 
 		loadinguse = (e->uses.add(new use_armour(e->script)));
 		if(DEBUG_CONF)
-			conoutf(CON_DEBUG, "DEBUG: created new armour use (%i) for item %i", e->uses.length() - 1, DEBUG_IND);
+			conoutf(CON_DEBUG, "\fs\f2DEBUG:\fr created new armour use (%i) for " DEBUG, e->uses.length() - 1, DEBUG_IND);
 	)
 
 	ICOMMAND(r_item_use_new_weapon, "", (),
-		item_base *e = checkitem_base();
+		item *e = checkitem();
 		if(!e) return;
 
 		loadinguse = (e->uses.add(new use_weapon(e->script)));
 		if(DEBUG_CONF)
-			conoutf(CON_DEBUG, "DEBUG: created new weapon use (%i) for item %i", e->uses.length() - 1, DEBUG_IND);
+			conoutf(CON_DEBUG, "\fs\f2DEBUG:\fr created new weapon use (%i) for " DEBUG, e->uses.length() - 1, DEBUG_IND);
 	)
 
 
 	STRING(name)
 	STRING(icon)
 	STRING(description)
-	STRING(mdl)
+	MODEL(mdl)
 
-	INT(script, 0, 0xFFFF)
-	INT(type, 0, ITEM_MAX - 1)
-	INT(flags, 0, item_base::F_MAX - 1)
-	INT(worth, 0, 0xFFFFFF)
+	INT(quantity, 0, 0xFFFFFF)
+	INT(script, 0, scripts.length() - 1)
+	INT(category, 0, categories.length() - 1)
+	INT(flags, 0, item::F_MAX)
+	INT(value, 0, 0xFFFFFF)
+	INT(maxdurability, 0, 0xFFFFFF)
 	FLOAT(weight, 0, 0xFFFF)
+	FLOAT(durability, 0, 0xFFFFFF)
+	FLOAT(recovery, 0, 1)
 
 	#undef START
 	#undef INIT
@@ -565,8 +726,8 @@ namespace game
 
 	#define START(n, f, a, b) ICOMMAND(r_item_use_ ##n, f, a, b)
 	#define INIT CAST *e = (CAST *) checkuse(TYPE);
-	#define DEBUG "item_base[%i]->uses[%i]"
-	#define DEBUG_IND items.find(loadingitem_base), loadingitem_base->uses.find(loadinguse)
+	#define DEBUG "item_base[%p]->uses[%i]"
+	#define DEBUG_IND loadingitem, loadingitem->uses.find(loadinguse)
 
 	#define CAST use
 	#define TYPE USE_CONSUME
@@ -578,9 +739,10 @@ namespace game
 
 	STRING(name)
 	STRING(description)
-	INT(script, 0, 0xFFFF)
+	STRING(icon)
+	INT(script, 0, scripts.length() - 1)
 	INT(cooldown, 0, 0xFFFF)
-	INT(chargeflags, 0, CHARGE_MAX - 1)
+	INT(chargeflags, 0, CHARGE_MAX)
 
 	START(new_status, "iif", (int *st, int *el, float *m),
 		INIT if(!e) return;
@@ -592,11 +754,11 @@ namespace game
 	#define CAST use_armour
 	#define TYPE USE_ARMOUR
 
-	STRING(vwepmdl)
-	STRING(hudmdl)
-	INT(idlefx, -1, 0xFFFF)
+	MODEL(vwepmdl)
+	MODEL(hudmdl)
+	INT(idlefx, -1, effects.length() - 1)
 	STATREQ(reqs)
-	INT(slots, 0, SLOT_MAX - 1)
+	INT(slots, 0, SLOT_MAX)
 	INT(skill, -1, SKILL_MAX - 1)
 
 	#undef CAST
@@ -608,12 +770,12 @@ namespace game
 	INT(angle, 0, 360)
 	INT(lifetime, 0, 0xFFFF)
 	INT(gravity, -1000, 1000)
-	INT(projeffect, -1, 0xFFFF)
-	INT(traileffect, -1, 0xFFFF)
-	INT(deatheffect, -1, 0xFFFF)
+	INT(projeffect, -1, effects.length() - 1)
+	INT(traileffect, -1, effects.length() - 1)
+	INT(deatheffect, -1, effects.length() - 1)
 	INT(cost, 0, 0xFFFF)
-	INT(pflags, 0, P_MAX - 1)
-	INT(ammo, -3, 0xFFFF)
+	INT(pflags, 0, P_MAX)
+	INT(ammo, -3, ammotypes.length() - 1)
 	INT(target, 0, T_MAX - 1)
 	INT(radius, 0, 0xFFFF)
 	INT(kickback, -0xFFFF, 0xFFFF)
@@ -641,36 +803,36 @@ namespace game
 	ICOMMAND(r_recipe_add_ingredient, "ii", (int *base, int *qty),
 		recipe *e = checkrecipe();
 		if(!e) return;
-		if(*qty <= 0) {conoutf(CON_ERROR, "ERROR: can't add an ingredient with a quantity of <= 0"); return;}
+		if(*qty <= 0) {conoutf(CON_ERROR, "\fs\f3ERROR:\fr can't add an ingredient with a quantity of <= 0"); return;}
 
 		if(DEBUG_CONF)
-			conoutf(CON_DEBUG, "DEBUG: added ingredient %i (%i) to recipe %i (%p)", *base, *qty, DEBUG_IND, e);
+			conoutf(CON_DEBUG, "\fs\f2DEBUG:\fr added ingredient %i (%i) to " DEBUG, *base, *qty, DEBUG_IND);
 
-		e->ingredients.add(invstack(*base, *qty));
+		e->ingredients.add(recipe::ingredient(*base, *qty));
 		e->optimise()
 	)
 
 	ICOMMAND(r_recipe_add_catalyst, "ii", (int *base, int *qty),
 		recipe *e = checkrecipe();
 		if(!e) return;
-		if(*qty <= 0) {conoutf(CON_ERROR, "ERROR: can't add an ingredient with a quantity of <= 0"); return;}
+		if(*qty <= 0) {conoutf(CON_ERROR, "\fs\f3ERROR:\fr can't add an ingredient with a quantity of <= 0"); return;}
 
 		if(DEBUG_CONF)
-			conoutf(CON_DEBUG, "DEBUG: added catalyst %i (%i) to recipe %i (%p)", *base, *qty, DEBUG_IND, e);
+			conoutf(CON_DEBUG, "\fs\f2DEBUG:\fr added catalyst %i (%i) to " DEBUG, *base, *qty, DEBUG_IND);
 
-		e->catalysts.add(invstack(*base, *qty));
+		e->catalysts.add(recipe::ingredient(*base, *qty));
 		e->optimise()
 	)
 
 	ICOMMAND(r_recipe_add_product, "ii", (int *base, int *qty),
 		recipe *e = checkrecipe();
 		if(!e) return;
-		if(*qty <= 0) {conoutf(CON_ERROR, "ERROR: can't add an ingredient with a quantity of <= 0"); return;}
+		if(*qty <= 0) {conoutf(CON_ERROR, "\fs\f3ERROR:\fr can't add an ingredient with a quantity of <= 0"); return;}
 
 		if(DEBUG_CONF)
-			conoutf(CON_DEBUG, "DEBUG: added product %i (%i) to recipe %i (%p)", *base, *qty, DEBUG_IND, e);
+			conoutf(CON_DEBUG, "\fs\f2DEBUG:\fr added product %i (%i) to " DEBUG, *base, *qty, DEBUG_IND);
 
-		e->products.add(invstack(*base, *qty));
+		e->products.add(recipe::ingredient(*base, *qty));
 		e->optimise()
 	)
 
@@ -725,7 +887,8 @@ namespace game
 		intret(e->products.length());
 	)
 
-	INT(flags, 0, recipe::MAX - 1)
+	STRING(name)
+	INT(flags, 0, recipe::MAX)
 	STATREQ(reqs)
 
 	#undef START
@@ -744,7 +907,7 @@ namespace game
 
 		e->items.add(*i);
 		if(DEBUG_CONF)
-			conoutf(CON_DEBUG, "DEBUG: Classified item %i as member of ammo %i (%s)", *i, DEBUG_IND, e->name ? e->name : "unnamed");
+			conoutf(CON_DEBUG, "\fs\f2DEBUG:\fr Classified item %i as member of " DEBUG " (%s)", *i, DEBUG_IND, e->name ? e->name : "unnamed");
 	)
 
 	ICOMMAND(r_ammo_num_item,  "", (),
@@ -769,17 +932,19 @@ namespace game
 	#undef DEBUG_IND
 
 	#define START(n, f, a, b) ICOMMAND(r_char_ ##n, f, a, b)
-	#define INIT char_base *e = checkchar_base();
-	#define DEBUG "char_base[%i]"
-	#define DEBUG_IND chars.find(e)
-
-	//ICOMMAND(r_char_additem, "ii", (int *i, int *q)
+	#define INIT rpgchar *e = checkrpgchar();
+	#define DEBUG "rpgchar[%p]"
+	#define DEBUG_IND loadingrpgchar
 
 	STRING(name)
-	STRING(mdl)
-	INT(script, 0, 0xFFFF)
-	INT(faction, 0, 0xFFFF)
+	MODEL(mdl)
+	STRING(portrait)
+	INT(script, 0, scripts.length() - 1)
+	INT(faction, 0, factions.length() - 1)
+	INT(merchant, -1, merchants.length() - 1)
 	STATS(base)
+	FLOAT(health, 0, e->base.getmaxhp())
+	FLOAT(mana, 0, e->base.getmaxmp())
 
 	#undef START
 	#undef INIT
@@ -796,7 +961,7 @@ namespace game
 		if(!e) return;
 		e->setrelation(*o, *f);
 		if(DEBUG_CONF)
-			conoutf(CON_DEBUG, "DEBUG: faction %i's liking of faction %i is now %i", DEBUG_IND, *o, *f);
+			conoutf(CON_DEBUG, "\fs\f2DEBUG:\fr faction %i's liking of faction %i is now %i", DEBUG_IND, *o, *f);
 	)
 
 	ICOMMAND(r_faction_get_relation, "i", (int *o),
@@ -816,14 +981,14 @@ namespace game
 	#undef DEBUG_IND
 
 	#define START(n, f, a, b) ICOMMAND(r_obstacle_ ##n, f, a, b)
-	#define INIT obstacle_base *e = checkobstacle_base();
-	#define DEBUG "obstacle_base[%i]"
-	#define DEBUG_IND obstacles.find(e)
+	#define INIT rpgobstacle *e = checkrpgobstacle();
+	#define DEBUG "rpgobstacle[%p]"
+	#define DEBUG_IND loadingrpgobstacle
 
-	STRING(mdl)
+	MODEL(mdl)
 	INT(weight, 0, 0xFFFF)
-	INT(script, 0, 0xFFFF)
-	INT(flags, 0, rpgobstacle::F_MAX - 1)
+	INT(script, 0, scripts.length() - 1)
+	INT(flags, 0, rpgobstacle::F_MAX)
 
 
 	#undef START
@@ -832,17 +997,18 @@ namespace game
 	#undef DEBUG_IND
 
 	#define START(n, f, a, b) ICOMMAND(r_container_ ##n, f, a, b)
-	#define INIT container_base *e = checkcontainer_base();
-	#define DEBUG "container_base[%i]"
-	#define DEBUG_IND containers.find(e)
+	#define INIT rpgcontainer *e = checkrpgcontainer();
+	#define DEBUG "rpgcontainer[%p]"
+	#define DEBUG_IND loadingrpgcontainer
 
-	//ICOMMAND(r_container_additem, "ii", (int *i, int *q)
-
-	STRING(mdl)
+	MODEL(mdl)
 	STRING(name)
+	INT(faction, -1, factions.length() - 1)
+	INT(merchant, -1, merchants.length() - 1)
 	INT(capacity, 0, 0xFFFF)
-	INT(script, 0, 0xFFFF)
-	INT(lock, 0, 101)
+	INT(script, 0, scripts.length() - 1)
+	INT(lock, 0, 100)
+	INT(magelock, 0, 100)
 
 
 	#undef START
@@ -851,26 +1017,27 @@ namespace game
 	#undef DEBUG_IND
 
 	#define START(n, f, a, b) ICOMMAND(r_platform_ ##n, f, a, b)
-	#define INIT platform_base *e = checkplatform_base();
-	#define DEBUG "platform_base[%i]"
-	#define DEBUG_IND platforms.find(e)
+	#define INIT rpgplatform *e = checkrpgplatform();
+	#define DEBUG "rpgplatform[%p]"
+	#define DEBUG_IND loadingrpgplatform
 
-	STRING(mdl)
+	MODEL(mdl)
 	INT(speed, 1, 1000)
-	INT(flags, 0, rpgplatform::F_MAX - 1)
-	INT(script, 0, 0xFFFF)
+	INT(flags, 0, rpgplatform::F_MAX)
+	INT(script, 0, scripts.length() - 1)
 
 	START(addroute, "ii", (int *from, int *to),
 		INIT
+		if(!e) return;
 		vector<int> &detours = e->routes.access(*from, vector<int>());
 		if(detours.find(*to) == -1)
 		{
 			detours.add(*to);
 			if(DEBUG_CONF)
-				conoutf(CON_DEBUG, "DEBUG: registered detour; " DEBUG "->routes[%i].add(%i)", DEBUG_IND, *from, *to);
+				conoutf(CON_DEBUG, "\fs\f2DEBUG:\fr registered detour; " DEBUG "->routes[%i].add(%i)", DEBUG_IND, *from, *to);
 		}
 		else if(DEBUG_CONF)
-			conoutf(CON_DEBUG, "DEBUG: route from %i to %i already registred for " DEBUG, *from, *to, DEBUG_IND);
+			conoutf(CON_DEBUG, "\fs\f2DEBUG:\fr route from %i to %i already registred for " DEBUG, *from, *to, DEBUG_IND);
 	)
 
 
@@ -880,32 +1047,61 @@ namespace game
 	#undef DEBUG_IND
 
 	#define START(n, f, a, b) ICOMMAND(r_trigger_ ##n, f, a, b)
-	#define INIT trigger_base *e = checktrigger_base();
-	#define DEBUG "trigger_base[%i]"
-	#define DEBUG_IND triggers.find(e)
+	#define INIT rpgtrigger *e = checkrpgtrigger();
+	#define DEBUG "rpgtrigger[%p]"
+	#define DEBUG_IND loadingrpgtrigger
 
 
-	STRING(mdl)
+	MODEL(mdl)
 	STRING(name)
-	INT(flags, 0, rpgtrigger::F_MAX - 1)
-	INT(script, 0, 0xFFFF)
-
+	INT(flags, 0, rpgtrigger::F_MAX)
+	INT(script, 0, scripts.length() - 1)
 
 	#undef START
 	#undef INIT
 	#undef DEBUG
 	#undef DEBUG_IND
 
-	ICOMMAND(r_global_new, "i", (int *v),
+	#define START(n, f, a, b) ICOMMAND(r_merchant_ ##n, f, a, b)
+	#define INIT merchant *e = checkmerchant();
+	#define DEBUG "merchants[%i]"
+	#define DEBUG_IND merchants.find(e)
+
+	INT(currency, 0, 0xFFFF)
+	INT(credit, 0, 0xFFFFFF)
+
+	START(setrate, "iff", (int *cat, float *buy, float *sell),
+		INIT
+		if(!e || *cat < 0) return;
 		if(DEBUG_CONF)
-			conoutf(CON_DEBUG, "DEBUG: created new global variable - variables[%i] = %i", variables.length(), *v);
-		variables.add(*v);
+			conoutf(CON_DEBUG, "\fs\f2DEBUG:\fr setting " DEBUG "->rate[%i] to %f%% and %f%%", DEBUG_IND, *cat, *buy * 100, *sell * 100);
+
+		while(!e->rates.inrange(*cat)) e->rates.add(merchant::rate(0, 0));
+		e->rates[*cat] = merchant::rate(*buy, *sell);
+	)
+
+	#undef START
+	#undef INIT
+	#undef DEBUG
+	#undef DEBUG_IND
+
+	ICOMMAND(r_global_new, "s", (const char *s),
+		if(DEBUG_CONF)
+			conoutf(CON_DEBUG, "\fs\f2DEBUG:\fr created new global variable - variables[%i] = \"%s\"", variables.length(), s);
+		variables.add(newstring(s));
 		intret(variables.length() - 1);
 	)
 
 	ICOMMAND(r_tip_new, "s", (const char *s),
 		if(DEBUG_CONF)
-			conoutf(CON_DEBUG, "DEBUG: added tip...\n%s", s);
+			conoutf(CON_DEBUG, "\fs\f2DEBUG:\fr added tip...\n%s", s);
 		tips.add(newstring(s));
 	)
+
+	ICOMMAND(r_cat_new, "s", (const char *s),
+		if(DEBUG_CONF)
+			conoutf(CON_DEBUG, "\fs\f2DEBUG:\fr Registering item category[%i] as  \"%s\"", categories.length(), s);
+		categories.add(newstring(s));
+		intret(categories.length() - 1);
+	)
 }
