@@ -15,11 +15,6 @@ namespace game
 	void lighteffects(dynent *d, vec &color, vec &dir) {}
 	void particletrack(physent *owner, vec &o, vec &d) {}
 	void dynlighttrack(physent *owner, vec &o, vec &hud) {}
-	#ifndef NEWGUI
-	bool serverinfostartcolumn(g3d_gui *g, int i) {return false;}
-	void serverinfoendcolumn(g3d_gui *g, int i) {}
-	bool serverinfoentry(g3d_gui *g, int i, const char *name, int port, const char *desc, const char *map, int ping, const vector<int> &attr, int np)  {return false;}
-	#endif
 	void vartrigger(ident *id) {}
 	void bounced(physent *d, const vec &surface) {}
 	void registeranimation(char *dir, char* anim, int num) {}
