@@ -2,9 +2,6 @@
 
 void rpgtrigger::update()
 {
-	resetmdl();
-	temp.light = vec4(0, 0, 0, 0);
-	temp.alpha = 1;
 	//hack to make models passthrough. (eg, doors)
 	if(flags & F_TRIGGERED)
 		state = (lastmillis - lasttrigger < 750) ? CS_ALIVE : CS_DEAD;
@@ -35,3 +32,13 @@ void rpgtrigger::hit(rpgent *attacker, use_weapon *weapon, use_weapon *ammo, flo
 
 	getsignal("hit", false, attacker);
 }
+
+void rpgtrigger::init(int base)
+{
+	game::loadingrpgtrigger = this;
+
+	defformatstring(file)("data/rpg/games/%s/triggers/%i.cfg", game::data, base);
+	execfile(file);
+
+	game::loadingrpgtrigger = NULL;
+}
