@@ -27,6 +27,9 @@
 
 #ifdef WIN32
   #define WIN32_LEAN_AND_MEAN
+  #ifdef _WIN32_WINNT
+  #undef _WIN32_WINNT
+  #endif
   #define _WIN32_WINNT 0x0500
   #include "windows.h"
   #ifndef _WINDOWS
