@@ -106,30 +106,20 @@ enum
 {
     ANIM_DEAD = 0, ANIM_DYING, ANIM_IDLE,
     ANIM_FORWARD, ANIM_BACKWARD, ANIM_LEFT, ANIM_RIGHT,
-    ANIM_HOLD1, ANIM_HOLD2, ANIM_HOLD3, ANIM_HOLD4, ANIM_HOLD5, ANIM_HOLD6, ANIM_HOLD7,
-    ANIM_ATTACK1, ANIM_ATTACK2, ANIM_ATTACK3, ANIM_ATTACK4, ANIM_ATTACK5, ANIM_ATTACK6, ANIM_ATTACK7,
     ANIM_PAIN,
     ANIM_JUMP, ANIM_SINK, ANIM_SWIM,
-    ANIM_EDIT, ANIM_LAG, ANIM_TAUNT, ANIM_WIN, ANIM_LOSE,
-    ANIM_GUN_IDLE, ANIM_GUN_SHOOT,
-    ANIM_VWEP_IDLE, ANIM_VWEP_SHOOT, ANIM_SHIELD, ANIM_POWERUP,
+    ANIM_EDIT, ANIM_LAG,
     ANIM_MAPMODEL, ANIM_TRIGGER,
-    NUMANIMS
+    ANIM_GAMESPECIFIC
 };
 
-static const char * const animnames[] =
-{
-    "dead", "dying", "idle",
-    "forward", "backward", "left", "right",
-    "hold 1", "hold 2", "hold 3", "hold 4", "hold 5", "hold 6", "hold 7",
-    "attack 1", "attack 2", "attack 3", "attack 4", "attack 5", "attack 6", "attack 7",
-    "pain",
-    "jump", "sink", "swim",
-    "edit", "lag", "taunt", "win", "lose",
-    "gun idle", "gun shoot",
-    "vwep idle", "vwep shoot", "shield", "powerup",
+#define ANIMNAMES \
+    "dead", "dying", "idle", \
+    "forward", "backward", "left", "right", \
+    "pain", \
+    "jump", "sink", "swim", \
+    "edit", "lag", \
     "mapmodel", "trigger"
-};
 
 #define ANIM_ALL         0x7F
 #define ANIM_INDEX       0x7F
@@ -137,6 +127,7 @@ static const char * const animnames[] =
 #define ANIM_START       (1<<8)
 #define ANIM_END         (1<<9)
 #define ANIM_REVERSE     (1<<10)
+#define ANIM_CLAMP       (ANIM_START|ANIM_END)
 #define ANIM_DIR         0x780
 #define ANIM_SECONDARY   11
 #define ANIM_NOSKIN      (1<<22)
@@ -146,7 +137,8 @@ static const char * const animnames[] =
 #define ANIM_NORENDER    (1<<26)
 #define ANIM_RAGDOLL     (1<<27)
 #define ANIM_SETSPEED    (1<<28)
-#define ANIM_GHOST       (1<<29)
+#define ANIM_NOPITCH     (1<<29)
+#define ANIM_GHOST       (1<<30)
 #define ANIM_FLAGS       (0x1FF<<22)
 
 struct animinfo // description of a character's animation
@@ -168,6 +160,8 @@ struct animinterpinfo // used for animation blending of animated characters
     void *lastmodel;
 
     animinterpinfo() : lastswitch(-1), lastmodel(NULL) {}
+
+    void reset() { lastswitch = -1; }
 };
 
 #define MAXANIMPARTS 3
@@ -212,6 +206,7 @@ struct dynent : physent                         // animated characters, or chara
     {
         physent::reset();
         stopmoving();
+        loopi(MAXANIMPARTS) animinterp[i].reset();
 //        animate = false;
 //        animation = -1;
     }
