@@ -31,6 +31,7 @@ enum // cube empty-space materials
     MAT_GAMECLIP = 3 << MATF_CLIP_SHIFT,  // game specific clip material
 
     MAT_DEATH    = 1 << MATF_FLAG_SHIFT,  // force player suicide
+    MAT_PAIN     = 2 << MATF_FLAG_SHIFT,  // damage player when submerged
     MAT_ALPHA    = 4 << MATF_FLAG_SHIFT   // alpha blended
 };
 
@@ -71,6 +72,19 @@ struct selinfo
     int size() const    { return s.x*s.y*s.z; }
     int us(int d) const { return s[d]*grid; }
     bool operator==(const selinfo &sel) const { return o==sel.o && s==sel.s && grid==sel.grid && orient==sel.orient; }
+    bool validate()
+    {
+        extern int worldsize;
+        if(grid <= 0 || grid >= worldsize) return false;
+        if(o.x >= worldsize || o.y >= worldsize || o.z >= worldsize) return false;
+        if(o.x < 0) { s.x -= (grid - 1 - o.x)/grid; o.x = 0; }
+        if(o.y < 0) { s.y -= (grid - 1 - o.y)/grid; o.y = 0; }
+        if(o.z < 0) { s.z -= (grid - 1 - o.z)/grid; o.z = 0; }
+        s.x = clamp(s.x, 0, (worldsize - o.x)/grid);
+        s.y = clamp(s.y, 0, (worldsize - o.y)/grid);
+        s.z = clamp(s.z, 0, (worldsize - o.z)/grid);
+        return s.x > 0 && s.y > 0 && s.z > 0;
+    }
 };
 
 struct editinfo;
@@ -96,6 +110,10 @@ extern void mpdelcube(selinfo &sel, bool local);
 extern void mpremip(bool local);
 
 // command
+extern void cleancode(ident &id);
+extern void pusharg(ident &id, const tagval &v, identstack &stack);
+extern void poparg(ident &id);
+
 extern int variable(const char *name, int min, int cur, int max, int *storage, identfun fun, int flags);
 extern float fvariable(const char *name, float min, float cur, float max, float *storage, identfun fun, int flags);
 extern char *svariable(const char *name, const char *cur, char **storage, identfun fun, int flags);
@@ -112,18 +130,28 @@ extern int getvarmax(const char *name);
 extern bool identexists(const char *name);
 extern ident *getident(const char *name);
 extern ident *newident(const char *name, int flags = 0);
+extern ident *readident(const char *name);
+extern ident *writeident(const char *name, int flags = 0);
 extern bool addcommand(const char *name, identfun fun, const char *narg);
 extern uint *compilecode(const char *p);
-extern void executeret(const uint *code, tagval &result);
-extern void executeret(const char *p, tagval &result);
-extern char *executeret(const char *p);
+extern void keepcode(uint *p);
+extern void freecode(uint *p);
+extern void executeret(const uint *code, tagval &result = *commandret);
+extern void executeret(const char *p, tagval &result = *commandret);
+extern char *executestr(const uint *code);
+extern char *executestr(const char *p);
 extern int execute(const uint *code);
 extern int execute(const char *p);
+extern bool executebool(const uint *code);
+extern bool executebool(const char *p);
 extern bool execfile(const char *cfgfile, bool msg = true);
 extern void alias(const char *name, const char *action);
 extern void alias(const char *name, tagval &v);
 extern const char *getalias(const char *name);
-extern void writeescapedstring(stream *f, const char *s);
+extern const char *escapestring(const char *s);
+extern const char *escapeid(const char *s);
+static inline const char *escapeid(ident &id) { return escapeid(id.name); }
+extern bool validateblock(const char *s);
 
 // console
 
@@ -146,27 +174,18 @@ extern void closelogfile();
 extern void logoutfv(const char *fmt, va_list args);
 extern void logoutf(const char *fmt, ...);
 
-// menus
-#ifndef NEWGUI
-
-extern vec menuinfrontofplayer();
-extern void newgui(char *name, char *contents, char *header = NULL);
-extern void showgui(const char *name);
-extern int cleargui(int n = 0);
-#endif
-
 // octa
 extern int lookupmaterial(const vec &o);
 
 static inline bool insideworld(const vec &o)
 {
-	extern int worldsize;
+    extern int worldsize;
     return o.x>=0 && o.x<worldsize && o.y>=0 && o.y<worldsize && o.z>=0 && o.z<worldsize;
 }
 
 static inline bool insideworld(const ivec &o)
 {
-	extern int worldsize;
+    extern int worldsize;
     return uint(o.x)<uint(worldsize) && uint(o.y)<uint(worldsize) && uint(o.z)<uint(worldsize);
 }
 
@@ -196,10 +215,31 @@ extern bool popfont();
 extern void gettextres(int &w, int &h);
 extern void draw_text(const char *str, int left, int top, int r = 255, int g = 255, int b = 255, int a = 255, int cursor = -1, int maxwidth = -1);
 extern void draw_textf(const char *fstr, int left, int top, ...);
-extern int text_width(const char *str);
-extern void text_bounds(const char *str, int &width, int &height, int maxwidth = -1);
-extern int text_visible(const char *str, int hitx, int hity, int maxwidth);
-extern void text_pos(const char *str, int cursor, int &cx, int &cy, int maxwidth);
+extern float text_widthf(const char *str);
+extern void text_boundsf(const char *str, float &width, float &height, int maxwidth = -1);
+extern int text_visible(const char *str, float hitx, float hity, int maxwidth);
+extern void text_posf(const char *str, int cursor, float &cx, float &cy, int maxwidth);
+
+static inline int text_width(const char *str)
+{
+    return int(ceil(text_widthf(str)));
+}
+
+static inline void text_bounds(const char *str, int &width, int &height, int maxwidth = -1)
+{
+    float widthf, heightf;
+    text_boundsf(str, widthf, heightf, maxwidth);
+    width = int(ceil(widthf));
+    height = int(ceil(heightf));
+}
+
+static inline void text_pos(const char *str, int cursor, int &cx, int &cy, int maxwidth)
+{
+    float cxf, cyf;
+    text_posf(str, cursor, cxf, cyf, maxwidth);
+    cx = int(cxf);
+    cy = int(cyf);
+}
 
 // renderva
 enum
@@ -243,12 +283,13 @@ enum
     PART_STREAK, PART_LIGHTNING,
     PART_EXPLOSION, PART_EXPLOSION_BLUE,
     PART_SPARK, PART_EDIT,
-    PART_MUZZLE_FLASH, PART_MUZZLE_FLASH2, PART_MUZZLE_FLASH3,
+    PART_MUZZLE_FLASH1, PART_MUZZLE_FLASH2, PART_MUZZLE_FLASH3,
     PART_HUD_ICON,
     PART_HUD_ICON_GREY,
     PART_TEXT,
     PART_METER, PART_METER_VS,
     PART_SNOW, PART_LEAVES,
+	PART_WATERLEAK,
     PART_LENS_FLARE
 };
 
@@ -271,11 +312,13 @@ extern void regularlensflare(vec o, uchar r, uchar g, uchar b, bool sun = false,
 // decal
 enum
 {
-    DECAL_EMPTY = 0,
-    DECAL_BURN, //activate when colision detection for particles are improved
+    DECAL_BURN = 0, //activate when colision detection for particles are improved
+    DECAL_BLOOD,
+    DECAL_WATERBURN,
     DECAL_STAIN,
     DECAL_RIPPLE,
     DECAL_LEAVES,
+	DECAL_WATERLEAK,
     DECAL_MAX
 };
 
@@ -287,6 +330,7 @@ extern bool load_world(const char *mname, const char *cname = NULL);
 extern bool save_world(const char *mname, bool nolms = false, bool octa = false);
 extern void getmapfilenames(const char *cname, bool fall = true);
 extern uint getmapcrc();
+extern void clearmapcrc();
 extern bool loadents(const char *fname, vector<entity> &ents, uint *crc = NULL);
 
 // physics
@@ -341,7 +385,7 @@ extern void abovemodel(vec &o, const char *mdl);
 extern void rendershadow(dynent *d);
 extern void renderclient(dynent *d, const char *mdlname, modelattach *attachments, int hold, int attack, int attackdelay, int lastaction, int lastpain, float fade = 1, bool ragdoll = false, int colour = 0xFFFFFF);
 extern void interpolateorientation(dynent *d, float &interpyaw, float &interppitch);
-extern void setbbfrommodel(dynent *d, const char *mdl);
+extern void setbbfrommodel(physent *d, const char *mdl);
 extern const char *mapmodelname(int i);
 extern model *loadmodel(const char *name, int i = -1, bool msg = false);
 extern void preloadmodel(const char *name);
@@ -405,6 +449,7 @@ extern const ENetAddress *connectedpeer();
 extern bool multiplayer(bool msg = true);
 extern void neterr(const char *s, bool disc = true);
 extern void gets2c();
+extern void notifywelcome();
 
 // crypto
 extern void genprivkey(const char *seed, vector<char> &privstr, vector<char> &pubstr);
@@ -424,78 +469,6 @@ extern bool settexture(Texture *&tex);
 
 enum { EDITORFOCUSED = 1, EDITORUSED, EDITORFOREVER };
 
-#ifndef NEWGUI
-enum { G3D_DOWN = 1, G3D_UP = 2, G3D_PRESSED = 4, G3D_ROLLOVER = 8, G3D_DRAGGED = 16 };
-struct g3d_gui
-{
-    virtual ~g3d_gui() {}
-
-    virtual void start(int starttime, float basescale, int *tab = NULL, bool allowinput = true) = 0;
-    virtual void end() = 0;
-
-    virtual int text(const char *text, int color, const char *icon = NULL) = 0;
-    int textf(const char *fmt, int color, const char *icon = NULL, ...)
-    {
-        defvformatstring(str, icon, fmt);
-        return text(str, color, icon);
-    }
-    virtual int button(const char *text, int color, const char *icon = NULL) = 0;
-    int buttonf(const char *fmt, int color, const char *icon = NULL, ...)
-    {
-        defvformatstring(str, icon, fmt);
-        return button(str, color, icon);
-    }
-    virtual int title(const char *text, int color, const char *icon = NULL) = 0;
-    int titlef(const char *fmt, int color, const char *icon = NULL, ...)
-    {
-        defvformatstring(str, icon, fmt);
-        return title(str, color, icon);
-    }
-    virtual void background(int color, int parentw = 0, int parenth = 0) = 0;
-
-    virtual void pushlist(int align = -1) {}
-    virtual void poplist() {}
-
-    virtual void allowautotab(bool on) = 0;
-    virtual bool shouldtab() { return false; }
-	virtual void tab(const char *name = NULL, int color = 0) = 0;
-    virtual int image(Texture *t, float scale, bool overlaid = false) = 0;
-    virtual int texture(VSlot &vslot, float scale, bool overlaid = true) = 0;
-    virtual void slider(int &val, int vmin, int vmax, int color, char *label = NULL, bool reverse = false) = 0;
-    virtual void separator() = 0;
-	virtual void progress(float percent) = 0;
-	virtual void strut(float size) = 0;
-    virtual void space(float size) = 0;
-    virtual char *keyfield(const char *name, int color, int length, int height = 0, const char *initval = NULL, int initmode = EDITORFOCUSED) = 0;
-    virtual char *field(const char *name, int color, int length, int height = 0, const char *initval = NULL, int initmode = EDITORFOCUSED) = 0;
-    virtual void textbox(const char *text, int width, int height, int color = 0xFFFFFF) = 0;
-    virtual void mergehits(bool on) = 0;
-};
-
-struct g3d_callback
-{
-    virtual ~g3d_callback() {}
-
-    int starttime() { return totalmillis; }
-
-    virtual void gui(g3d_gui &g, bool firstpass) = 0;
-};
-
-enum
-{
-    GUI_2D       = 1<<0,
-    GUI_FOLLOW   = 1<<1,
-    GUI_FORCE_2D = 1<<2,
-    GUI_BOTTOM   = 1<<3
-};
-
-extern void g3d_addgui(g3d_callback *cb, vec &origin, int flags = 0);
-extern bool g3d_movecursor(int dx, int dy);
-extern void g3d_cursorpos(float &x, float &y);
-extern void g3d_resetcursor();
-extern void g3d_limitscale(float scale);
-#endif
-
 extern vector<int> entgroup;
 extern int efocus, enthover, entorient;
 
