@@ -78,10 +78,10 @@ static inline T min(T a, T b)
     return a < b ? a : b;
 }
 
-template<class T>
-static inline T clamp(T a, T b, T c) //max min val
+template<class T, class U>
+static inline T clamp(T a, U b, U c) //val min max
 {
-    return max(b, min(a, c));
+    return max(T(b), min(a, T(c)));
 }
 
 #define rnd(x) ((int)(randomMT()&0xFFFFFF)%(x))
@@ -161,6 +161,8 @@ struct stringformatter
 #define loopvj(v)   for(int j = 0; j<(v).length(); j++)
 #define loopvk(v)   for(int k = 0; k<(v).length(); k++)
 #define loopvrev(v) for(int i = (v).length()-1; i>=0; i--)
+#define loopvjrev(v) for(int j = (v).length()-1; j>=0; j--)
+#define loopvkrev(v) for(int k = (v).length()-1; k>=0; k--)
 
 template <class T>
 struct databuf
@@ -204,7 +206,7 @@ struct databuf
     void put(const T *vals, int numvals)
     {
         if(maxlen-len<numvals) flags |= OVERWROTE;
-        memcpy(&buf[len], vals, min(maxlen-len, numvals)*sizeof(T));
+        memcpy(&buf[len], (const void *)vals, min(maxlen-len, numvals)*sizeof(T));
         len += min(maxlen-len, numvals);
     }
 
@@ -212,7 +214,7 @@ struct databuf
     {
         int read = min(maxlen-len, numvals);
         if(read<numvals) flags |= OVERREAD;
-        memcpy(vals, &buf[len], read*sizeof(T));
+        memcpy(vals, (void *)&buf[len], read*sizeof(T));
         len += read;
         return read;
     }
@@ -380,6 +382,13 @@ static inline void quicksort(T *buf, int n)
     quicksort(buf, buf+n, compareless<T>);
 }
 
+template<class T> struct isclass
+{
+    template<class C> static char test(void (C::*)(void));
+    template<class C> static int test(...);
+    enum { yes = sizeof(test<T>(0)) == 1 ? 1 : 0, no = yes^1 };
+};
+
 template <class T> struct vector
 {
     static const int MINSIZE = 8;
@@ -438,7 +447,7 @@ template <class T> struct vector
         else
         {
             growbuf(ulen+v.ulen);
-            if(v.ulen) memcpy(&buf[ulen], v.buf, v.ulen*sizeof(T));
+            if(v.ulen) memcpy(&buf[ulen], (void  *)v.buf, v.ulen*sizeof(T));
             ulen += v.ulen;
             v.ulen = 0;
         }
@@ -459,7 +468,7 @@ template <class T> struct vector
 
     void disown() { buf = NULL; alen = ulen = 0; }
 
-    void shrink(int i)         { ASSERT(i<=ulen); while(ulen>i) drop(); }
+    void shrink(int i) { ASSERT(i<=ulen); if(isclass<T>::no) ulen = i; else while(ulen>i) drop(); }
     void setsize(int i) { ASSERT(i<=ulen); ulen = i; }
 
     void deletecontents() { while(!empty()) delete   pop(); }
@@ -486,7 +495,7 @@ template <class T> struct vector
         uchar *newbuf = new uchar[alen*sizeof(T)];
         if(olen > 0)
         {
-            memcpy(newbuf, buf, olen*sizeof(T));
+            memcpy(newbuf, (void *)buf, olen*sizeof(T));
             delete[] (uchar *)buf;
         }
         buf = (T *)newbuf;
@@ -699,6 +708,20 @@ template<class T> struct hashset
         deletechunks();
     }
 
+    int length()
+    {
+        int ret = 0;
+        loopi(size)
+        {
+            for(chain *enumc = chains[i]; enumc;)
+            {
+                enumc = enumc->next;
+                ret++;
+            }
+        }
+        return ret;
+    }
+
     chain *insert(uint h)
     {
         if(!unused)
@@ -744,6 +767,18 @@ template<class T> struct hashset
         HTFIND(key, c->elem, insert(h)->elem);
     }
 
+    template<class K>
+    T &find(const K &key, T &notfound)
+    {
+        HTFIND(key, c->elem, notfound);
+    }
+
+    template<class K>
+    const T &find(const K &key, const T &notfound)
+    {
+        HTFIND(key, c->elem, notfound);
+    }
+
     template<class K>
     bool remove(const K &key)
     {
@@ -838,6 +873,16 @@ template<class K, class T> struct hashtable : hashset<hashtableentry<K, T> >
         HTFIND(key, c->elem.data, insert(key, h).data);
     }
 
+    T &find(const K &key, T &notfound)
+    {
+        HTFIND(key, c->elem.data, notfound);
+    }
+
+    const T &find(const K &key, const T &notfound)
+    {
+        HTFIND(key, c->elem.data, notfound);
+    }
+
     static inline chain *getnext(void *i) { return ((chain *)i)->next; }
     static inline K &getkey(void *i) { return ((chain *)i)->elem.key; }
     static inline T &getdata(void *i) { return ((chain *)i)->elem.data; }
@@ -1030,6 +1075,7 @@ struct stream
     virtual void close() = 0;
     virtual bool end() = 0;
     virtual offset tell() { return -1; }
+    virtual offset rawtell() { return tell(); }
     virtual bool seek(offset pos, int whence = SEEK_SET) { return false; }
     virtual offset size();
     virtual int read(void *buf, int len) { return 0; }
@@ -1055,11 +1101,37 @@ struct stream
 #endif
 };
 
-extern int iscubeprint(int c);
-extern int uni2cube(int c);
-extern int cube2uni(int c);
-extern int decodeutf8(uchar *dst, uchar *src, int len, int *carry = NULL);
+enum
+{
+    CT_PRINT   = 1<<0,
+    CT_SPACE   = 1<<1,
+    CT_DIGIT   = 1<<2,
+    CT_ALPHA   = 1<<3,
+    CT_LOWER   = 1<<4,
+    CT_UPPER   = 1<<5,
+    CT_UNICODE = 1<<6
+};
+extern const uchar cubectype[256];
+static inline int iscubeprint(uchar c) { return cubectype[c]&CT_PRINT; }
+static inline int iscubespace(uchar c) { return cubectype[c]&CT_SPACE; }
+static inline int iscubealpha(uchar c) { return cubectype[c]&CT_ALPHA; }
+static inline int iscubealnum(uchar c) { return cubectype[c]&(CT_ALPHA|CT_DIGIT); }
+static inline int iscubelower(uchar c) { return cubectype[c]&CT_LOWER; }
+static inline int iscubeupper(uchar c) { return cubectype[c]&CT_UPPER; }
+static inline int cube2uni(uchar c)
+{
+    extern const int cube2unichars[256];
+    return cube2unichars[c];
+}
+static inline uchar uni2cube(int c)
+{
+    extern const int uni2cubeoffsets[8];
+    extern const uchar uni2cubechars[];
+    return uint(c) <= 0x7FF ? uni2cubechars[uni2cubeoffsets[c>>8] + (c&0xFF)] : 0;
+}
+extern int decodeutf8(uchar *dst, int dstlen, uchar *src, int srclen, int *carry = NULL);
 extern int encodeutf8(uchar *dstbuf, int dstlen, uchar *srcbuf, int srclen, int *carry = NULL);
+
 extern char *makerelpath(const char *dir, const char *file, const char *prefix = NULL, const char *cmd = NULL);
 extern char *path(char *s);
 extern char *path(const char *s, bool copy);
