@@ -25,7 +25,7 @@ namespace game
 
 	void addcoin(char *s, int *x)
 	{
-		pickups.add(new pickup_generic(s));
+		pickups.add(new pickup_generic(s, PICKUP_COIN));
 		pickup_generic *p = (pickup_generic *) pickups[pickups.length() - 1];
 		p->amount = *x;
 	}
@@ -33,27 +33,24 @@ namespace game
 
 	void addhealth(char *s, int *x)
 	{
-		pickups.add(new pickup_generic(s));
+		pickups.add(new pickup_generic(s, PICKUP_HEALTH));
 		pickup_generic *p = (pickup_generic *) pickups[pickups.length() - 1];
-		p->type = PICKUP_HEALTH;
 		p->amount = *x;
 	}
 	COMMAND(addhealth, "si");
 
 	void addtime(char *s, int *x)
 	{
-		pickups.add(new pickup_generic(s));
+		pickups.add(new pickup_generic(s, PICKUP_TIME));
 		pickup_generic *p = (pickup_generic *) pickups[pickups.length() - 1];
-		p->type = PICKUP_TIME;
 		p->amount = *x;
 	}
 	COMMAND(addtime, "si");
 
 	void addlife(char *s, int *x)
 	{
-		pickups.add(new pickup_generic(s));
+		pickups.add(new pickup_generic(s, PICKUP_LIVES));
 		pickup_generic *p = (pickup_generic *) pickups[pickups.length() - 1];
-		p->type = PICKUP_LIVES;
 		p->amount = *x;
 	}
 	COMMAND(addlife, "si");
@@ -62,7 +59,6 @@ namespace game
 	{
 		pickups.add(new pickup_weapon(s));
 		pickup_weapon *p = (pickup_weapon *) pickups[pickups.length() - 1];
-		p->type = PICKUP_WEAPON;
 		p->projectile = *x;
 		p->sound = *y;
 		p->cooldown = *z;
@@ -70,15 +66,21 @@ namespace game
 	}
 	COMMAND(addweapon, "ssiii");
 
-	void addarmour(char *s, char *t, int *x)
+	void addarmour(char *s, char *t, int *x, char *tex, int *col)
 	{
 		pickups.add(new pickup_armour(s));
 		pickup_armour *p = (pickup_armour *) pickups[pickups.length() - 1];
-		p->type = PICKUP_ARMOUR;
-		p->armour = min((int) ARM_SPIKE, max((int) ARM_PLAIN, *x));
+		p->armour = clamp<int, int>(*x, ARM_PLAIN, ARM_MAX);
 		p->attachmdl = newstring(t);
+		if(*tex)
+		{
+			delete[] p->tex;
+			p->tex = newstring(tex);
+			p->icon = textureload(tex, 3);
+		}
+		if(*col) p->iconcolour = *col;
 	}
-	COMMAND(addarmour, "ssi");
+	COMMAND(addarmour, "ssisi");
 
 	ICOMMAND(resetpickups, "", (), pickups.deletecontents(); boxdefs.deletecontents();)
 
@@ -225,13 +227,13 @@ namespace game
 
 	void startmap(const char *name)   // called just after a map load
 	{
-		if(sspobjs.length())
-		{
-			sspobjs.remove(0); //remove the player
-		}
+		sspobjs.removeobj(player1); //remove the player
 		sspobjs.deletecontents();
 		sspobjs.add(player1);
 
+		player1->~sspchar();
+		new(player1) sspchar();
+
 		secsallowed = 300;
 		secsremain = 300;
 
@@ -240,16 +242,16 @@ namespace game
 
 		setcamera(0); //set the camera if it exists
 
-        	copystring(clientmap, name ? name : "");
+		copystring(clientmap, name ? name : "");
 		intermission = false;
 		maptime = lastmillis;
 		entities::prepareents();
 		initialisemonsters();
 		findplayerspawn(player1, -1);
 		player1->lastpain = lastmillis; //for spawning invulnerability
-        	conoutf(CON_GAMEINFO, "\f2D to move right, A to move left, W to jump, and click to attack, have fun!");
+		conoutf(CON_GAMEINFO, "\f2D to move right, A to move left, W to jump, and click to attack, have fun!");
 
-        	if(identexists("mapstart")) execute("mapstart");
+		if(identexists("mapstart")) execute("mapstart");
 	}
 
 	void edittoggled(bool on)
@@ -296,9 +298,17 @@ namespace game
 		return player1->state!=CS_DEAD && !intermission;
 	}
 
-	bool allowdoublejump()
+	bool allowdoublejump(physent *d)
 	{
-		if(player1->armour==ARM_FLY && lastmillis - player1->lastjump > 600) return true;
+		sspchar *sc = (sspchar *) d;
+		if(sc->etype != ENT_CHAR && sc->etype != ENT_ENEMY)
+			return false;
+
+		if(sc->armour==ARM_FLY && lastmillis - sc->lastjump > 600)
+			return true;
+		if(sc->armour == ARM_DJUMP && lastmillis - sc->lastjump > sc->timeinair)
+			return true;
+
 		return false;
 	}
 
@@ -402,54 +412,60 @@ namespace game
 			secsremain = 300;
 			intermission = false;
 			maptime = lastmillis;
+
+			physicsframe();
+			moveplayer(player1, 10, true);
+			return;
+		}
+
+		disablezoom();
+		fixplayeryaw();
+
+		if(mousetarget)
+		{
+			vec delta = vec(player1->yaw * RAD, 0);
+			//if(delta.x < -1e-5f || delta.y < -1e-5f)
+				//delta.mul(-1);
+
+			float yaw = 0;
+			if(entities::ents.inrange(cameraent) && entities::ents[cameraent]->type == CAMERA)
+				yaw = entities::ents[cameraent]->attr[1];
+			if(closestyaw(player1->yaw - yaw) == 180 || closestyaw(player1->yaw - yaw) == 270)
+				delta.mul(vec(-1, -1, 0));
+
+			delta.mul(mousex);
+			delta.z = mousey;
+
+			if(!delta.iszero())
+				vectoyawpitch(delta, player1->yaw, player1->pitch);
 		}
 		else
-		{
-			disablezoom();
-			fixplayeryaw();
-
-			if(mousetarget)
-			{
-				vec delta = vec(player1->yaw * RAD, 0);
-				//if(delta.x < -1e-5f || delta.y < -1e-5f)
-					//delta.mul(-1);
-
-				float yaw = 0;
-				if(entities::ents.inrange(cameraent) && entities::ents[cameraent]->type == CAMERA)
-					yaw = entities::ents[cameraent]->attr[1];
-				if(closestyaw(player1->yaw - yaw) == 180 || closestyaw(player1->yaw - yaw) == 270)
-					delta.mul(vec(-1, -1, 0));
-
-				delta.mul(mousex);
-				delta.z = mousey;
-
-				if(!delta.iszero())
-					vectoyawpitch(delta, player1->yaw, player1->pitch);
-			}
-			else
-				player1->pitch = 0;
-		}
+			player1->pitch = 0;
 
 		if (secsremain <= 0) intermission = true;
 
-		physicsframe();
-		updateprojs();
-
-		if (!intermission)
+		if(!intermission)
 		{
+			physicsframe();
+			updateprojs();
 			secsremain = secsallowed - (lastmillis - maptime)/1000;
 			loopvrev(sspobjs)
 			{
 				if(!sspobjs[i]->update())
 				{
-					delete sspobjs[i];
-					sspobjs.remove(i);
+					sspent *s = sspobjs.remove(i);
+					clearprojs(s);
+					delete s;
 				}
 				else if(sspobjs[i]->stool())
 				{
 					sspobjs[i]->takedamage(1);
 				}
 			}
+			loopvrev(entities::platforms)
+			{
+				entities::platforms[i]->update();
+			}
 			loopvrev(entities::items)
 			{
 				if(!entities::items[i]->update())
@@ -480,13 +496,20 @@ namespace game
 
 	void gameplayhud(int w, int h)
 	{
+		static Texture *character = NULL, *overlay = NULL, *hp = NULL, *time = NULL, *coin = NULL, *shield = NULL;
+		if(!character) character = textureload("data/ssp/hud/character", 3);
+		if(!overlay) overlay = textureload("data/ssp/hud/overlay", 3);
+		if(!hp) hp = textureload("data/ssp/hud/hp", 3);
+		if(!time) time = textureload("data/ssp/hud/time", 3);
+		if(!coin) coin = textureload("data/ssp/hud/coin", 3);
+		if(!shield) shield = textureload("data/ssp/hud/shield", 3);
+
 		glPushMatrix();
 		float scale = min (w / 1600.0f, h / 1200.0f);
 		glScalef(scale, scale, 1);
 
 		float right = w / scale, bottom = h / scale; // top and left are ALWAYS 0
 
-
 		loopvrev(eventimages)
 		{
 			if(eventimages[i]->deltime < lastmillis)
@@ -502,9 +525,7 @@ namespace game
 			numw = 1;
 
 		while(eventimages.length() > numw * numw)
-		{
 			numw++;
-		}
 
 		float size = 768.0f / numw;
 
@@ -517,8 +538,7 @@ namespace game
 					break;
 
 				eventimage *img = eventimages[index];
-
-				settexture(img->tex, 3);
+				settexture(img->tex);
 
 				if(img->deltime - lastmillis <= 500)
 					glColor4f(1, 1, 1, (img->deltime - lastmillis) / 500.0);
@@ -529,12 +549,12 @@ namespace game
 
 				quad(right / 2 - 384 + size * j, bottom / 2 - 384 + size * i, size, size);
 
-				settexture("data/hud/2.3/guioverlay", 3);
+				settexture(overlay);
 				quad(right / 2 - 384 + size * j, bottom / 2 - 384 + size * i, size, size);
 			}
 		}
 
-		settexture("data/hud/2.3/icons/rc", 3); //character
+		settexture(character);
 
 		if(lastmillis < player1->lastpain + 2000)
 			glColor4f(1, .5, .5, 1); //set colours, he obviously had a recent owie
@@ -543,40 +563,53 @@ namespace game
 
 		quad(32, bottom - 224, 192, 192);
 
-		settexture("data/ssp/hud/hp", 3); //HP, try a cube or something
-		glColor4f(1- (0.5f * (player1->health - 1)), 0.5f * (player1->health - 1) , 0, 1);
-		loopi(player1->health)
+		settexture(hp); //HP, try a cube or something
+
+		switch(player1->health)
 		{
-			quad(256 + 80 * i, bottom - 96, 64, 64);
+			case 0:
+			case 1:
+				glColor3f(1, 0, 0);
+				break;
+			case 2:
+				glColor3f(.8, 8, 0);
+				break;
+			default:
+			case 3:
+				glColor3f(0, 1, 0);
+				break;
 		}
 
-		settexture("data/ssp/hud/time", 3); //clock
+		loopi(player1->health)
+			quad(256 + 80 * i, bottom - 96, 64, 64);
+
+		settexture(time); //clock
 		glColor4f(1, 1, 1, 1);
 		quad(right / 2 - 224, bottom - 160, 128, 128);
 
-		settexture("data/ssp/hud/coin", 3); //bling bling!
+		settexture(coin); //bling bling!
 		glColor4f(1, 1, 1, 1);
 		quad(right - 448, bottom - 160, 128, 128);
 
-		settexture("data/ssp/hud/shield", 3); //are you protected? :P
-		switch(player1->armour)
+
+		if(player1->armour && pickups.inrange(player1->armourvec))
 		{
-			case ARM_PLAIN:
-				glColor4f(.75, 1, .5, 1);
-				break;
-			case ARM_ATTRACT:
-				glColor4f(0., .5, 1, 1);
-				break;
-			case ARM_FLY:
-				glColor4f(1, 1, 1, 1);
-				break;
-			case ARM_SPIKE:
-				glColor4f(0.75, 0.75, 0.75, 1);
-				break;
-			default:
-				glColor4f(1, 1, 1, .25);
-				break;
+			pickup_armour *arm = (pickup_armour *) pickups[player1->armourvec];
+			settexture(arm->icon);
+
+			uchar col[3] = {
+				uchar((arm->iconcolour >> 16) & 255),
+				uchar((arm->iconcolour >> 8 ) & 255),
+				uchar((arm->iconcolour      ) & 255)
+			};
+			glColor3ubv(col);
 		}
+		else
+		{
+			settexture(shield); //are you protected? :P
+			glColor4f(1, 1, 1, .25);
+		}
+
 		quad(32, bottom - 384, 128, 128);
 
 		draw_textf("%d", 256, bottom - 160, player1->lives);
@@ -652,13 +685,16 @@ namespace game
 		}
 	}
 
-	int numdynents() { return sspobjs.length(); }
+	int numdynents() { return sspobjs.length() + entities::platforms.length(); }
 
 	dynent *iterdynents(int i)
 	{
 		if(i < sspobjs.length())
 			return sspobjs[i];
-		// i -= sspobjs.length();
+		i -= sspobjs.length();
+		if(i < entities::platforms.length())
+			return entities::platforms[i];
+		// i -= entities::platforms.length();
 		return NULL;
 	}
 
@@ -685,19 +721,19 @@ namespace game
 					pickup_generic *gp = (pickup_generic *) p;
 					static const char *cmds[] = {"addcoin", "addhealth", "addtime", "addlife"};
 
-					f->printf("%s %s %i\n", cmds[gp->type], gp->mdl, gp->amount);
+					f->printf("%s %s %i //%i\n", cmds[gp->type], escapestring(gp->mdl), gp->amount, i);
 					break;
 				}
 				case PICKUP_WEAPON:
 				{
 					pickup_weapon *wp = (pickup_weapon *) p;
-					f->printf("addweapon %s %s %i %i %i\n", wp->mdl, wp->attachmdl, wp->projectile, wp->sound, wp->cooldown);
+					f->printf("addweapon %s %s %i %i %i //%i\n", escapestring(wp->mdl), escapestring(wp->attachmdl), wp->projectile, wp->sound, wp->cooldown, i);
 					break;
 				}
 				case PICKUP_ARMOUR:
 				{
 					pickup_armour *ap = (pickup_armour *) p;
-					f->printf("addarmour %s %s %i\n", ap->mdl, ap->attachmdl, ap->armour);
+					f->printf("addarmour %s %s %i %s 0x%.6X //%i\n", escapestring(ap->mdl), escapestring(ap->attachmdl), ap->armour, escapestring(ap->tex), ap->iconcolour, i);
 					break;
 				}
 			}
@@ -707,7 +743,8 @@ namespace game
 		{
 			loopvj(boxdefs[i]->inv)
 			{
-				f->printf(j ? "addboxitem %i\n" : "addbox %i\n", boxdefs[i]->inv[j]);
+				if(j) f->printf("addboxitem %i\n", boxdefs[i]->inv[j]);
+				else f->printf("addbox %i //%i\n", boxdefs[i]->inv[j], i);
 			}
 		}
 		f->putchar('\n');
