@@ -1,5 +1,38 @@
 #include "sspgame.h"
 
+void platform::update()
+{
+	extentity *orig = dest;
+
+	vec dir = vec(dest->o).sub(o);
+	while(dir.magnitude() == 0)
+	{
+		loopv(entities::ents)
+		{
+			extentity &e = *entities::ents[i];
+			if(e.type == PLATFORMROUTE && e.attr[0] == dest->attr[1])
+			{
+				dest = entities::ents[i];
+				break;
+			}
+		}
+
+		if(dest == orig) return;
+		dir = vec(dest->o).sub(o);
+	}
+
+	dir.rescale(min<float>(speed * curtime / 1000.f, dir.magnitude()));
+
+	moveplatform(this, dir);
+}
+
+void platform::render()
+{
+	rendermodel(&light, mapmodelname(model), ANIM_MAPMODEL|ANIM_LOOP,
+		feetpos(), yaw + 90, 0, 0, MDL_CULL_VFC|MDL_CULL_DIST|MDL_CULL_OCCLUDED
+	);
+}
+
 #define PAINIMMUNITY 2000
 
 bool sspchar::stool()
@@ -117,9 +150,9 @@ void sspchar::render()
 	if(state == CS_EDITING && !isthirdperson())
 		return;
 
-	int anim = ANIM_ATTACK1+gunselect,
+	int anim = ANIM_ATTACK,
 	delay = 300,
-	hold = ANIM_HOLD1|ANIM_LOOP;
+	hold = ANIM_IDLE|ANIM_LOOP;
 
 	if(game::intermission && state!=CS_DEAD)
 	{
@@ -137,7 +170,7 @@ void sspchar::render()
 			a[ai].m = NULL;
 			a[ai].name = ((pickup_weapon *) game::pickups[gunselect])->attachmdl;
 			a[ai].tag = "tag_weapon";
-			a[ai].anim = ANIM_VWEP_IDLE|ANIM_LOOP;
+			a[ai].anim = ANIM_IDLE|ANIM_LOOP;
 			a[ai].basetime = 0;
 			ai++;
 		}
@@ -146,7 +179,7 @@ void sspchar::render()
 			a[ai].m = NULL;
 			a[ai].name = ((pickup_armour *) game::pickups[armourvec])->attachmdl;
 			a[ai].tag = "tag_shield";
-			a[ai].anim = ANIM_VWEP_IDLE|ANIM_LOOP;
+			a[ai].anim = ANIM_IDLE|ANIM_LOOP;
 			a[ai].basetime = 0;
 			ai++;
 		}
@@ -162,6 +195,8 @@ void sspchar::render()
 		renderclient(this, "rc", a[0].tag ? a : NULL, hold, anim, delay, lastaction, game::intermission ? 0 : lastpain, 1.0f, true);
 }
 
+VAR(noai, 0, 0, 1);
+
 bool sspmonster::update()
 {
 	if(state == CS_DEAD && ragdoll)
@@ -175,7 +210,24 @@ bool sspmonster::update()
 	if(state == CS_ALIVE)
 	{
 		game::checkmonsteritems(this);
+		if(noai)
+		{
+			stopmoving();
+			goto end;
+		}
+
+		vec d = vec(yaw * RAD, 0);
+		if(!move && !timeinair) move = 1;
+		if(game::player1->o.dist(o) < 40)
+		{
+			if(d.dot(vec(game::player1->o).sub(o)) < 0)
+				yaw += 180;
+			jumping = true;
+		}
+		else if(blocked) yaw += 180;
 	}
+
+	end:
 	return !(state == CS_DEAD && lastpain + PAINIMMUNITY < lastmillis);
 }
 
@@ -195,7 +247,7 @@ void sspmonster::render()
 	if(game::monstertypes.inrange(mtype))
 	{
 		if(lastpain + PAINIMMUNITY > lastmillis && (lastmillis % 100) > 50) return; //flicker if hurt
-			renderclient(this, game::monstertypes[mtype]->mdl, NULL, 0, ANIM_HOLD1|ANIM_LOOP, 300, lastaction, lastpain);
+			renderclient(this, game::monstertypes[mtype]->mdl, NULL, 0, ANIM_IDLE|ANIM_LOOP, 300, lastaction, lastpain);
 
 	}
 }
@@ -217,7 +269,7 @@ bool sspitem::update()
 
 		if(game::player1->armour==ARM_ATTRACT)
 		{
-			regularshape(PART_LIGHTNING, 8, 0x007FFF, 31, 2, 200, vec(o.x+rnd(8)-4, o.y+rnd(8)-4, o.z+rnd(8)), .25, 0, 0 ); //make it look pretty
+			regularshape(PART_LIGHTNING, 8, 0x007FFF, 22, 2, 200, vec(o.x+rnd(8)-4, o.y+rnd(8)-4, o.z+rnd(8)), .25, 0, 0 ); //make it look pretty
 		}
 
 		if(dist < 8 && entities::pickuppowerup(game::player1, this))
@@ -245,39 +297,15 @@ void sspitem::render()
 	);
 }
 
-bool sspbox::update()
+void sspbox::trigger()
 {
-	if(explode && explode > lastmillis)
+	lasttouch = lastmillis;
+
+	defformatstring(ds)("box_%i", script);
+	if(identexists(ds)) execute(ds);
+
+	if(inventory.length())
 	{
-		//explosion here
-		return false;
-	}
-
-	vec pos = feetpos(), &pl = game::player1->o;
-
-	float xdelta = xradius + game::player1->xradius;
-	float ydelta = yradius + game::player1->yradius;
-
-	float x = abs(pos.x - pl.x);
-	float y = abs(pos.y - pl.y);
-	float z = abs(pos.z - pl.z);
-
-	if(game::player1->vel.z + game::player1->falling.z > 0 &&
-		x < xdelta &&
-		y < ydelta &&
-		z <= game::player1->aboveeye + (game::player1->vel.z + game::player1->falling.z) * curtime / 800.0f
-	)
-	{
-		game::player1->vel.z = 0;
-		lasttouch = lastmillis;
-
-		defformatstring(ds)("box_%i", script);
-		if(identexists(ds)) execute(ds);
-
-		if(!inventory.length())
-			return true;
-
-
 		if(flags & BOX_PINJATA)
 		{
 			while(inventory.length())
@@ -297,66 +325,62 @@ bool sspbox::update()
 
 			inventory.remove(0);
 		}
+	}
 
-		if(flags & BOX_DESTROY && !inventory.length())
+	if(explode)
+	{
+		explode = lastmillis;
+		return;
+	};
+
+	if(!inventory.length() && (flags & BOX_DESTROY))
+	{
+		if(flags & BOX_EXPLODE)
+			explode = lastmillis + 3000; // explode in 3 seconds
+	}
+	else if((flags & (BOX_DESTROY|BOX_EXPLODE)) == BOX_EXPLODE)
+	{
+		flags |= BOX_PINJATA;
+		explode = lastmillis + 3000;
+	}
+}
+
+bool sspbox::update()
+{
+	if(explode)
+	{
+		if(explode < lastmillis)
 		{
-			if(flags & BOX_EXPLODE)
-			{
-				explode = lastmillis + 3000; // explode in 3 seconds
-			}
-			else
-			{
-				return false;
-			}
+			game::explode(o, 32, 1);
+			return false;
 		}
 	}
 
-	return true;
+	vec pos = feetpos(), &pl = game::player1->o;
+
+	float xdelta = xradius + game::player1->xradius;
+	float ydelta = yradius + game::player1->yradius;
+
+	float x = abs(pos.x - pl.x);
+	float y = abs(pos.y - pl.y);
+	float z = abs(pos.z - pl.z);
+
+	if(game::player1->vel.z + game::player1->falling.z > 0 &&
+		x < xdelta &&
+		y < ydelta &&
+		z <= game::player1->aboveeye + (game::player1->vel.z + game::player1->falling.z) * curtime / 800.0f
+	)
+	{
+		game::player1->vel.z = 0;
+		trigger();
+	}
+
+	return inventory.length() || (flags & (BOX_DESTROY|BOX_EXPLODE)) != BOX_DESTROY;
 }
 
 void sspbox::takedamage(int amount, bool immunity)
 {
-	if(!explode)
-		lasttouch = lastmillis;
-
-	defformatstring(ds)("box_%i", script);
-	if(identexists(ds)) execute(ds);
-
-	if(!inventory.length())
-		return;
-
-
-	if(flags & BOX_PINJATA)
-	{
-		while(inventory.length())
-		{
-			sspitem *item = new sspitem();
-			item->init(vec(o).add(vec(0, 0, 4)), inventory[0], 0, false, lastmillis + 50000);
-			entities::items.add(item);
-
-			inventory.remove(0);
-		}
-	}
-	else
-	{
-		sspitem *item = new sspitem();
-		item->init(vec(o).add(vec(0, 0, 4)), inventory[0], false, lastmillis + 50000);
-		entities::items.add(item);
-
-		inventory.remove(0);
-	}
-
-	if(flags & BOX_DESTROY && !inventory.length())
-	{
-		if(flags & BOX_EXPLODE)
-		{
-			explode = lastmillis + 3000; // explode in 3 seconds
-		}
-		else
-		{
-			return;
-		}
-	}
+	trigger();
 }
 
 void sspbox::render()
@@ -364,7 +388,9 @@ void sspbox::render()
 	if(editmode)
 		return;
 
-	vec pos = o;
+	if(explode && lastmillis % 750 < 375) return;
+
+	vec pos = o; pos.z -= eyeheight;
 	float yaw = 0;
 	if(lasttouch + 500 > lastmillis)
 	{
@@ -374,4 +400,4 @@ void sspbox::render()
 	rendermodel(&light, mapmodelname(mdl), ANIM_MAPMODEL|ANIM_LOOP, pos, yaw, 0, 0,
 		MDL_SHADOW|MDL_CULL_VFC|MDL_CULL_DIST|MDL_CULL_OCCLUDED|MDL_LIGHT
 	);
-}
\ No newline at end of file
+}
