@@ -10,6 +10,55 @@ enum
 	CON_KILL = 1<<10 //just when an enemy is defeated :P
 };
 
+struct eventimage;
+struct box_def;
+struct enemy;
+struct pickup;
+struct platform;
+struct sspchar;
+struct sspent;
+struct sspitem;
+struct sspmonster;
+
+namespace entities
+{
+	extern vector<extentity *> ents;
+	extern vector<sspitem *> items;
+	extern vector<platform *> platforms;
+	extern void prepareents();
+	extern void trypickup(int n, sspchar *d);
+	extern void renderentities();
+	extern void checkitems(sspchar *d);
+	extern bool pickuppowerup(sspchar *d, sspitem *i);
+}
+
+namespace game
+{
+	extern sspchar *player1;
+	extern vector<sspent *> sspobjs;
+	extern vector<eventimage *> eventimages;
+	extern int secsallowed, secsremain;
+	extern bool intermission;
+	extern vector<pickup *> pickups;
+	extern vector<box_def *> boxdefs;
+	extern int closestyaw(int yaw);
+	extern void clearprojs();
+	extern void explode(const vec &o, int radius, int damage = 1, const sspent *owner = NULL);
+	extern void clearprojs(sspent *d);
+	extern void updateprojs();
+	extern void renderprojs();
+	extern void initialisemonsters();
+	extern void checkmonsteritems(sspmonster *d);
+
+	extern int debug;
+	extern int mousetarget;
+
+	extern vector<enemy *> monstertypes;
+
+	extern void writemonsters(stream *f);
+	extern void writeprojectiles(stream *f);
+}
+
 enum                            // static entity types
 {
 	NOTUSED = ET_EMPTY,         // entity slot not in use in map
@@ -52,6 +101,7 @@ enum //armour : if armour or a weapon is equipped, lose it instead of health
 	ARM_ATTRACT, //quadruples pickup range of well.. pickups
 	ARM_FLY, //the player can jump repeatedly in the air for extra height
 	ARM_SPIKE, //kills whatever lands on the player, at the cost of the armour
+	ARM_DJUMP,
 	ARM_MAX
 };
 
@@ -67,16 +117,12 @@ enum
 struct pickup
 {
 	const char *mdl;
-	int type;
+	const int type;
 
-	pickup()
-	{
-		mdl = NULL;
-		type = PICKUP_COIN;
-	}
+	pickup(const char *m, int t) : mdl(newstring(m)), type (t) {}
 	virtual ~pickup()
 	{
-		if(mdl) delete[] mdl;
+		delete[] mdl;
 	}
 };
 
@@ -84,11 +130,7 @@ struct pickup_generic : pickup //hp, time, coins and lives
 {
 	int amount;
 
-	pickup_generic(const char *m)
-	{
-		mdl = newstring(m);
-		amount = 1;
-	}
+	pickup_generic(const char *m, int type) : pickup(m, type), amount(1) {}
 	~pickup_generic() {}
 };
 
@@ -96,17 +138,15 @@ struct pickup_armour : pickup
 {
 	const char *attachmdl;
 	int armour; //the armour type
+	const char *tex;
+	Texture *icon;
+	int iconcolour;
 
-	pickup_armour(const char *m)
-	{
-		mdl = newstring(m);
-		attachmdl = NULL;
-		armour = ARM_PLAIN;
-	}
+	pickup_armour(const char *m) : pickup(m, PICKUP_ARMOUR), attachmdl(NULL), armour(ARM_PLAIN), tex(newstring("data/ssp/hud/shield")), icon(textureload(tex, 3)), iconcolour(0xFFFFFF) {}
 	~pickup_armour()
 	{
-
-		if(attachmdl) delete[] attachmdl;
+		delete[] attachmdl;
+		delete[] tex;
 	}
 };
 
@@ -117,16 +157,10 @@ struct pickup_weapon : pickup
 	int sound;
 	int cooldown;
 
-	pickup_weapon(const char *m)
-	{
-		mdl = newstring(m);
-		projectile = 0;
-		sound = 0;
-		cooldown = 0;
-	}
+	pickup_weapon(const char *m) : pickup(m, PICKUP_WEAPON), attachmdl(NULL), projectile(0), sound(0), cooldown(0) {}
 	~pickup_weapon()
 	{
-		if(attachmdl) delete[] attachmdl;
+		delete[] attachmdl;
 	}
 };
 
@@ -134,37 +168,41 @@ struct sspentity : extentity {}; //extend with additional properties if needed
 
 enum
 {
-    S_JUMP = 0, S_LAND, S_UNUSED, S_TELEPORT, S_SPLASH1, S_SPLASH2, S_CG,
-    S_RLFIRE, S_RUMBLE, S_JUMPPAD, S_WEAPLOAD, S_ITEMAMMO, S_ITEMHEALTH,
-    S_ITEMARMOUR, S_ITEMPUP, S_ITEMSPAWN,  S_NOAMMO, S_PUPOUT,
-    S_PAIN1, S_PAIN2, S_PAIN3, S_PAIN4, S_PAIN5, S_PAIN6,
-    S_DIE1, S_DIE2,
-    S_FLAUNCH, S_FEXPLODE,
-    S_SG, S_PUNCH1,
-    S_GRUNT1, S_GRUNT2, S_RLHIT,
-    S_PAINO,
-    S_PAINR, S_DEATHR,
-    S_PAINE, S_DEATHE,
-    S_PAINS, S_DEATHS,
-    S_PAINB, S_DEATHB,
-    S_PAINP, S_PIGGR2,
-    S_PAINH, S_DEATHH,
-    S_PAIND, S_DEATHD,
-    S_PIGR1, S_ICEBALL, S_SLIMEBALL, S_PISTOL,
+	S_JUMP = 0, S_LAND, S_RIFLE, S_TELEPORT, S_SPLASH1, S_SPLASH2, S_CG,
+	S_RLFIRE, S_RUMBLE, S_JUMPPAD, S_WEAPLOAD, S_ITEMAMMO, S_ITEMHEALTH,
+	S_ITEMARMOUR, S_ITEMPUP, S_ITEMSPAWN,  S_NOAMMO, S_PUPOUT,
+	S_PAIN,
+	S_DIE,
+	S_FLAUNCH, S_FEXPLODE,
+	S_SG, S_PUNCH1,
+	S_GRUNT1, S_GRUNT2, S_RLHIT,
+	S_PAINO,
+	S_PAINR, S_DEATHR,
+	S_PAINE, S_DEATHE,
+	S_PAINS, S_DEATHS,
+	S_PAINB, S_DEATHB,
+	S_PAINP, S_PIGGR2,
+	S_PAINH, S_DEATHH,
+	S_PAIND, S_DEATHD,
+	S_PIGR1, S_ICEBALL, S_SLIMEBALL, S_PISTOL,
 
-    S_V_BASECAP, S_V_BASELOST,
-    S_V_FIGHT,
-    S_V_BOOST, S_V_BOOST10,
-    S_V_QUAD, S_V_QUAD10,
-    S_V_RESPAWNPOINT,
+	S_V_BASECAP, S_V_BASELOST,
+	S_V_FIGHT,
+	S_V_BOOST, S_V_BOOST10,
+	S_V_QUAD, S_V_QUAD10,
+	S_V_RESPAWNPOINT,
 
-    S_FLAGPICKUP,
-    S_FLAGDROP,
-    S_FLAGRETURN,
-    S_FLAGSCORE,
-    S_FLAGRESET,
+	S_FLAGPICKUP,
+	S_FLAGDROP,
+	S_FLAGRETURN,
+	S_FLAGSCORE,
+	S_FLAGRESET,
 
-    S_BURN
+	S_BURN,
+	S_CHAINSAW_ATTACK,
+	S_CHAINSAW_IDLE,
+
+	S_HIT
 };
 
 enum
@@ -177,9 +215,9 @@ enum
 
 struct sspent : dynent
 {
-	int etype;
-	sspent() {}
-	~sspent() {}
+	const int etype;
+	sspent(int t) : etype(t) {}
+	virtual ~sspent() {}
 
 	virtual void respawn()
 	{
@@ -208,7 +246,7 @@ struct sspchar : sspent
 	editinfo *edit;
 	int lives, coins, checkpoint;
 
-	sspchar() : maxhealth(3), weight(100), lastpain(0), edit(NULL), lives(5), coins(0) { etype = ENT_CHAR; respawn(); }
+	sspchar(int t = ENT_CHAR) : sspent(t), maxhealth(3), weight(100), lastpain(0), edit(NULL), lives(5), coins(0) { respawn(); }
 
 	void respawn()
 	{
@@ -242,25 +280,24 @@ struct enemy //enemy declaration
 		mdl = newstring(_mdl[0] ? _mdl : "rc/red");
 		health = _hlt ? _hlt : 3;
 		speed = _spd ? _spd : 80;
-		painsound = _ps ? _ps : S_PAIN1;
-		diesound = _ds ? _ds : S_DIE1;
+		painsound = _ps ? _ps : S_PAIN;
+		diesound = _ds ? _ds : S_DIE;
 	}
 	~enemy() {DELETEA(mdl);}
 };
 
 struct sspmonster : sspchar
 {
-	int mtype;
+	const int mtype;
 
-	sspmonster(extentity &e, enemy &d)
+	sspmonster(extentity &e, enemy &d) : sspchar(ENT_ENEMY), mtype(e.attr[1])
 	{
-		etype = ENT_ENEMY;
 		type = ENT_AI;
-		mtype = e.attr[1];
 		collidetype = COLLIDE_AABB;
 
 		health = maxhealth = d.health;
 		maxspeed = d.speed;
+		setbbfrommodel(this, d.mdl);
 
 		o = newpos = e.o; yaw = e.attr[0];
 		o.z += eyeheight;
@@ -279,6 +316,23 @@ struct sspmonster : sspchar
 	void render();
 };
 
+struct platform : dynent
+{
+	extentity *dest;
+	int model;
+	int speed;
+
+	void update();
+	void render();
+
+	platform(int m) : dest(NULL), model(m)
+	{
+		physent::type = ENT_INANIMATE;
+		setbbfrommodel(this, mapmodelname(model));
+	}
+	~platform() {}
+};
+
 enum
 {
 	BOX_PERSIST = 0, //stays after clearing
@@ -305,9 +359,8 @@ struct sspbox : sspent
 
 	int explode, lasttouch; //explode is non zero when touched and last item is rejected
 
-	sspbox()
+	sspbox() : sspent(ENT_BOX)
 	{
-		etype = ENT_BOX;
 		explode = lasttouch = 0;
 	}
 	~sspbox() {}
@@ -317,17 +370,24 @@ struct sspbox : sspent
 		loopv(b.inv)
 			inventory.add(b.inv[i]);
 
-		collidetype = COLLIDE_OBB;
-
-		setbbfrommodel(this, mapmodelname(e.attr[1]));
-		if(eyeheight <= 0) eyeheight = 2 * xradius; //WARNING above function might set a negative eyeheight - this causes a lot of ANNOYING problems
-
 		mdl = e.attr[1];
 		flags = e.attr[3];
 		script = e.attr[4];
 		o = e.o;
-		yaw = 0;
+		yaw = e.attr[0];
+
+
+		collidetype = COLLIDE_OBB;
+		setbbfrommodel(this, mapmodelname(e.attr[1]));
+		if(eyeheight <= 0)
+		{
+			conoutf("WARNING: negative eyeheight for %s, expect odd behaviour", mapmodelname(e.attr[1]));
+			eyeheight = xradius;
+		}
+
+		o.z += eyeheight;
 	}
+	void trigger();
 	bool update();
 	void takedamage(int amount, bool immunity = false);
 	void render();
@@ -373,33 +433,25 @@ struct proj //ectiles
 		radius = _rad ? _rad : 1;
 		force = _frce ? _frce : 1;
 		travelsound = _ts ? _ts : 1;
-		speed = _spd ? _spd : 1;
+		speed = _spd;
 
 		didx = DECAL_BURN,dcol.x = dcol.y = dcol.z = 0xFF, drad = 50;
 	}
 	~proj() {DELETEA(mdl);}
-
-
-
-
 };
 
 struct projectile //the actual flying ones
 {
-	int prj;
+	const int prj;
 	vec o, d;
 	sspent *owner;
 
-	projectile(int _prj, vec _o, float _yaw, float _pitch, sspent *_owner = NULL)
+	projectile(int _prj, vec _o, float _yaw, float _pitch, sspent *_owner = NULL) : prj(_prj), o(_o), owner(_owner)
 	{
-		prj = _prj;
-		owner = _owner;
-
 		vecfromyawpitch(_yaw, _pitch, 1, 0, d);
 		vec p = d;
 		p.mul(owner ? owner->radius * 2 : 4);
 
-		o = _o;
 		o.add(p);
 	}
 	~projectile() {}
@@ -407,53 +459,29 @@ struct projectile //the actual flying ones
 
 struct eventimage
 {
-	const char *tex;
+	Texture *tex;
 	int deltime;
 	int starttime;
 
-	eventimage(const char *_tex, int _dt)
+	eventimage(const char *_tex, int _dt) : starttime(lastmillis)
 	{
-		tex = newstring(_tex[0] ? _tex : "data/sandboxlogo");
+		tex = textureload(*_tex ? _tex : "data/sandboxlogo", 3);
 		deltime = lastmillis + (_dt ? _dt : 2000);
-		starttime = lastmillis;
 	}
-	~eventimage() {DELETEA(tex);}
+	~eventimage() {}
 };
 
-namespace entities
+enum
 {
-	extern vector<extentity *> ents;
-	extern vector<sspitem *> items;
-	extern void prepareents();
-	extern void trypickup(int n, sspchar *d);
-	extern void renderentities();
-	extern void checkitems(sspchar *d);
-	extern bool pickuppowerup(sspchar *d, sspitem *i);
-}
+	ANIM_POWERUP = ANIM_GAMESPECIFIC,
+	ANIM_ATTACK, ANIM_LOSE, ANIM_WIN,
+	NUMANIMS
+};
 
-namespace game
+static const char * const animnames[] =
 {
-	extern sspchar *player1;
-	extern vector<sspent *> sspobjs;
-	extern vector<eventimage *> eventimages;
-	extern int secsallowed, secsremain;
-	extern bool intermission;
-	extern vector<pickup *> pickups;
-	extern vector<box_def *> boxdefs;
-	extern int closestyaw(int yaw);
-	extern void clearprojs();
-	extern void updateprojs();
-	extern void renderprojs();
-	extern void initialisemonsters();
-	extern void checkmonsteritems(sspmonster *d);
-
-	extern int debug;
-	extern int mousetarget;
-
-	extern vector<enemy *> monstertypes;
-
-	extern void writemonsters(stream *f);
-	extern void writeprojectiles(stream *f);
-}
+    ANIMNAMES,
+	"powerup", "attack", "lose", "win"
+};
 
 #endif
