@@ -54,7 +54,7 @@ namespace game
 		loopv(monstertypes)
 		{
 			enemy &m = *monstertypes[i];
-			f->printf("addmonster %s %i %i %i %i\n", m.mdl, m.health, m.speed, m.painsound, m.diesound);
+			f->printf("addmonster %s %i %i %i %i //%i\n", escapestring(m.mdl), m.health, m.speed, m.painsound, m.diesound, i);
 		}
 	}
 }
