@@ -6,10 +6,13 @@ namespace game
 	{
 		startmodelbatches();
 
-		loopvrev(sspobjs)
+		loopv(sspobjs)
 			sspobjs[i]->render();
 
-		loopvrev(entities::items)
+		loopv(entities::platforms)
+			entities::platforms[i]->render();
+
+		loopv(entities::items)
 			entities::items[i]->render();
 
 		entities::renderentities();
@@ -25,6 +28,12 @@ namespace game
 				defformatstring(ds)("%p", o);
 				particle_textcopy(o->abovehead(), ds, PART_TEXT, 5, 0xFFFFFF, 4);
 			}
+			loopv(entities::platforms)
+			{
+				platform *o = entities::platforms[i];
+				defformatstring(ds)("%p", o);
+				particle_textcopy(o->abovehead(), ds, PART_TEXT, 5, 0xFFFFFF, 4);
+			}
 			loopv(entities::items)
 			{
 				sspitem *o = entities::items[i];
@@ -33,4 +42,13 @@ namespace game
 			}
 		}
 	}
+
+	const char *animname(int i)
+	{
+		i &= 0x7F;
+		if(i >= NUMANIMS) return "";
+		return animnames[i];
+	}
+
+	const int numanims() {return NUMANIMS;}
 }
