@@ -19,11 +19,6 @@ namespace game
 	void dynlighttrack(physent *owner, vec &o, vec &hud) {}
 	void preload() {}
 	void g3d_gamemenus() {}
-	#ifndef NEWGUI
-	bool serverinfostartcolumn(g3d_gui *g, int i) {return false;}
-	void serverinfoendcolumn(g3d_gui *g, int i) {}
-	bool serverinfoentry(g3d_gui *g, int i, const char *name, int port, const char *desc, const char *map, int ping, const vector<int> &attr, int np)  {return false;}
-	#endif
 	bool ispaused() {return false;}
 	void vartrigger(ident *id) {}
 	void bounced(physent *d, const vec &surface) {}
